@@ -681,10 +681,19 @@ void checkresources(void)
                     || !strcmp(list1->string, "FFMPEGVideoHalveFramerate")
                   ) {
                     printf("(FFMPEG only, might be disabled)");
+                } else if(0
+                    || !strcmp(list1->string, "MIDIEnable")
+                    || !strcmp(list1->string, "MIDIMode")
+                    || !strcmp(list1->string, "MIDIInDev")
+                    || !strcmp(list1->string, "MIDIOutDev")
+                    || !strcmp(list1->string, "MIDIDriver")
+                  ) {
+                    printf("(MIDI only, might be disabled)");
                 } else if(0
                     || !strcmp(list1->string, "KeepMonitorOpen")
                     || !strcmp(list1->string, "KeepAspectRatio")
                     || !strcmp(list1->string, "TrueAspectRatio")
+                    || !strcmp(list1->string, "VSync")
                     || !strcmp(list1->string, "Window0Width")
                     || !strcmp(list1->string, "Window0Height")
                     || !strcmp(list1->string, "Window0Xpos")
@@ -1014,13 +1023,26 @@ void checkoptions(void)
                     || !strcmp(list1->string, "-ffmpegvideobitrate")
                   ) {
                     printf("(FFMPEG only, might be disabled)");
+                } else if(0
+                    || !strcmp(list1->string, "-miditype")
+                    || !strcmp(list1->string, "-midi")
+                    || !strcmp(list1->string, "+midi")
+                    || !strcmp(list1->string, "-midiin")
+                    || !strcmp(list1->string, "-midiout")
+                    || !strcmp(list1->string, "-mididrv")
+                  ) {
+                    printf("(MIDI only, might be disabled)");
                 } else if(0
                     || !strcmp(list1->string, "-keepaspect")
                     || !strcmp(list1->string, "+keepaspect")
                     || !strcmp(list1->string, "-trueaspect")
                     || !strcmp(list1->string, "+trueaspect")
+                    || !strcmp(list1->string, "-vsync")
+                    || !strcmp(list1->string, "+vsync")
                     || !strcmp(list1->string, "-keepmonopen")
                     || !strcmp(list1->string, "+keepmonopen")
+                    || !strcmp(list1->string, "-refreshonbreak")
+                    || !strcmp(list1->string, "+refreshonbreak")
                     || !strcmp(list1->string, "-minimized")
                     || !strcmp(list1->string, "+minimized")
                     || !strcmp(list1->string, "-displaydepth")
