@@ -59,7 +59,7 @@ static Boolean IOHIDDevice_GetLongProperty( IOHIDDeviceRef inIOHIDDeviceRef, CFS
             // if this is a number
             if ( CFNumberGetTypeID() == CFGetTypeID( tCFTypeRef ) ) {
                 // get it's value
-                result = CFNumberGetValue( ( CFNumberRef ) tCFTypeRef, kCFNumberSInt32Type, outValue );
+                result = CFNumberGetValue((CFNumberRef)tCFTypeRef, kCFNumberLongType, outValue);
             }
         }
     }
@@ -231,9 +231,9 @@ void joy_hidlib_free_devices(joy_hid_device_array_t *devices)
         devices->internal_devices = NULL;
     }
 
-    if(devices != NULL) {
-        lib_free(devices);
-        devices = NULL;
+    if(devices->devices != NULL) {
+        lib_free(devices->devices);
+        devices->devices = NULL;
     }
 }
 
