@@ -46,32 +46,6 @@ void mousedrv_init(void);
  *         enabled or disabled. */
 void mousedrv_mouse_changed(void);
 
-/** \brief Returns the current mouse X value.
- *
- *  This is a running total of mouse movements and does not
- *  necessarily correspond to any particular screen position.
- *
- *  \return The current X value, in the range 0-65536.
- */
-int mousedrv_get_x(void);
-
-/** \brief Returns the current mouse Y value.
- *
- *  This is a running total of mouse movements and does not
- *  necessarily correspond to any particular screen position.
- *
- *  \return The current Y value, in the range 0-65536.
- */
-int mousedrv_get_y(void);
-
-/** \brief Returns the last time the mouse position changed.
- *
- *  \note A button press or release is not a change.
- *
- *  \return The current X value, in the range 0-65536.
- */
-unsigned long mousedrv_get_timestamp(void);
-
 /** \brief Called by the UI event handler to announce the user has
  *         pressed or released a button.
  *  \param bnumber Which button was pressed or released.
@@ -79,12 +53,4 @@ unsigned long mousedrv_get_timestamp(void);
  */
 void mouse_button(int bnumber, int state);
 
-/** \brief Called by the UI event handler to announce that the mouse
- *         has moved.
- *  \param dx Amount the mouse moved in the X direction
- *  \param dy Amount the mouse moved in the Y direction
- *  \todo  Determine what the actual units here are.
- */    
-void mouse_move(float dx, float dy);
-
 #endif
