@@ -38,6 +38,16 @@
 #include "uicompiletimefeatures.h"
 
 
+/** \brief  Column indexes
+ */
+enum {
+    COL_DESCR = 0,  /**< Symbol description */
+    COL_SYMBOL,     /**< Symbol name in config.h */
+    COL_ISDEFINED   /**< Symbol is defined */
+};
+
+
+
 /** \brief  Handler for the 'response' event of the dialog
  *
  * \param[in,out]   dialog      dialog triggering the event
@@ -54,6 +64,43 @@ static void on_response(GtkDialog *dialog,
 }
 
 
+/** \brief  Sort callback for column \a data
+ *
+ * \param[in,out]   model   tree model
+ * \param[in]       a       tree iterator to first element
+ * \param[in]       b       tree iterator to second element
+ * \param[in]       data    column ID
+ *
+ * \return  0 if equal, -1 if a < b, 1 if a > b
+ */
+static int sort_symbol_func(GtkTreeModel *model,
+                            GtkTreeIter *a,
+                            GtkTreeIter *b,
+                            gpointer data)
+{
+    gchar *name_a = NULL;
+    gchar *name_b = NULL;
+    gint result = 0;
+    gint column_id = GPOINTER_TO_INT(data);
+
+    gtk_tree_model_get(model, a, column_id, &name_a, -1);
+    gtk_tree_model_get(model, b, column_id, &name_b, -1);
+
+    if (name_a == NULL || name_b == NULL) {
+        if (name_a == NULL && name_b == NULL) {
+            /* no data for some reason */
+            return 0;
+        }
+        result =  name_a == NULL ? -1 : 1;
+    }
+    result = g_utf8_collate(name_a, name_b);
+    g_free(name_a);
+    g_free(name_b);
+
+    return result;
+}
+
+
 /** \brief  Create tree model with compile time features
  *
  * \return  GtkTreeStore
@@ -62,7 +109,7 @@ static GtkTreeStore *create_store(void)
 {
     GtkTreeStore *store;
     GtkTreeIter iter;
-    feature_list_t *list;
+    const feature_list_t *list;
 
     store = gtk_tree_store_new(3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
     list = vice_get_feature_list();
@@ -70,9 +117,9 @@ static GtkTreeStore *create_store(void)
     while (list->symbol != NULL) {
         gtk_tree_store_append(store, &iter, NULL);
         gtk_tree_store_set(store, &iter,
-                0, list->descr,
-                1, list->symbol,
-                2, list->isdefined ? "yes" : "no",
+                COL_DESCR, list->descr,
+                COL_SYMBOL, list->symbol,
+                COL_ISDEFINED, list->isdefined ? "yes" : "no",
                 -1);
         list++;
     }
@@ -89,34 +136,52 @@ static GtkWidget *create_content_widget(void)
     GtkWidget *view;
     GtkWidget *scrolled;
     GtkTreeStore *store;
+    GtkTreeSortable *sortable;
     GtkCellRenderer *text_renderer;
-    GtkTreeViewColumn *column_desc;
-    GtkTreeViewColumn *column_macro;
-    GtkTreeViewColumn *column_state;
-
+    GtkTreeViewColumn *column_descr;
+    GtkTreeViewColumn *column_symbol;
+    GtkTreeViewColumn *column_isdefined;
 
     scrolled = gtk_scrolled_window_new(NULL, NULL);
-   store = create_store();
+    store = create_store();
+
+    /* make model sortable */
+    sortable = GTK_TREE_SORTABLE(store);
+    gtk_tree_sortable_set_sort_func(sortable, COL_DESCR, sort_symbol_func,
+            GINT_TO_POINTER(COL_DESCR), NULL);
+    gtk_tree_sortable_set_sort_func(sortable, COL_SYMBOL, sort_symbol_func,
+            GINT_TO_POINTER(COL_SYMBOL), NULL);
+    gtk_tree_sortable_set_sort_func(sortable, COL_ISDEFINED, sort_symbol_func,
+            GINT_TO_POINTER(COL_ISDEFINED), NULL);
+    /* set default sort order */
+    gtk_tree_sortable_set_sort_column_id(sortable, COL_SYMBOL,
+            GTK_SORT_ASCENDING);
+
+    /* create view */
     view =  gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
+    gtk_tree_view_set_headers_clickable(GTK_TREE_VIEW(view), TRUE);
 
     text_renderer = gtk_cell_renderer_text_new();
 
-    column_desc = gtk_tree_view_column_new_with_attributes(
+    column_descr = gtk_tree_view_column_new_with_attributes(
             "Description", text_renderer,
-            "text", 0,
+            "text", COL_DESCR,
             NULL);
-    column_macro = gtk_tree_view_column_new_with_attributes(
-            "Macro", text_renderer,
-            "text", 1,
+    gtk_tree_view_column_set_sort_column_id(column_descr, COL_DESCR);
+    column_symbol = gtk_tree_view_column_new_with_attributes(
+            "Symbol", text_renderer,
+            "text", COL_SYMBOL,
             NULL);
-    column_state = gtk_tree_view_column_new_with_attributes(
-            "Enabled", text_renderer,
-            "text", 2,
+    gtk_tree_view_column_set_sort_column_id(column_symbol, COL_SYMBOL);
+    column_isdefined = gtk_tree_view_column_new_with_attributes(
+            "Defined", text_renderer,
+            "text", COL_ISDEFINED,
             NULL);
+    gtk_tree_view_column_set_sort_column_id(column_isdefined, COL_ISDEFINED);
 
-    gtk_tree_view_append_column(GTK_TREE_VIEW(view), column_desc);
-    gtk_tree_view_append_column(GTK_TREE_VIEW(view), column_macro);
-    gtk_tree_view_append_column(GTK_TREE_VIEW(view), column_state);
+    gtk_tree_view_append_column(GTK_TREE_VIEW(view), column_descr);
+    gtk_tree_view_append_column(GTK_TREE_VIEW(view), column_symbol);
+    gtk_tree_view_append_column(GTK_TREE_VIEW(view), column_isdefined);
 
     gtk_widget_set_size_request(scrolled, 800, 600);
     gtk_container_add(GTK_CONTAINER(scrolled), view);
@@ -139,7 +204,7 @@ gboolean uicompiletimefeatures_dialog_show(GtkWidget *widget, gpointer user_data
     GtkWidget *content;
     gchar title[256];
 
-    g_snprintf(title, 256, "%s compile time features", machine_name);
+    g_snprintf(title, sizeof(title), "%s compile time features", machine_name);
 
     dialog = gtk_dialog_new_with_buttons(title,
             ui_get_active_window(),
@@ -151,7 +216,7 @@ gboolean uicompiletimefeatures_dialog_show(GtkWidget *widget, gpointer user_data
     gtk_box_pack_start(GTK_BOX(content), create_content_widget(),
             TRUE, TRUE, 0);
 
-    g_signal_connect(dialog, "response", G_CALLBACK(on_response), NULL);
+    g_signal_connect_unlocked(dialog, "response", G_CALLBACK(on_response), NULL);
     gtk_widget_show_all(dialog);
     return TRUE;
 }
