@@ -30,7 +30,7 @@
 #include "vice.h"
 #include <gtk/gtk.h>
 
-gboolean ui_disk_attach_callback(GtkWidget *widget, gpointer user_data);
+gboolean ui_disk_attach_dialog_show(GtkWidget *widget, gpointer user_data);
 gboolean ui_disk_detach_callback(GtkWidget *widget, gpointer user_data);
 gboolean ui_disk_detach_all_callback(GtkWidget *widget, gpointer data);
 
