@@ -30,9 +30,9 @@
 
 #include "basewidgets.h"
 #include "basedialogs.h"
-#include "debug_gtk3.h"
 #include "widgethelpers.h"
 #include "driveunitwidget.h"
+#include "drivenowidget.h"
 #include "drive.h"
 #include "diskimage.h"
 #include "filechooserhelpers.h"
@@ -74,19 +74,25 @@ static disk_image_type_t disk_image_types[] = {
     { "d80", DISK_IMAGE_TYPE_D80 },
     { "d81", DISK_IMAGE_TYPE_D81 },
     { "d82", DISK_IMAGE_TYPE_D82 },
+    { "d90", DISK_IMAGE_TYPE_D90 },
     { "d1m", DISK_IMAGE_TYPE_D1M },
     { "d2m", DISK_IMAGE_TYPE_D2M },
     { "d4m", DISK_IMAGE_TYPE_D4M },
+    { "dhd", DISK_IMAGE_TYPE_DHD },
     { "g64", DISK_IMAGE_TYPE_G64 },
     { "g71", DISK_IMAGE_TYPE_G71 },
     { "p64", DISK_IMAGE_TYPE_P64 },
+#ifdef HAVE_X64_IMAGE
     { "x64", DISK_IMAGE_TYPE_X64 },
+#endif
     { NULL, -1 }
 };
 
 
 /** \brief  Drive unit to attach image to */
 static int unit_number = DRIVE_UNIT_MIN;
+/** \brief  Drive number to attach image to in unit*/
+static int drive_number = 0;
 /** \brief  Disk image type to create */
 static int image_type = 1541;
 
@@ -133,7 +139,6 @@ static void on_response(GtkWidget *widget, gint response_id, gpointer data)
             gtk_widget_destroy(widget);
             break;
         default:
-            debug_gtk3("warning: unhandled response ID %d\n", response_id);
             break;
     }
 }
@@ -154,16 +159,18 @@ static void on_disk_image_type_changed(GtkComboBox *combo, gpointer data)
         model = gtk_combo_box_get_model(combo);
         if (gtk_combo_box_get_active_iter(combo, &iter)) {
             gtk_tree_model_get(model, &iter, 1, &image_type, -1);
-            debug_gtk3("got disk image type %d\n", image_type);
         }
     }
 }
 
 
+/** \brief  Attempt to set drive to match image type
+ *
+ * \return  TRUE if succesful
+ */
 static gboolean attempt_to_set_drive_type(void)
 {
     if (resources_set_int_sprintf("Drive%dType", image_type, unit_number) < 0) {
-        debug_gtk3("failed to set Drive%dType to %d\n", unit_number, image_type);
         return FALSE;
     }
     return TRUE;
@@ -253,7 +260,7 @@ static gboolean create_disk_image(const char *filename)
         }
 
         /* finally attach the disk image */
-        if (file_system_attach_disk(unit_number, fname_copy) < 0) {
+        if (file_system_attach_disk(unit_number, drive_number, fname_copy) < 0) {
             vice_gtk3_message_error("fail", "Could not attach image '%s'",
                     fname_copy);
             status = FALSE;
@@ -325,6 +332,7 @@ static GtkWidget *create_extra_widget(GtkWidget *parent, int unit)
 {
     GtkWidget *grid;
     GtkWidget *unit_widget;
+    GtkWidget *drive_widget;
     GtkWidget *type_widget;
     GtkWidget *label;
 
@@ -337,6 +345,10 @@ static GtkWidget *create_extra_widget(GtkWidget *parent, int unit)
     gtk_widget_set_valign(unit_widget, GTK_ALIGN_CENTER);
     gtk_grid_attach(GTK_GRID(grid), unit_widget, 0, 0, 1, 1);
 
+    drive_widget = drive_no_widget_create(0, &drive_number, NULL);
+    gtk_widget_set_valign(drive_widget, GTK_ALIGN_CENTER);
+    gtk_grid_attach(GTK_GRID(grid), drive_widget, 0, 1, 1, 1);
+
     /* disk name */
     label = gtk_label_new("Name:");
     gtk_widget_set_halign(label, GTK_ALIGN_START);
@@ -366,7 +378,7 @@ static GtkWidget *create_extra_widget(GtkWidget *parent, int unit)
             "Set proper drive type when attaching image");
     /* disable by default */
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(set_drive_type), FALSE);
-    gtk_grid_attach(GTK_GRID(grid), set_drive_type, 0, 1, 4, 1);
+    gtk_grid_attach(GTK_GRID(grid), set_drive_type, 4, 1, 4, 1);
 
     gtk_widget_show_all(grid);
     return grid;
@@ -380,7 +392,7 @@ static GtkWidget *create_extra_widget(GtkWidget *parent, int unit)
  *
  * \return  TRUE;
  */
-gboolean uidiskcreate_dialog_show(GtkWidget *parent, gpointer data)
+gboolean ui_disk_create_dialog_show(GtkWidget *parent, gpointer data)
 {
     GtkWidget *dialog;
     GtkFileFilter *filter;
