@@ -7,15 +7,13 @@
 /*
  * $VICERES SoundRecordDeviceArg        all
  * $VICERES SoundRecordDeviceName       all
- * $VICERES DoodleOversizeHandling      -vsid
- * $VICERES DoodleUndersizeHandling     -vsid
- * $VICERES DoodleMultiColorHandling    -vsid
- * $VICERES DoodleTEDLumHandling        -vsid
- * $VICERES DoodleCRTCTextColor         -vsid
+ * $VICERES OCPOversizeHandling         -vsid
+ * $VICERES OCPUndersizeHandling        -vsid
+ * $VICERES OCPMultiColorHandling       -vsid
+ * $VICERES OCPTEDLumHandling           -vsid
  * $VICERES KoalaOversizeHandling       -vsid
  * $VICERES KoalaUndersizeHandling      -vsid
  * $VICERES KoalaTEDLumHandling         -vsid
- * $VICERES KoalaCRTCTextColor          -vsid
  */
 
 /*
@@ -44,22 +42,21 @@
 #include <string.h>
 #include <stdint.h>
 
-#include "basedialogs.h"
-#include "basewidgets.h"
-#include "debug_gtk3.h"
-#include "filechooserhelpers.h"
 #include "gfxoutput.h"
 #include "lib.h"
 #include "machine.h"
+#include "mainlock.h"
 #include "openfiledialog.h"
 #include "resources.h"
 #include "savefiledialog.h"
 #include "screenshot.h"
 #include "selectdirectorydialog.h"
 #include "sound.h"
+#include "statusbarrecordingwidget.h"
 #include "ui.h"
 #include "uiapi.h"
-#include "widgethelpers.h"
+#include "uistatusbar.h"
+#include "vice_gtk3.h"
 
 #ifdef HAVE_FFMPEG
 #include "ffmpegwidget.h"
@@ -204,26 +201,16 @@ static const vice_gtk3_combo_entry_int_t multicolor_modes[] = {
  */
 static const vice_gtk3_combo_entry_int_t ted_luma_modes[] = {
     { "ignore", 0 },
-    { "dither", 1 },
+    { "Best cell colors", 1 },
     { NULL, -1 },
 };
 
 
-/** \brief  List of available colors to use for CRTC screenshots
- */
-static const vice_gtk3_combo_entry_int_t crtc_colors[] = {
-    { "White", 0 },
-    { "Amber", 1 },
-    { "Green", 2 },
-    { NULL, -1 }
-};
-
-
 /* forward declarations of helper functions */
 static GtkWidget *create_screenshot_param_widget(const char *prefix);
-static void save_screenshot_handler(void);
-static void save_audio_recording_handler(void);
-static void save_video_recording_handler(void);
+static void save_screenshot_handler(GtkWidget *parent);
+static void save_audio_recording_handler(GtkWidget *parent);
+static void save_video_recording_handler(GtkWidget *parent);
 
 
 /** \brief  Reference to the GtkStack containing the media types
@@ -239,13 +226,19 @@ static int old_pause_state = 0;
 
 
 /* references to widgets, used from various event handlers */
+
+/** \brief  Screenshot options grid reference */
 static GtkWidget *screenshot_options_grid = NULL;
+/** \brief  Screenshot "Oversize" widget reference */
 static GtkWidget *oversize_widget = NULL;
+/** \brief  Screenshot "Undersize" widget reference */
 static GtkWidget *undersize_widget = NULL;
+/** \brief  Screenshot "Multicolor mode" widget reference */
 static GtkWidget *multicolor_widget = NULL;
+/** \brief  Screenshot "TED luma" widget reference */
 static GtkWidget *ted_luma_widget = NULL;
-static GtkWidget *crtc_textcolor_widget = NULL;
 #ifdef HAVE_FFMPEG
+/** \brief  FFMPEG video driver options grid reference */
 static GtkWidget *video_driver_options_grid = NULL;
 #endif
 
@@ -262,7 +255,6 @@ static GtkWidget *video_driver_options_grid = NULL;
 static void on_dialog_destroy(GtkWidget *widget, gpointer data)
 {
     if (machine_class != VICE_MACHINE_VSID) {
-        debug_gtk3("called: cleaning up driver list.");
         lib_free(video_driver_list);
     }
 
@@ -297,18 +289,17 @@ static void update_screenshot_options_grid(GtkWidget *new)
  *
  * \param[in,out]   dialog      dialog triggering the event
  * \param[in]       response_id response ID
- * \param[in]       data        extra event data (unused)
+ * \param[in]       data        parent dialog
  */
 static void on_response(GtkDialog *dialog, gint response_id, gpointer data)
 {
     const gchar *child_name;
 
-    debug_gtk3("got response ID %d.", response_id);
-
     switch (response_id) {
         case GTK_RESPONSE_DELETE_EVENT:
-            debug_gtk3("destroying dialog.");
+            mainlock_release();
             gtk_widget_destroy(GTK_WIDGET(dialog));
+            mainlock_obtain();
             break;
 
         case RESPONSE_SAVE:
@@ -316,35 +307,27 @@ static void on_response(GtkDialog *dialog, gint response_id, gpointer data)
             if (machine_class != VICE_MACHINE_VSID) {
                 /* stack child name determines what to do next */
                 child_name = gtk_stack_get_visible_child_name(GTK_STACK(stack));
-                debug_gtk3("Saving media, tab '%s' selected.", child_name);
 
                 if (strcmp(child_name, CHILD_SCREENSHOT) == 0) {
-                    debug_gtk3("Screenshot requested, driver %d.",
-                            screenshot_driver_index);
-                    save_screenshot_handler();
+                    save_screenshot_handler(data);
                 } else if (strcmp(child_name, CHILD_SOUND) == 0) {
-                    debug_gtk3("Audio recording requested, driver %d.",
-                            audio_driver_index);
-                    save_audio_recording_handler();
+                    save_audio_recording_handler(data);
                     ui_display_recording(1);
                 } else if (strcmp(child_name, CHILD_VIDEO) == 0) {
-#ifdef HAVE_FFMPEG
-                    debug_gtk3("Video recording requested, driver %d.",
-                            video_driver_index);
-#endif
-                    save_video_recording_handler();
+                    save_video_recording_handler(data);
                     ui_display_recording(1);
                 }
             } else {
-                debug_gtk3("Audio recording requested, driver %d.",
-                        audio_driver_index);
-                save_audio_recording_handler();
+                save_audio_recording_handler(data);
                 ui_display_recording(1);
             }
+#if 0
+            mainlockk_release();
             gtk_widget_destroy(GTK_WIDGET(dialog));
+            mainlock_obtain();
+#endif
             break;
 
-
         default:
             break;
     }
@@ -360,8 +343,6 @@ static void on_screenshot_driver_toggled(GtkWidget *widget, gpointer data)
 {
     if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))) {
         int index = GPOINTER_TO_INT(data);
-        debug_gtk3("screenshot driver %d (%s) selected.",
-                index, video_driver_list[index].name);
         screenshot_driver_index = index;
         update_screenshot_options_grid(
                 create_screenshot_param_widget(video_driver_list[index].name));
@@ -378,8 +359,6 @@ static void on_audio_driver_toggled(GtkWidget *widget, gpointer data)
 {
     if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))) {
         int index = GPOINTER_TO_INT(data);
-        debug_gtk3("audio driver %d (%s) selected.",
-                index, audio_driver_list[index].name);
         audio_driver_index = index;
     }
 }
@@ -390,19 +369,24 @@ static void on_audio_driver_toggled(GtkWidget *widget, gpointer data)
  *                              Helpers functions                            *
  ****************************************************************************/
 
-/** \brief  Create a string in the format 'yyyymmddHHMMss' of the current time
+/** \brief  Create a string in the format 'yyyymmddHHMMssffffff' of the current time
  *
  * \return  string owned by GLib, free with g_free()
  */
 static gchar *create_datetime_string(void)
 {
     GDateTime *d;
+    gint m;
     gchar *s;
+    gchar *t;
 
     d = g_date_time_new_now_local();
+    m = g_date_time_get_microsecond(d);
     s = g_date_time_format(d, "%Y%m%d%H%M%S");
     g_date_time_unref(d);
-    return s;
+    t = g_strdup_printf("%s%06d", s, m);
+    g_free(s);
+    return t;
 }
 
 
@@ -462,29 +446,20 @@ static char *create_proposed_audio_recording_name(const char *ext)
 }
 
 
-
-/** \brief  Save a screenshot
+/** \brief  Callback for the save-screenshot dialog
  *
- * Pops up a save-file dialog with a proposed filename (ie
- * 'screenshot-197411151210.png'.
+ * \param[in,out]   dialog      dialog
+ * \param[in,out]   filename    screenshot filename
+ * \param[in]       data        extra data (unused)
  */
-static void save_screenshot_handler(void)
+static void on_save_screenshot_filename(GtkDialog *dialog,
+                                        gchar *filename,
+                                        gpointer data)
 {
-    const char *display;
     const char *name;
-    const char *ext;
-    gchar *filename;
-    char *title;
-    char *proposed;
 
-    display = video_driver_list[screenshot_driver_index].display;
     name = video_driver_list[screenshot_driver_index].name;
-    ext = video_driver_list[screenshot_driver_index].ext;
 
-    title = lib_msprintf("Save %s file", display);
-    proposed = create_proposed_screenshot_name(ext);
-
-    filename = vice_gtk3_save_file_dialog(title, proposed, TRUE, NULL);
     if (filename != NULL) {
 
         gchar *filename_locale = file_chooser_convert_to_locale(filename);
@@ -497,78 +472,129 @@ static void save_screenshot_handler(void)
         g_free(filename);
         g_free(filename_locale);
     }
+    mainlock_release();
+    gtk_widget_destroy(GTK_WIDGET(dialog));
+    mainlock_obtain();
+}
+
+
+
+/** \brief  Save a screenshot
+ *
+ * Pops up a save-file dialog with a proposed filename (ie
+ * 'screenshot-197411151210.png'.
+ *
+ * \note    Destroys \a parent along with the dialog
+ *
+ * \param[in,out]   parent  parent dialog
+ */
+static void save_screenshot_handler(GtkWidget *parent)
+{
+    GtkWidget *dialog;
+    const char *display;
+    char *title;
+    char *proposed;
+    const char *ext;
+
+    ext = video_driver_list[screenshot_driver_index].ext;
+    display = video_driver_list[screenshot_driver_index].display;
+    title = lib_msprintf("Save %s file", display);
+    proposed = create_proposed_screenshot_name(ext);
+
+    dialog = vice_gtk3_save_file_dialog(
+            title, proposed, TRUE, NULL,
+            on_save_screenshot_filename,
+            NULL);
+    /* destroy parent dialog when the dialog is destroyed */
+    g_signal_connect_swapped(
+            dialog,
+            "destroy",
+            G_CALLBACK(gtk_widget_destroy),
+            parent);
+
     lib_free(proposed);
     lib_free(title);
 }
 
 
-/** \brief  Start an audio recording
+/** \brief  Callback for the save-audio dialog
  *
- * Pops up a save-file dialog with a proposed filename (ie
- * 'audio-recording-197411151210.png'.
+ * \param[in,out]   dialog      dialog
+ * \param[in,out]   filename    audio recording filename
+ * \param[in]       data        extra data (unused)
  */
-static void save_audio_recording_handler(void)
+static void on_save_audio_filename(GtkDialog *dialog,
+                                   gchar *filename,
+                                   gpointer data)
 {
-    const char *display;
-    const char *name;
-    const char *ext;
-    gchar *filename;
     gchar *filename_locale;
-    char *title;
-    char *proposed;
-
-    display = audio_driver_list[audio_driver_index].display;
-    name = audio_driver_list[audio_driver_index].name;
-    ext = audio_driver_list[audio_driver_index].ext;
-
-    title = lib_msprintf("Save %s file", display);
-    proposed = create_proposed_audio_recording_name(ext);
-
-    filename = vice_gtk3_save_file_dialog(title, proposed, TRUE, NULL);
-    filename_locale = file_chooser_convert_to_locale(filename);
 
     if (filename != NULL) {
+        filename_locale = file_chooser_convert_to_locale(filename);
+        const char *name = audio_driver_list[audio_driver_index].name;
         /* XXX: setting resources doesn't exactly help with catching errors */
         resources_set_string("SoundRecordDeviceArg", filename_locale);
         resources_set_string("SoundRecordDeviceName", name);
         g_free(filename);
         g_free(filename_locale);
     }
+    mainlock_release();
+    gtk_widget_destroy(GTK_WIDGET(dialog));
+    mainlock_obtain();
 }
 
 
-/** \brief  Start recording a video
+
+/** \brief  Start an audio recording
  *
  * Pops up a save-file dialog with a proposed filename (ie
- * 'video-recording-197411151210.png'.
+ * 'audio-recording-197411151210.png'.
+ *
+ * \note    Destroys \a parent along with the dialog
+ *
+ * \param[in,out]   parent dialog
  */
-static void save_video_recording_handler(void)
+static void save_audio_recording_handler(GtkWidget *parent)
 {
-    /* these may be useful once QuickTime is supported */
-#if 0
+    GtkWidget *dialog;
     const char *display;
-    const char *name;
-#endif
     const char *ext;
-    gchar *filename;
     char *title;
     char *proposed;
-#ifdef HAVE_FFMPEG
-    debug_gtk3("video driver index = %d.", video_driver_index);
-#endif
 
-#if 0
-    display = video_driver_list[video_driver_index].display;
-    name = video_driver_list[video_driver_index].name;
-#endif
-    /* we don't have a format->extension mapping, so the format name itself is
-       better than `video_driver_list[video_driver_index].ext' */
-    resources_get_string("FFMPEGFormat", &ext);
+    ext = audio_driver_list[audio_driver_index].ext;
+    display = audio_driver_list[audio_driver_index].display;
+    title = lib_msprintf("Save %s file", display);
+    proposed = create_proposed_audio_recording_name(ext);
 
-    title = lib_msprintf("Save %s file", "FFMPEG");
-    proposed = create_proposed_video_recording_name(ext);
+    dialog = vice_gtk3_save_file_dialog(
+            title, proposed, TRUE, NULL,
+            on_save_audio_filename,
+            NULL);
 
-    filename = vice_gtk3_save_file_dialog(title, proposed, TRUE, NULL);
+    /* destroy parent dialog when the dialog is destroyed */
+    g_signal_connect_swapped(
+            dialog,
+            "destroy",
+            G_CALLBACK(gtk_widget_destroy),
+            parent);
+
+    lib_free(title);
+    lib_free(proposed);
+}
+
+
+
+/** \brief  Callback for the save-video dialog
+ *
+ * \param[in,out]   dialog      dialog
+ * \param[in,out]   filename    video recording filename
+ * \param[in]       data        extra data (unused)
+ */
+static void on_save_video_filename(GtkDialog *dialog,
+                                   gchar *filename,
+                                   gpointer data)
+{
     if (filename != NULL) {
 
         const char *driver;
@@ -588,10 +614,8 @@ static void save_video_recording_handler(void)
         debug_gtk3("Video = %d, bitrate %d.", vc, vb);
         debug_gtk3("Audio = %d, bitrate %d.", ac, ab);
 
-
         ui_pause_disable();
 
-
         filename_locale = file_chooser_convert_to_locale(filename);
 
         /* TODO: add extension if not present? */
@@ -602,6 +626,50 @@ static void save_video_recording_handler(void)
         g_free(filename);
         g_free(filename_locale);
     }
+    mainlock_release();
+    gtk_widget_destroy(GTK_WIDGET(dialog));
+    mainlock_obtain();
+}
+
+
+/** \brief  Start recording a video
+ *
+ * Pops up a save-file dialog with a proposed filename (ie
+ * 'video-recording-197411151210.png'.
+ *
+ * \note    Destroys \a parent along with the dialog
+ *
+ * \param[in,out]   parent  parent dialog
+ */
+static void save_video_recording_handler(GtkWidget *parent)
+{
+    GtkWidget *dialog;
+    const char *ext;
+    char *title;
+    char *proposed;
+
+#if 0
+    display = video_driver_list[video_driver_index].display;
+    name = video_driver_list[video_driver_index].name;
+#endif
+    /* we don't have a format->extension mapping, so the format name itself is
+       better than `video_driver_list[video_driver_index].ext' */
+    resources_get_string("FFMPEGFormat", &ext);
+
+    title = lib_msprintf("Save %s file", "FFMPEG");
+    proposed = create_proposed_video_recording_name(ext);
+
+    dialog = vice_gtk3_save_file_dialog(
+            title, proposed, TRUE, NULL,
+            on_save_video_filename, NULL);
+
+    /* destroy parent dialog when the dialog is destroyed */
+    g_signal_connect_swapped(
+            dialog,
+            "destroy",
+            G_CALLBACK(gtk_widget_destroy),
+            parent);
+
     lib_free(proposed);
     lib_free(title);
 }
@@ -619,8 +687,6 @@ static void create_video_driver_list(void)
     int index;
 
     video_driver_count = gfxoutput_num_drivers();
-    debug_gtk3("got %d output drivers.", video_driver_count);
-
     video_driver_list = lib_malloc((size_t)(video_driver_count + 1) *
             sizeof *video_driver_list);
 
@@ -629,9 +695,6 @@ static void create_video_driver_list(void)
     if (video_driver_count > 0) {
         driver = gfxoutput_drivers_iter_init();
         while (driver != NULL) {
-            debug_gtk3(".. adding driver '%s'. ext: %s.",
-                    driver->name,
-                    driver->default_extension);
             video_driver_list[index].display = driver->displayname;
             video_driver_list[index].name = driver->name;
             video_driver_list[index].ext = driver->default_extension;
@@ -657,13 +720,7 @@ static int driver_is_video(const char *name)
 {
     int result;
 
-    debug_gtk3("Got driver '%s'.", name);
-#if 0
-    result =  strcmp(name, "FFMPEG") == 0 || strcmp(name, "QuickTime") == 0;
-#else
     result = strcmp(name, "FFMPEG") == 0;
-#endif
-    debug_gtk3("Result = %s", result ? "TRUE" : "FALSE");
     return result;
 }
 
@@ -673,14 +730,16 @@ static int driver_is_video(const char *name)
  * This widget exposes controls to alter screenshot parameters, depending on
  * machine class and output file format
  *
- * \param[in]   prefix  resource prefix (either "Doodle", "Koala", or ""/NULL)
+ * \param[in]   prefix  resource prefix (either "OCP", "Koala", or ""/NULL)
+ *
+ * \return  GtkGrid
  */
 static GtkWidget *create_screenshot_param_widget(const char *prefix)
 {
     GtkWidget *grid;
     GtkWidget *label;
     int row;
-    int doodle = 0;
+    int artstudio = 0;
     int koala = 0;
 
     grid = gtk_grid_new();
@@ -694,19 +753,17 @@ static GtkWidget *create_screenshot_param_widget(const char *prefix)
         return grid;
     }
 
-    if ((strcmp(prefix, "DOODLE") == 0)
-            || (strcmp(prefix, "DOODLE_COMPRESSED") == 0)) {
-        prefix = "Doodle";  /* XXX: not strictly required since resource names
+    if (strcmp(prefix, "ARTSTUDIO") == 0) {
+        prefix = "OCP";  /* XXX: not strictly required since resource names
                                     seem to be case insensitive, but better
                                     safe than sorry */
-        doodle = 1;
-    } else if ((strcmp(prefix, "KOALA") == 0)
-            || (strcmp(prefix, "KOALA_COMPRESSED") == 0)) {
+        artstudio = 1;
+    } else if (strcmp(prefix, "KOALA") == 0) {
         prefix = "Koala";
         koala = 1;
     }
 
-    if (!koala && !doodle) {
+    if (!koala && !artstudio) {
         label = gtk_label_new("No parameters required");
         g_object_set(label, "margin-left", 16, NULL);
         gtk_grid_attach(GTK_GRID(grid), label, 0, 0, 1, 1);
@@ -736,8 +793,8 @@ static GtkWidget *create_screenshot_param_widget(const char *prefix)
 
     row = 2;    /* from now on, the widgets depend on machine and image type */
 
-    /* DoodleMultiColorHandling */
-    if (doodle) {
+    /* OCPMultiColorHandling */
+    if (artstudio) {
         label = gtk_label_new("Multi color handling");
         g_object_set(label, "margin-left", 16, NULL);
         gtk_widget_set_halign(label, GTK_ALIGN_START);
@@ -760,19 +817,6 @@ static GtkWidget *create_screenshot_param_widget(const char *prefix)
         row++;
     }
 
-    /* ${FORMAT}CRTCTextColor */
-    if (machine_class == VICE_MACHINE_PET
-            || machine_class == VICE_MACHINE_CBM6x0) {
-        label = gtk_label_new("CRTC text color");
-        g_object_set(label, "margin-left", 16, NULL);
-        gtk_widget_set_halign(label, GTK_ALIGN_START);
-        crtc_textcolor_widget = vice_gtk3_resource_combo_box_int_new_sprintf(
-                "%sCRTCTextColor", crtc_colors, prefix);
-        gtk_grid_attach(GTK_GRID(grid), label, 0, row, 1, 1);
-        gtk_grid_attach(GTK_GRID(grid), crtc_textcolor_widget, 1, row, 1, 1);
-        row++;
-    }
-
     gtk_widget_show_all(grid);
     return grid;
 }
@@ -788,6 +832,7 @@ static GtkWidget *create_screenshot_widget(void)
 {
     GtkWidget *grid;
     GtkWidget *drv_grid;
+    GtkWidget *label;
     GtkWidget *radio;
     GtkWidget *last;
     GSList *group = NULL;
@@ -798,7 +843,14 @@ static GtkWidget *create_screenshot_widget(void)
     gtk_grid_set_column_spacing(GTK_GRID(grid), 16);
     gtk_grid_set_row_spacing(GTK_GRID(grid), 8);
 
-    drv_grid = uihelpers_create_grid_with_label("Driver", 1);
+    /* grid without extra row spacing */
+    drv_grid = vice_gtk3_grid_new_spaced_with_label(-1, 0, "Driver", 1);
+    g_object_set(drv_grid, "margin-top", 8, "margin-left", 16, NULL);
+    /* add some padding to the label */
+    label = gtk_grid_get_child_at(GTK_GRID(drv_grid), 0, 0);
+    g_object_set(label, "margin-bottom", 8, NULL);
+
+    /* add drivers */
     grid_index = 1;
     last = NULL;
     for (index = 0; video_driver_list[index].name != NULL; index++) {
@@ -845,10 +897,11 @@ static GtkWidget *create_screenshot_widget(void)
     }
 
     /* this is where the various options go per screenshot driver (for example
-     * Koala or Doodle) */
-    screenshot_options_grid = uihelpers_create_grid_with_label(
-            "Driver options", 1);
-
+     * Koala or Artstudio) */
+    screenshot_options_grid = vice_gtk3_grid_new_spaced_with_label(
+            -1, -1, "Driver options", 1);
+    g_object_set(screenshot_options_grid,
+                 "margin-top", 8, "margin-left", 16, NULL);
     gtk_grid_attach(GTK_GRID(grid), drv_grid, 0, 0, 1, 1);
     gtk_grid_attach(GTK_GRID(grid), screenshot_options_grid, 1, 0, 1, 1);
 
@@ -867,6 +920,7 @@ static GtkWidget *create_sound_widget(void)
 {
     GtkWidget *grid;
     GtkWidget *drv_grid;
+    GtkWidget *label;
     GtkWidget *radio;
     GtkWidget *last;
     GSList *group = NULL;
@@ -876,7 +930,11 @@ static GtkWidget *create_sound_widget(void)
     gtk_grid_set_column_spacing(GTK_GRID(grid), 16);
     gtk_grid_set_row_spacing(GTK_GRID(grid), 8);
 
-    drv_grid = uihelpers_create_grid_with_label("Driver", 1);
+    drv_grid = vice_gtk3_grid_new_spaced_with_label(-1, 0, "Driver", 1);
+    label = gtk_grid_get_child_at(GTK_GRID(drv_grid), 0, 0);
+    g_object_set(label, "margin-bottom", 8, NULL);
+    g_object_set(drv_grid, "margin-top", 8, "margin-left", 16, NULL);
+
     last = NULL;
     for (index = 0; audio_driver_list[index].name != NULL; index++) {
         const char *display = audio_driver_list[index].display;
@@ -930,9 +988,7 @@ static GtkWidget *create_video_widget(void)
     GtkWidget *selection_grid;
     GtkWidget *options_grid;
 #endif
-    grid = gtk_grid_new();
-    gtk_grid_set_column_spacing(GTK_GRID(grid), 16);
-    gtk_grid_set_row_spacing(GTK_GRID(grid), 8);
+    grid = vice_gtk3_grid_new_spaced(16, 8);
 
 
 #ifdef HAVE_FFMPEG
@@ -961,7 +1017,9 @@ static GtkWidget *create_video_widget(void)
     gtk_widget_set_hexpand(combo, TRUE);
     gtk_combo_box_set_active(GTK_COMBO_BOX(combo), 0);
 
-    selection_grid = uihelpers_create_grid_with_label("Driver selection", 2);
+    selection_grid = vice_gtk3_grid_new_spaced_with_label
+        (-1, -1, "Driver selection", 2);
+    g_object_set(selection_grid, "margin-top", 8, "margin-left", 16, NULL);
     gtk_grid_set_column_spacing(GTK_GRID(selection_grid), 16);
     gtk_grid_set_row_spacing(GTK_GRID(selection_grid), 8);
     gtk_grid_attach(GTK_GRID(selection_grid), label, 0, 1, 1, 1);
@@ -970,14 +1028,13 @@ static GtkWidget *create_video_widget(void)
 
     gtk_grid_attach(GTK_GRID(grid), selection_grid, 0, 0, 1, 1);
 
-    /* grid around ffmpeg/quicktime options */
-    options_grid = uihelpers_create_grid_with_label("Driver options", 1);
+    /* grid around ffmpeg */
+    options_grid = vice_gtk3_grid_new_spaced_with_label(
+            -1, -1, "Driver options", 1);
+    g_object_set(options_grid, "margin-top", 8, "margin-left", 16, NULL);
     gtk_grid_set_column_spacing(GTK_GRID(options_grid), 16);
     gtk_grid_set_row_spacing(GTK_GRID(options_grid), 8);
 
-/* XXX: this obviously needs a cleaner solution which also handles QuickTime
- *      on MacOS
- */
     gtk_grid_attach(GTK_GRID(options_grid), ffmpeg_widget_create(), 0, 1, 1,1);
     video_driver_options_grid = options_grid;
 
@@ -1058,7 +1115,7 @@ static GtkWidget *create_content_widget(void)
  *
  * \return  TRUE
  */
-gboolean uimedia_dialog_show(GtkWidget *parent, gpointer data)
+gboolean ui_media_dialog_show(GtkWidget *parent, gpointer data)
 {
     GtkWidget *dialog;
     GtkWidget *content;
@@ -1070,7 +1127,7 @@ gboolean uimedia_dialog_show(GtkWidget *parent, gpointer data)
     /* remember pause state before entering the widget */
     old_pause_state = ui_pause_active();
 
-    /* pause emulartion */
+    /* pause emulation */
     ui_pause_enable();
 
     /* create driver list */
@@ -1095,8 +1152,8 @@ gboolean uimedia_dialog_show(GtkWidget *parent, gpointer data)
     }
 
     gtk_window_set_resizable(GTK_WINDOW(dialog), FALSE);
-    g_signal_connect(dialog, "response", G_CALLBACK(on_response), NULL);
-    g_signal_connect(dialog, "destroy", G_CALLBACK(on_dialog_destroy), NULL);
+    g_signal_connect(dialog, "response", G_CALLBACK(on_response), (gpointer)dialog);
+    g_signal_connect_unlocked(dialog, "destroy", G_CALLBACK(on_dialog_destroy), NULL);
 
     gtk_widget_show_all(dialog);
     return TRUE;
@@ -1104,13 +1161,14 @@ gboolean uimedia_dialog_show(GtkWidget *parent, gpointer data)
 
 
 /** \brief  Stop audio or video recording, if active
+ *
+ * \param[in]   parent      parent widget (unused)
+ * \param[in]   data        extra event data (unused)
  *
  * \return  TRUE, so the emulated machine doesn't get the shortcut key
  */
-gboolean uimedia_stop_recording(GtkWidget *parent, gpointer data)
+gboolean ui_media_stop_recording(GtkWidget *parent, gpointer data)
 {
-    debug_gtk3("Stopping media recording.");
-
     /* stop sound recording, if active */
     if (sound_is_recording()) {
         sound_stop_recording();
@@ -1121,19 +1179,35 @@ gboolean uimedia_stop_recording(GtkWidget *parent, gpointer data)
     }
 
     ui_display_recording(0);
+    statusbar_recording_widget_hide_all(ui_statusbar_get_recording_widget(), 10);
 
     return TRUE;
 }
 
 
-void uimedia_auto_screenshot(void)
+/** \brief  Create screenshot with autogenerated filename
+ *
+ * Creates a PNG screenshot with an autogenerated filename with an ISO-8601-ish
+ * timestamp:
+ * "vice-screenshot-<year><month><day><hour><month><seconds><sec-frac>.png"
+ */
+void ui_media_auto_screenshot(void)
 {
-    debug_gtk3("called!");
     char *filename;
 
+    /* remember pause state before entering the widget */
+    old_pause_state = ui_pause_active();
+
+    /* pause emulation */
+    ui_pause_enable();
+
     /* no need for locale bullshit */
     filename = create_proposed_screenshot_name("png");
     if (screenshot_save("PNG", filename, ui_get_active_canvas()) < 0) {
         debug_gtk3("OOPS");
     }
+
+    if (!old_pause_state) {
+        ui_pause_disable();
+    }
 }
