@@ -31,8 +31,8 @@
 #include <gtk/gtk.h>
 
 
-gboolean uimedia_dialog_show(GtkWidget *parent, gpointer user_data);
-gboolean uimedia_stop_recording(GtkWidget *parent, gpointer data);
-void     uimedia_auto_screenshot(void);
+gboolean ui_media_dialog_show(GtkWidget *parent, gpointer user_data);
+gboolean ui_media_stop_recording(GtkWidget *parent, gpointer data);
+void     ui_media_auto_screenshot(void);
 
 #endif
