@@ -78,7 +78,14 @@ GtkWidget *ui_menu_submenu_create(GtkWidget *bar, const char *label)
     return new_submenu;
 }
 
-/** \brief  Constructor for accelerator data */
+
+/** \brief  Constructor for accelerator data
+ *
+ * \param[in]   widget  widget for the accelerator data
+ * \param[in]   item    menu item for the accelerator data
+ *
+ * \return  heap-allocated accelerator data (owned by VICE)
+ */
 static ui_accel_data_t *ui_accel_data_new(GtkWidget *widget, ui_menu_item_t *item)
 {
     ui_accel_data_t *accel_data = lib_malloc(sizeof(ui_accel_data_t));
@@ -87,17 +94,47 @@ static ui_accel_data_t *ui_accel_data_new(GtkWidget *widget, ui_menu_item_t *ite
     return accel_data;
 }
 
+
 /** \brief  Destructor for accelerator data
  *
  * FIXME:   this doesn't get triggered
+ *
+ * \param[in,out]   data    accelerator data
+ * \param[in]       closure closure (unused)
  */
 static void ui_accel_data_delete(gpointer data, GClosure *closure)
 {
     debug_gtk3("Freeing accelerator data\n");
+#if 0
     lib_free(data);
+#endif
 }
 
-/** \brief  Callback that forwards accelerator codes.
+
+/** \brief  Handler for the 'destroy' event of a menu item
+ *
+ * This 'hack' is needed since the 'finalize' callback of the GClosures we use
+ * for accelerators doesn't get called, which means the accelerator data doesn't
+ * get cleaned up, which means we leak memory.
+ *
+ * \param[in]       item        menu item
+ * \param[in,out]   accel_data  accelator data (optional)
+ */
+static void on_menu_item_destroy(GtkWidget *item, gpointer accel_data)
+{
+    if (accel_data != NULL) {
+        lib_free(accel_data);
+    }
+}
+
+
+/** \brief  Callback that forwards accelerator codes
+ *
+ * \param[in]       accel_grp       accelerator group (unused)
+ * \param[in]       acceleratable   ? (unused)
+ * \param[in]       keyval          GDK keyval (unused)
+ * \param[in]       modifier        GDK key modifier(s) (unused)
+ * \param[in,out]   user_data       accelerator data
  */
 static void handle_accelerator(GtkAccelGroup *accel_grp,
                                GObject *acceleratable,
@@ -109,6 +146,7 @@ static void handle_accelerator(GtkAccelGroup *accel_grp,
     accel_data->item->callback(accel_data->widget, accel_data->item->data);
 }
 
+
 /** \brief  Add menu \a items to \a menu
  *
  * \param[in,out]   menu    Gtk menu
@@ -122,18 +160,26 @@ GtkWidget *ui_menu_add(GtkWidget *menu, ui_menu_item_t *items)
     while (items[i].label != NULL || items[i].type >= 0) {
         GtkWidget *item = NULL;
         GtkWidget *submenu;
+        ui_accel_data_t *accel_data = NULL;
 
         switch (items[i].type) {
             case UI_MENU_TYPE_ITEM_ACTION:  /* fall through */
                 /* normal callback item */
-                /* debug_gtk3("adding menu item '%s'\n", items[i].label); */
                 item = gtk_menu_item_new_with_mnemonic(items[i].label);
                 if (items[i].callback != NULL) {
-                    g_signal_connect(
+                    if (items[i].unlocked) {
+                        g_signal_connect_unlocked(
                             item,
                             "activate",
                             G_CALLBACK(items[i].callback),
                             (gpointer)(items[i].data));
+                    } else {
+                        g_signal_connect(
+                            item,
+                            "activate",
+                            G_CALLBACK(items[i].callback),
+                            (gpointer)(items[i].data));
+                    }
                 } else {
                     /* no callback: 'grey-out'/'ghost' the item */
                     gtk_widget_set_sensitive(item, FALSE);
@@ -154,11 +200,19 @@ GtkWidget *ui_menu_add(GtkWidget *menu, ui_menu_item_t *items)
                     }
                     /* connect signal handler AFTER setting the state, otherwise
                      * the callback gets triggered, leading to odd results */
-                    g_signal_connect(
+                    if (items[i].unlocked) {
+                        g_signal_connect_unlocked(
                             item,
                             "activate",
                             G_CALLBACK(items[i].callback),
                             items[i].data);
+                    } else {
+                        g_signal_connect(
+                            item,
+                            "activate",
+                            G_CALLBACK(items[i].callback),
+                            items[i].data);
+                    }
                 } else {
                     /* grey out */
                     gtk_widget_set_sensitive(item, FALSE);
@@ -186,24 +240,31 @@ GtkWidget *ui_menu_add(GtkWidget *menu, ui_menu_item_t *items)
 
             if (items[i].keysym != 0 && items[i].callback != NULL) {
                 GClosure *accel_closure;
-#if 0
-                debug_gtk3("adding accelerator %d to item %s'\n",
-                        items[i].keysym, items[i].label);
-#endif
+
                 /* Normally you would use gtk_widget_add_accelerator
                  * here, but that will disable the accelerators if the
                  * menu is hidden, which can be configured to happen
                  * while in fullscreen. We instead create the closure
                  * by hand, add it to the GtkAccelGroup, and update
                  * the accelerator information. */
+                accel_data = ui_accel_data_new(item, &items[i]);
                 accel_closure = g_cclosure_new(G_CALLBACK(handle_accelerator),
-                                               ui_accel_data_new(item, &items[i]),
+                                               accel_data,
                                                ui_accel_data_delete);
-                gtk_accel_group_connect(accel_group, items[i].keysym, items[i].modifier, GTK_ACCEL_MASK, accel_closure);
+                if (items[i].unlocked) {
+                    gtk_accel_group_connect(accel_group, items[i].keysym, items[i].modifier, GTK_ACCEL_MASK, accel_closure);
+                } else {
+                    vice_locking_gtk_accel_group_connect(accel_group, items[i].keysym, items[i].modifier, GTK_ACCEL_MASK, accel_closure);
+                }
                 gtk_accel_label_set_accel(GTK_ACCEL_LABEL(gtk_bin_get_child(GTK_BIN(item))), items[i].keysym, items[i].modifier);
             }
 
             gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+            /* the closure's callback doesn't trigger due to mysterious reasons,
+             * so we use the menu item to free the accelerator's data
+             */
+            g_signal_connect_unlocked(item, "destroy", G_CALLBACK(on_menu_item_destroy),
+                    accel_data);
         }
         i++;
     }
@@ -220,3 +281,4 @@ void ui_menu_init_accelerators(GtkWidget *window)
     accel_group = gtk_accel_group_new();
     gtk_window_add_accel_group(GTK_WINDOW(window), accel_group);
 }
+
