@@ -2,6 +2,9 @@
  * \brief   Native GTK3 UI monitor stuff
  *
  * \author  Fabrizio Gennari <fabrizio.ge@tiscali.it>
+ * \author  David Hogan <david.q.hogan@gmail.com>
+ *
+ * \todo    Properly document this, please.
  */
 
 /*
@@ -27,8 +30,10 @@
 
 #include "vice.h"
 
+#include <pthread.h>
 #include <stdlib.h>
 #include <string.h>
+#include <stdbool.h>
 
 #include "novte/novte.h"
 
@@ -58,18 +63,28 @@
 
 #include "console.h"
 #include "debug_gtk3.h"
+#include "machine.h"
 #include "monitor.h"
+#include "mainlock.h"
 #include "resources.h"
 #include "lib.h"
+#include "log.h"
 #include "ui.h"
 #include "linenoise.h"
+#include "tick.h"
 #include "uimon.h"
 #include "uimonarch.h"
 #include "uimon-fallback.h"
 #include "mon_command.h"
 #include "vsync.h"
+#include "vsyncapi.h"
 #include "uidata.h"
 
+static gboolean uimon_window_open_impl(gpointer user_data);
+static gboolean uimon_window_resume_impl(gpointer user_data);
+
+#define VTE_CSS \
+    "vte-terminal { font-size: 300%; }"
 
 /** \brief  Monitor console window object
  *
@@ -79,12 +94,18 @@ struct console_private_s {
     GtkWidget *window;  /**< windows */
     GtkWidget *term;    /**< could be a VTE instance? */
     char *input_buffer;
-} fixed = {NULL, NULL, NULL};
+
+    pthread_mutex_t output_lock;
+    char *output_buffer;
+    size_t output_buffer_allocated_size;
+    size_t output_buffer_used_size;
+} fixed = {NULL, NULL, NULL, PTHREAD_MUTEX_INITIALIZER, NULL, 0, 0};
 
 static console_t vte_console;
 static linenoiseCompletions command_lc = {0, NULL};
 static linenoiseCompletions need_filename_lc = {0, NULL};
 
+
 /* FIXME: this should perhaps be done using some function from archdep */
 static int is_dir(struct dirent *de)
 {
@@ -114,20 +135,69 @@ static int native_monitor(void)
     return res;
 }
 
-void uimon_write_to_terminal(struct console_private_s *t,
-                       const char *data,
-                       glong length)
+static gboolean uimon_write_to_terminal_impl(gpointer _unused)
 {
-    if(t->term) {
-        vte_terminal_feed(VTE_TERMINAL(t->term), data, length);
+    pthread_mutex_lock(&fixed.output_lock);
+
+    if (fixed.output_buffer) {
+        vte_terminal_feed(VTE_TERMINAL(fixed.term), fixed.output_buffer, fixed.output_buffer_used_size);
+
+        lib_free(fixed.output_buffer);
+        fixed.output_buffer = NULL;
+        fixed.output_buffer_allocated_size = 0;
+        fixed.output_buffer_used_size = 0;
     }
+
+    pthread_mutex_unlock(&fixed.output_lock);
+
+    return FALSE;
+}
+
+void uimon_write_to_terminal(struct console_private_s *t,
+                             const char *data,
+                             glong length)
+{
+    if (!t->term) {
+        return;
+    }
+
+    size_t output_buffer_required_size;
+
+    pthread_mutex_lock(&fixed.output_lock);
+
+    /*
+     * If the output buffer exists, we've already scheduled a write
+     * so we just append to the existing buffer.
+     */
+
+    output_buffer_required_size = fixed.output_buffer_used_size + length;
+
+    if (output_buffer_required_size > fixed.output_buffer_allocated_size) {
+        output_buffer_required_size += 4096;
+
+        if (fixed.output_buffer) {
+            fixed.output_buffer = lib_realloc(fixed.output_buffer, output_buffer_required_size);
+        } else {
+            fixed.output_buffer = lib_malloc(output_buffer_required_size);
+
+            /* schedule a call on the ui thread */
+            gdk_threads_add_timeout(10, uimon_write_to_terminal_impl, NULL);
+        }
+
+        fixed.output_buffer_allocated_size = output_buffer_required_size;
+    }
+
+    memcpy(fixed.output_buffer + fixed.output_buffer_used_size, data, length);
+    fixed.output_buffer_used_size += length;
+
+    pthread_mutex_unlock(&fixed.output_lock);
 }
 
 
 int uimon_get_columns(struct console_private_s *t)
 {
     if(t->term) {
-        return vte_terminal_get_column_count(VTE_TERMINAL(t->term));
+        return (int)vte_terminal_get_column_count(VTE_TERMINAL(t->term));
     }
     return 80;
 }
@@ -158,6 +228,8 @@ static char* append_string_to_input_buffer(char *old_input_buffer, GtkWidget *te
             if (*char_in >= 32) {
 #endif
                 *char_out++ = *char_in;
+            } else if (*char_in == 10) {
+                *char_out++ = 13;
             }
         }
         *char_out = 0;
@@ -176,40 +248,81 @@ static gboolean plain_key_pressed(char **input_buffer, guint keyval)
                 *input_buffer = append_char_to_input_buffer(*input_buffer, (char)keyval);
                 return TRUE;
             }
+            if(keyval >= GDK_KEY_KP_0 && keyval <= GDK_KEY_KP_9) {
+                *input_buffer =
+                    append_char_to_input_buffer(
+                        *input_buffer,
+                        (char)keyval - GDK_KEY_KP_0 + 48);
+                return TRUE;
+            }
             return FALSE;
-        case GDK_KEY_Return:
-            *input_buffer = append_char_to_input_buffer(*input_buffer, 13);
-            return TRUE;
-        case GDK_KEY_BackSpace:
-            *input_buffer = append_char_to_input_buffer(*input_buffer, 127);
+        case GDK_KEY_Home:
+        case GDK_KEY_KP_Home:
+            *input_buffer = append_char_to_input_buffer(*input_buffer, 1);
             return TRUE;
         case GDK_KEY_Left:
+        case GDK_KEY_KP_Left:
             *input_buffer = append_char_to_input_buffer(*input_buffer, 2);
             return TRUE;
+        case GDK_KEY_Delete:
+        case GDK_KEY_KP_Delete:
+            *input_buffer = append_char_to_input_buffer(*input_buffer, 4);
+            return TRUE;
+        case GDK_KEY_End:
+        case GDK_KEY_KP_End:
+            *input_buffer = append_char_to_input_buffer(*input_buffer, 5);
+            return TRUE;
         case GDK_KEY_Right:
+        case GDK_KEY_KP_Right:
             *input_buffer = append_char_to_input_buffer(*input_buffer, 6);
             return TRUE;
-        case GDK_KEY_Up:
-            *input_buffer = append_char_to_input_buffer(*input_buffer, 16);
-            return TRUE;
-        case GDK_KEY_Down:
-            *input_buffer = append_char_to_input_buffer(*input_buffer, 14);
-            return TRUE;
         case GDK_KEY_Tab:
             *input_buffer = append_char_to_input_buffer(*input_buffer, 9);
             return TRUE;
-        case GDK_KEY_Delete:
-            *input_buffer = append_char_to_input_buffer(*input_buffer, 4);
+        case GDK_KEY_Return:
+        case GDK_KEY_KP_Enter:
+            *input_buffer = append_char_to_input_buffer(*input_buffer, 13);
             return TRUE;
-        case GDK_KEY_Home:
-            *input_buffer = append_char_to_input_buffer(*input_buffer, 1);
+        case GDK_KEY_Down:
+        case GDK_KEY_KP_Down:
+            *input_buffer = append_char_to_input_buffer(*input_buffer, 14);
             return TRUE;
-        case GDK_KEY_End:
-            *input_buffer = append_char_to_input_buffer(*input_buffer, 5);
+        case GDK_KEY_Up:
+        case GDK_KEY_KP_Up:
+            *input_buffer = append_char_to_input_buffer(*input_buffer, 16);
+            return TRUE;
+        case GDK_KEY_dead_diaeresis:
+            *input_buffer = append_char_to_input_buffer(*input_buffer, 34);
+            return TRUE;
+        case GDK_KEY_dead_acute:
+            *input_buffer = append_char_to_input_buffer(*input_buffer, 39);
+            return TRUE;
+        case GDK_KEY_KP_Multiply:
+            *input_buffer = append_char_to_input_buffer(*input_buffer, 42);
+            return TRUE;
+        case GDK_KEY_KP_Add:
+            *input_buffer = append_char_to_input_buffer(*input_buffer, 43);
+            return TRUE;
+        case GDK_KEY_KP_Subtract:
+            *input_buffer = append_char_to_input_buffer(*input_buffer, 45);
+            return TRUE;
+        case GDK_KEY_KP_Decimal:
+            *input_buffer = append_char_to_input_buffer(*input_buffer, 46);
+            return TRUE;
+        case GDK_KEY_KP_Divide:
+            *input_buffer = append_char_to_input_buffer(*input_buffer, 47);
+            return TRUE;
+        case GDK_KEY_dead_circumflex:
+            *input_buffer = append_char_to_input_buffer(*input_buffer, 94);
+            return TRUE;
+        case GDK_KEY_dead_grave:
+            *input_buffer = append_char_to_input_buffer(*input_buffer, 96);
             return TRUE;
         case GDK_KEY_dead_tilde:
-            *input_buffer =
-                append_char_to_input_buffer(*input_buffer, GDK_KEY_asciitilde);
+            *input_buffer = append_char_to_input_buffer(*input_buffer, 126);
+            return TRUE;
+        case GDK_KEY_BackSpace:
+            *input_buffer = append_char_to_input_buffer(*input_buffer, 127);
             return TRUE;
     }
 }
@@ -268,6 +381,27 @@ static gboolean ctrl_plus_key_pressed(char **input_buffer, guint keyval, GtkWidg
             /* ctrl+e, go to the end of the line */
             *input_buffer = append_char_to_input_buffer(*input_buffer, 5);
             return TRUE;
+#ifndef MACOSX_SUPPORT
+        case GDK_KEY_c:
+        case GDK_KEY_C:
+            vte_terminal_copy_clipboard(VTE_TERMINAL(terminal));
+            /* _format only exists in bleeding edge VTE 0.50 */
+            /* vte_terminal_copy_clipboard_format(VTE_TERMINAL(terminal), VTE_FORMAT_TEXT); */
+            return TRUE;
+        case GDK_KEY_v:
+        case GDK_KEY_V:
+            *input_buffer = append_string_to_input_buffer(*input_buffer, terminal, GDK_SELECTION_CLIPBOARD);
+            return TRUE;
+#endif
+    }
+}
+
+#ifdef MACOSX_SUPPORT
+static gboolean cmd_plus_key_pressed(char **input_buffer, guint keyval, GtkWidget *terminal)
+{
+    switch (keyval) {
+        default:
+            return FALSE;
         case GDK_KEY_c:
         case GDK_KEY_C:
             vte_terminal_copy_clipboard(VTE_TERMINAL(terminal));
@@ -280,6 +414,7 @@ static gboolean ctrl_plus_key_pressed(char **input_buffer, guint keyval, GtkWidg
             return TRUE;
     }
 }
+#endif
 
 static gboolean key_press_event (GtkWidget   *widget,
                                  GdkEventKey *event,
@@ -291,15 +426,15 @@ static gboolean key_press_event (GtkWidget   *widget,
     gdk_event_get_state((GdkEvent*)event, &state);
 
     if (*input_buffer && event->type == GDK_KEY_PRESS) {
-        switch (state & (GDK_SHIFT_MASK | GDK_CONTROL_MASK)) {
-            case 0:
-            case GDK_SHIFT_MASK:
-                return plain_key_pressed(input_buffer, event->keyval);
-            case GDK_CONTROL_MASK:
-                return ctrl_plus_key_pressed(input_buffer, event->keyval, widget);
-            default:
-                return FALSE;
+        if (state & GDK_CONTROL_MASK) {
+            return ctrl_plus_key_pressed(input_buffer, event->keyval, widget);
         }
+#ifdef MACOSX_SUPPORT
+        if (state & GDK_MOD2_MASK) {
+            return cmd_plus_key_pressed(input_buffer, event->keyval, widget);
+        }
+#endif
+        return plain_key_pressed(input_buffer, event->keyval);
     }
     return FALSE;
 }
@@ -329,14 +464,16 @@ static gboolean close_window(GtkWidget *widget, GdkEvent *event, gpointer user_d
     return gtk_widget_hide_on_delete(widget);
 }
 
-
+/* \brief Block until some monitor input event happens */
 int uimon_get_string(struct console_private_s *t, char* string, int string_len)
 {
     int retval=0;
     while(retval<string_len) {
         int i;
 
-        gtk_main_iteration();
+        /* give the ui a chance to do stuff, and also don't max out the CPU waiting for input */
+        tick_sleep(tick_per_second() / 60);
+
         if (!t->input_buffer) {
             return -1;
         }
@@ -369,9 +506,9 @@ static void screen_resize_window_cb (VteTerminal *terminal,
 static void screen_resize_window_cb2 (VteTerminal *terminal,
                          gpointer* window)
 {
-    int width, height;
-    int cwidth, cheight;
-    int newwidth, newheight;
+    gint width, height;
+    glong cwidth, cheight;
+    glong newwidth, newheight;
 
     gtk_window_get_size (GTK_WINDOW(fixed.window), &width, &height);
     cwidth = vte_terminal_get_char_width (VTE_TERMINAL(fixed.term));
@@ -391,25 +528,79 @@ static void screen_resize_window_cb2 (VteTerminal *terminal,
 
 /** \brief  Create an icon by loading it from the vice.gresource file
  *
- * \return  Standard C= icon ripped from the internet (but at least scalable)
- *          Which of course looks weird on Windows for some reason, *sigh*.
+ * \return  Current emulator's icon
+ *
+ * \note    If we want something else, we should ask whoever created the current
+ *          icon set.
  */
 static GdkPixbuf *get_default_icon(void)
 {
-    return uidata_get_pixbuf("CBM_Logo.svg");
+    char buffer[1024];
+
+    g_snprintf(buffer, sizeof(buffer), "%s.svg", machine_name);
+    return uidata_get_pixbuf(buffer);
 }
 
 console_t *uimonfb_window_open(void);
 
-console_t *uimon_window_open(void)
+
+/** \brief  Try to set VTE monitor font
+ *
+ * \return  boolean
+ */
+bool uimon_set_font(void)
+{
+    const PangoFontDescription *desc_tmp;
+    PangoFontDescription* desc;
+    const char *monitor_font = NULL;
+    GList *widgets;
+    GList *box;
+
+
+    if (resources_get_string("MonitorFont", &monitor_font) < 0) {
+        log_error(LOG_ERR, "Failed to read 'MonitorFont' resource.");
+        return false;
+    }
+
+    if (fixed.term == NULL) {
+        log_error(LOG_ERR, "No monitor instance found.");
+        return false;
+    }
+
+    /* try to set monitor font */
+    desc = pango_font_description_from_string(monitor_font);
+    if (desc == NULL) {
+        /* fall back */
+        log_warning(LOG_ERR, "Failed to parse Pango font description, falling"
+                " back to default font.");
+
+        desc_tmp = vte_terminal_get_font(VTE_TERMINAL(fixed.term));
+        desc = pango_font_description_copy_static(desc_tmp);
+        pango_font_description_set_family(desc, "Consolas,monospace");
+        pango_font_description_set_size(desc, 11 * PANGO_SCALE);
+    }
+    vte_terminal_set_font(VTE_TERMINAL(fixed.term), desc);
+    pango_font_description_free(desc);
+
+    gtk_widget_set_size_request(GTK_WIDGET(fixed.window), -1, -1);
+    gtk_widget_set_size_request(GTK_WIDGET(fixed.term), -1, -1);
+
+    /* get GtkBox */
+    widgets = gtk_container_get_children(GTK_CONTAINER(fixed.window));
+    box = g_list_first(widgets);
+
+    gtk_widget_set_size_request(GTK_WIDGET(box->data), -1 , -1);
+    return true;
+}
+
+static gboolean uimon_window_open_impl(gpointer user_data)
 {
     GtkWidget *scrollbar, *horizontal_container;
     GdkGeometry hints;
     GdkPixbuf *icon;
+    int sblines;
 
-    if (native_monitor()) {
-        return uimonfb_window_open();
-    }
+    resources_get_int("MonitorScrollbackLines", &sblines);
 
     if (fixed.window == NULL) {
         fixed.window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
@@ -425,14 +616,14 @@ console_t *uimon_window_open(void)
         }
 
         fixed.term = vte_terminal_new();
-        vte_terminal_set_scrollback_lines (VTE_TERMINAL(fixed.term), 1000);
+        vte_terminal_set_scrollback_lines (VTE_TERMINAL(fixed.term), sblines);
         vte_terminal_set_scroll_on_output (VTE_TERMINAL(fixed.term), TRUE);
 
         /* allowed window widths are base_width + width_inc * N
          * allowed window heights are base_height + height_inc * N
          */
-        hints.width_inc = vte_terminal_get_char_width (VTE_TERMINAL(fixed.term));
-        hints.height_inc = vte_terminal_get_char_height (VTE_TERMINAL(fixed.term));
+        hints.width_inc = (gint)vte_terminal_get_char_width (VTE_TERMINAL(fixed.term));
+        hints.height_inc = (gint)vte_terminal_get_char_height (VTE_TERMINAL(fixed.term));
         /* min size should be multiple of .._inc, else we get funky effects */
         hints.min_width = hints.width_inc;
         hints.min_height = hints.height_inc;
@@ -446,12 +637,21 @@ console_t *uimon_window_open(void)
                                      GDK_HINT_MIN_SIZE |
                                      GDK_HINT_BASE_SIZE);
         scrollbar = gtk_scrollbar_new(GTK_ORIENTATION_VERTICAL,
-        gtk_scrollable_get_vadjustment (GTK_SCROLLABLE(fixed.term)));
+                gtk_scrollable_get_vadjustment(GTK_SCROLLABLE(fixed.term)));
 
         horizontal_container = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
         gtk_container_add(GTK_CONTAINER(fixed.window), horizontal_container);
+
+#if 0
         gtk_container_add(GTK_CONTAINER(horizontal_container), fixed.term);
         gtk_container_add(GTK_CONTAINER(horizontal_container), scrollbar);
+#else
+        gtk_box_pack_start(GTK_BOX(horizontal_container), fixed.term,
+                TRUE, TRUE, 0);
+        gtk_box_pack_end(GTK_BOX(horizontal_container), scrollbar,
+                FALSE, FALSE, 0);
+#endif
+
 
         g_signal_connect(G_OBJECT(fixed.window), "delete-event",
             G_CALLBACK(close_window), &fixed.input_buffer);
@@ -469,18 +669,19 @@ console_t *uimon_window_open(void)
             G_CALLBACK (screen_resize_window_cb2), NULL);
 
         vte_console.console_can_stay_open = 1;
+
+        uimon_set_font();
+    } else {
+        vte_terminal_set_scrollback_lines (VTE_TERMINAL(fixed.term), sblines);
     }
-    return uimon_window_resume();
+
+    uimon_window_resume_impl(user_data);
+
+    return FALSE;
 }
 
-console_t *uimon_window_resume(void)
+static gboolean uimon_window_resume_impl(gpointer user_data)
 {
-    GtkWindow *active;
-
-    if (native_monitor()) {
-        return uimonfb_window_resume();
-    }
-
     gtk_widget_show_all(fixed.window);
     screen_resize_window_cb (VTE_TERMINAL(fixed.term), NULL);
 
@@ -489,15 +690,89 @@ console_t *uimon_window_resume(void)
      * window. This makes the monitor window show when the emulated machine
      * window is in fullscreen mode. (only tested on Windows 10)
      */
+#if 0
     active = ui_get_active_window();
     if (active != GTK_WINDOW(fixed.window)) {
         debug_gtk3("setting monitor window transient for emulator window.");
         gtk_window_set_transient_for(GTK_WINDOW(fixed.window), active);
     }
-
+#endif
     gtk_window_present(GTK_WINDOW(fixed.window));
 
-    ui_dispatch_events();
+    return FALSE;
+}
+
+static gboolean uimon_window_suspend_impl(gpointer user_data)
+{
+    if (fixed.window != NULL) {
+        int keep_open = 0;
+
+        /* do need to keep the monitor window open? */
+        resources_get_int("KeepMonitorOpen", &keep_open);
+        if (!keep_open) {
+            gtk_widget_hide(fixed.window);
+        } else {
+            /* move monitor window behind the emu window */
+            GtkWidget *window = ui_get_window_by_index(ui_get_main_window_index());
+            gtk_window_present(GTK_WINDOW(window));
+        }
+    }
+
+    return FALSE;
+}
+
+static gboolean uimon_out_impl(gpointer user_data)
+{
+    char *buffer = (char*)user_data;
+    const char *c;
+
+    for(c = buffer; *c; c++) {
+        if(*c == '\n') {
+            uimon_write_to_terminal(&fixed, "\r", 1);
+        }
+        uimon_write_to_terminal(&fixed, c, 1);
+    }
+
+    lib_free(buffer);
+
+    return FALSE;
+}
+
+static gboolean uimon_window_close_impl(gpointer user_data)
+{
+    /* Flush any queued writes */
+    uimon_write_to_terminal_impl(NULL);
+
+    /* only close window if there is one: this avoids a GTK_CRITICAL warning
+     * when using a remote monitor */
+    if (fixed.window != NULL) {
+        gtk_widget_hide(fixed.window);
+    }
+
+    return FALSE;
+}
+
+console_t *uimon_window_open(void)
+{
+    if (native_monitor()) {
+        return uimonfb_window_open();
+    }
+
+    /* call from ui thread */
+    gdk_threads_add_timeout(0, uimon_window_open_impl, NULL);
+
+    return &vte_console;
+}
+
+console_t *uimon_window_resume(void)
+{
+    if (native_monitor()) {
+        return uimonfb_window_resume();
+    }
+
+    /* call from ui thread */
+    gdk_threads_add_timeout(0, uimon_window_resume_impl, NULL);
+
     return &vte_console;
 }
 
@@ -507,20 +782,20 @@ void uimon_window_suspend(void)
         uimonfb_window_suspend();
         return;
     }
+
+    /* call from ui thread */
+    gdk_threads_add_timeout(0, uimon_window_suspend_impl, NULL);
 }
 
 int uimon_out(const char *buffer)
 {
-    const char *c;
     if (native_monitor()) {
         return uimonfb_out(buffer);
     }
-    for(c = buffer; *c; c++) {
-        if(*c == '\n') {
-            uimon_write_to_terminal(&fixed, "\r", 1);
-        }
-        uimon_write_to_terminal(&fixed, c, 1);
-    }
+
+    /* call from ui thread */
+    gdk_threads_add_timeout(0, uimon_out_impl, (gpointer)lib_strdup(buffer));
+
     return 0;
 }
 
@@ -531,11 +806,8 @@ void uimon_window_close(void)
         return;
     }
 
-    /* only close window if there is one: this avoids a GTK_CRITICAL warning
-     * when using a remote monitor */
-    if (fixed.window != NULL) {
-        gtk_widget_hide(fixed.window);
-    }
+    /* call from ui thread */
+    gdk_threads_add_timeout(0, uimon_window_close_impl, NULL);
 }
 
 void uimon_notify_change(void)
@@ -677,20 +949,20 @@ char *uimon_get_in(char **ppchCommandLine, const char *prompt)
         return uimonfb_get_in(ppchCommandLine, prompt);
     }
 
-    fixed.input_buffer = lib_strdup("");;
+    if (!fixed.input_buffer) {
+        fixed.input_buffer = lib_strdup("");
+    }
     vte_linenoiseSetCompletionCallback(monitor_completions);
     p = vte_linenoise(prompt, &fixed);
     if (p) {
         if (*p) {
             vte_linenoiseHistoryAdd(p);
         }
-        ret_string = lib_strdup(p); /* LEAKS */
+        ret_string = lib_strdup(p);
         free(p);
     } else {
         ret_string = lib_strdup("x");
     }
-    lib_free(fixed.input_buffer);
-    fixed.input_buffer = NULL;
 
     return ret_string;
 }
@@ -702,10 +974,12 @@ int console_init(void)
     char *short_name;
     int takes_filename_as_arg;
 
+
     if (native_monitor()) {
         return consolefb_init();
     }
 
+
     while (mon_get_nth_command(i++,
                 &full_name,
                 &short_name,
@@ -730,6 +1004,14 @@ int console_close_all(void)
 {
     int i;
 
+    if (fixed.input_buffer) {
+        /* This happens if the application exits with the monitor open, as the VICE thread
+         * exits while the monitor is waiting for user input into this buffer.
+         */
+        lib_free(fixed.input_buffer);
+        fixed.input_buffer = NULL;
+    }
+
     if (native_monitor()) {
         return consolefb_close_all();
     }
@@ -740,6 +1022,7 @@ int console_close_all(void)
     for(i = 0; i < need_filename_lc.len; i++) {
         free(need_filename_lc.cvec[i]);
     }
+
     return 0;
 }
 
@@ -774,10 +1057,11 @@ gboolean ui_monitor_activate_callback(GtkWidget *widget, gpointer user_data)
         /* FIXME: restore mouse in case it was grabbed */
         /* ui_restore_mouse(); */
 #endif
-        if (!ui_pause_active()) {
-            monitor_startup_trap();
+
+        if (ui_pause_active()) {
+            ui_pause_enter_monitor();
         } else {
-            monitor_startup(e_default_space);
+            monitor_startup_trap();
         }
     }
     return TRUE;
