@@ -30,7 +30,6 @@
 
 #include "basewidgets.h"
 #include "basedialogs.h"
-#include "debug_gtk3.h"
 #include "widgethelpers.h"
 #include "filechooserhelpers.h"
 #include "util.h"
@@ -89,7 +88,6 @@ static void on_response(GtkWidget *widget, gint response_id, gpointer data)
             gtk_widget_destroy(widget);
             break;
         default:
-            debug_gtk3("warning: unhandled response ID %d\n", response_id);
             break;
     }
 }
@@ -154,8 +152,9 @@ static GtkWidget *create_extra_widget(void)
  * \param[in]   parent  parent widget (ignored)
  * \param[in]   data    extra data (ignored)
  *
+ * \return  TRUE
  */
-gboolean uitapecreate_dialog_show(GtkWidget *parent, gpointer data)
+gboolean ui_tape_create_dialog_show(GtkWidget *parent, gpointer data)
 {
     GtkWidget *dialog;
     GtkFileFilter *filter;
