@@ -37,6 +37,23 @@
 #include "video.h"
 
 #include <gtk/gtk.h>
+#include <pthread.h>
+
+
+/** \brief  Enum for rendering backends for the Gtk3 port
+ *
+ * Currently OpenGL and DirectX map to the same integer value since that's how
+ * our current 'GtkBackend' resource works: 0 = Cario (SW), 1 = HW (OS-dependent).
+ * The Metal and Vulkan enums are added for future renderers, should we add them.
+ */
+enum {
+    VICE_RENDER_BACKEND_CAIRO = 0,      /**< Cairo (software) rendering */
+    VICE_RENDER_BACKEND_OPENGL = 1,     /**< OpenGL rendering (unix, mac) */
+    VICE_RENDER_BACKEND_DIRECTX = 1,    /**< DirectX rendering (windows) */
+    VICE_RENDER_BACKEND_METAL = 2,      /**< Metal rendering (unsupported) */
+    VICE_RENDER_BACKEND_VULKAN = 3      /**< Vulkan rendering (unsupported) */
+};
+
 
 struct vice_renderer_backend_s;
 
@@ -47,50 +64,62 @@ typedef struct video_canvas_s {
     /** \brief Nonzero if it is safe to access other members of the
      *         structure. */
     unsigned int initialized;
+    
     /** \brief Nonzero if the structure has been fully realized. */
     unsigned int created;
 
+    /** \brief Used to coordinate vice thread access */
+    pthread_mutex_t lock;
+
     /** \brief Top-level widget that contains the full contents of the
      *         machine window. */
     GtkWidget *grid;
-    /** \brief Child widget to which the emulated screen is drawn. 
-     *
-     *  Depending on what renderer backend is in use this will be
-     *  either a GtkDrawingArea or a GtkGLArea. */
-    GtkWidget *drawing_area;
+
+    /** \brief Widget for mouse input and to size the rendering overlay.
+     */
+    GtkWidget *event_box;
+    
     /** \brief The renderer backend selected for use this run. */
     struct vice_renderer_backend_s *renderer_backend;
+    
     /** \brief Data unique to the renderer backend. This value is
      *         passed to all renderer methods. and is managed by
      *         them. */
     void *renderer_context;
+    
     /** \brief Special "blank" cursor for cases where the mouse
      *         pointer should disappear. */
     GdkCursor *blank_ptr;
+    
     /** \brief Special "target" cursor for active light pens. */
     GdkCursor *pen_ptr;
-    /** \brief Number of frames the mouse hasn't moved while still on
-     *         the canvas. */
-    unsigned int still_frames;
+    
     /** \brief Handle to the timer callback that will make the mouse
      *         disappear if it's hovered for too long over the screen
      *         display. */
     guint still_frame_callback_id;
+    
     /** \brief Light pen X coordinate, in window coordinates. */
     int pen_x;
+    
     /** \brief Light pen Y coordinate, in window coordinates. */
     int pen_y;
+    
     /** \brief Light pen button status. */
     int pen_buttons;
+    
     /** \brief Leftmost X coordinate of the actual machine's screen,
      *         in window coordinates. */
     double screen_origin_x;
+    
     /** \brief Topmost Y coordinate of the actual machine's screen, in
      *         window coordinates. */
     double screen_origin_y;
+    
     /** \brief Width of the actual machine's screen, in window
      *         coordinates. */
     double screen_display_w;
+    
     /** \brief Height of the actual machine's screen, in window
      *         coordinates. */
     double screen_display_h;
@@ -98,16 +127,21 @@ typedef struct video_canvas_s {
     /** \brief Rendering configuration as seen by the emulator
      *         core. */
     struct video_render_config_s *videoconfig;
+    
     /** \brief Drawing buffer as seen by the emulator core. */
     struct draw_buffer_s *draw_buffer;
+    
     /** \brief Display window as seen by the emulator core. */
     struct viewport_s *viewport;
+    
     /** \brief Machine screen geometry as seen by the emulator
      *         core. */
     struct geometry_s *geometry;
+    
     /** \brief Color palette for translating display results into
      *         window colors. */
     struct palette_s *palette;
+    
     /** \brief Methods for managing the draw buffer when the core
      *         rasterizer handles it. */
     struct video_draw_buffer_callback_s *video_draw_buffer_callback;
@@ -131,15 +165,12 @@ void video_canvas_adjust_aspect_ratio(struct video_canvas_s *canvas);
  *  scaled) pixel content or incrementally updating it. These routines
  *  let us keep those differences contained. */
 typedef struct vice_renderer_backend_s {
-    /** \brief Creates a widget suitable for this renderer to target.
+    /** \brief Add event handlers to the event box and create context.
      *
-     *  Also initializes the opaque video_canvas_s::renderer_context
-     *  field if needed, and sets other necessary fields.
-     *
-     *  \param canvas The canvas to create the widget for.
-     *  \return The newly created canvas.
+     *  \param canvas The canvas to initialise.
+     *  \return The newly created widget.
      */
-    GtkWidget *(*create_widget)(video_canvas_t *canvas);
+    void (*initialise)(video_canvas_t *canvas);
     /** \brief Creates or resizes the pixel buffer that this renderer
      *         backend is using for the screen.
      *
@@ -161,8 +192,7 @@ typedef struct vice_renderer_backend_s {
     void (*destroy_context)(video_canvas_t *canvas);
     /** \brief Render pixels in the specified rectangle.
      *
-     * This both asks the emulator core to update the renderer context
-     * and asks the UI to display the changed results.
+     * This asks the emulator core to update the renderer context.
      *
      * \param canvas The canvas being rendered to
      * \param xs     A parameter to forward to video_canvas_render()
@@ -176,6 +206,12 @@ typedef struct vice_renderer_backend_s {
                          unsigned int xs, unsigned int ys,
                          unsigned int xi, unsigned int yi,
                          unsigned int w, unsigned int h);
+    /** \brief Queue a redraw operation from the UI thread
+     *
+     * \param clock The window GtkFrameClock generating the event 
+     * \param widget UI widget to queue a redraw for
+     */
+    void (*queue_redraw)(GdkFrameClock *clock, video_canvas_t *canvas);
     /** \brief Initialize the palette for this renderer.
      *
      * \param canvas The canvas being initialized
