@@ -30,8 +30,6 @@
 
 #include <stdio.h>
 
-#include "debug_gtk3.h"
-
 #include "vice_gtk3.h"
 #include "machine.h"
 #include "ui.h"
@@ -262,7 +260,5 @@ int vsid_ui_init(void)
     ui_create_main_window(canvas);
     ui_display_main_window(canvas->window_index);
 
-    /* for debugging */
-    debug_gtk3("libhvsc version: %s.", hvsc_lib_version_str());
     return 0;
 }
