@@ -30,16 +30,20 @@
 #include "vice.h"
 #include <gtk/gtk.h>
 
-gchar *vice_gtk3_open_file_dialog(
+GtkWidget *vice_gtk3_open_file_dialog(
         const char *title,
         const char *filter_desc,
         const char **filter_list,
-        const char *path);
+        const char *path,
+        void (*callback)(GtkDialog *dialog, gchar *, gpointer),
+        gpointer param);
 
-gchar *vice_gtk3_open_create_file_dialog(
+GtkWidget *vice_gtk3_open_create_file_dialog(
         const char *title,
         const char *proposed,
         gboolean confirm,
-        const char *path);
+        const char *path,
+        void (*callback)(GtkDialog *dialog, gchar *, gpointer),
+        gpointer param);
 
 #endif
