@@ -33,17 +33,24 @@
 
 GtkWidget *vice_gtk3_resource_browser_new(
         const char *resource,
-        const char **patterns,
+        const char * const *patterns,
         const char *pattern_name,
         const char *browser_title,
         const char *label,
         void (*callback)(GtkWidget *, gpointer));
 
+GtkWidget *vice_gtk3_resource_browser_save_new(
+        const char *resource,
+        char *browser_title,
+        char *label,
+        const char *suggested,
+        void (*callback)(GtkWidget *, gpointer));
+
+
 gboolean vice_gtk3_resource_browser_set(GtkWidget *widget, const char *new);
 gboolean vice_gtk3_resource_browser_get(GtkWidget *widget, const char **dest);
 gboolean vice_gtk3_resource_browser_reset(GtkWidget *widget);
 gboolean vice_gtk3_resource_browser_sync(GtkWidget *widget);
 gboolean vice_gtk3_resource_browser_factory(GtkWidget *widget);
-gboolean vice_gtk3_resource_browser_apply(GtkWidget *widget);
 
 #endif
