@@ -141,11 +141,10 @@ static GtkWidget *resource_switch_new_helper(GtkWidget *widget)
             widget,
             vice_gtk3_resource_switch_reset,
             vice_gtk3_resource_switch_factory,
-            vice_gtk3_resource_switch_sync,
-            vice_gtk3_resource_switch_apply);
+            vice_gtk3_resource_switch_sync);
     g_signal_connect(widget, "state-set", G_CALLBACK(on_switch_state_set),
             (gpointer)resource);
-    g_signal_connect(widget, "destroy", G_CALLBACK(on_switch_destroy),
+    g_signal_connect_unlocked(widget, "destroy", G_CALLBACK(on_switch_destroy),
             NULL);
 
     gtk_widget_show(widget);
@@ -313,16 +312,3 @@ gboolean vice_gtk3_resource_switch_sync(GtkWidget *widget)
     }
     return TRUE;
 }
-
-
-/** \brief  Set resource to the widget's value
- *
- * \param[in,out]   widget  resource switch button widget
- *
- * \return  bool
- */
-gboolean vice_gtk3_resource_switch_apply(GtkWidget *widget)
-{
-    NOT_IMPLEMENTED_WARN_ONLY();
-    return FALSE;
-}
