@@ -38,6 +38,8 @@
 #include "vice.h"
 
 #include <gtk/gtk.h>
+#include <string.h>
+#include <stdbool.h>
 
 #include "lib.h"
 #include "resources.h"
@@ -73,6 +75,9 @@ GtkWidget *uihelpers_create_grid_with_label(const gchar *text, gint columns)
     GtkWidget *label;
     gchar buffer[LABEL_BUFFER_SIZE];
 
+
+    debug_gtk3("Deprecated in favout of vice_gtk3_grid_new_spaced()");
+
     /* sanitize columns input */
     if (columns < 1) {
         columns = 1;
@@ -94,56 +99,6 @@ GtkWidget *uihelpers_create_grid_with_label(const gchar *text, gint columns)
     return grid;
 }
 
-/** \brief  Create a GtkGrid with a label radio buttons with text/id pairs
- *
- * \param[in]   label       label text
- * \param[in]   data        text/value pairs for the radio buttons
- * \param[in]   callback    optional callback to trigger when a radio button
- *                          is toggled (`NULL` == no callback)
- *
- * \note    keep in mind that the callback is also triggered when a radio button
- *          is deactivated, so use gtk_toggle_button_get_active() if you only
- *          want to respond to the currently activated radio button
- *
- * \return  GtkGrid
- */
-GtkWidget *uihelpers_radiogroup_create(
-        const gchar *label,
-        const vice_gtk3_radiogroup_entry_t *data,
-        void (*callback)(GtkWidget *, gpointer),
-        int active)
-{
-    GtkWidget *grid;
-    GtkRadioButton *last;
-    GSList *group = NULL;
-    size_t i;
-
-    debug_gtk3("DEPRECATED in favour of vice_gtk3_resource_radiogroup!");
-    grid = uihelpers_create_grid_with_label(label, 1);
-
-    last = NULL;
-    for (i = 0; data[i].name != NULL; i++) {
-        GtkWidget *radio = gtk_radio_button_new_with_label(group, data[i].name);
-        gtk_radio_button_join_group(GTK_RADIO_BUTTON(radio), last);
-        gtk_grid_attach(GTK_GRID(grid), radio, 0, i + 1, 1, 1);
-        g_object_set(radio, "margin-left", 16, NULL);  /* indent 16 units */
-
-        if (active == i) {
-            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio), TRUE);
-        }
-
-        if (callback != NULL) {
-            g_signal_connect(radio, "toggled", G_CALLBACK(callback),
-                    GINT_TO_POINTER(data[i].id));
-        }
-        gtk_widget_show(radio);
-        last = GTK_RADIO_BUTTON(radio);
-    }
-    g_object_set(grid, "margin", 8, NULL);
-    gtk_widget_show(grid);
-    return grid;
-}
-
 
 /** \brief  Get index of \a value in \a list
  *
@@ -286,4 +241,88 @@ GtkWidget *vice_gtk3_grid_new_spaced_with_label(int column_spacing,
 }
 
 
+/** \brief  Convert petscii encoded string to utf8 string we can show using the CBM font
+ *
+ * this function handles all characters that may appear in a directory listing,
+ * including "non printable" control characters, which appear as inverted characters
+ * in so called "quote mode".
+ *
+ * \param[in]   s           PETSCII string to convert to UTF-8
+ * \param[in]   inverted    use inverted mode
+ * \param[in]   lowercase   use the lowercase chargen
+ */
+unsigned char *vice_gtk3_petscii_to_utf8(unsigned char *s,
+                                         bool inverted,
+                                         bool lowercase)
+{
+    unsigned char *d, *r;
+    unsigned int codepoint;
+
+    r = d = lib_malloc((size_t)(strlen((char *)s) * 3 + 1));
+
+    while (*s) {
+
+        /* 0xe000-0xe0ff codepoints cover the regular, uppercase, petscii codes
+                         in ranges 0x20-0x7f and 0xa0-0xff
+           0xe200-0xe2ff codepoints cover the same characters, but contain the
+                         respective inverted glyphs.
+
+           regular valid petscii codes are converted as is, petscii control 
+           codes will produce the glyph that the petscii code would produce
+           in so called "quote mode".
+        */
+
+        /* first convert petscii to utf8 codepoint */
+        if (*s < 0x20) {
+            /* petscii 0x00-0x1f  control codes (inverted @ABC..etc) */
+            codepoint  = *s + 0xe240;           /* 0xe240-0xe25f */
+        } else if (*s < 0x80) {
+            /* petscii 0x20-0x7f  printable petscii codes */
+            codepoint = *s + 0xe000;            /* 0xe020-0xe07f */
+        } else if (*s < 0xa0) {
+            /* petscii 0x80-0x9f  control codes (inverted SHIFT+@ABC..etc) */
+            codepoint = (*s - 0x80) + 0xe260;   /* 0xe260-0xe27f */
+        } else {
+            /* petscii 0xa0-0xff  printable petscii codes */
+            codepoint = *s + 0xe000;            /* 0xe0a0-0xe0ff */
+        }
+        if (inverted) {
+            codepoint ^= 0x0200;                /* 0xe0XX <-> 0xe2XX */
+        }
+        /* switch to lower case if requested */
+        if (lowercase) {
+            codepoint ^= 0x0100;
+        }
+        s++;
+
+        /* now copy to the destination string and convert to utf8 */
+#if 0
+        if (codepoint < 0x80) {
+            /* one byte form - 0xxxxxxx */
+            *d = codepoint;
+        } else if (codepoint < 0x800) {
+            /* two byte form - 110xxxxx 10xxxxxx */
+
+            /* for some reason 0xad will not result in output in the popup
+             * menu, so we remap it to 0xed, which works */
+            if (codepoint == 0xad) { /* Unicode U+00AD SOFT HYPHEN */
+                codepoint = 0xed;
+            }
+
+            *d++ = 0xc0 | (codepoint >> 6);
+            *d = (codepoint & ~0xc0) | 0x80;
+        } else
+#endif
+        /* we can get away with just this, because all codepoints are > 4095 */
+        {
+            /* three byte form - 1110xxxx 10xxxxxx 10xxxxxx */
+            *d++ = 0xe0 | ((codepoint >> 12) & 0x0f);
+            *d++ = 0x80 | ((codepoint >> (6)) & 0x3f);
+            *d   = 0x80 | ((codepoint >> (0)) & 0x3f);
+        }
+        d++;
+    }
+    *d = '\0';
+    return r;
+}
 
