@@ -31,20 +31,26 @@
  */
 
 #include "vice.h"
+
 #include <gtk/gtk.h>
 
-#include "machine.h"
-#include "resources.h"
-#include "debug_gtk3.h"
-#include "basewidgets.h"
-#include "widgethelpers.h"
 #include "basedialogs.h"
-#include "openfiledialog.h"
+#include "basewidgets.h"
 #include "cartridge.h"
+#include "debug_gtk3.h"
+#include "machine.h"
+#include "openfiledialog.h"
+#include "resources.h"
+#include "ui.h"
+#include "widgethelpers.h"
 
 #include "ieee488widget.h"
 
 
+
+static GtkWidget *entry_widget;
+
+
 /** \brief  Handler for the "toggled" event of the 'enable' check button
  *
  * Toggles the 'enabled' state of the IEEE-488 adapter/cart, but only if an
@@ -87,24 +93,19 @@ static void on_enable_toggled(GtkWidget *widget, gpointer data)
 }
 
 
-/** \brief  Handler for the "clicked" event of the browse button
+/** \brief  Callback for the open-file dialog
  *
- * Activates a file-open dialog and stores the file name in the GtkEntry passed
- * in \a user_data if valid, triggering a resource update.
- *
- * \param[in]   widget      button
- * \param[in]   user_data   entry to store filename in
+ * \param[in,out]   dialog      open-file dialog
+ * \param[in]       filename    filename or NULL on cancel
+ * \param[in]       data        extra data (unused)
  */
-static void on_browse_clicked(GtkWidget *widget, gpointer user_data)
+static void browse_filename_callback(GtkDialog *dialog,
+                                     gchar *filename,
+                                     gpointer data)
 {
-    gchar *filename;
-
-    filename = vice_gtk3_open_file_dialog("Open IEEE-488 image", NULL, NULL,
-            NULL);
     if (filename != NULL) {
-        GtkEntry *entry = GTK_ENTRY(user_data);
-        debug_gtk3("setting IEEE488Image to '%s'.", filename);
-        gtk_entry_set_text(entry, filename);
+        gtk_entry_set_text(GTK_ENTRY(entry_widget), filename);
+
         /* required, since setting the text of the entry doesn't trigger an
          * update of the connected resource (it only responds to focus-out and
          * pressing 'Enter' */
@@ -115,6 +116,26 @@ static void on_browse_clicked(GtkWidget *widget, gpointer user_data)
         }
         g_free(filename);
     }
+
+    gtk_widget_destroy(GTK_WIDGET(dialog));
+}
+
+
+/** \brief  Handler for the "clicked" event of the browse button
+ *
+ * Activates a file-open dialog and stores the file name in the GtkEntry passed
+ * in \a user_data if valid, triggering a resource update.
+ *
+ * \param[in]   widget      button
+ * \param[in]   user_data   entry to store filename in
+ */
+static void on_browse_clicked(GtkWidget *widget, gpointer user_data)
+{
+    vice_gtk3_open_file_dialog(
+            "Open IEEE-488 image",
+            NULL, NULL, NULL,
+            browse_filename_callback,
+            NULL);
 }
 
 
@@ -129,7 +150,6 @@ GtkWidget *ieee488_widget_create(GtkWidget *parent)
     GtkWidget *grid;
     GtkWidget *enable_widget;
     GtkWidget *label;
-    GtkWidget *entry;
     GtkWidget *browse;
 
     const char *image;
@@ -157,14 +177,13 @@ GtkWidget *ieee488_widget_create(GtkWidget *parent)
 
     label = gtk_label_new("IEEE-488 image");
     gtk_widget_set_halign(label, GTK_ALIGN_START);
-    entry = vice_gtk3_resource_entry_full_new("IEEE488Image");
-    gtk_widget_set_hexpand(entry, TRUE);
+    entry_widget = vice_gtk3_resource_entry_full_new("IEEE488Image");
+    gtk_widget_set_hexpand(entry_widget, TRUE);
     browse = gtk_button_new_with_label("Browse ...");
-    g_signal_connect(browse, "clicked", G_CALLBACK(on_browse_clicked),
-            (gpointer)entry);
+    g_signal_connect(browse, "clicked", G_CALLBACK(on_browse_clicked), NULL);
 
     gtk_grid_attach(GTK_GRID(grid), label, 0, 1, 1, 1);
-    gtk_grid_attach(GTK_GRID(grid), entry, 1, 1, 1, 1);
+    gtk_grid_attach(GTK_GRID(grid), entry_widget, 1, 1, 1, 1);
     gtk_grid_attach(GTK_GRID(grid), browse, 2, 1, 1, 1);
 
     g_signal_connect(enable_widget, "toggled", G_CALLBACK(on_enable_toggled), NULL);
