@@ -1,9 +1,10 @@
-/**
+/** \file   petvideosizewidget.h
  * \brief   Widget to set the PET video size - header
  *
- * Written by
- *  Bas Wassink <b.wassink@ziggo.nl>
- *
+ * \author  Bas Wassink <b.wassink@ziggo.nl>
+ */
+
+/*
  * This file is part of VICE, the Versatile Commodore Emulator.
  * See README for copyright notice.
  *
@@ -30,6 +31,9 @@
 #include "vice.h"
 #include <gtk/gtk.h>
 
-GtkWidget *pet_video_size_widget_create(void);
+GtkWidget * pet_video_size_widget_create(void);
+void        pet_video_size_widget_set_callback(GtkWidget *widget,
+                                               void (*func)(int));
+void        pet_video_size_widget_sync(GtkWidget *widget);
 
 #endif
