@@ -2,6 +2,7 @@
  * \brief   Model settings dialog
  *
  * \author  Bas Wassink <b.wassink@ziggo.nl>
+ * \author  Groepaz <groepaz@gmx.de>
  */
 
 /*
@@ -10,6 +11,7 @@
  * $VICERES Go64Mode            x128
  * $VICERES DtvRevision         x64dtv
  * $VICERES VICIINewLuminances  x64dtv
+ * $VICERES HummerADC           x64dtv
  *
  *  (for more, see used widgets)
  */
@@ -53,11 +55,17 @@
 #include "machine.h"
 #include "machinemodelwidget.h"
 #include "petiosizewidget.h"
+#include "petram9widget.h"
+#include "petramawidget.h"
 #include "petkeyboardtypewidget.h"
 #include "petmiscwidget.h"
+#include "petmodel.h"
 #include "petramsizewidget.h"
 #include "petvideosizewidget.h"
+#include "plus4aciawidget.h"
+#include "plus4memhacks.h"
 #include "plus4memoryexpansionwidget.h"
+#include "plus4memorysizewidget.h"
 #include "resourcecheckbutton.h"
 #include "resources.h"
 #include "sidmodelwidget.h"
@@ -66,6 +74,7 @@
 #include "vic20memoryexpansionwidget.h"
 #include "vice_gtk3.h"
 #include "videomodelwidget.h"
+#include "v364speechwidget.h"
 
 #include "c64model.h"
 
@@ -73,6 +82,12 @@
 #include "settings_model.h"
 
 
+/*
+ * Forward declarations
+ */
+static void plus4_debug_dump_resources(void);
+
+
 /** \brief  List of C64DTV revisions
  */
 static const vice_gtk3_radiogroup_entry_t c64dtv_revisions[] = {
@@ -98,12 +113,17 @@ static GtkWidget *cia_widget = NULL;
 
 static int (*get_model_func)(void);
 
+static const char *(*get_memhack_func)(int hack);
+
 
 /** \brief  Video model widget
  */
 static GtkWidget *video_widget = NULL;
 
 static GtkWidget *ram_widget = NULL;
+static GtkWidget *memhack_widget = NULL;
+static GtkWidget *acia_widget = NULL;
+static GtkWidget *speech_widget = NULL;
 static GtkWidget *vdc_widget = NULL;
 static GtkWidget *sid_widget = NULL;
 static GtkWidget *kernal_widget = NULL;
@@ -111,8 +131,12 @@ static GtkWidget *pet_video_size_widget = NULL;
 static GtkWidget *pet_keyboard_widget = NULL;
 static GtkWidget *pet_misc_widget = NULL;
 static GtkWidget *pet_io_widget = NULL;
+static GtkWidget *pet_ram9_widget = NULL;
+static GtkWidget *pet_rama_widget = NULL;
 
 static GtkWidget *c64dtv_rev_widget = NULL;
+static GtkWidget *c64dtv_hummer_adc_widget = NULL;
+static GtkWidget *reset_with_iec_widget = NULL;
 
 
 static void video_model_callback(int model)
@@ -129,11 +153,18 @@ static void video_model_callback(int model)
 #endif
         debug_gtk3("got true model %d", true_model);
         machine_model_widget_update(machine_widget);
+        if (machine_class == VICE_MACHINE_PLUS4) {
+            plus4_debug_dump_resources();
+        }
     }
-
 }
 
 
+/* {{{ C128 glue logic
+ *
+ * x128-specific callbacks
+ */
+
 static void vdc_revision_callback(int revision)
 {
     debug_gtk3("got VDC revision %d.", revision);
@@ -144,13 +175,13 @@ static void vdc_revision_callback(int revision)
 
 
 static void vdc_ram_callback(int state)
-{
-    debug_gtk3("Got VDC 64KB RAM state %d.", state);
+{    debug_gtk3("Got VDC 64KiB RAM state %d.", state);
     if (get_model_func != NULL) {
         machine_model_widget_update(machine_widget);
     }
 }
 
+/* }}} */
 
 static void sid_model_callback(int model)
 {
@@ -170,6 +201,36 @@ static void sid_model_callback(int model)
 }
 
 
+/** \brief  Custom callback for the Kernal Revision widget
+ *
+ * Triggers an update of the 'machine model' widget when a different kernal rev
+ * has been selected. Only valid for x64/x64sc as far as I know.
+ *
+ * \param[in]   rev     new KERNAL revision
+ */
+static void kernal_revision_callback(int rev)
+{
+#ifdef HAVE_DEBUG_GTK3UI
+    int true_model = -1;
+
+    if (get_model_func != NULL) {
+        true_model = get_model_func();
+        debug_gtk3("Got model ID: %d.", true_model);
+    }
+#endif
+    debug_gtk3("Got revision %d.", rev);
+    machine_model_widget_update(machine_widget);
+}
+
+
+static void iec_callback(GtkWidget *widget, gpointer data)
+{
+    debug_gtk3("Called.");
+    machine_model_widget_update(machine_widget);
+}
+
+
+
 /** \brief  Callback for CIA model changes
  *
  * \param[in]   cia_num     CIA number (1 or 2)
@@ -185,9 +246,11 @@ static void cia_model_callback(int cia_num, int cia_model)
 }
 
 
+/* {{{ PET glue logic */
+
 /** \brief  Callback for PET RAM size changes
  *
- * \param[in]   size    RAM size in KB
+ * \param[in]   size    RAM size in KiB
  */
 static void pet_ram_size_callback(int size)
 {
@@ -240,11 +303,169 @@ static void pet_io_callback(int state)
     }
 }
 
+static void pet_ram9_callback(int state)
+{
+    debug_gtk3("called with RAM9 type %d.", state);
+    if (get_model_func != NULL) {
+        machine_model_widget_update(machine_widget);
+    }
+}
+
+static void pet_rama_callback(int state)
+{
+    debug_gtk3("called with RAMA type %d.", state);
+    if (get_model_func != NULL) {
+        machine_model_widget_update(machine_widget);
+    }
+}
+/* }}} */
+
+
+/* {{{ Plus4 glue logic and helpers */
+
+
+static void plus4_debug_dump_resources(void)
+{
+#ifdef HAVE_DEBUG_GTK3UI
+    int model = -1;
+    int video = 0;
+    int ram = 0;
+    int hack = -1;
+    const char *rom = NULL;
+
+    const char *vidmodes[] = { "UNKNOWN", "PAL", "NTSC" };
+
+    /* get model */
+    if (get_model_func != NULL) {
+        model = get_model_func();
+    }
+
+    /* get TED PAL/NTSC mode */
+    if (resources_get_int("MachineVideoStandard", &video) < 0) {
+        video = 0;
+    }
+
+    /* get RAM size */
+    resources_get_int("RamSize", &ram);
+    /* get memory exp hack */
+    resources_get_int("MemoryHack", &hack);
+
+    g_print("Plus4 resources dump:\n");
+    g_print("    get_model_func()    : %d\n", model);
+    g_print("    MachineVideoStandard: %d (%s)\n", video, vidmodes[video]);
+    g_print("    RAM size            : %dKiB\n", ram);
+    g_print("    MemoryHack          : %d (%s)\n",
+            hack,
+            get_memhack_func != NULL ?
+            get_memhack_func(hack) : "get_memhack_func not set");
+
+    /* dump active ROMs */
+    resources_get_string("KernalName", &rom);
+    g_print("    KernalName          : %s\n", rom);
+    resources_get_string("BasicName", &rom);
+    g_print("    BasicName           : %s\n", rom);
+    resources_get_string("FunctionLoWName", &rom);
+    g_print("    FunctionLoWName     : %s\n", rom);
+    resources_get_string("FunctionHighName", &rom);
+    g_print("    FunctionHighName:   : %s\n", rom);
+    resources_get_string("c1loName", &rom);
+    g_print("    c1loName            : %s\n", rom);
+    resources_get_string("c1hiName", &rom);
+    g_print("    c1hiName            : %s\n", rom);
+    resources_get_string("c2loName", &rom);
+    g_print("    c2loName            : %s\n", rom);
+    resources_get_string("c2hiName", &rom);
+    g_print("    c2hiName            : %s\n", rom);
+
+#endif
+}
+
+
+/** \brief  Extra calback for the Plus4 memory size widget
+ *
+ * Triggered when the widget changes value.
+ *
+ * \param[in,out]   widget      plus4 ram size widget
+ * \param[in]       value       new size in KiB
+ */
+static void plus4_mem_size_callback(GtkWidget *widget, int value)
+{
+    int size = 0;
+
+    resources_get_int("RamSize", &size);
+    debug_gtk3("Got new value: %dKiB, RamSize = %d", value, size);
+    debug_gtk3("Calling plus4_memory_expansion_widget_sync(): ");
+    if (plus4_memory_expansion_widget_sync()) {
+        debug_gtk3("OK.");
+    } else {
+        debug_gtk3("failed.");
+    }
+    machine_model_widget_update(machine_widget);
+    plus4_debug_dump_resources();
+}
+
+
+/** \brief  Extra calback for the Plus4 memory expansion hack widget
+ *
+ * Triggered when the widget changes value.
+ *
+ * \param[in,out]   widget      plus4 memory expansion hack widget
+ * \param[in]       value       new size in KiB
+ */
+static void plus4_mem_hack_callback(GtkWidget *widget, int value)
+{
+    int size = 0;
+
+    resources_get_int("RamSize", &size);
+    debug_gtk3("Got new MemoryHack value: %d, RamSize = %d", value, size);
+
+    plus4_memory_size_widget_sync();
+    debug_gtk3("Setting RamSize widget sensitivity to %s",
+            value == MEMORY_HACK_NONE ? "TRUE" : "FALSE");
+    gtk_widget_set_sensitive(ram_widget, value == MEMORY_HACK_NONE);
+    debug_gtk3("Calling plus4_memory_size_widget_sync(): ");
+    if (plus4_memory_size_widget_sync()) {
+        debug_gtk3("OK.");
+    } else {
+        debug_gtk3("failed.");
+    }
+
+    machine_model_widget_update(machine_widget);
+    plus4_debug_dump_resources();
+}
+
+
+/** \brief  Callback to update the Plus4 model widget on ACIA widget change
+ *
+ * \param[in]   widget  ACIA widget
+ * \param[in]   value   new value
+ */
+static void plus4_acia_widget_callback(GtkWidget *widget, int value)
+{
+    debug_gtk3("Called with value %d\n", value);
+    machine_model_widget_update(machine_widget);
+}
+
+
+/** \brief  Callback to update the Plus4 model widget on v364 widget change
+ *
+ * \param[in]   widget  v364 widget
+ * \param[in]   value   new value
+ */
+static void v364_speech_widget_callback(GtkWidget *widget, int value)
+{
+    debug_gtk3("Called with value %d\n", value);
+    machine_model_widget_update(machine_widget);
+}
+
+
+/* }}} */
+
 
 /*
  * C64(sc) model change handling
  */
-
+static void c64_misc_widget_sync(void);
 
 static void machine_model_handler_c64(int model)
 {
@@ -263,6 +484,11 @@ static void machine_model_handler_c64(int model)
 
     /* synchronize CIA widget */
     cia_model_widget_sync(cia_widget);
+
+    /* synchronize kernal-revision widget */
+    kernal_revision_widget_sync(kernal_widget);
+    /* synchronize misc widget */
+    c64_misc_widget_sync();
 }
 
 
@@ -273,9 +499,38 @@ static void machine_model_handler_c64(int model)
 static void machine_model_handler_c128(int model)
 {
     GtkWidget *sid_group;
+#ifdef HAVE_DEBUG_GTK3UI
+    int res_board_type = -1;
+    int res_vdc_revision = -1;
+    int res_vdc_64kb = -1;
+    int res_machine_type = -1;
+    int res_video_standard = -1;
+    int res_cia1 = -1;
+    int res_cia2 = -1;
+    int res_sid = -1;
 
     debug_gtk3("Got model change for C128: %d.", model);
 
+    resources_get_int("BoardType",      &res_board_type);
+    resources_get_int("VDCRevision",    &res_vdc_revision);
+    resources_get_int("VDC64KB",        &res_vdc_64kb);
+    resources_get_int("MachineType",    &res_machine_type);
+    resources_get_int("MachineVideoStandard",    &res_video_standard);
+    resources_get_int("CIA1Model",      &res_cia1);
+    resources_get_int("CIA2Model",      &res_cia2);
+    resources_get_int("SIDModel",       &res_sid);
+
+    printf("=== %s ===\n", __func__);
+    printf("    BoardType             : %d\n", res_board_type);
+    printf("    VDCRevision           : %d\n", res_vdc_revision);
+    printf("    VDC64KB               : %d\n", res_vdc_64kb);
+    printf("    MachineType           : %d\n", res_machine_type);
+    printf("    MachineVideoStandard: : %d\n", res_video_standard);
+    printf("    CIA1                  : %d\n", res_cia1);
+    printf("    CIA2                  : %d\n", res_cia2);
+    printf("    SIDModel              : %d\n", res_sid);
+#endif
+
     /* sync video chip (VICIIe) widget */
     video_model_widget_update(video_widget);
 
@@ -307,7 +562,9 @@ static void machine_model_handler_c128(int model)
 static void dtv_revision_callback(GtkWidget *widget, int revision)
 {
     debug_gtk3("got revision %d.", revision);
-    machine_model_widget_update(machine_widget);
+    if (get_model_func != NULL) {
+        machine_model_widget_update(machine_widget);
+    }
 }
 
 
@@ -323,6 +580,27 @@ static void dtv_video_callback(int model)
     machine_model_widget_update(machine_widget);
 }
 
+/** \brief  Sync "Hummer ADC" widget with the associated resource
+ *
+ */
+static void c64dtv_hummer_adc_sync(void)
+{
+    int hummeradc = 0;
+    resources_get_int("HummerAdc", &hummeradc);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(c64dtv_hummer_adc_widget), hummeradc);
+}
+
+
+/** \brief  Update DTV widget on 'Hummer ADC' toggle
+ *
+ * \param[in]   widget  check button
+ * \param[in]   value   new value (bool)
+ */
+static void c64dtv_hummer_adc_callback(GtkWidget *widget, int value)
+{
+    machine_model_widget_update(machine_widget);
+}
+
 
 /** \brief  Callback for DTV machine model changes
  *
@@ -360,6 +638,8 @@ static void machine_model_handler_c64dtv(int model)
 
     /* update VIC-II model widget */
     video_model_widget_update(video_widget);
+    /* update Hummer ADC widget */
+    c64dtv_hummer_adc_sync();
 }
 
 
@@ -383,13 +663,15 @@ static void vic20_video_callback(int model)
 
 /** \brief  Callback for VIC-20 machine model changes
  *
- * \param[in]   model   VIC-29 model
+ * \param[in]   model   VIC-20 model
  */
 static void machine_model_handler_vic20(int model)
 {
-    /* update VIC-II model widget */
+    /* update VIC model widget */
     video_model_widget_update(video_widget);
 
+    /* FIXME: */
+    /* vic20_memory_expansion_widget_sync(); */
 }
 
 
@@ -415,26 +697,36 @@ static void plus4_video_callback(int model)
 {
     debug_gtk3("got video model %d.", model);
     machine_model_widget_update(machine_widget);
+    plus4_debug_dump_resources();
 }
 
-
+#if 0
 /** \brief  Callback for the Plus4 memory size/hack
  *
  * Calls model widget update
  *
- * \param[in]   ram     ram size in KB
+ * \param[in]   ram     ram size in KiB
  * \param[in]   hack    memory hack type
  */
 static void plus4_memory_callback(int ram, int hack)
 {
     machine_model_widget_update(machine_widget);
 }
+#endif
 
 
+/** \brief  Handler for the model change for Plus4
+ *
+ * \param[in]   model   new model (unused, it seems)
+ */
 static void machine_model_handler_plus4(int model)
 {
     debug_gtk3("called with model %d.", model);
     video_model_widget_update(video_widget);
+    plus4_memory_size_widget_sync();
+    plus4_acia_widget_sync();
+    v364_speech_widget_sync();
+    plus4_debug_dump_resources();
 }
 
 
@@ -498,6 +790,24 @@ static void machine_model_handler_cbm6x0(int model)
 }
 
 
+/** \brief  Set sensitivity of PET Ram9 and RamA widgets
+ *
+ * Only the 8296 model has the Ram9 and RamA resources
+ */
+static void pet_set_ram9a_sensitivity(void)
+{
+    gboolean model_is_8296;
+
+    if (get_model_func != NULL) {
+        int true_model = get_model_func();
+
+        model_is_8296 = true_model == PETMODEL_8296;
+        gtk_widget_set_sensitive(pet_ram9_widget, model_is_8296);
+        gtk_widget_set_sensitive(pet_rama_widget, model_is_8296);
+    }
+}
+
+
 static void machine_model_handler_pet(int model)
 {
     debug_gtk3("Called.");
@@ -506,6 +816,9 @@ static void machine_model_handler_pet(int model)
     pet_keyboard_type_widget_sync(pet_keyboard_widget);
     pet_misc_widget_sync(pet_misc_widget);
     pet_io_size_widget_sync(pet_io_widget);
+    pet_ram9_widget_sync(pet_ram9_widget);
+    pet_rama_widget_sync(pet_rama_widget);
+    pet_set_ram9a_sensitivity();
 }
 
 
@@ -519,8 +832,9 @@ static void machine_model_callback(int model)
     debug_gtk3("got model %d.", model);
 
     switch (machine_class) {
-        case VICE_MACHINE_C64:  /* fall through */
-        case VICE_MACHINE_C64SC:
+        case VICE_MACHINE_C64:      /* fall through */
+        case VICE_MACHINE_C64SC:    /* fall through */
+        case VICE_MACHINE_SCPU64:
             machine_model_handler_c64(model);
             break;
         case VICE_MACHINE_C64DTV:
@@ -564,21 +878,34 @@ static void on_c64_glue_toggled(GtkWidget *widget, gpointer user_data)
         debug_gtk3("setting GlueLogic to %s.",
                 glue == 0 ? "discrete" : "252535-01");
         resources_set_int("GlueLogic", glue);
+        machine_model_widget_update(machine_widget);
     }
 }
 
 
+/** \brief  Sync "Reset-to-IEC" widget with the associated resource
+ *
+ */
+static void c64_reset_with_iec_sync(void)
+{
+    int iecreset = 0;
+    resources_get_int("IECReset", &iecreset);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(reset_with_iec_widget), iecreset);
+}
 
 /** \brief  Create widget to toggle "Reset-to-IEC"
  *
  * \return  GtkGrid
  */
-static GtkWidget *create_reset_to_iec_widget(void)
+static GtkWidget *create_reset_with_iec_widget(void)
 {
-    return vice_gtk3_resource_check_button_new("IECReset",
+    reset_with_iec_widget = vice_gtk3_resource_check_button_new("IECReset",
             "Reset goes to IEC");
-}
+    g_signal_connect(GTK_WIDGET(reset_with_iec_widget), "toggled",
+            G_CALLBACK(iec_callback), NULL);
 
+    return reset_with_iec_widget;
+}
 
 /** \brief  Create widget to toggle "Go64Mode"
  *
@@ -591,6 +918,23 @@ static GtkWidget *create_go64_widget(void)
 }
 
 
+GtkWidget *c64_discrete_radio = NULL;
+GtkWidget *c64_custom_radio = NULL;
+
+/** \brief  Sync "Glue Logic" widget with the associated resource
+ *
+ */
+static void c64_glue_widget_sync(void)
+{
+    int glue;
+    GtkWidget *radio;
+
+    resources_get_int("GlueLogic", &glue);
+    radio = (glue == 0) ? c64_discrete_radio : c64_custom_radio;
+    if (radio) {
+        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio), TRUE);
+    }
+}
 
 /** \brief  Create widget to select C64SC Glue Logic
  *
@@ -600,8 +944,6 @@ static GtkWidget *create_c64_glue_widget(void)
 {
     GtkWidget *grid;
     GtkWidget *label;
-    GtkWidget *discrete_radio;
-    GtkWidget *custom_radio;
     GtkWidget *radio;
     GSList *group = NULL;
 
@@ -616,21 +958,21 @@ static GtkWidget *create_c64_glue_widget(void)
     g_object_set(label, "margin-left", 16, NULL);
     gtk_grid_attach(GTK_GRID(grid), label, 0, 0, 1, 1);
 
-    discrete_radio = gtk_radio_button_new_with_label(group, "Discrete");
-    custom_radio = gtk_radio_button_new_with_label(group, "Custom IC");
-    gtk_radio_button_join_group(GTK_RADIO_BUTTON(custom_radio),
-            GTK_RADIO_BUTTON(discrete_radio));
+    c64_discrete_radio = gtk_radio_button_new_with_label(group, "Discrete");
+    c64_custom_radio = gtk_radio_button_new_with_label(group, "Custom IC");
+    gtk_radio_button_join_group(GTK_RADIO_BUTTON(c64_custom_radio),
+            GTK_RADIO_BUTTON(c64_discrete_radio));
 
-    radio = glue == 0 ? discrete_radio : custom_radio;
+    radio = glue == 0 ? c64_discrete_radio : c64_custom_radio;
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio), TRUE);
 
-    g_signal_connect(discrete_radio, "toggled",
+    g_signal_connect(c64_discrete_radio, "toggled",
             G_CALLBACK(on_c64_glue_toggled), GINT_TO_POINTER(0));
-    g_signal_connect(custom_radio, "toggled",
+    g_signal_connect(c64_custom_radio, "toggled",
             G_CALLBACK(on_c64_glue_toggled), GINT_TO_POINTER(1));
 
-    gtk_grid_attach(GTK_GRID(grid), discrete_radio, 1, 0, 1, 1);
-    gtk_grid_attach(GTK_GRID(grid), custom_radio, 2, 0, 1, 1);
+    gtk_grid_attach(GTK_GRID(grid), c64_discrete_radio, 1, 0, 1, 1);
+    gtk_grid_attach(GTK_GRID(grid), c64_custom_radio, 2, 0, 1, 1);
 
 
     gtk_widget_show_all(grid);
@@ -648,9 +990,12 @@ static GtkWidget *create_c64_misc_widget(void)
     GtkWidget *iec_widget;
     GtkWidget *glue_widget = NULL;
 
+#if 0
     grid = uihelpers_create_grid_with_label("Miscellaneous", 1);
-
-    iec_widget = create_reset_to_iec_widget();
+#else
+    grid = vice_gtk3_grid_new_spaced_with_label(-1, -1, "Miscellaneous", 1);
+#endif
+    iec_widget = create_reset_with_iec_widget();
     g_object_set(iec_widget, "margin-left", 16, NULL);
     gtk_grid_attach(GTK_GRID(grid), iec_widget, 0, 1, 1, 1);
 
@@ -669,6 +1014,15 @@ static GtkWidget *create_c64_misc_widget(void)
 }
 
 
+/** \brief  Synchronize Glue logic and IEC widget with their resources
+ */
+static void c64_misc_widget_sync(void)
+{
+    c64_glue_widget_sync();
+    c64_reset_with_iec_sync();
+}
+
+
 /** \brief  Create 'misc' widget for C128
  *
  * \return  GtkGrid
@@ -678,7 +1032,7 @@ static GtkWidget *create_c128_misc_widget(void)
     GtkWidget *grid;
     GtkWidget *go64_widget;
 
-    grid = uihelpers_create_grid_with_label("Miscellaneous", 1);
+    grid = vice_gtk3_grid_new_spaced_with_label(-1, -1, "Miscellaneous", 1);
 
     go64_widget = create_go64_widget();
     g_object_set(go64_widget, "margin-left", 16, NULL);
@@ -729,6 +1083,8 @@ static GtkWidget *create_c64dtv_revision_widget(void)
  */
 static GtkWidget *create_c64_layout(GtkWidget *grid)
 {
+    GtkWidget *misc_widget;
+
     /* add machine widget */
     gtk_grid_attach(GTK_GRID(grid), machine_widget, 0, 0, 1, 2);
 
@@ -751,11 +1107,14 @@ static GtkWidget *create_c64_layout(GtkWidget *grid)
     if (machine_class != VICE_MACHINE_SCPU64) {
         kernal_widget = kernal_revision_widget_create();
         gtk_grid_attach(GTK_GRID(grid), kernal_widget, 2, 0, 1, 1);
+        /* add custom callback */
+        kernal_revision_widget_add_callback(kernal_revision_callback);
     }
 
     /* C64 misc. model settings */
-    gtk_grid_attach(GTK_GRID(grid), create_c64_misc_widget(),
-            2, 1, 1, 1);
+    misc_widget = create_c64_misc_widget();
+    g_object_set(misc_widget, "margin", 8, NULL);
+    gtk_grid_attach(GTK_GRID(grid), misc_widget, 2, 1, 1, 1);
 
     return grid;
 }
@@ -769,8 +1128,9 @@ static GtkWidget *create_c64_layout(GtkWidget *grid)
  */
 static GtkWidget *create_c128_layout(GtkWidget *grid)
 {
-    GtkWidget *video_wrapper;
+    GtkWidget *col2_wrapper;
     GtkWidget *machine_wrapper;
+    GtkWidget *misc_widget;
 
     /* wrap machine model and machine type widgets in a single widget */
     machine_wrapper = gtk_grid_new();
@@ -785,12 +1145,12 @@ static GtkWidget *create_c128_layout(GtkWidget *grid)
     gtk_grid_attach(GTK_GRID(grid), machine_wrapper, 0, 0, 1, 1);
 
     /* wrap VIC-II, VDC and CIA1/2 in a single widget */
-    video_wrapper = gtk_grid_new();
+    col2_wrapper = gtk_grid_new();
 
     /* VIC-II model widget */
     video_widget = video_model_widget_create(machine_widget);
     video_model_widget_set_callback(video_widget, video_model_callback);
-    gtk_grid_attach(GTK_GRID(video_wrapper), video_widget, 0, 0, 1, 1);
+    gtk_grid_attach(GTK_GRID(col2_wrapper), video_widget, 0, 0, 1, 1);
 
     /* VDC model widget */
     vdc_widget = vdc_model_widget_create();
@@ -800,21 +1160,23 @@ static GtkWidget *create_c128_layout(GtkWidget *grid)
     vdc_model_widget_set_ram_callback(vdc_widget, vdc_ram_callback);
     /* align with other widgets */
     g_object_set(vdc_widget, "margin-left", 8, NULL);
-    gtk_grid_attach(GTK_GRID(video_wrapper), vdc_widget, 0, 1, 1, 1);
+    gtk_grid_attach(GTK_GRID(col2_wrapper), vdc_widget, 0, 1, 1, 1);
 
     /* CIA1 & CIA2 widget */
     cia_widget = cia_model_widget_create(machine_widget, 2);
     cia_model_widget_set_callback(cia_widget, cia_model_callback);
-    gtk_grid_attach(GTK_GRID(video_wrapper), cia_widget, 0, 2, 1, 1);
-    gtk_grid_attach(GTK_GRID(grid), video_wrapper, 1, 0, 1, 1);
+    gtk_grid_attach(GTK_GRID(col2_wrapper), cia_widget, 0, 2, 1, 1);
+    gtk_grid_attach(GTK_GRID(grid), col2_wrapper, 1, 0, 1, 1);
 
     /* SID widget */
     sid_widget = sid_model_widget_create(machine_widget);
     sid_model_widget_set_callback(sid_widget, sid_model_callback);
-    gtk_grid_attach(GTK_GRID(grid), sid_widget, 2, 0, 1, 1);
+    gtk_grid_attach(GTK_GRID(col2_wrapper), sid_widget, 0, 3, 1, 1);
 
     /* Misc widget */
-    gtk_grid_attach(GTK_GRID(grid), create_c128_misc_widget(), 0, 1, 3, 1);
+    misc_widget = create_c128_misc_widget();
+    g_object_set(misc_widget, "margin", 8, NULL);
+    gtk_grid_attach(GTK_GRID(grid), misc_widget, 0, 1, 3, 1);
     return grid;
 }
 
@@ -853,6 +1215,20 @@ static GtkWidget *create_c64dtv_layout(GtkWidget *grid)
             "Enable LumaFix (use new VICII luminances)");
     g_object_set(G_OBJECT(luma_widget), "margin-left", 8, "margin-top", 16, NULL);
     gtk_grid_attach(GTK_GRID(grid), luma_widget, 0, 3, 2, 1);
+
+    /* Hummer ADC widget */
+    c64dtv_hummer_adc_widget = vice_gtk3_resource_check_button_new(
+            "HummerADC",
+            "Enable Hummer ADC");
+    vice_gtk3_resource_check_button_add_callback(c64dtv_hummer_adc_widget,
+                                                 c64dtv_hummer_adc_callback);
+    g_object_set(
+            c64dtv_hummer_adc_widget,
+            "margin-left", 8,
+            "margin-top", 8,
+            NULL);
+    gtk_grid_attach(GTK_GRID(grid), c64dtv_hummer_adc_widget, 0, 4, 2, 1);
+
     return grid;
 }
 
@@ -889,18 +1265,38 @@ static GtkWidget *create_vic20_layout(GtkWidget *grid)
  */
 static GtkWidget *create_plus4_layout(GtkWidget *grid)
 {
+    int hack;
+
     /* add machine widget */
     gtk_grid_attach(GTK_GRID(grid), machine_widget, 0, 0, 1, 1);
 
-    /* PET model widget */
+    /* Plus4 model widget */
     video_widget = video_model_widget_create(machine_widget);
     video_model_widget_set_callback(video_widget, plus4_video_callback);
     gtk_grid_attach(GTK_GRID(grid), video_widget, 1, 0, 1, 1);
 
-    /* RAM size/expansion hacks */
-    ram_widget = plus4_memory_expansion_widget_create();
-    plus4_memory_expansion_widget_set_callback(plus4_memory_callback);
-    gtk_grid_attach(GTK_GRID(grid), ram_widget, 2, 0, 1, 1);
+    /* memory expansion hacks */
+    memhack_widget = plus4_memory_expansion_widget_create();
+    plus4_memory_expansion_widget_add_callback(plus4_mem_hack_callback);
+    gtk_grid_attach(GTK_GRID(grid), memhack_widget, 0, 1, 1, 1);
+
+    /* memory size */
+    ram_widget = plus4_memory_size_widget_create();
+    plus4_memory_size_widget_add_callback(plus4_mem_size_callback);
+    gtk_grid_attach(GTK_GRID(grid), ram_widget, 1, 1, 1, 1);
+
+    resources_get_int("MemoryHack", &hack);
+    gtk_widget_set_sensitive(memhack_widget, hack == MEMORY_HACK_NONE);
+
+    /* ACIA widget */
+    acia_widget = plus4_acia_widget_create();
+    plus4_acia_widget_add_callback(plus4_acia_widget_callback);
+    gtk_grid_attach(GTK_GRID(grid), acia_widget, 0, 2, 1, 1);
+
+    /* V364 speech widget */
+    speech_widget = v364_speech_widget_create();
+    v364_speech_widget_add_callback(v364_speech_widget_callback);
+    gtk_grid_attach(GTK_GRID(grid), speech_widget, 1, 2, 1, 1);
 
     gtk_widget_show_all(grid);
     return grid;
@@ -944,6 +1340,15 @@ static GtkWidget *create_pet_layout(GtkWidget *grid)
     pet_io_size_widget_set_callback(pet_io_widget, pet_io_callback);
     gtk_grid_attach(GTK_GRID(pet_grid), pet_io_widget, 2, 1, 1, 1);
 
+    pet_ram9_widget = pet_ram9_widget_create();
+    pet_ram9_widget_set_callback(pet_ram9_widget, pet_ram9_callback);
+    gtk_grid_attach(GTK_GRID(pet_grid), pet_ram9_widget, 3, 0, 1, 1);
+
+    pet_rama_widget = pet_rama_widget_create();
+    pet_rama_widget_set_callback(pet_rama_widget, pet_rama_callback);
+    pet_set_ram9a_sensitivity();
+    gtk_grid_attach(GTK_GRID(pet_grid), pet_rama_widget, 3, 1, 1, 1);
+
     pet_misc_widget = pet_misc_widget_create();
     pet_misc_widget_set_crtc_callback(pet_misc_widget, pet_crtc_callback);
     pet_misc_widget_set_blank_callback(pet_misc_widget, pet_blank_callback);
@@ -1248,3 +1653,13 @@ void settings_model_widget_set_model_func(int (*func)(void))
 {
     get_model_func = func;
 }
+
+
+/** \brief  Set function to get a memory hack description
+ *
+ * \param[in]   func    function to get a string for a memhack ID (int)
+ */
+void settings_model_widget_set_memhack_func(const char *(func)(int))
+{
+    get_memhack_func = func;
+}
