@@ -55,11 +55,13 @@
  * $VICERES DosName1581         x64 x64sc xscpu64 xvic x128 xplus4
  * $VICERES DosName2000         x64 x64sc xscpu64 xvic x128 xplus4
  * $VICERES DosName4000         x64 x64sc xscpu64 xvic x128 xplus4
+ * $VICERES DosNameCMDHD        x64 x64sc xscpu64 xvic x128 xplus4
  * $VICERES DosName2031         x64 x64sc xscpu64 xvic x128 xcbm5x0 xcbm2 xpet
  * $VICERES DosName2040         x64 x64sc xscpu64 xvic x128 xcbm5x0 xcbm2 xpet
  * $VICERES DosName3040         x64 x64sc xscpu64 xvic x128 xcbm5x0 xcbm2 xpet
  * $VICERES DosName4040         x64 x64sc xscpu64 xvic x128 xcbm5x0 xcbm2 xpet
  * $VICERES DosName1001         x64 x64sc xscpu64 xvic x128 xcbm5x0 xcbm2 xpet
+ * $VICERES DosName9000         x64 x64sc xscpu64 xvic x128 xcbm5x0 xcbm2 xpet
  * $VICERES DriveProfDOS1571Name    x64 x64sc xscpu64 x128
  * $VICERES DriveSuperCardName      x64 x64sc xscpu64 x128
  * $VICERES DriveStarDosName        x64 x64sc xscpu64 x128
@@ -274,11 +276,13 @@ static const romset_entry_t c64_vic20_drive_roms[] = {
     { "DosName1581",    "1581",     NULL },
     { "DosName2000",    "2000",     NULL },
     { "DosName4000",    "4000",     NULL },
+    { "DosNameCMDHD",   "CMD HD",   NULL },
     { "DosName2031",    "2031",     NULL },
     { "DosName2040",    "2040",     NULL },
     { "DosName3040",    "3040",     NULL },
     { "DosName4040",    "4040",     NULL },
     { "DosName1001",    "1001",     NULL },
+    { "DosName9000",    "D9090/60", NULL },
     { NULL,         NULL,           NULL }
 };
 
@@ -295,11 +299,13 @@ static const romset_entry_t c128_drive_roms[] = {
     { "DosName1581",    "1581",     NULL },
     { "DosName2000",    "2000",     NULL },
     { "DosName4000",    "4000",     NULL },
+    { "DosNameCMDHD",   "CMD HD",   NULL },
     { "DosName2031",    "2031",     NULL },
     { "DosName2040",    "2040",     NULL },
     { "DosName3040",    "3040",     NULL },
     { "DosName4040",    "4040",     NULL },
     { "DosName1001",    "1001",     NULL },
+    { "DosName9000",    "D9090/60", NULL },
     { NULL,         NULL,           NULL }
 };
 
@@ -320,6 +326,7 @@ static const romset_entry_t pet_cbm2_drive_roms[] = {
     { "DosName3040",    "3040",     NULL },
     { "DosName4040",    "4040",     NULL },
     { "DosName1001",    "1001",     NULL },
+    { "DosName9000",    "D9090/60", NULL },
     { NULL,         NULL,           NULL }
 };
 
@@ -336,6 +343,7 @@ static const romset_entry_t plus4_drive_roms[] = {
     { "DosName1581",    "1581",     NULL },
     { "DosName2000",    "2000",     NULL },
     { "DosName4000",    "4000",     NULL },
+    { "DosNameCMDHD",   "CMD HD",   NULL },
     { NULL,         NULL,           NULL }
 };
 
