@@ -300,6 +300,13 @@ static void on_drag_data_received(
 
 }
 
+/** \brief Called each frame for UI updates
+ */
+void vsid_main_widget_update(void)
+{
+    vsid_tune_info_widget_update();
+}
+
 
 /** \brief  Create VSID main widget
  *
