@@ -415,6 +415,11 @@ int joy_arch_init(void)
     return 0;
 }
 
+void joystick(void)
+{
+    /* Provided only for archdep joy.h. TODO: Migrate joystick polling in here if any needed? */
+}
+
 void joystick_close(void)
 {
     int i;
@@ -564,7 +569,7 @@ int joy_arch_mapping_dump(const char *filename)
             "#\n"
             "# action [action_parameters]:\n"
             "# 0               none\n"
-            "# 1 port pin      joystick (pin: 1/2/4/8/16 = u/d/l/r/fire)\n"
+            "# 1 port pin      joystick (pin: 1/2/4/8/16/32/64 = u/d/l/r/fire/fire2/fire3)\n"
             "# 2 row col       keyboard\n"
             "# 3               map\n"
             "# 4               UI activate\n"
@@ -751,16 +756,22 @@ int joy_arch_mapping_load(const char *filename)
                 *p = 0;
             }
 
-            switch (*buffer) {
+            /* remove whitespace at the beginning of the line */
+            p = buffer;
+            while (((*p == ' ') || (*p == '\t')) && (*p != 0)) {
+                ++p;
+            }
+
+            switch (*p) {
                 case 0:
                     break;
                 case '!':
                     /* keyword handling */
-                    joy_arch_parse_keyword(buffer);
+                    joy_arch_parse_keyword(p);
                     break;
                 default:
                     /* table entry handling */
-                    joy_arch_parse_entry(buffer);
+                    joy_arch_parse_entry(p);
                     break;
             }
         }
@@ -1213,6 +1224,11 @@ void sdljoy_swap_ports(void)
     resources_set_int("JoyDevice2", i);
 }
 
+void joystick(void)
+{
+    /* Provided only for archdep joy.h. TODO: Migrate joystick polling in here if any needed? */
+}
+
 void joystick_close(void)
 {
 }
