@@ -38,6 +38,7 @@
 #include "uimenu.h"
 
 extern void joystick_close(void);
+extern void joystick(void);
 
 extern void joy_arch_resources_shutdown(void);
 
@@ -71,6 +72,8 @@ extern int sdljoy_get_swap_userport_ports(void) ;
 
 #ifdef ANDROID_COMPILE
 #define JOYDEV_DEFAULT JOYDEV_JOYSTICK
+#else
+#define JOYDEV_DEFAULT JOYDEV_NUMPAD
 #endif
 
 #ifdef HAVE_SDL_NUMJOYSTICKS
