@@ -118,19 +118,19 @@ static const ui_menu_entry_t cbm5x0_model_menu[] = {
 
 static const ui_menu_entry_t cbm2_memory_menu[] = {
     SDL_MENU_ITEM_TITLE("CBM2 memory size"),
-    { "128kB",
+    { "128KiB",
       MENU_ENTRY_RESOURCE_RADIO,
       radio_RamSize_callback,
       (ui_callback_data_t)128 },
-    { "256kB",
+    { "256KiB",
       MENU_ENTRY_RESOURCE_RADIO,
       radio_RamSize_callback,
       (ui_callback_data_t)256 },
-    { "512kB",
+    { "512KiB",
       MENU_ENTRY_RESOURCE_RADIO,
       radio_RamSize_callback,
       (ui_callback_data_t)512 },
-    { "1024kB",
+    { "1MiB",
       MENU_ENTRY_RESOURCE_RADIO,
       radio_RamSize_callback,
       (ui_callback_data_t)1024 },
@@ -165,23 +165,23 @@ static const ui_menu_entry_t cbm2_memory_menu[] = {
 
 static const ui_menu_entry_t cbm5x0_memory_menu[] = {
     SDL_MENU_ITEM_TITLE("CBM2 memory size"),
-    { "64kB",
+    { "64KiB",
       MENU_ENTRY_RESOURCE_RADIO,
       radio_RamSize_callback,
       (ui_callback_data_t)64 },
-    { "128kB",
+    { "128KiB",
       MENU_ENTRY_RESOURCE_RADIO,
       radio_RamSize_callback,
       (ui_callback_data_t)128 },
-    { "256kB",
+    { "256KiB",
       MENU_ENTRY_RESOURCE_RADIO,
       radio_RamSize_callback,
       (ui_callback_data_t)256 },
-    { "512kB",
+    { "512KiB",
       MENU_ENTRY_RESOURCE_RADIO,
       radio_RamSize_callback,
       (ui_callback_data_t)512 },
-    { "1024kB",
+    { "1MiB",
       MENU_ENTRY_RESOURCE_RADIO,
       radio_RamSize_callback,
       (ui_callback_data_t)1024 },
