@@ -34,14 +34,17 @@
 
 #include "vice.h"
 #include "types.h"
+#include "uifonts.h"
 #include "uimenu.h"
 
-/* #define MENU_SUBMENU_STRING "->" */
-#define MENU_SUBMENU_STRING "\xa7>"
+#define MENU_MENUHEADERLEFT_STRING UIFONT_MENUHEADERLEFT_STRING
+#define MENU_MENUHEADERRIGHT_STRING UIFONT_MENUHEADERRIGHT_STRING
 
-/* #define MENU_CHECKMARK_CHECKED_STRING   "*" */
-#define MENU_CHECKMARK_CHECKED_STRING   "\xba"
-#define MENU_CHECKMARK_UNCHECKED_STRING "."
+#define MENU_SUBMENU_STRING UIFONT_SUBMENU_STRING
+
+#define MENU_CHECKMARK_UNCHECKED_STRING UIFONT_CHECKMARK_UNCHECKED_STRING
+#define MENU_CHECKMARK_UNCHECKED_CHAR UIFONT_CHECKMARK_UNCHECKED_CHAR
+#define MENU_CHECKMARK_CHECKED_STRING UIFONT_CHECKMARK_CHECKED_STRING
 
 #define MENU_NOT_AVAILABLE_STRING "(N/A)"
 
