@@ -83,23 +83,23 @@ const ui_menu_entry_t scpu64_burstmod_menu[] = {
 UI_MENU_DEFINE_RADIO(SIMMSize)
 
 const ui_menu_entry_t scpu64_simmsize_menu[] = {
-    { "0 MB",
+    { "0 MiB",
       MENU_ENTRY_RESOURCE_RADIO,
       radio_SIMMSize_callback,
       (ui_callback_data_t)0 },
-    { "1 MB",
+    { "1 MiB",
       MENU_ENTRY_RESOURCE_RADIO,
       radio_SIMMSize_callback,
       (ui_callback_data_t)1 },
-    { "4 MB",
+    { "4 MiB",
       MENU_ENTRY_RESOURCE_RADIO,
       radio_SIMMSize_callback,
       (ui_callback_data_t)4 },
-    { "8 MB",
+    { "8 MiB",
       MENU_ENTRY_RESOURCE_RADIO,
       radio_SIMMSize_callback,
       (ui_callback_data_t)8 },
-    { "16 MB",
+    { "16 MiB",
       MENU_ENTRY_RESOURCE_RADIO,
       radio_SIMMSize_callback,
       (ui_callback_data_t)16 },
