@@ -151,20 +151,13 @@ UI_MENU_DEFINE_SLIDER(SidResid8580FilterBias, -5000, 5000)
 
 UI_MENU_DEFINE_TOGGLE(SidFilters)
 UI_MENU_DEFINE_RADIO(SidStereo)
-UI_MENU_DEFINE_RADIO(SidStereoAddressStart)
-UI_MENU_DEFINE_RADIO(SidTripleAddressStart)
-UI_MENU_DEFINE_RADIO(SidQuadAddressStart)
-
-static UI_MENU_CALLBACK(show_SidStereoAddressStart_callback)
-{
-    static char buf[20];
-    int value;
-
-    resources_get_int("SidStereoAddressStart", &value);
-
-    sprintf(buf, "$%04x", (unsigned int)value);
-    return buf;
-}
+UI_MENU_DEFINE_RADIO(Sid2AddressStart)
+UI_MENU_DEFINE_RADIO(Sid3AddressStart)
+UI_MENU_DEFINE_RADIO(Sid4AddressStart)
+UI_MENU_DEFINE_RADIO(Sid5AddressStart)
+UI_MENU_DEFINE_RADIO(Sid6AddressStart)
+UI_MENU_DEFINE_RADIO(Sid7AddressStart)
+UI_MENU_DEFINE_RADIO(Sid8AddressStart)
 
 #define SID_D4XX_MENU(menu, txt, showcb, cb) \
 static const ui_menu_entry_t menu[] = {      \
@@ -296,216 +289,88 @@ static const ui_menu_entry_t menu[] = {      \
     SDL_MENU_LIST_END                        \
 };
 
-SID_D4XX_MENU(sid_d4x0_menu, "Second SID base address", show_SidStereoAddressStart_callback, radio_SidStereoAddressStart_callback)
-SID_D5XX_MENU(sid_d5x0_menu, "Second SID base address", show_SidStereoAddressStart_callback, radio_SidStereoAddressStart_callback)
-SID_D6XX_MENU(sid_d6x0_menu, "Second SID base address", show_SidStereoAddressStart_callback, radio_SidStereoAddressStart_callback)
-SID_D7XX_MENU(sid_d7x0_menu, "Second SID base address", show_SidStereoAddressStart_callback, radio_SidStereoAddressStart_callback)
-SID_DEXX_MENU(sid_dex0_menu, "Second SID base address", show_SidStereoAddressStart_callback, radio_SidStereoAddressStart_callback)
-SID_DFXX_MENU(sid_dfx0_menu, "Second SID base address", show_SidStereoAddressStart_callback, radio_SidStereoAddressStart_callback)
+#define SID_EXTRA_MENU(sid_nr, sid_text)                                                                                                                    \
+    static UI_MENU_CALLBACK(show_Sid##sid_nr##AddressStart_callback)                                                                                        \
+    {                                                                                                                                                       \
+        static char buf[20];                                                                                                                                \
+        int value;                                                                                                                                          \
+                                                                                                                                                            \
+        resources_get_int_sprintf("Sid%dAddressStart", &value, sid_nr);                                                                                     \
+                                                                                                                                                            \
+        sprintf(buf, "$%04x", (unsigned int)value);                                                                                                         \
+        return buf;                                                                                                                                         \
+    }                                                                                                                                                       \
+                                                                                                                                                            \
+    SID_D4XX_MENU(sid##sid_nr##_d4x0_menu, sid_text " SID base address", show_Sid##sid_nr##AddressStart_callback, radio_Sid##sid_nr##AddressStart_callback) \
+    SID_D5XX_MENU(sid##sid_nr##_d5x0_menu, sid_text " SID base address", show_Sid##sid_nr##AddressStart_callback, radio_Sid##sid_nr##AddressStart_callback) \
+    SID_D6XX_MENU(sid##sid_nr##_d6x0_menu, sid_text " SID base address", show_Sid##sid_nr##AddressStart_callback, radio_Sid##sid_nr##AddressStart_callback) \
+    SID_D7XX_MENU(sid##sid_nr##_d7x0_menu, sid_text " SID base address", show_Sid##sid_nr##AddressStart_callback, radio_Sid##sid_nr##AddressStart_callback) \
+    SID_DEXX_MENU(sid##sid_nr##_dex0_menu, sid_text " SID base address", show_Sid##sid_nr##AddressStart_callback, radio_Sid##sid_nr##AddressStart_callback) \
+    SID_DFXX_MENU(sid##sid_nr##_dfx0_menu, sid_text " SID base address", show_Sid##sid_nr##AddressStart_callback, radio_Sid##sid_nr##AddressStart_callback) \
+                                                                                                                                                            \
+    static const ui_menu_entry_t c128_sid##sid_nr##_base_menu[] = {                                                                                         \
+        { sid_text " SID base address",                                                                                                                     \
+          MENU_ENTRY_TEXT,                                                                                                                                  \
+          show_Sid##sid_nr##AddressStart_callback,                                                                                                          \
+          NULL},                                                                                                                                            \
+        { "$D4x0",                                                                                                                                          \
+          MENU_ENTRY_SUBMENU,                                                                                                                               \
+          submenu_callback,                                                                                                                                 \
+          (ui_callback_data_t)sid##sid_nr##_d4x0_menu },                                                                                                    \
+        { "$D7x0",                                                                                                                                          \
+          MENU_ENTRY_SUBMENU,                                                                                                                               \
+          submenu_callback,                                                                                                                                 \
+          (ui_callback_data_t)sid##sid_nr##_d7x0_menu },                                                                                                    \
+        { "$DEx0",                                                                                                                                          \
+          MENU_ENTRY_SUBMENU,                                                                                                                               \
+          submenu_callback,                                                                                                                                 \
+          (ui_callback_data_t)sid##sid_nr##_dex0_menu },                                                                                                    \
+        { "$DFx0",                                                                                                                                          \
+          MENU_ENTRY_SUBMENU,                                                                                                                               \
+          submenu_callback,                                                                                                                                 \
+          (ui_callback_data_t)sid##sid_nr##_dfx0_menu },                                                                                                    \
+        SDL_MENU_LIST_END                                                                                                                                   \
+    };                                                                                                                                                      \
+                                                                                                                                                            \
+    static const ui_menu_entry_t c64_sid##sid_nr##_base_menu[] = {                                                                                          \
+        { sid_text " SID base address",                                                                                                                     \
+          MENU_ENTRY_TEXT,                                                                                                                                  \
+          show_Sid##sid_nr##AddressStart_callback,                                                                                                          \
+          NULL },                                                                                                                                           \
+        { "$D4x0",                                                                                                                                          \
+          MENU_ENTRY_SUBMENU,                                                                                                                               \
+          submenu_callback,                                                                                                                                 \
+          (ui_callback_data_t)sid##sid_nr##_d4x0_menu },                                                                                                    \
+        { "$D5x0",                                                                                                                                          \
+          MENU_ENTRY_SUBMENU,                                                                                                                               \
+          submenu_callback,                                                                                                                                 \
+          (ui_callback_data_t)sid##sid_nr##_d5x0_menu },                                                                                                    \
+        { "$D6x0",                                                                                                                                          \
+          MENU_ENTRY_SUBMENU,                                                                                                                               \
+          submenu_callback,                                                                                                                                 \
+          (ui_callback_data_t)sid##sid_nr##_d6x0_menu },                                                                                                    \
+        { "$D7x0",                                                                                                                                          \
+          MENU_ENTRY_SUBMENU,                                                                                                                               \
+          submenu_callback,                                                                                                                                 \
+          (ui_callback_data_t)sid##sid_nr##_d7x0_menu },                                                                                                    \
+        { "$DEx0",                                                                                                                                          \
+          MENU_ENTRY_SUBMENU,                                                                                                                               \
+          submenu_callback,                                                                                                                                 \
+          (ui_callback_data_t)sid##sid_nr##_dex0_menu },                                                                                                    \
+        { "$DFx0",                                                                                                                                          \
+          MENU_ENTRY_SUBMENU,                                                                                                                               \
+          submenu_callback,                                                                                                                                 \
+          (ui_callback_data_t)sid##sid_nr##_dfx0_menu },                                                                                                    \
+        SDL_MENU_LIST_END                                                                                                                                   \
+    };
 
-static const ui_menu_entry_t c128_stereo_sid_base_menu[] = {
-    { "Second SID base address",
-      MENU_ENTRY_TEXT,
-      show_SidStereoAddressStart_callback,
-      NULL},
-    { "$D4x0",
-      MENU_ENTRY_SUBMENU,
-      submenu_callback,
-      (ui_callback_data_t)sid_d4x0_menu },
-    { "$D7x0",
-      MENU_ENTRY_SUBMENU,
-      submenu_callback,
-      (ui_callback_data_t)sid_d7x0_menu },
-    { "$DEx0",
-      MENU_ENTRY_SUBMENU,
-      submenu_callback,
-      (ui_callback_data_t)sid_dex0_menu },
-    { "$DFx0",
-      MENU_ENTRY_SUBMENU,
-      submenu_callback,
-      (ui_callback_data_t)sid_dfx0_menu },
-    SDL_MENU_LIST_END
-};
-
-static const ui_menu_entry_t c64_stereo_sid_base_menu[] = {
-    { "Second SID base address",
-      MENU_ENTRY_TEXT,
-      show_SidStereoAddressStart_callback,
-      NULL },
-    { "$D4x0",
-      MENU_ENTRY_SUBMENU,
-      submenu_callback,
-      (ui_callback_data_t)sid_d4x0_menu },
-    { "$D5x0",
-      MENU_ENTRY_SUBMENU,
-      submenu_callback,
-      (ui_callback_data_t)sid_d5x0_menu },
-    { "$D6x0",
-      MENU_ENTRY_SUBMENU,
-      submenu_callback,
-      (ui_callback_data_t)sid_d6x0_menu },
-    { "$D7x0",
-      MENU_ENTRY_SUBMENU,
-      submenu_callback,
-      (ui_callback_data_t)sid_d7x0_menu },
-    { "$DEx0",
-      MENU_ENTRY_SUBMENU,
-      submenu_callback,
-      (ui_callback_data_t)sid_dex0_menu },
-    { "$DFx0",
-      MENU_ENTRY_SUBMENU,
-      submenu_callback,
-      (ui_callback_data_t)sid_dfx0_menu },
-    SDL_MENU_LIST_END
-};
-
-static UI_MENU_CALLBACK(show_SidTripleAddressStart_callback)
-{
-    static char buf[20];
-    int value;
-
-    resources_get_int("SidTripleAddressStart", &value);
-
-    sprintf(buf, "$%04x", (unsigned int)value);
-    return buf;
-}
-
-SID_D4XX_MENU(sid3_d4x0_menu, "Third SID base address", show_SidTripleAddressStart_callback, radio_SidTripleAddressStart_callback)
-SID_D5XX_MENU(sid3_d5x0_menu, "Third SID base address", show_SidTripleAddressStart_callback, radio_SidTripleAddressStart_callback)
-SID_D6XX_MENU(sid3_d6x0_menu, "Third SID base address", show_SidTripleAddressStart_callback, radio_SidTripleAddressStart_callback)
-SID_D7XX_MENU(sid3_d7x0_menu, "Third SID base address", show_SidTripleAddressStart_callback, radio_SidTripleAddressStart_callback)
-SID_DEXX_MENU(sid3_dex0_menu, "Third SID base address", show_SidTripleAddressStart_callback, radio_SidTripleAddressStart_callback)
-SID_DFXX_MENU(sid3_dfx0_menu, "Third SID base address", show_SidTripleAddressStart_callback, radio_SidTripleAddressStart_callback)
-
-static const ui_menu_entry_t c128_triple_sid_base_menu[] = {
-    { "Third SID base address",
-      MENU_ENTRY_TEXT,
-      show_SidTripleAddressStart_callback,
-      NULL},
-    { "$D4x0",
-      MENU_ENTRY_SUBMENU,
-      submenu_callback,
-      (ui_callback_data_t)sid3_d4x0_menu },
-    { "$D7x0",
-      MENU_ENTRY_SUBMENU,
-      submenu_callback,
-      (ui_callback_data_t)sid3_d7x0_menu },
-    { "$DEx0",
-      MENU_ENTRY_SUBMENU,
-      submenu_callback,
-      (ui_callback_data_t)sid3_dex0_menu },
-    { "$DFx0",
-      MENU_ENTRY_SUBMENU,
-      submenu_callback,
-      (ui_callback_data_t)sid3_dfx0_menu },
-    SDL_MENU_LIST_END
-};
-
-static const ui_menu_entry_t c64_triple_sid_base_menu[] = {
-    { "Third SID base address",
-      MENU_ENTRY_TEXT,
-      show_SidTripleAddressStart_callback,
-      NULL },
-    { "$D4x0",
-      MENU_ENTRY_SUBMENU,
-      submenu_callback,
-      (ui_callback_data_t)sid3_d4x0_menu },
-    { "$D5x0",
-      MENU_ENTRY_SUBMENU,
-      submenu_callback,
-      (ui_callback_data_t)sid3_d5x0_menu },
-    { "$D6x0",
-      MENU_ENTRY_SUBMENU,
-      submenu_callback,
-      (ui_callback_data_t)sid3_d6x0_menu },
-    { "$D7x0",
-      MENU_ENTRY_SUBMENU,
-      submenu_callback,
-      (ui_callback_data_t)sid3_d7x0_menu },
-    { "$DEx0",
-      MENU_ENTRY_SUBMENU,
-      submenu_callback,
-      (ui_callback_data_t)sid3_dex0_menu },
-    { "$DFx0",
-      MENU_ENTRY_SUBMENU,
-      submenu_callback,
-      (ui_callback_data_t)sid3_dfx0_menu },
-    SDL_MENU_LIST_END
-};
-
-static UI_MENU_CALLBACK(show_SidQuadAddressStart_callback)
-{
-    static char buf[20];
-    int value;
-
-    resources_get_int("SidQuadAddressStart", &value);
-
-    sprintf(buf, "$%04x", (unsigned int)value);
-    return buf;
-}
-
-SID_D4XX_MENU(sid4_d4x0_menu, "Fourth SID base address", show_SidQuadAddressStart_callback, radio_SidQuadAddressStart_callback)
-SID_D5XX_MENU(sid4_d5x0_menu, "Fourth SID base address", show_SidQuadAddressStart_callback, radio_SidQuadAddressStart_callback)
-SID_D6XX_MENU(sid4_d6x0_menu, "Fourth SID base address", show_SidQuadAddressStart_callback, radio_SidQuadAddressStart_callback)
-SID_D7XX_MENU(sid4_d7x0_menu, "Fourth SID base address", show_SidQuadAddressStart_callback, radio_SidQuadAddressStart_callback)
-SID_DEXX_MENU(sid4_dex0_menu, "Fourth SID base address", show_SidQuadAddressStart_callback, radio_SidQuadAddressStart_callback)
-SID_DFXX_MENU(sid4_dfx0_menu, "Fourth SID base address", show_SidQuadAddressStart_callback, radio_SidQuadAddressStart_callback)
-
-static const ui_menu_entry_t c128_quad_sid_base_menu[] = {
-    { "Fourth SID base address",
-      MENU_ENTRY_TEXT,
-      show_SidQuadAddressStart_callback,
-      NULL},
-    { "$D4x0",
-      MENU_ENTRY_SUBMENU,
-      submenu_callback,
-      (ui_callback_data_t)sid4_d4x0_menu },
-    { "$D7x0",
-      MENU_ENTRY_SUBMENU,
-      submenu_callback,
-      (ui_callback_data_t)sid4_d7x0_menu },
-    { "$DEx0",
-      MENU_ENTRY_SUBMENU,
-      submenu_callback,
-      (ui_callback_data_t)sid4_dex0_menu },
-    { "$DFx0",
-      MENU_ENTRY_SUBMENU,
-      submenu_callback,
-      (ui_callback_data_t)sid4_dfx0_menu },
-    SDL_MENU_LIST_END
-};
-
-static const ui_menu_entry_t c64_quad_sid_base_menu[] = {
-    { "Fourth SID base address",
-      MENU_ENTRY_TEXT,
-      show_SidQuadAddressStart_callback,
-      NULL },
-    { "$D4x0",
-      MENU_ENTRY_SUBMENU,
-      submenu_callback,
-      (ui_callback_data_t)sid4_d4x0_menu },
-    { "$D5x0",
-      MENU_ENTRY_SUBMENU,
-      submenu_callback,
-      (ui_callback_data_t)sid4_d5x0_menu },
-    { "$D6x0",
-      MENU_ENTRY_SUBMENU,
-      submenu_callback,
-      (ui_callback_data_t)sid4_d6x0_menu },
-    { "$D7x0",
-      MENU_ENTRY_SUBMENU,
-      submenu_callback,
-      (ui_callback_data_t)sid4_d7x0_menu },
-    { "$DEx0",
-      MENU_ENTRY_SUBMENU,
-      submenu_callback,
-      (ui_callback_data_t)sid4_dex0_menu },
-    { "$DFx0",
-      MENU_ENTRY_SUBMENU,
-      submenu_callback,
-      (ui_callback_data_t)sid4_dfx0_menu },
-    SDL_MENU_LIST_END
-};
+SID_EXTRA_MENU(2, "Second")
+SID_EXTRA_MENU(3, "Third")
+SID_EXTRA_MENU(4, "Fourth")
+SID_EXTRA_MENU(5, "Fifth")
+SID_EXTRA_MENU(6, "Sixth")
+SID_EXTRA_MENU(7, "Seventh")
+SID_EXTRA_MENU(8, "Eight")
 
 static UI_MENU_CALLBACK(show_SidStereo_callback)
 {
@@ -519,6 +384,14 @@ static UI_MENU_CALLBACK(show_SidStereo_callback)
             return "Two";
         case 3:
             return "Three";
+        case 4:
+            return "Four";
+        case 5:
+            return "Five";
+        case 6:
+            return "Six";
+        case 7:
+            return "Seven";
     }
     return "None";
 }
@@ -540,6 +413,22 @@ static const ui_menu_entry_t c64_stereo_sid_menu[] = {
       MENU_ENTRY_RESOURCE_RADIO,
       radio_SidStereo_callback,
       (ui_callback_data_t)3 },
+    { "Four",
+      MENU_ENTRY_RESOURCE_RADIO,
+      radio_SidStereo_callback,
+      (ui_callback_data_t)4 },
+    { "Five",
+      MENU_ENTRY_RESOURCE_RADIO,
+      radio_SidStereo_callback,
+      (ui_callback_data_t)5 },
+    { "Six",
+      MENU_ENTRY_RESOURCE_RADIO,
+      radio_SidStereo_callback,
+      (ui_callback_data_t)6 },
+    { "Seven",
+      MENU_ENTRY_RESOURCE_RADIO,
+      radio_SidStereo_callback,
+      (ui_callback_data_t)7 },
     SDL_MENU_LIST_END
 };
 
@@ -554,16 +443,32 @@ ui_menu_entry_t sid_c64_menu[] = {
       (ui_callback_data_t)c64_stereo_sid_menu },
     { "Second SID base address",
       MENU_ENTRY_SUBMENU,
-      show_SidStereoAddressStart_callback,
-      (ui_callback_data_t)c64_stereo_sid_base_menu },
+      show_Sid2AddressStart_callback,
+      (ui_callback_data_t)c64_sid2_base_menu },
     { "Third SID base address",
       MENU_ENTRY_SUBMENU,
-      show_SidTripleAddressStart_callback,
-      (ui_callback_data_t)c64_triple_sid_base_menu },
+      show_Sid3AddressStart_callback,
+      (ui_callback_data_t)c64_sid3_base_menu },
     { "Fourth SID base address",
       MENU_ENTRY_SUBMENU,
-      show_SidQuadAddressStart_callback,
-      (ui_callback_data_t)c64_quad_sid_base_menu },
+      show_Sid4AddressStart_callback,
+      (ui_callback_data_t)c64_sid4_base_menu },
+    { "Fifth SID base address",
+      MENU_ENTRY_SUBMENU,
+      show_Sid5AddressStart_callback,
+      (ui_callback_data_t)c64_sid5_base_menu },
+    { "Sixth SID base address",
+      MENU_ENTRY_SUBMENU,
+      show_Sid6AddressStart_callback,
+      (ui_callback_data_t)c64_sid6_base_menu },
+    { "Seventh SID base address",
+      MENU_ENTRY_SUBMENU,
+      show_Sid7AddressStart_callback,
+      (ui_callback_data_t)c64_sid7_base_menu },
+    { "Eight SID base address",
+      MENU_ENTRY_SUBMENU,
+      show_Sid8AddressStart_callback,
+      (ui_callback_data_t)c64_sid8_base_menu },
     { "Emulate filters",
       MENU_ENTRY_RESOURCE_TOGGLE,
       toggle_SidFilters_callback,
@@ -585,16 +490,32 @@ ui_menu_entry_t sid_c128_menu[] = {
       (ui_callback_data_t)c64_stereo_sid_menu },
     { "Second SID base address",
       MENU_ENTRY_SUBMENU,
-      show_SidStereoAddressStart_callback,
-      (ui_callback_data_t)c128_stereo_sid_base_menu },
+      show_Sid2AddressStart_callback,
+      (ui_callback_data_t)c128_sid2_base_menu },
     { "Third SID base address",
       MENU_ENTRY_SUBMENU,
-      show_SidTripleAddressStart_callback,
-      (ui_callback_data_t)c128_triple_sid_base_menu },
+      show_Sid3AddressStart_callback,
+      (ui_callback_data_t)c128_sid3_base_menu },
     { "Fourth SID base address",
       MENU_ENTRY_SUBMENU,
-      show_SidQuadAddressStart_callback,
-      (ui_callback_data_t)c128_quad_sid_base_menu },
+      show_Sid4AddressStart_callback,
+      (ui_callback_data_t)c128_sid4_base_menu },
+    { "Fift SID base address",
+      MENU_ENTRY_SUBMENU,
+      show_Sid5AddressStart_callback,
+      (ui_callback_data_t)c128_sid5_base_menu },
+    { "Sixth SID base address",
+      MENU_ENTRY_SUBMENU,
+      show_Sid6AddressStart_callback,
+      (ui_callback_data_t)c128_sid6_base_menu },
+    { "Seventh SID base address",
+      MENU_ENTRY_SUBMENU,
+      show_Sid7AddressStart_callback,
+      (ui_callback_data_t)c128_sid7_base_menu },
+    { "Eighth SID base address",
+      MENU_ENTRY_SUBMENU,
+      show_Sid8AddressStart_callback,
+      (ui_callback_data_t)c128_sid8_base_menu },
     { "Emulate filters",
       MENU_ENTRY_RESOURCE_TOGGLE,
       toggle_SidFilters_callback,
@@ -802,4 +723,3 @@ void uisid_menu_shutdown(void)
         lib_free(sid_model_menu);
     }
 }
-
