@@ -107,19 +107,19 @@ static const ui_menu_entry_t vic20_memory_common_menu[] = {
       MENU_ENTRY_RESOURCE_RADIO,
       custom_memory_callback,
       (ui_callback_data_t)0 },
-    { "3kB",
+    { "3KiB",
       MENU_ENTRY_RESOURCE_RADIO,
       custom_memory_callback,
       (ui_callback_data_t)(BLOCK_0) },
-    { "8kB",
+    { "8KiB",
       MENU_ENTRY_RESOURCE_RADIO,
       custom_memory_callback,
       (ui_callback_data_t)(BLOCK_1) },
-    { "16kB",
+    { "16KiB",
       MENU_ENTRY_RESOURCE_RADIO,
       custom_memory_callback,
       (ui_callback_data_t)(BLOCK_1 | BLOCK_2) },
-    { "24kB",
+    { "24KiB",
       MENU_ENTRY_RESOURCE_RADIO,
       custom_memory_callback,
       (ui_callback_data_t)(BLOCK_1 | BLOCK_2 | BLOCK_3) },
@@ -198,7 +198,6 @@ static const ui_menu_entry_t userport_menu[] = {
     SDL_MENU_LIST_END
 };
 
-UI_MENU_DEFINE_TOGGLE(Mouse)
 UI_MENU_DEFINE_TOGGLE(RAMBlock0)
 UI_MENU_DEFINE_TOGGLE(RAMBlock1)
 UI_MENU_DEFINE_TOGGLE(RAMBlock2)
@@ -276,12 +275,6 @@ const ui_menu_entry_t vic20_hardware_menu[] = {
       MENU_ENTRY_SUBMENU,
       submenu_callback,
       (ui_callback_data_t)ethernetcart20_menu },
-#endif
-#ifdef HAVE_MOUSE
-    { "Paddle emulation",
-      MENU_ENTRY_RESOURCE_TOGGLE,
-      toggle_Mouse_callback,
-      NULL },
 #endif
     SDL_MENU_ITEM_SEPARATOR,
     SDL_MENU_ITEM_TITLE("Memory expansions"),
@@ -289,23 +282,23 @@ const ui_menu_entry_t vic20_hardware_menu[] = {
       MENU_ENTRY_SUBMENU,
       submenu_radio_callback,
       (ui_callback_data_t)vic20_memory_common_menu },
-    { "Block 0 (3kB at $0400-$0FFF)",
+    { "Block 0 (3KiB at $0400-$0FFF)",
       MENU_ENTRY_RESOURCE_TOGGLE,
       toggle_RAMBlock0_callback,
       NULL },
-    { "Block 1 (8kB at $2000-$3FFF)",
+    { "Block 1 (8KiB at $2000-$3FFF)",
       MENU_ENTRY_RESOURCE_TOGGLE,
       toggle_RAMBlock1_callback,
       NULL },
-    { "Block 2 (8kB at $4000-$5FFF)",
+    { "Block 2 (8KiB at $4000-$5FFF)",
       MENU_ENTRY_RESOURCE_TOGGLE,
       toggle_RAMBlock2_callback,
       NULL },
-    { "Block 3 (8kB at $6000-$7FFF)",
+    { "Block 3 (8KiB at $6000-$7FFF)",
       MENU_ENTRY_RESOURCE_TOGGLE,
       toggle_RAMBlock3_callback,
       NULL },
-    { "Block 5 (8kB at $A000-$BFFF)",
+    { "Block 5 (8KiB at $A000-$BFFF)",
       MENU_ENTRY_RESOURCE_TOGGLE,
       toggle_RAMBlock5_callback,
       NULL },
