@@ -32,6 +32,7 @@
 #include "log.h"
 #include "machine.h"
 #include "main.h"
+#include "uimenu.h"
 
 #include "vice_sdl.h"
 
@@ -44,6 +45,12 @@ void main_exit(void)
 {
     log_message(LOG_DEFAULT, "\nExiting...");
 
+    /*
+     * Clean up dangling resources due to the 'Quit emu' callback not returning
+     * to the calling menu code.
+     */
+    sdl_ui_menu_shutdown();
+
     machine_shutdown();
 
     putchar('\n');
