@@ -60,7 +60,9 @@
 #include "uimenu.h"
 #include "vkbd.h"
 
-static const ui_menu_entry_t x64dtv_main_menu[] = {
+static UI_MENU_CALLBACK(pause_callback_wrapper);
+
+static ui_menu_entry_t x64dtv_main_menu[] = {
     { "Autostart image",
       MENU_ENTRY_DIALOG,
       autostart_callback,
@@ -119,8 +121,9 @@ static const ui_menu_entry_t x64dtv_main_menu[] = {
 #endif
     { "Pause",
       MENU_ENTRY_OTHER_TOGGLE,
-      pause_callback,
+      pause_callback_wrapper,
       NULL },
+    /* Caution: index is hardcoded below */
     { "Advance Frame",
       MENU_ENTRY_OTHER,
       advance_frame_callback,
@@ -129,6 +132,7 @@ static const ui_menu_entry_t x64dtv_main_menu[] = {
       MENU_ENTRY_SUBMENU,
       submenu_callback,
       (ui_callback_data_t)monitor_menu },
+    /* Caution: index is hardcoded below */
     { "Virtual keyboard",
       MENU_ENTRY_OTHER,
       vkbd_callback,
@@ -164,6 +168,22 @@ static const ui_menu_entry_t x64dtv_main_menu[] = {
     SDL_MENU_LIST_END
 };
 
+#ifdef HAVE_NETWORK
+# define MENU_ADVANCE_FRAME_IDX      14
+# define MENU_VIRTUAL_KEYBOARD_IDX   16
+#else
+# define MENU_ADVANCE_FRAME_IDX      13
+# define MENU_VIRTUAL_KEYBOARD_IDX   15
+#endif
+static UI_MENU_CALLBACK(pause_callback_wrapper)
+{
+    x64dtv_main_menu[MENU_ADVANCE_FRAME_IDX].status = 
+        sdl_pause_state || !sdl_menu_state ? MENU_STATUS_ACTIVE : MENU_STATUS_INACTIVE;
+    x64dtv_main_menu[MENU_VIRTUAL_KEYBOARD_IDX].status =
+        sdl_pause_state ? MENU_STATUS_INACTIVE : MENU_STATUS_ACTIVE;
+    return pause_callback(activated, param);
+}
+
 static void c64dtvui_set_menu_params(int index, menu_draw_t *menu_draw)
 {
     /* VICII-DTV */
