@@ -67,7 +67,10 @@
 #include "videoarch.h"
 #include "vkbd.h"
 
-static const ui_menu_entry_t xcbm6x0_7x0_main_menu[] = {
+static UI_MENU_CALLBACK(pause_callback_wrapper);
+static UI_MENU_CALLBACK(pause_callback_wrapper2);
+
+static ui_menu_entry_t xcbm6x0_7x0_main_menu[] = {
     { "Autostart image",
       MENU_ENTRY_DIALOG,
       autostart_callback,
@@ -129,8 +132,9 @@ static const ui_menu_entry_t xcbm6x0_7x0_main_menu[] = {
 #endif
     { "Pause",
       MENU_ENTRY_OTHER_TOGGLE,
-      pause_callback,
+      pause_callback_wrapper,
       NULL },
+    /* Caution: index is hardcoded below */
     { "Advance Frame",
       MENU_ENTRY_OTHER,
       advance_frame_callback,
@@ -139,6 +143,7 @@ static const ui_menu_entry_t xcbm6x0_7x0_main_menu[] = {
       MENU_ENTRY_SUBMENU,
       submenu_callback,
       (ui_callback_data_t)monitor_menu },
+    /* Caution: index is hardcoded below */
     { "Virtual keyboard",
       MENU_ENTRY_OTHER,
       vkbd_callback,
@@ -174,7 +179,23 @@ static const ui_menu_entry_t xcbm6x0_7x0_main_menu[] = {
     SDL_MENU_LIST_END
 };
 
-static const ui_menu_entry_t xcbm5x0_main_menu[] = {
+#ifdef HAVE_NETWORK
+# define MENU_ADVANCE_FRAME_IDX      15
+# define MENU_VIRTUAL_KEYBOARD_IDX   17
+#else
+# define MENU_ADVANCE_FRAME_IDX      14
+# define MENU_VIRTUAL_KEYBOARD_IDX   16
+#endif
+static UI_MENU_CALLBACK(pause_callback_wrapper)
+{
+    xcbm6x0_7x0_main_menu[MENU_ADVANCE_FRAME_IDX].status = 
+        sdl_pause_state || !sdl_menu_state ? MENU_STATUS_ACTIVE : MENU_STATUS_INACTIVE;
+    xcbm6x0_7x0_main_menu[MENU_VIRTUAL_KEYBOARD_IDX].status =
+        sdl_pause_state ? MENU_STATUS_INACTIVE : MENU_STATUS_ACTIVE;
+    return pause_callback(activated, param);
+}
+
+static ui_menu_entry_t xcbm5x0_main_menu[] = {
     { "Autostart image",
       MENU_ENTRY_DIALOG,
       autostart_callback,
@@ -239,8 +260,9 @@ static const ui_menu_entry_t xcbm5x0_main_menu[] = {
 #endif
     { "Pause",
       MENU_ENTRY_OTHER_TOGGLE,
-      pause_callback,
+      pause_callback_wrapper2,
       NULL },
+    /* Caution: index is hardcoded below */
     { "Advance Frame",
       MENU_ENTRY_OTHER,
       advance_frame_callback,
@@ -249,6 +271,7 @@ static const ui_menu_entry_t xcbm5x0_main_menu[] = {
       MENU_ENTRY_SUBMENU,
       submenu_callback,
       (ui_callback_data_t)monitor_menu },
+    /* Caution: index is hardcoded below */
     { "Virtual keyboard",
       MENU_ENTRY_OTHER,
       vkbd_callback,
@@ -284,6 +307,22 @@ static const ui_menu_entry_t xcbm5x0_main_menu[] = {
     SDL_MENU_LIST_END
 };
 
+#ifdef HAVE_NETWORK
+# define MENU2_ADVANCE_FRAME_IDX      16
+# define MENU2_VIRTUAL_KEYBOARD_IDX   18
+#else
+# define MENU2_ADVANCE_FRAME_IDX      15
+# define MENU2_VIRTUAL_KEYBOARD_IDX   17
+#endif
+static UI_MENU_CALLBACK(pause_callback_wrapper2)
+{
+    xcbm5x0_main_menu[MENU2_ADVANCE_FRAME_IDX].status = 
+        sdl_pause_state || !sdl_menu_state ? MENU_STATUS_ACTIVE : MENU_STATUS_INACTIVE;
+    xcbm5x0_main_menu[MENU2_VIRTUAL_KEYBOARD_IDX].status =
+        sdl_pause_state ? MENU_STATUS_INACTIVE : MENU_STATUS_ACTIVE;
+    return pause_callback(activated, param);
+}
+
 static void cbm2ui_set_menu_params(int index, menu_draw_t *menu_draw)
 {
     int model;
