@@ -38,7 +38,7 @@
 #include "util.h"
 #include "archdep_defs.h"
 
-#include "archdep_atexit.h"
+#include "archdep_exit.h"
 #include "archdep_home_path.h"
 
 /* TODO:    Include required headers for AmigaOS */
@@ -77,44 +77,9 @@ int archdep_expand_path(char **return_path, const char *orig_name)
 #elif defined(ARCHDEP_OS_WINDOWS)
     /* taken from the old WinVICE port (src/arch/win32/archdep.c): */
     *return_path = lib_strdup(orig_name);
-#elif defined(ARCHDEP_OS_AMIGA)
-    /* taken from src/arch/sdl/archdep_amiga.c: */
-    BPTR lock;
-
-    lock = Lock(orig_name, ACCESS_READ);
-    if (lock) {
-        char name[1024];
-        LONG rc;
-        rc = NameFromLock(lock, name, 1024);
-        UnLock(lock);
-        if (rc != 0) {
-            *return_path = lib_strdup(name);
-            return 0;
-        }
-    }
-    *return_path = lib_strdup(orig_name);
 #elif defined(ARCHDEP_OS_BEOS)
     /* taken from src/arch/sdl/archdep_beos.c: */
     *return_path = lib_strdup(orig_name);
-#elif defined(ARCHDEP_OS_OS2)
-    /* the OS/2 code is too terrible to include, so just exit: */
-    log_err(LOG_ERR, "OS/2 code is too screwed up, sorry.");
-    archdep_vice_exit(1);
-# if 0
-    if (filename[0] == '\\' || filename[1] == ':') {
-        *return_path = lib_strdup(filename);
-    } else {
-        char *p = (char *)malloc(512);
-        while (getcwd(p, 512) == NULL) {
-            return 0;
-        }
-
-        *return_path = util_concat(p, "\\", filename, NULL);
-        lib_free(p);
-    }
-    return 0;
-# endif
-
 #else
     /* fallback */
     log_error(LOG_ERR, "unsupported OS: just returning input.");
