@@ -13,9 +13,6 @@
  *  - Windows
  *  - MacOS
  *  - BeOS/Haiku (always returns '/boot/home')
- *  - AmigaOS (untested, always returns 'PROGDIR:')
- *  - OS/2 (untested, always returns '.')
- *  - MS-DOS (untested, always returns '.')
  *
  */
 
@@ -50,10 +47,6 @@
 #include "lib.h"
 #include "log.h"
 
-#ifdef ARCHDEP_OS_AMIGA
-/* some includes */
-#endif
-
 #ifdef ARCHDEP_OS_UNIX
 # include <unistd.h>
 # include <sys/types.h>
@@ -90,6 +83,11 @@ const char *archdep_home_path(void)
     /* stupid vice code rules, only declare vars at the top */
 #ifdef ARCHDEP_OS_UNIX
     char *home;
+#elif defined(ARCHDEP_OS_WINDOWS)
+    HANDLE token_handle;
+    DWORD bufsize;
+    LPDWORD lpcchSize;
+    DWORD err;
 #endif
 
     if (home_dir != NULL) {
@@ -110,10 +108,8 @@ const char *archdep_home_path(void)
     }
     home_dir = lib_strdup(home);
 #elif defined(ARCHDEP_OS_WINDOWS)
-    HANDLE token_handle;
-    DWORD bufsize = 4096;
-    LPDWORD lpcchSize = &bufsize;
-    DWORD err;
+    bufsize = 4096;
+    lpcchSize = &bufsize;
 
     /* get process token handle, whatever the hell that means */
     if (!OpenProcessToken(GetCurrentProcess(),
@@ -140,9 +136,6 @@ const char *archdep_home_path(void)
 #elif defined(ARCHDEP_OS_BEOS)
     /* Beos/Haiku is single-user */
     home_dir = lib_strdup("/boot/home");
-#elif defined(ARCHDEP_OS_AMIGA)
-    /* single user: use the path to the executable as the "home" dir */
-    home_dir = lib_strdup("PROGDIR:");
 #else
     /* all others: */
     home_dir = lib_strdup(".");
