@@ -84,7 +84,7 @@ int machine_drive_cmdline_options_init(void)
     return 0;
 }
 
-void machine_drive_init(struct drive_context_s *drv)
+void machine_drive_init(struct diskunit_context_s *drv)
 {
     iec_drive_init(drv);
     iec128dcr_drive_init(drv);
@@ -93,7 +93,7 @@ void machine_drive_init(struct drive_context_s *drv)
     ieee_drive_init(drv);
 }
 
-void machine_drive_shutdown(struct drive_context_s *drv)
+void machine_drive_shutdown(struct diskunit_context_s *drv)
 {
     iec_drive_shutdown(drv);
     iec128dcr_drive_shutdown(drv);
@@ -101,7 +101,7 @@ void machine_drive_shutdown(struct drive_context_s *drv)
     ieee_drive_shutdown(drv);
 }
 
-void machine_drive_reset(struct drive_context_s *drv)
+void machine_drive_reset(struct diskunit_context_s *drv)
 {
     iec_drive_reset(drv);
     iec128dcr_drive_reset(drv);
@@ -110,7 +110,7 @@ void machine_drive_reset(struct drive_context_s *drv)
     ieee_drive_reset(drv);
 }
 
-void machine_drive_mem_init(struct drive_context_s *drv, unsigned int type)
+void machine_drive_mem_init(struct diskunit_context_s *drv, unsigned int type)
 {
     iec_drive_mem_init(drv, type);
     iec128dcr_drive_mem_init(drv, type);
@@ -118,7 +118,7 @@ void machine_drive_mem_init(struct drive_context_s *drv, unsigned int type)
     ieee_drive_mem_init(drv, type);
 }
 
-void machine_drive_setup_context(struct drive_context_s *drv)
+void machine_drive_setup_context(struct diskunit_context_s *drv)
 {
     iec_drive_setup_context(drv);
     iec128dcr_drive_setup_context(drv);
@@ -166,7 +166,7 @@ void machine_drive_rom_do_checksum(unsigned int dnr)
     ieee_drive_rom_do_checksum(dnr);
 }
 
-int machine_drive_snapshot_read(struct drive_context_s *ctxptr, struct snapshot_s *s)
+int machine_drive_snapshot_read(struct diskunit_context_s *ctxptr, struct snapshot_s *s)
 {
     if (iec_drive_snapshot_read(ctxptr, s) < 0) {
         return -1;
@@ -181,7 +181,7 @@ int machine_drive_snapshot_read(struct drive_context_s *ctxptr, struct snapshot_
     return 0;
 }
 
-int machine_drive_snapshot_write(struct drive_context_s *ctxptr, struct snapshot_s *s)
+int machine_drive_snapshot_write(struct diskunit_context_s *ctxptr, struct snapshot_s *s)
 {
     if (iec_drive_snapshot_write(ctxptr, s) < 0) {
         return -1;
@@ -196,17 +196,17 @@ int machine_drive_snapshot_write(struct drive_context_s *ctxptr, struct snapshot
     return 0;
 }
 
-int machine_drive_image_attach(struct disk_image_s *image, unsigned int unit)
+int machine_drive_image_attach(struct disk_image_s *image, unsigned int unit, unsigned int drive)
 {
-    return iec_drive_image_attach(image, unit) & ieee_drive_image_attach(image, unit);
+    return iec_drive_image_attach(image, unit, drive) & ieee_drive_image_attach(image, unit, drive);
 }
 
-int machine_drive_image_detach(struct disk_image_s *image, unsigned int unit)
+int machine_drive_image_detach(struct disk_image_s *image, unsigned int unit, unsigned int drive)
 {
-    return iec_drive_image_detach(image, unit) & ieee_drive_image_detach(image, unit);
+    return iec_drive_image_detach(image, unit, drive) & ieee_drive_image_detach(image, unit, drive);
 }
 
-void machine_drive_port_default(struct drive_context_s *drv)
+void machine_drive_port_default(struct diskunit_context_s *drv)
 {
     iec_drive_port_default(drv);
 }
@@ -236,6 +236,7 @@ static drive_type_info_t drive_type_info_list[] = {
     { DRIVE_NAME_1581, DRIVE_TYPE_1581 },
     { DRIVE_NAME_2000, DRIVE_TYPE_2000 },
     { DRIVE_NAME_4000, DRIVE_TYPE_4000 },
+    { DRIVE_NAME_CMDHD, DRIVE_TYPE_CMDHD },
     { DRIVE_NAME_2031, DRIVE_TYPE_2031 },
     { DRIVE_NAME_2040, DRIVE_TYPE_2040 },
     { DRIVE_NAME_3040, DRIVE_TYPE_3040 },
@@ -243,6 +244,7 @@ static drive_type_info_t drive_type_info_list[] = {
     { DRIVE_NAME_1001, DRIVE_TYPE_1001 },
     { DRIVE_NAME_8050, DRIVE_TYPE_8050 },
     { DRIVE_NAME_8250, DRIVE_TYPE_8250 },
+    { DRIVE_NAME_9000, DRIVE_TYPE_9000 },
     { NULL, -1 }
 };
 
