@@ -183,7 +183,7 @@ static int set_kernal_revision(int val, void *param)
 {
     int trapfl;
 
-    log_verbose("set_kernal_revision val:%d kernal_revision: %d", val, kernal_revision);
+    log_verbose("set_kernal_revision (\"KernalRev\") val:%d kernal_revision: %d", val, kernal_revision);
     if(!c64rom_isloaded()) {
         return 0;
     }
@@ -205,7 +205,7 @@ static int set_kernal_revision(int val, void *param)
         resources_set_int("VirtualDevices", trapfl);
     }
     kernal_revision = val;
-    log_verbose("set_kernal_revision new kernal_revision: %d", kernal_revision);
+    log_verbose("set_kernal_revision (\"KernalRev\") new kernal_revision: %d", kernal_revision);
     return 0;
 }
 
@@ -219,32 +219,18 @@ static int set_sync_factor(int val, void *param)
 
     switch (val) {
         case MACHINE_SYNC_PAL:
-            sync_factor = val;
-            if (change_timing) {
-                machine_change_timing(MACHINE_SYNC_PAL, vicii_resources.border_mode);
-            }
-            break;
         case MACHINE_SYNC_NTSC:
-            sync_factor = val;
-            if (change_timing) {
-                machine_change_timing(MACHINE_SYNC_NTSC, vicii_resources.border_mode);
-            }
-            break;
         case MACHINE_SYNC_NTSCOLD:
-            sync_factor = val;
-            if (change_timing) {
-                machine_change_timing(MACHINE_SYNC_NTSCOLD, vicii_resources.border_mode);
-            }
-            break;
         case MACHINE_SYNC_PALN:
-            sync_factor = val;
-            if (change_timing) {
-                machine_change_timing(MACHINE_SYNC_PALN, vicii_resources.border_mode);
-            }
             break;
         default:
             return -1;
     }
+    sync_factor = val;
+    if (change_timing) {
+        vicii_comply_with_video_standard(val);
+        machine_change_timing(val, vicii_resources.border_mode);
+    }
 
     return 0;
 }
@@ -275,12 +261,20 @@ static const resource_int_t resources_int[] = {
       &cia2_model, set_cia2_model, NULL },
     { "KernalRev", C64_KERNAL_REV3, RES_EVENT_SAME, NULL,
       &kernal_revision, set_kernal_revision, NULL },
-    { "SidStereoAddressStart", 0xde00, RES_EVENT_SAME, NULL,
-      (int *)&sid_stereo_address_start, sid_set_sid_stereo_address, NULL },
-    { "SidTripleAddressStart", 0xdf00, RES_EVENT_SAME, NULL,
-      (int *)&sid_triple_address_start, sid_set_sid_triple_address, NULL },
-    { "SidQuadAddressStart", 0xdf80, RES_EVENT_SAME, NULL,
-      (int *)&sid_quad_address_start, sid_set_sid_quad_address, NULL },
+    { "Sid2AddressStart", 0xde00, RES_EVENT_SAME, NULL,
+      (int *)&sid2_address_start, sid_set_sid2_address, NULL },
+    { "Sid3AddressStart", 0xdf00, RES_EVENT_SAME, NULL,
+      (int *)&sid3_address_start, sid_set_sid3_address, NULL },
+    { "Sid4AddressStart", 0xdf80, RES_EVENT_SAME, NULL,
+      (int *)&sid4_address_start, sid_set_sid4_address, NULL },
+    { "Sid5AddressStart", 0xde80, RES_EVENT_SAME, NULL,
+      (int *)&sid5_address_start, sid_set_sid5_address, NULL },
+    { "Sid6AddressStart", 0xdf40, RES_EVENT_SAME, NULL,
+      (int *)&sid6_address_start, sid_set_sid6_address, NULL },
+    { "Sid7AddressStart", 0xde40, RES_EVENT_SAME, NULL,
+      (int *)&sid7_address_start, sid_set_sid7_address, NULL },
+    { "Sid8AddressStart", 0xdfc0, RES_EVENT_SAME, NULL,
+      (int *)&sid8_address_start, sid_set_sid8_address, NULL },
     { "BurstMod", BURST_MOD_NONE, RES_EVENT_NO, NULL,
       &burst_mod, set_burst_mod, NULL },
     RESOURCE_INT_LIST_END
