@@ -53,6 +53,10 @@
 #include "actionreplay4.h"
 #include "actionreplay.h"
 #include "atomicpower.h"
+#include "bisplus.h"
+#include "blackbox3.h"
+#include "blackbox4.h"
+#include "blackbox8.h"
 #include "c64acia.h"
 #include "c64-generic.h"
 #include "c64-midi.h"
@@ -78,11 +82,14 @@
 #include "gamekiller.h"
 #include "georam.h"
 #include "gmod2.h"
+#include "gmod3.h"
 #include "gs.h"
+#include "hero.h"
 #include "ide64.h"
 #include "isepic.h"
 #include "kcs.h"
 #include "kingsoft.h"
+#include "ltkernal.h"
 #include "mach5.h"
 #include "magicdesk.h"
 #include "magicformel.h"
@@ -91,16 +98,20 @@
 #include "mikroass.h"
 #include "mmc64.h"
 #include "mmcreplay.h"
+#include "multimax.h"
 #include "ocean.h"
 #include "pagefox.h"
 #include "prophet64.h"
 #include "ramcart.h"
+#include "ramlink.h"
 #include "retroreplay.h"
 #include "reu.h"
 #include "rexep256.h"
+#include "rexramfloppy.h"
 #include "rexutility.h"
 #include "rrnetmk3.h"
 #include "ross.h"
+#include "sdbox.h"
 #include "simonsbasic.h"
 #include "snapshot64.h"
 #include "stardos.h"
@@ -112,6 +123,7 @@
 #include "warpspeed.h"
 #include "westermann.h"
 #include "zaxxon.h"
+#include "zippcode48.h"
 #undef CARTRIDGE_INCLUDE_PRIVATE_API
 
 /* #define DEBUGCART */
@@ -151,10 +163,6 @@ void cart_set_port_game_slot1(int);
 void cart_port_config_changed_slot1(void);
 void cart_config_changed_slot1(uint8_t mode_phi1, uint8_t mode_phi2, unsigned int wflag);
 
-
-
-
-
 /*
   mode_phiN:
 
@@ -177,7 +185,7 @@ void cart_config_changed_slot1(uint8_t mode_phi1, uint8_t mode_phi2, unsigned in
 */
 const char *cart_config_string(uint8_t mode)
 {
-    const char *modes[4] = {"8k game", "16k game", "Off", "Ultimax"};
+    static const char * const modes[4] = {"8k game", "16k game", "Off", "Ultimax"};
     return modes[mode & 3];
 }
 
@@ -515,6 +523,8 @@ void cart_romlbank_set_slotmain(unsigned int bank)
 
         magic voice
 
+        ltkernal
+
         isepic
         expert
 
@@ -522,6 +532,7 @@ void cart_romlbank_set_slotmain(unsigned int bank)
         exos
         final plus
         game killer
+        gmod3       (Uses ultimax to change the hw vectors at fff8-ffff)
         capture
         magicformel
         mmcreplay
@@ -551,7 +562,7 @@ void cart_romlbank_set_slotmain(unsigned int bank)
         midi
         acia
         clockport
-	ethernetcart
+        ethernetcart
 
               hook                    default
 
@@ -609,18 +620,28 @@ static uint8_t roml_read_slotmain(uint16_t addr)
             return freezemachine_roml_read(addr);
         case CARTRIDGE_GMOD2:
             return gmod2_roml_read(addr);
+        case CARTRIDGE_GMOD3:
+            return gmod3_roml_read(addr);
         case CARTRIDGE_IDE64:
             return ide64_rom_read(addr);
         case CARTRIDGE_KINGSOFT:
             return kingsoft_roml_read(addr);
+        case CARTRIDGE_LT_KERNAL:
+            return ltkernal_roml_read(addr);
         case CARTRIDGE_MAX_BASIC:
             return maxbasic_roml_read(addr);
         case CARTRIDGE_MMC_REPLAY:
             return mmcreplay_roml_read(addr);
+        case CARTRIDGE_MULTIMAX:
+            return multimax_roml_read(addr);
         case CARTRIDGE_PAGEFOX:
             return pagefox_roml_read(addr);
+        case CARTRIDGE_RAMLINK:
+            return ramlink_roml_read(addr);
         case CARTRIDGE_RETRO_REPLAY:
             return retroreplay_roml_read(addr);
+        case CARTRIDGE_REX_RAMFLOPPY:
+            return rexramfloppy_roml_read(addr);
 #ifdef HAVE_RAWNET
         case CARTRIDGE_RRNETMK3:
             return rrnetmk3_roml_read(addr);
@@ -637,6 +658,8 @@ static uint8_t roml_read_slotmain(uint16_t addr)
             return se5_roml_read(addr);
         case CARTRIDGE_ZAXXON:
             return zaxxon_roml_read(addr);
+        case CARTRIDGE_ZIPPCODE48:
+            return zippcode48_roml_read(addr);
         case CARTRIDGE_CAPTURE:
         case CARTRIDGE_EXOS:
         case CARTRIDGE_FORMEL64:
@@ -754,6 +777,9 @@ void roml_store(uint16_t addr, uint8_t value)
         case CARTRIDGE_EASYFLASH:
             easyflash_roml_store(addr, value);
             return;
+        case CARTRIDGE_LT_KERNAL:
+            ltkernal_roml_store(addr, value);
+            return;
         case CARTRIDGE_MMC_REPLAY:
             mmcreplay_roml_store(addr, value);
             return;
@@ -775,6 +801,8 @@ void roml_store(uint16_t addr, uint8_t value)
         case CARTRIDGE_EXOS:
         case CARTRIDGE_FORMEL64:
         case CARTRIDGE_GAME_KILLER:
+        case CARTRIDGE_GMOD3:
+        case CARTRIDGE_RAMLINK:
         case CARTRIDGE_STARDOS:
         case CARTRIDGE_MAGIC_FORMEL: /* ? */
             /* fake ultimax hack */
@@ -821,25 +849,33 @@ static uint8_t romh_read_slotmain(uint16_t addr)
             return ide64_rom_read(addr);
         case CARTRIDGE_KINGSOFT:
             return kingsoft_romh_read(addr);
+        case CARTRIDGE_LT_KERNAL:
+            return ltkernal_romh_read(addr);
         case CARTRIDGE_MAGIC_FORMEL:
             return magicformel_romh_read(addr);
         case CARTRIDGE_MAX_BASIC:
             return maxbasic_romh_read(addr);
         case CARTRIDGE_MMC_REPLAY:
             return mmcreplay_romh_read(addr);
+        case CARTRIDGE_MULTIMAX:
+            return multimax_romh_read(addr);
         case CARTRIDGE_OCEAN:
             return ocean_romh_read(addr);
         case CARTRIDGE_PAGEFOX:
             return pagefox_romh_read(addr);
+        case CARTRIDGE_RAMLINK:
+            return ramlink_romh_read(addr);
         case CARTRIDGE_RETRO_REPLAY:
             return retroreplay_romh_read(addr);
         case CARTRIDGE_SNAPSHOT64:
             return snapshot64_romh_read(addr);
         case CARTRIDGE_EXOS:
-        case CARTRIDGE_STARDOS:
         case CARTRIDGE_GMOD2:
+        case CARTRIDGE_STARDOS:
             /* fake ultimax hack, read from ram */
             return ram_read(addr);
+        case CARTRIDGE_GMOD3:
+            return gmod3_romh_read(addr);
         /* return mem_read_without_ultimax(addr); */
         case CARTRIDGE_ACTION_REPLAY4:
         case CARTRIDGE_FINAL_III:
@@ -928,14 +964,20 @@ static uint8_t ultimax_romh_read_hirom_slotmain(uint16_t addr)
             return ide64_rom_read(addr);
         case CARTRIDGE_KINGSOFT:
             return kingsoft_romh_read(addr);
+        case CARTRIDGE_LT_KERNAL:
+            return ltkernal_romh_read(addr);
         case CARTRIDGE_MAGIC_FORMEL:
             return magicformel_romh_read_hirom(addr);
         case CARTRIDGE_MAX_BASIC:
             return maxbasic_romh_read(addr);
         case CARTRIDGE_MMC_REPLAY:
             return mmcreplay_romh_read(addr);
+        case CARTRIDGE_MULTIMAX:
+            return multimax_romh_read(addr);
         case CARTRIDGE_OCEAN:
             return ocean_romh_read(addr);
+        case CARTRIDGE_RAMLINK:
+            return ramlink_romh_read(addr);
         case CARTRIDGE_RETRO_REPLAY:
             return retroreplay_romh_read(addr);
         case CARTRIDGE_SNAPSHOT64:
@@ -945,6 +987,8 @@ static uint8_t ultimax_romh_read_hirom_slotmain(uint16_t addr)
         case CARTRIDGE_GMOD2:
             /* ultimax only enabled on writes */
             return mem_read_without_ultimax(addr);
+        case CARTRIDGE_GMOD3:
+            return gmod3_romh_read(addr);
         case CARTRIDGE_ACTION_REPLAY4:
         case CARTRIDGE_FINAL_III:
         case CARTRIDGE_FREEZE_FRAME:
@@ -1030,14 +1074,19 @@ void romh_store(uint16_t addr, uint8_t value)
         case CARTRIDGE_GMOD2:
             gmod2_romh_store(addr, value);
             return;
+        case CARTRIDGE_LT_KERNAL:
+            ltkernal_romh_store(addr, value);
+            return;
         case CARTRIDGE_MMC_REPLAY:
             mmcreplay_romh_store(addr, value);
             return;
         case CARTRIDGE_EXOS:
         case CARTRIDGE_FINAL_PLUS:
+        case CARTRIDGE_GMOD3:
+        case CARTRIDGE_MAGIC_FORMEL: /* ? */
+        case CARTRIDGE_RAMLINK:
         case CARTRIDGE_STARDOS:
         case CARTRIDGE_SNAPSHOT64: /* ? */
-        case CARTRIDGE_MAGIC_FORMEL: /* ? */
             /* fake ultimax hack, c64 ram */
             /* mem_store_without_ultimax(addr, value); */
             ram_store(addr, value);
@@ -1072,15 +1121,17 @@ void romh_no_ultimax_store(uint16_t addr, uint8_t value)
     switch (mem_cartridge_type) {
         case CARTRIDGE_ATOMIC_POWER:
             atomicpower_romh_store(addr, value);
+            return; /* writes to cartridge RAM should not fall through to C64 RAM in mode 0x22 */
+            break;
+        case CARTRIDGE_IDE64:
+            ide64_rom_store(addr, value);
             break;
         case CARTRIDGE_PAGEFOX:
             pagefox_romh_store(addr, value);
             break;
         case CARTRIDGE_RETRO_REPLAY:
             retroreplay_romh_store(addr, value);
-            break;
-        case CARTRIDGE_IDE64:
-            ide64_rom_store(addr, value);
+            return; /* writes to cartridge RAM should not fall through to C64 RAM in mode 0x22 */
             break;
         case CARTRIDGE_CRT: /* invalid */
             DBG(("CARTMEM: BUG! invalid type %d for main cart (addr %04x)\n", mem_cartridge_type, addr));
@@ -1124,6 +1175,9 @@ void roml_no_ultimax_store(uint16_t addr, uint8_t value)
         case CARTRIDGE_ATOMIC_POWER:
             atomicpower_roml_store(addr, value);
             break;
+        case CARTRIDGE_IDE64:
+            ide64_rom_store(addr, value);
+            break;
         case CARTRIDGE_PAGEFOX:
             pagefox_roml_store(addr, value);
             break;
@@ -1132,6 +1186,10 @@ void roml_no_ultimax_store(uint16_t addr, uint8_t value)
                 return; /* FIXME: this is weird */
             }
             break;
+        case CARTRIDGE_REX_RAMFLOPPY:
+            rexramfloppy_roml_store(addr, value);
+            return; /* writes to cartridge RAM should not fall through to C64 RAM */
+            break;
 #ifdef HAVE_RAWNET
         case CARTRIDGE_RRNETMK3:
             if (rrnetmk3_roml_store(addr, value)) {
@@ -1139,9 +1197,6 @@ void roml_no_ultimax_store(uint16_t addr, uint8_t value)
             }
             break;
 #endif
-        case CARTRIDGE_IDE64:
-            ide64_rom_store(addr, value);
-            break;
         case CARTRIDGE_CRT: /* invalid */
             DBG(("CARTMEM: BUG! invalid type %d for main cart (addr %04x)\n", mem_cartridge_type, addr));
             break;
@@ -1240,6 +1295,8 @@ uint8_t ultimax_0800_0fff_read(uint16_t addr)
     switch (mem_cartridge_type) {
         case CARTRIDGE_MAX_BASIC:
             return maxbasic_0800_0fff_read(addr);
+        case CARTRIDGE_MULTIMAX:
+            return multimax_0800_0fff_read(addr);
         case CARTRIDGE_CRT: /* invalid */
             DBG(("CARTMEM: BUG! invalid type %d for main cart (addr %04x)\n", mem_cartridge_type, addr));
             break;
@@ -1256,6 +1313,9 @@ void ultimax_0800_0fff_store(uint16_t addr, uint8_t value)
         case CARTRIDGE_MAX_BASIC:
             maxbasic_0800_0fff_store(addr, value);
             break;
+        case CARTRIDGE_MULTIMAX:
+            multimax_0800_0fff_store(addr, value);
+            break;
         case CARTRIDGE_CRT: /* invalid */
             DBG(("CARTMEM: BUG! invalid type %d for main cart (addr %04x)\n", mem_cartridge_type, addr));
             break;
@@ -1283,13 +1343,16 @@ static uint8_t ultimax_1000_7fff_read_slot1(uint16_t addr)
             return ide64_ram_read(addr);
         case CARTRIDGE_MMC_REPLAY:
             return mmcreplay_1000_7fff_read(addr);
+        case CARTRIDGE_EXOS:
+        case CARTRIDGE_FINAL_PLUS:
         case CARTRIDGE_FORMEL64:
-        case CARTRIDGE_MAGIC_FORMEL:
         case CARTRIDGE_GAME_KILLER:
         case CARTRIDGE_GMOD2:
+        case CARTRIDGE_GMOD3:
         case CARTRIDGE_KINGSOFT:
-        case CARTRIDGE_FINAL_PLUS:
-        case CARTRIDGE_EXOS:
+        case CARTRIDGE_LT_KERNAL:
+        case CARTRIDGE_MAGIC_FORMEL:
+        case CARTRIDGE_RAMLINK:
         case CARTRIDGE_STARDOS:
             /* fake ultimax hack, c64 ram */
             return mem_read_without_ultimax(addr);
@@ -1355,14 +1418,17 @@ void ultimax_1000_7fff_store(uint16_t addr, uint8_t value)
         case CARTRIDGE_CAPTURE:
             capture_1000_7fff_store(addr, value);
             break;
+        case CARTRIDGE_EXOS:
+        case CARTRIDGE_FINAL_PLUS:
         case CARTRIDGE_FORMEL64:
-        case CARTRIDGE_MAGIC_FORMEL:
         case CARTRIDGE_GAME_KILLER:
         case CARTRIDGE_GMOD2:
-        case CARTRIDGE_FINAL_PLUS:
-        case CARTRIDGE_EXOS:
-        case CARTRIDGE_STARDOS:
+        case CARTRIDGE_GMOD3:
         case CARTRIDGE_KINGSOFT:
+        case CARTRIDGE_LT_KERNAL:
+        case CARTRIDGE_MAGIC_FORMEL:
+        case CARTRIDGE_RAMLINK:
+        case CARTRIDGE_STARDOS:
             /* fake ultimax hack, c64 ram */
             mem_store_without_ultimax(addr, value);
             break;
@@ -1394,14 +1460,18 @@ static uint8_t ultimax_a000_bfff_read_slot1(uint16_t addr)
             return ide64_rom_read(addr);
         case CARTRIDGE_MMC_REPLAY:
             return mmcreplay_a000_bfff_read(addr);
+        case CARTRIDGE_RAMLINK:
+            return ramlink_a000_bfff_read(addr);
         case CARTRIDGE_RETRO_REPLAY:
             return retroreplay_a000_bfff_read(addr);
-        case CARTRIDGE_FORMEL64:
-        case CARTRIDGE_MAGIC_FORMEL:
         case CARTRIDGE_CAPTURE:
+        case CARTRIDGE_EXOS:
+        case CARTRIDGE_FORMEL64:
         case CARTRIDGE_GAME_KILLER:
         case CARTRIDGE_GMOD2:
-        case CARTRIDGE_EXOS:
+        case CARTRIDGE_GMOD3:
+        case CARTRIDGE_LT_KERNAL:
+        case CARTRIDGE_MAGIC_FORMEL:
         case CARTRIDGE_STARDOS:
             /* fake ultimax hack, c64 basic, ram */
             return mem_read_without_ultimax(addr);
@@ -1462,11 +1532,14 @@ void ultimax_a000_bfff_store(uint16_t addr, uint8_t value)
             retroreplay_a000_bfff_store(addr, value);
             break;
         case CARTRIDGE_CAPTURE:
-        case CARTRIDGE_MAGIC_FORMEL:
-        case CARTRIDGE_GAME_KILLER:
+        case CARTRIDGE_EXOS:
         case CARTRIDGE_FINAL_PLUS:
         case CARTRIDGE_FORMEL64:
-        case CARTRIDGE_EXOS:
+        case CARTRIDGE_GAME_KILLER:
+        case CARTRIDGE_GMOD3:
+        case CARTRIDGE_LT_KERNAL:
+        case CARTRIDGE_MAGIC_FORMEL:
+        case CARTRIDGE_RAMLINK:
         case CARTRIDGE_STARDOS:
             /* fake ultimax hack, c64 ram */
             mem_store_without_ultimax(addr, value);
@@ -1495,15 +1568,18 @@ static uint8_t ultimax_c000_cfff_read_slot1(uint16_t addr)
     switch (mem_cartridge_type) {
         case CARTRIDGE_MMC_REPLAY:
             return mmcreplay_c000_cfff_read(addr);
-        case CARTRIDGE_MAGIC_FORMEL:
         case CARTRIDGE_CAPTURE:
-        case CARTRIDGE_GAME_KILLER:
-        case CARTRIDGE_GMOD2:
+        case CARTRIDGE_EXOS:
         case CARTRIDGE_FINAL_PLUS:
         case CARTRIDGE_FORMEL64:
-        case CARTRIDGE_EXOS:
-        case CARTRIDGE_STARDOS:
+        case CARTRIDGE_GAME_KILLER:
+        case CARTRIDGE_GMOD2:
+        case CARTRIDGE_GMOD3:
         case CARTRIDGE_KINGSOFT:
+        case CARTRIDGE_LT_KERNAL:
+        case CARTRIDGE_MAGIC_FORMEL:
+        case CARTRIDGE_RAMLINK:
+        case CARTRIDGE_STARDOS:
             /* fake ultimax hack, c64 ram */
             return mem_read_without_ultimax(addr);
         case CARTRIDGE_CRT: /* invalid */
@@ -1561,14 +1637,17 @@ void ultimax_c000_cfff_store(uint16_t addr, uint8_t value)
         case CARTRIDGE_MMC_REPLAY:
             mmcreplay_c000_cfff_store(addr, value);
             break;
-        case CARTRIDGE_MAGIC_FORMEL:
         case CARTRIDGE_CAPTURE:
-        case CARTRIDGE_GAME_KILLER:
+        case CARTRIDGE_EXOS:
         case CARTRIDGE_FINAL_PLUS:
         case CARTRIDGE_FORMEL64:
-        case CARTRIDGE_EXOS:
-        case CARTRIDGE_STARDOS:
+        case CARTRIDGE_GAME_KILLER:
+        case CARTRIDGE_GMOD3:
         case CARTRIDGE_KINGSOFT:
+        case CARTRIDGE_LT_KERNAL:
+        case CARTRIDGE_MAGIC_FORMEL:
+        case CARTRIDGE_RAMLINK:
+        case CARTRIDGE_STARDOS:
             /* fake ultimax hack, c64 ram */
             mem_store_without_ultimax(addr, value);
             break;
@@ -1595,10 +1674,13 @@ static uint8_t ultimax_d000_dfff_read_slot1(uint16_t addr)
         case CARTRIDGE_EXOS:
         case CARTRIDGE_FINAL_PLUS:
         case CARTRIDGE_FORMEL64:
+        case CARTRIDGE_GMOD3:
+        case CARTRIDGE_KINGSOFT:
+        case CARTRIDGE_LT_KERNAL:
         case CARTRIDGE_MAGIC_FORMEL:
+        case CARTRIDGE_RAMLINK:
         case CARTRIDGE_SNAPSHOT64: /* ? */
         case CARTRIDGE_STARDOS:
-        case CARTRIDGE_KINGSOFT:
             /* fake ultimax hack, c64 io,colram,ram */
             return mem_read_without_ultimax(addr);
         case CARTRIDGE_CRT: /* invalid */
@@ -1657,10 +1739,13 @@ void ultimax_d000_dfff_store(uint16_t addr, uint8_t value)
         case CARTRIDGE_EXOS:
         case CARTRIDGE_FINAL_PLUS:
         case CARTRIDGE_FORMEL64:
+        case CARTRIDGE_GMOD3:
+        case CARTRIDGE_KINGSOFT:
+        case CARTRIDGE_LT_KERNAL:
         case CARTRIDGE_MAGIC_FORMEL:
+        case CARTRIDGE_RAMLINK:
         case CARTRIDGE_SNAPSHOT64: /* ? */
         case CARTRIDGE_STARDOS:
-        case CARTRIDGE_KINGSOFT:
             /* fake ultimax hack, c64 io,colram,ram */
             mem_store_without_ultimax(addr, value);
             return;
@@ -1702,6 +1787,9 @@ static int ultimax_romh_phi1_read_slotmain(uint16_t addr, uint8_t *value)
         case CARTRIDGE_FINAL_PLUS:
             res = final_plus_romh_phi1_read(addr, value);
             break;
+        case CARTRIDGE_GMOD3:
+            res = gmod3_romh_phi1_read(addr, value);
+            break;
         case CARTRIDGE_MAGIC_FORMEL:
             res = magicformel_romh_phi1_read(addr, value);
             break;
@@ -1805,6 +1893,9 @@ static int ultimax_romh_phi2_read_slotmain(uint16_t addr, uint8_t *value)
         case CARTRIDGE_FINAL_PLUS:
             res = final_plus_romh_phi2_read(addr, value);
             break;
+        case CARTRIDGE_GMOD3:
+            res = gmod3_romh_phi2_read(addr, value);
+            break;
         case CARTRIDGE_MAGIC_FORMEL:
             res = magicformel_romh_phi2_read(addr, value);
             break;
@@ -2017,9 +2108,18 @@ static uint8_t cartridge_peek_mem_slotmain(uint16_t addr)
         case CARTRIDGE_GMOD2:
             res = gmod2_peek_mem(&export_slotmain, addr, &value);
             break;
+        case CARTRIDGE_GMOD3:
+            res = gmod3_peek_mem(&export_slotmain, addr, &value);
+            break;
+        case CARTRIDGE_LT_KERNAL:
+            res = ltkernal_peek_mem(&export_slotmain, addr, &value);
+            break;
         case CARTRIDGE_MAGIC_FORMEL:
             res = magicformel_peek_mem(&export_slotmain, addr, &value);
             break;
+        case CARTRIDGE_RAMLINK:
+            res = ramlink_peek_mem(&export_slotmain, addr, &value);
+            break;
         case CARTRIDGE_RETRO_REPLAY:
             res = retroreplay_peek_mem(&export_slotmain, addr, &value);
             break;
@@ -2044,7 +2144,7 @@ static uint8_t cartridge_peek_mem_slotmain(uint16_t addr)
                 if (addr >= 0xe000) {
                     return ultimax_romh_read_hirom_slotmain(addr);
                 }
-            } else if (!export_slotmain.exrom && !export_slotmain.game) {
+            } else if (export_slotmain.exrom && export_slotmain.game) {
                 /* 16k Game */
                 if (addr >= 0x8000 && addr <= 0x9fff) {
                     return roml_read_slotmain(addr);
