@@ -300,7 +300,7 @@ void capture_detach(void)
    ARRAY | RAM             | 8192 BYTES of RAM data
  */
 
-static char snap_module_name[] = "CARTCAPTURE";
+static const char snap_module_name[] = "CARTCAPTURE";
 #define SNAP_MAJOR   0
 #define SNAP_MINOR   0
 
