@@ -241,7 +241,7 @@ void comal80_detach(void)
    ARRAY | ROMH     | 32768 or 65536 BYTES of ROMH data
  */
 
-static char snap_module_name[] = "CARTCOMAL";
+static const char snap_module_name[] = "CARTCOMAL";
 #define SNAP_MAJOR   0
 #define SNAP_MINOR   1
 
