@@ -180,7 +180,7 @@ void dinamic_detach(void)
    ARRAY | ROML | 8192 BYTES of ROML data
  */
 
-static char snap_module_name[] = "CARTDINAMIC";
+static const char snap_module_name[] = "CARTDINAMIC";
 #define SNAP_MAJOR   0
 #define SNAP_MINOR   0
 
