@@ -256,7 +256,7 @@ void epyxfastload_detach(void)
    ARRAY | ROML   |   0.0+  | 8192 BYTES of ROML data
  */
 
-static char snap_module_name[] = "CARTEPYX";
+static const char snap_module_name[] = "CARTEPYX";
 #define SNAP_MAJOR   0
 #define SNAP_MINOR   1
 
