@@ -121,7 +121,7 @@ static int georam_enabled = 0;
 /* Size of the GEORAM.  */
 static int georam_size = 0;
 
-/* Size of the GEORAM in KB.  */
+/* Size of the GEORAM in KiB.  */
 static int georam_size_kb = 0;
 
 /* Filename of the GEORAM image.  */
@@ -227,7 +227,7 @@ static void georam_io2_store(uint16_t addr, uint8_t byte)
 
 static int georam_dump(void)
 {
-    mon_out("Size: %d Kb, Bank: %d, Window: %d\n", georam_size_kb, georam[1], georam[0]);
+    mon_out("Size: %d KiB, Bank: %d, Window: %d\n", georam_size_kb, georam[1], georam[0]);
     return 0;
 }
 
@@ -248,7 +248,7 @@ static int georam_activate(void)
 
     old_georam_ram_size = georam_size;
 
-    log_message(georam_log, "%dKB unit installed.", georam_size >> 10);
+    log_message(georam_log, "%dKiB unit installed.", georam_size >> 10);
 
     if (!util_check_null_string(georam_filename)) {
         if (util_file_load(georam_filename, georam_ram, (size_t)georam_size, UTIL_FILE_LOAD_RAW) < 0) {
@@ -468,7 +468,7 @@ static const cmdline_option_t cmdline_options[] =
       NULL, "Disable the GEO-RAM expansion unit" },
     { "-georamsize", SET_RESOURCE, CMDLINE_ATTRIB_NEED_ARGS,
       NULL, NULL, "GEORAMsize", NULL,
-      "<size in KB>", "Size of the GEORAM expansion unit" },
+      "<size in KiB>", "Size of the GEORAM expansion unit" },
     { "-georamimage", SET_RESOURCE, CMDLINE_ATTRIB_NEED_ARGS,
       NULL, NULL, "GEORAMfilename", NULL,
       "<Name>", "Specify name of GEORAM image" },
@@ -551,7 +551,7 @@ void georam_config_setup(uint8_t *rawcart)
 int georam_bin_attach(const char *filename, uint8_t *rawcart)
 {
     FILE *fd;
-    int size;
+    size_t size;
 
     fd = fopen(filename, MODE_READ);
     if (fd == NULL) {
@@ -560,7 +560,7 @@ int georam_bin_attach(const char *filename, uint8_t *rawcart)
     size = util_file_length(fd);
     fclose(fd);
 
-    if (set_georam_size(size / 1024, NULL) < 0) {
+    if (set_georam_size((uint32_t)(size / 1024), NULL) < 0) {
         return -1;
     }
 
@@ -604,12 +604,12 @@ int georam_flush_image(void)
    type  | name    | version | description
    ---------------------------------------
    BYTE  | io swap |   0.1   | VIC20 I/O swap flag
-   DWORD | size    |   0.0+  | size in KB
+   DWORD | size    |   0.0+  | size in KiB
    ARRAY | regs    |   0.0+  | 2 BYTES of register data
    ARRAY | RAM     |   0.0+  | 65536, 131072, 262144, 524288, 1048576, 2097152 or 4194304 BYTES of RAM data
  */
 
-static char snap_module_name[] = "GEORAM";
+static const char snap_module_name[] = "GEORAM";
 #define SNAP_MAJOR 0
 #define SNAP_MINOR 1
 
