@@ -174,7 +174,7 @@ void mikroass_detach(void)
    ARRAY | ROML | 8192 BYTES of ROML data
  */
 
-static char snap_module_name[] = "CARTMIKROASS";
+static const char snap_module_name[] = "CARTMIKROASS";
 #define SNAP_MAJOR   0
 #define SNAP_MINOR   0
 
