@@ -270,7 +270,7 @@ void pagefox_detach(void)
    ARRAY | ROMH    |   0.0+  | 32768 BYTES of ROMH data
  */
 
-static char snap_module_name[] = "CARTPAGEFOX";
+static const char snap_module_name[] = "CARTPAGEFOX";
 #define SNAP_MAJOR   0
 #define SNAP_MINOR   1
 
