@@ -267,7 +267,7 @@ static int ramcart_dump(void)
     bank += ramcart[0];
 
     mon_out("RAM size: %s, bank: %d, status: %s\n",
-            (ramcart_size_kb == 128) ? "128Kb" : "64Kb",
+            (ramcart_size_kb == 128) ? "128KiB" : "64KiB",
             bank,
             (ramcart_readonly) ? ((mirrored) ? "read-only and mirrored at $8000-$80FF" : "read-only") : "read/write");
     return 0;
@@ -290,7 +290,7 @@ static int ramcart_activate(void)
 
     old_ramcart_ram_size = ramcart_size;
 
-    log_message(ramcart_log, "%dKB unit installed.", ramcart_size >> 10);
+    log_message(ramcart_log, "%dKiB unit installed.", ramcart_size >> 10);
 
     if (!util_check_null_string(ramcart_filename)) {
         if (util_file_load(ramcart_filename, ramcart_ram, (size_t)ramcart_size, UTIL_FILE_LOAD_RAW) < 0) {
@@ -495,7 +495,7 @@ static const cmdline_option_t cmdline_options[] =
       NULL, "Disable the RamCart expansion" },
     { "-ramcartsize", SET_RESOURCE, CMDLINE_ATTRIB_NEED_ARGS,
       NULL, NULL, "RAMCARTsize", NULL,
-      "<size in KB>", "Size of the RAMCART expansion. (64/128)" },
+      "<size in KiB>", "Size of the RAMCART expansion. (64/128)" },
     { "-ramcartimage", SET_RESOURCE, CMDLINE_ATTRIB_NEED_ARGS,
       NULL, NULL, "RAMCARTfilename", NULL,
       "<Name>", "Specify name of RAMCART image" },
@@ -668,7 +668,7 @@ int ramcart_peek_mem(uint16_t addr, uint8_t *value)
    ARRAY | RAM       | 65536 or 131072 BYTES of RAM data
  */
 
-static char snap_module_name[] = "CARTRAMCART";
+static const char snap_module_name[] = "CARTRAMCART";
 #define SNAP_MAJOR   0
 #define SNAP_MINOR   0
 
