@@ -270,7 +270,7 @@ void rexep256_detach(void)
    ARRAY | ROML         |   0.0+  | 270336 BYTES of ROML data
  */
 
-static char snap_module_name[] = "CARTREXEP256";
+static const char snap_module_name[] = "CARTREXEP256";
 #define SNAP_MAJOR   0
 #define SNAP_MINOR   1
 
