@@ -32,9 +32,9 @@
 #include "types.h"
 
 extern void rexep256_config_init(void);
-extern void rexep256_config_setup(BYTE *rawcart);
-extern int rexep256_bin_attach(const char *filename, BYTE *rawcart);
-extern int rexep256_crt_attach(FILE *fd, BYTE *rawcart);
+extern void rexep256_config_setup(uint8_t *rawcart);
+extern int rexep256_bin_attach(const char *filename, uint8_t *rawcart);
+extern int rexep256_crt_attach(FILE *fd, uint8_t *rawcart);
 extern void rexep256_detach(void);
 
 struct snapshot_s;
