@@ -237,7 +237,7 @@ void se5_detach(void)
    ARRAY | ROML |   0.0+  | 16384 BYTES of ROML data
  */
 
-static char snap_module_name[] = "CARTSE5";
+static const char snap_module_name[] = "CARTSE5";
 #define SNAP_MAJOR   0
 #define SNAP_MINOR   1
 
