@@ -215,7 +215,7 @@ void warpspeed_detach(void)
    ARRAY | ROMH     |   0.0+  | 8192 BYTES of ROMH data
  */
 
-static char snap_module_name[] = "CARTWARP";
+static const char snap_module_name[] = "CARTWARP";
 #define SNAP_MAJOR   0
 #define SNAP_MINOR   1
 
