@@ -219,8 +219,21 @@ extern void cartridge_sound_chip_init(void);
 #define CARTRIDGE_EASYCALC             59 /* easycalc.c */
 #define CARTRIDGE_GMOD2                60 /* gmod2.c */
 #define CARTRIDGE_MAX_BASIC            61 /* maxbasic.c */
+#define CARTRIDGE_GMOD3                62 /* gmod3.c */
+#define CARTRIDGE_ZIPPCODE48           63 /* zippcode48.c */
+#define CARTRIDGE_BLACKBOX8            64 /* blackbox8.c */
+#define CARTRIDGE_BLACKBOX3            65 /* blackbox3.c */
+#define CARTRIDGE_BLACKBOX4            66 /* blackbox4.c */
+#define CARTRIDGE_REX_RAMFLOPPY        67 /* rexramfloppy.c */
+#define CARTRIDGE_BISPLUS              68 /* bisplus.c */
+#define CARTRIDGE_SDBOX                69 /* sdbox.c */
+#define CARTRIDGE_MULTIMAX             70 /* multimax.c */
+#define CARTRIDGE_BLACKBOX9            71 /* blackbox9.c */
+#define CARTRIDGE_LT_KERNAL            72 /* ltkernal.c */
+#define CARTRIDGE_RAMLINK              73 /* ramlink.c */
+#define CARTRIDGE_HERO                 74 /* hero.c */
 
-#define CARTRIDGE_LAST                 61 /* cartconv: last cartridge in list */
+#define CARTRIDGE_LAST                 74 /* cartconv: last cartridge in list */
 
 /* list of canonical names for the c64 cartridges:
    note: often it is hard to determine "the" official name, let alone the way it
@@ -242,6 +255,11 @@ extern void cartridge_sound_chip_init(void);
 #define CARTRIDGE_NAME_ACTION_REPLAY3     "Action Replay MK3" /* http://rr.pokefinder.org/wiki/Action_Replay */
 #define CARTRIDGE_NAME_ACTION_REPLAY4     "Action Replay MK4" /* http://rr.pokefinder.org/wiki/Action_Replay */
 #define CARTRIDGE_NAME_ATOMIC_POWER       "Atomic Power" /* also: "Nordic Power" */ /* http://rr.pokefinder.org/wiki/Nordic_Power */
+#define CARTRIDGE_NAME_BISPLUS            "BIS-Plus"
+#define CARTRIDGE_NAME_BLACKBOX3          "Blackbox V3"
+#define CARTRIDGE_NAME_BLACKBOX4          "Blackbox V4"
+#define CARTRIDGE_NAME_BLACKBOX8          "Blackbox V8"
+#define CARTRIDGE_NAME_BLACKBOX9          "Blackbox V9"
 #define CARTRIDGE_NAME_GS                 "C64 Games System" /* http://retro.lonningdal.net/home.php?page=Computers&select=c64gs&image=c64gs4.jpg */
 #define CARTRIDGE_NAME_CAPTURE            "Capture" /* see manual http://rr.pokefinder.org/wiki/Capture */
 #define CARTRIDGE_NAME_COMAL80            "Comal 80" /* http://www.retroport.de/C64_C128_Hardware.html */
@@ -274,11 +292,14 @@ extern void cartridge_sound_chip_init(void);
 #define CARTRIDGE_NAME_GAME_KILLER        "Game Killer" /* http://rr.pokefinder.org/wiki/Game_Killer */
 #define CARTRIDGE_NAME_GEORAM             "GEO-RAM" /* http://www.retroport.de/Rex.html */
 #define CARTRIDGE_NAME_GMOD2              "GMod2" /* http://wiki.icomp.de/wiki/GMod2 */
+#define CARTRIDGE_NAME_GMOD3              "GMod3" /* http://wiki.icomp.de/wiki/GMod3 */
+#define CARTRIDGE_NAME_HERO               "H.E.R.O. (Drean)"
 #define CARTRIDGE_NAME_IDE64              "IDE64" /* see http://www.ide64.org/ */
 #define CARTRIDGE_NAME_IEEE488            "IEEE-488 Interface"
 #define CARTRIDGE_NAME_ISEPIC             "ISEPIC" /* http://rr.pokefinder.org/wiki/Isepic */
 #define CARTRIDGE_NAME_KCS_POWER          "KCS Power Cartridge" /* http://rr.pokefinder.org/wiki/Power_Cartridge */
 #define CARTRIDGE_NAME_KINGSOFT           "Kingsoft"
+#define CARTRIDGE_NAME_LT_KERNAL          "Lt. Kernal Host Adaptor"
 #define CARTRIDGE_NAME_MACH5              "MACH 5" /* http://rr.pokefinder.org/wiki/MACH_5 */
 #define CARTRIDGE_NAME_MAGIC_DESK         "Magic Desk" /* also: "Domark, Hes Australia" */
 #define CARTRIDGE_NAME_MAGIC_FORMEL       "Magic Formel" /* http://rr.pokefinder.org/wiki/Magic_Formel */
@@ -291,22 +312,26 @@ extern void cartridge_sound_chip_init(void);
 #define CARTRIDGE_NAME_MIDI_NAMESOFT      "Namesoft MIDI"
 #define CARTRIDGE_NAME_MIDI_PASSPORT      "Passport MIDI"
 #define CARTRIDGE_NAME_MIDI_SEQUENTIAL    "Sequential MIDI"
+#define CARTRIDGE_NAME_MULTIMAX           "MultiMAX" /* http://www.multimax.co/ */
 #define CARTRIDGE_NAME_NORDIC_REPLAY      "Nordic Replay" /* "Retro Replay v2" see manual */
 #define CARTRIDGE_NAME_OCEAN              "Ocean"
 #define CARTRIDGE_NAME_PAGEFOX            "Pagefox"
 #define CARTRIDGE_NAME_P64                "Prophet64" /* see http://www.prophet64.com/ */
 #define CARTRIDGE_NAME_RAMCART            "RamCart" /* see cc65 driver */
+#define CARTRIDGE_NAME_RAMLINK            "RAMLink"
 #define CARTRIDGE_NAME_REU                "RAM Expansion Module" /* http://www.retroport.de/C64_C128_Hardware.html */
-#define CARTRIDGE_NAME_REX_EP256          "REX 256k EPROM Cart" /* http://www.retroport.de/Rex.html */
+#define CARTRIDGE_NAME_REX_EP256          "REX 256K EPROM Cart" /* http://www.retroport.de/Rex.html */
 #define CARTRIDGE_NAME_REX                "REX Utility"
+#define CARTRIDGE_NAME_REX_RAMFLOPPY      "REX RAM-Floppy"
 #define CARTRIDGE_NAME_RGCD               "RGCD"
 #define CARTRIDGE_NAME_RRNET              "RR-Net" /* see manual */
 #define CARTRIDGE_NAME_RRNETMK3           "RR-Net MK3" /* see manual */
 #define CARTRIDGE_NAME_RETRO_REPLAY       "Retro Replay" /* see manual */
 #define CARTRIDGE_NAME_ROSS               "ROSS"
+#define CARTRIDGE_NAME_SDBOX              "SD-BOX" /* http://c64.com.pl/index.php/sdbox106.html */
 #define CARTRIDGE_NAME_SFX_SOUND_EXPANDER "SFX Sound Expander" /* http://www.floodgap.com/retrobits/ckb/secret/cbm-sfx-fmbport.jpg */
 #define CARTRIDGE_NAME_SFX_SOUND_SAMPLER  "SFX Sound Sampler" /* http://www.floodgap.com/retrobits/ckb/secret/cbm-ssm-box.jpg */
-#define CARTRIDGE_NAME_SILVERROCK_128     "Silverrock 128K Cartridge"
+#define CARTRIDGE_NAME_SILVERROCK_128     "Silverrock 128KiB Cartridge"
 #define CARTRIDGE_NAME_SIMONS_BASIC       "Simons' BASIC" /* http://en.wikipedia.org/wiki/Simons'_BASIC */
 #define CARTRIDGE_NAME_SNAPSHOT64         "Snapshot 64" /* http://rr.pokefinder.org/wiki/Super_Snapshot */
 #define CARTRIDGE_NAME_STARDOS            "Stardos" /* see manual http://rr.pokefinder.org/wiki/StarDOS */
@@ -320,10 +345,11 @@ extern void cartridge_sound_chip_init(void);
 #define CARTRIDGE_NAME_WARPSPEED          "Warp Speed" /* see manual http://rr.pokefinder.org/wiki/WarpSpeed */
 #define CARTRIDGE_NAME_WESTERMANN         "Westermann Learning"
 #define CARTRIDGE_NAME_ZAXXON             "Zaxxon"
+#define CARTRIDGE_NAME_ZIPPCODE48         "ZIPP-CODE 48"
 
-#define CARTRIDGE_NAME_GENERIC_8KB        "8k game"
-#define CARTRIDGE_NAME_GENERIC_16KB       "16k game"
-#define CARTRIDGE_NAME_ULTIMAX            "Ultimax"
+#define CARTRIDGE_NAME_GENERIC_8KB        "generic 8KiB game"
+#define CARTRIDGE_NAME_GENERIC_16KB       "generic 16KiB game"
+#define CARTRIDGE_NAME_ULTIMAX            "generic Ultimax"
 
 /*
  * VIC20 cartridge system
@@ -367,23 +393,49 @@ extern void cartridge_sound_chip_init(void);
  * They will always be converted to CARTRIDGE_VIC20_GENERIC when
  * attached.  This also means they can be remapped at will.
  *
- * VIC20: &1 -> 0=4k, 1=8k; &16 -> 0= < 16k, 1= 16k 2nd half at $a000
- * (this logic is not used AFAIK /tlr)
+ * bit 0:  uses block 5 (a000-bfff)
+ *     1:  uses block 3 (6000-7fff)
+ *     2:  uses block 2 (4000-5fff)
+ *     3:  uses block 1 (2000-3fff)
+ * 
+ * bit 4:  0=4k 1=8k (only useful if just one block is used)
+ * bit 5:  0=first, 1=second 4k in block (only useful for 4k images)
  */
 #define CARTRIDGE_VIC20_DETECT       0x8000
-#define CARTRIDGE_VIC20_4KB_2000     0x8002
-#define CARTRIDGE_VIC20_8KB_2000     0x8003
-#define CARTRIDGE_VIC20_4KB_6000     0x8004
-#define CARTRIDGE_VIC20_8KB_6000     0x8005
-#define CARTRIDGE_VIC20_4KB_A000     0x8006
-#define CARTRIDGE_VIC20_8KB_A000     0x8007
-#define CARTRIDGE_VIC20_4KB_B000     0x8008
-#define CARTRIDGE_VIC20_8KB_4000     0x8009
-#define CARTRIDGE_VIC20_4KB_4000     0x800a
 
-#define CARTRIDGE_VIC20_16KB_2000    0x8013
-#define CARTRIDGE_VIC20_16KB_4000    0x8019
-#define CARTRIDGE_VIC20_16KB_6000    0x8015
+#define CARTRIDGE_VIC20_TYPEDEF(h, s, b1, b2, b3, b5) \
+    (0x8000 | ((h) << 5) | ((s) << 4) | ((b1) << 3) | ((b2) << 2) | ((b3) << 1) | ((b5) << 0))
+
+/* block 1 */    
+#define CARTRIDGE_VIC20_4KB_2000        CARTRIDGE_VIC20_TYPEDEF(0, 0, 1, 0, 0, 0)
+#define CARTRIDGE_VIC20_8KB_2000        CARTRIDGE_VIC20_TYPEDEF(0, 1, 1, 0, 0, 0)
+#define CARTRIDGE_VIC20_4KB_3000        CARTRIDGE_VIC20_TYPEDEF(1, 0, 1, 0, 0, 0)
+/* block 2 */
+#define CARTRIDGE_VIC20_4KB_4000        CARTRIDGE_VIC20_TYPEDEF(0, 0, 0, 1, 0, 0)
+#define CARTRIDGE_VIC20_8KB_4000        CARTRIDGE_VIC20_TYPEDEF(0, 1, 0, 1, 0, 0)
+#define CARTRIDGE_VIC20_4KB_5000        CARTRIDGE_VIC20_TYPEDEF(1, 0, 0, 1, 0, 0)
+/* block 3 */
+#define CARTRIDGE_VIC20_4KB_6000        CARTRIDGE_VIC20_TYPEDEF(0, 0, 0, 0, 1, 0)
+#define CARTRIDGE_VIC20_8KB_6000        CARTRIDGE_VIC20_TYPEDEF(0, 1, 0, 0, 1, 0)
+#define CARTRIDGE_VIC20_4KB_7000        CARTRIDGE_VIC20_TYPEDEF(1, 0, 0, 0, 1, 0)
+/* block 5 */
+#define CARTRIDGE_VIC20_4KB_A000        CARTRIDGE_VIC20_TYPEDEF(0, 0, 0, 0, 0, 1)
+#define CARTRIDGE_VIC20_8KB_A000        CARTRIDGE_VIC20_TYPEDEF(0, 1, 0, 0, 0, 1)
+#define CARTRIDGE_VIC20_4KB_B000        CARTRIDGE_VIC20_TYPEDEF(1, 0, 0, 0, 0, 1)
+
+/* block 1 + block 2 */
+#define CARTRIDGE_VIC20_16KB_2000       CARTRIDGE_VIC20_TYPEDEF(0, 1, 1, 1, 0, 0) 
+/* block 1 + block 5 */
+#define CARTRIDGE_VIC20_16KB_2000_A000  CARTRIDGE_VIC20_TYPEDEF(0, 1, 1, 0, 0, 1) 
+/* block 2 + block 3 */
+#define CARTRIDGE_VIC20_16KB_4000       CARTRIDGE_VIC20_TYPEDEF(0, 1, 0, 1, 1, 0) 
+/* block 2 + block 5 */
+#define CARTRIDGE_VIC20_16KB_4000_A000  CARTRIDGE_VIC20_TYPEDEF(0, 1, 0, 1, 0, 1) 
+/* block 3 + block 5 */
+#define CARTRIDGE_VIC20_16KB_6000       CARTRIDGE_VIC20_TYPEDEF(0, 1, 0, 0, 1, 1) 
+
+/* block 1,2,3,5 */
+#define CARTRIDGE_VIC20_32KB_2000       CARTRIDGE_VIC20_TYPEDEF(0, 1, 1, 1, 1, 1)
 
 /* list of canonical names for the VIC20 cartridges: */
 #define CARTRIDGE_VIC20_NAME_BEHRBONZ        "Behr Bonz"
@@ -429,6 +481,7 @@ extern void cartridge_sound_chip_init(void);
 #define CARTRIDGE_CBM2_16KB_6000       0x9008
 
 /* FIXME: cartconv: the sizes are used in a bitfield and also by their absolute values */
+#define CARTRIDGE_SIZE_2KB     0x00000800
 #define CARTRIDGE_SIZE_4KB     0x00001000
 #define CARTRIDGE_SIZE_8KB     0x00002000
 #define CARTRIDGE_SIZE_12KB    0x00003000
@@ -442,6 +495,11 @@ extern void cartridge_sound_chip_init(void);
 #define CARTRIDGE_SIZE_256KB   0x00040000
 #define CARTRIDGE_SIZE_512KB   0x00080000
 #define CARTRIDGE_SIZE_1024KB  0x00100000
+#define CARTRIDGE_SIZE_2048KB  0x00200000
+#define CARTRIDGE_SIZE_4096KB  0x00400000
+#define CARTRIDGE_SIZE_8192KB  0x00800000
+#define CARTRIDGE_SIZE_16384KB 0x01000000
+#define CARTRIDGE_SIZE_MAX     CARTRIDGE_SIZE_16384KB
 
 #define CARTRIDGE_FILETYPE_BIN  1
 #define CARTRIDGE_FILETYPE_CRT  2
