@@ -83,6 +83,18 @@ static uint8_t memmap_mem_read(unsigned int addr)
     memmap_mark_read(addr);
     return (*_mem_read_tab_ptr[(addr) >> 8])((uint16_t)(addr));
 }
+
+static uint8_t memmap_mem_read_dummy(unsigned int addr)
+{
+    memmap_mark_read(addr);
+    return (*_mem_read_tab_ptr_dummy[(addr) >> 8])((uint16_t)(addr));
+}
+
+static void memmap_mem_store_dummy(unsigned int addr, unsigned int value)
+{
+    monitor_memmap_store(addr, MEMMAP_RAM_W);
+    (*_mem_write_tab_ptr_dummy[(addr) >> 8])((uint16_t)(addr), (uint8_t)(value));
+}
 #endif
 
 #include "../maincpu.c"
