@@ -29,10 +29,15 @@
 #define VICE_CHARSET_H
 
 #include "types.h"
+#include <stddef.h>
 
 #define a2p(c) charset_petconvstring(c, 0)
 #define p2a(c) charset_petconvstring(c, 1)
 
+
+/* TODO:    Fix these functions to use size_t for lenght, not int
+ */
+
 extern uint8_t *charset_petconvstring(uint8_t *c, int dir);
 extern uint8_t charset_p_toascii(uint8_t c, int cs);
 extern uint8_t charset_p_topetcii(uint8_t c);
@@ -45,7 +50,7 @@ extern void charset_petcii_to_screencode_line(const uint8_t *line, uint8_t **buf
                                               unsigned int *len);
 
 extern int charset_petscii_to_ucs(uint8_t c);
-extern int charset_ucs_to_utf8(uint8_t *out, int code, int len);
+extern int charset_ucs_to_utf8(uint8_t *out, int code, size_t len);
 
 #define CONVERT_TO_PETSCII 0
 #define CONVERT_TO_ASCII 1
