@@ -86,7 +86,9 @@ unsigned int disk_image_speed_map(unsigned int format, unsigned int track)
         /* single sided format, these are straight forward */
         case DISK_IMAGE_TYPE_D67:
         case DISK_IMAGE_TYPE_D64:
+#ifdef HAVE_X64_IMAGE
         case DISK_IMAGE_TYPE_X64:
+#endif
         case DISK_IMAGE_TYPE_G64:
         case DISK_IMAGE_TYPE_P64:
             return (track < 31) + (track < 25) + (track < 18);
@@ -128,30 +130,31 @@ unsigned int disk_image_speed_map(unsigned int format, unsigned int track)
 /** \brief  Sectors per speed zone for D64/D71 images
  */
 static const unsigned int sector_map_d64[SPEED_ZONE_COUNT] = {
-    17, /* tracks 1-17, 36-52 (D71 only) */
-    18, /* tracks 18-24, 53-59 (D71 only)  */
-    19, /* track 25-30, 60-65 (D71 only) */
-    21  /* tracks 31-[35-40], 66-70 (D71 only) */
+    17, /* tracks 31-[35-40], 66-70 (D71 only) */
+    18, /* track 25-30, 60-65 (D71 only) */
+    19, /* tracks 18-24, 53-59 (D71 only)  */
+    21  /* tracks 1-17, 36-52 (D71 only) */
 };
 
 
 /** \brief  Sectors per speed zone for D67 images
  */
+/* the 2040 tried to squeeze one more sector into speed zone 2 */
 static const unsigned int sector_map_d67[SPEED_ZONE_COUNT] = {
-    17, /* tracks 1-17 */
-    18, /* tracks 18-24 */
-    20, /* track 25-30 */
-    21  /* tracks 31-[35-40} */
+    17, /* tracks 31-[35-40} */
+    18, /* track 25-30 */
+    20, /* tracks 18-24 */
+    21  /* tracks 1-17 */
 };
 
 
 /** \brief  Sectors per speed zone for D80/D82 images
  */
 static const unsigned int sector_map_d80[SPEED_ZONE_COUNT] = {
-    23, /* tracks 1-39, 78-116 (D82 only) */
-    25, /* tracks 40-53, 117-130 (D82 only) */
-    27, /* tracks 54-64, 131,141 (D82 only) */
-    29  /* tracks 65-77, 142-154 (D82 only) */
+    23, /* tracks 65-77, 142-154 (D82 only) */
+    25, /* tracks 54-64, 131,141 (D82 only) */
+    27, /* tracks 40-53, 117-130 (D82 only) */
+    29  /* tracks 1-39, 78-116 (D82 only) */
 };
 
 
@@ -169,7 +172,9 @@ unsigned int disk_image_sector_per_track(unsigned int format,
 {
     switch (format) {
         case DISK_IMAGE_TYPE_D64:
+#ifdef HAVE_X64_IMAGE
         case DISK_IMAGE_TYPE_X64:
+#endif
         case DISK_IMAGE_TYPE_G64:
         case DISK_IMAGE_TYPE_P64:
         case DISK_IMAGE_TYPE_D71:
@@ -193,14 +198,36 @@ unsigned int disk_image_sector_per_track(unsigned int format,
 
 /** \brief  Size of a raw (GCR) track in bytes in a D64 image per speed zone
  */
-static const unsigned int raw_track_size_d64[4] = {
-    6250, 6666, 7142, 7692
+static const unsigned int raw_track_size_d64[SPEED_ZONE_COUNT] = {
+    6250, /* (50000 bits) tracks 31-[35-40], 66-70 (D71 only) */
+    6666, /* (53333 bits) track 25-30, 60-65 (D71 only) */
+    7142, /* (57143 bits) tracks 18-24, 53-59 (D71 only)  */
+    7692  /* (61538 bits) tracks 1-17, 36-52 (D71 only) */
+};
+
+/** \brief  Size of a raw (GCR) track in bytes in a D67 image per speed zone
+ */
+/* the 2040 tried to squeeze one more sector into speed zone 2 */
+/* FIXME: D67 has 20 sectors per track for speed zone 2,
+          D64/D71 has 19 sectors for that speed zone 
+
+          note: the number of GCR bits per track "should" be the same as in D64,
+                as that is pretty much defined by the clock and rotation speed.
+ */
+static const unsigned int raw_track_size_d67[SPEED_ZONE_COUNT] = {
+    6250, /* tracks 31-[35-40} */
+    6666, /* track 25-30 */
+    7142, /* tracks 18-24 */ /* FIXME: is this correct? */
+    7692  /* tracks 1-17 */
 };
 
 /** \brief  Size of a raw (GCR) track in bytes in a D80 image per speed zone
  */
-static const unsigned int raw_track_size_d80[4] = {
-    9375, 10000, 10714, 11538
+static const unsigned int raw_track_size_d80[SPEED_ZONE_COUNT] = {
+    9375,  /* tracks 65-77, 142-154 (D82 only) */
+    10000, /* tracks 54-64, 131,141 (D82 only) */
+    10714, /* tracks 40-53, 117-130 (D82 only) */
+    11538  /* tracks 1-39, 78-116 (D82 only) */
 };
 
 
@@ -216,17 +243,17 @@ unsigned int disk_image_raw_track_size(unsigned int format,
 {
     switch (format) {
         case DISK_IMAGE_TYPE_D64:
+#ifdef HAVE_X64_IMAGE
         case DISK_IMAGE_TYPE_X64:   /* FIXME: X64 can contain a lot of different
                                               formats, not just D64 (BW) */
+#endif
         case DISK_IMAGE_TYPE_G64:
         case DISK_IMAGE_TYPE_P64:
         case DISK_IMAGE_TYPE_D71:
         case DISK_IMAGE_TYPE_G71:
-        case DISK_IMAGE_TYPE_D67:   /* XXX: D67 has 20 sectors per track for
-                                            speed zone 2, D64/D71 has 19 sectors
-                                            for that speed zone, so is this
-                                            correct? (BW) */
             return raw_track_size_d64[disk_image_speed_map(format, track)];
+        case DISK_IMAGE_TYPE_D67:
+            return raw_track_size_d67[disk_image_speed_map(format, track)];
         case DISK_IMAGE_TYPE_D80:
         case DISK_IMAGE_TYPE_D82:
             return raw_track_size_d80[disk_image_speed_map(format, track)];
@@ -242,20 +269,37 @@ unsigned int disk_image_raw_track_size(unsigned int format,
 /* Gap between sectors */
 
 static const unsigned int gap_size_d64[4] = {
-    9, 12, 17, 8
+    9,  /* (72 bits) tracks 31-[35-40], 66-70 (D71 only) */
+    12, /* (96 bits) track 25-30, 60-65 (D71 only) */
+    17, /* (136 bits) tracks 18-24, 53-59 (D71 only)  */
+    8   /* (64 bits) tracks 1-17, 36-52 (D71 only) */
+};
+
+/* the 2040 tried to squeeze one more sector into speed zone 2 */
+/* FIXME: D67 has 20 sectors per track for speed zone 2,
+ *        D64/D71 has 19 sectors for that speed zone */
+static const unsigned int gap_size_d67[4] = {
+    9,  /* tracks 31-[35-40} */ 
+    12, /* track 25-30 */
+    4,  /* tracks 18-24 */ /* FIXME: is this correct? */
+    8   /* tracks 1-17 */
 };
 
 unsigned int disk_image_gap_size(unsigned int format, unsigned int track)
 {
     switch (format) {
         case DISK_IMAGE_TYPE_D64:
-        case DISK_IMAGE_TYPE_X64:
+#ifdef HAVE_X64_IMAGE
+        case DISK_IMAGE_TYPE_X64:   /* FIXME: X64 can contain a lot of different
+                                              formats, not just D64 (BW) */
+#endif
         case DISK_IMAGE_TYPE_G64:
         case DISK_IMAGE_TYPE_P64:
         case DISK_IMAGE_TYPE_D71:
         case DISK_IMAGE_TYPE_G71:
-        case DISK_IMAGE_TYPE_D67:
             return gap_size_d64[disk_image_speed_map(format, track)];
+        case DISK_IMAGE_TYPE_D67:
+            return gap_size_d67[disk_image_speed_map(format, track)];
         case DISK_IMAGE_TYPE_D80:
         case DISK_IMAGE_TYPE_D82:
             return 25;
@@ -267,6 +311,65 @@ unsigned int disk_image_gap_size(unsigned int format, unsigned int track)
     return 1;
 }
 
+/*-----------------------------------------------------------------------*/
+/* Gap between header and sector */
+
+unsigned int disk_image_header_gap_size(unsigned int format, unsigned int track)
+{
+    switch (format) {
+        case DISK_IMAGE_TYPE_D64:
+#ifdef HAVE_X64_IMAGE
+        case DISK_IMAGE_TYPE_X64:   /* FIXME: X64 can contain a lot of different
+                                              formats, not just D64 (BW) */
+#endif
+        case DISK_IMAGE_TYPE_G64:
+        case DISK_IMAGE_TYPE_P64:
+        case DISK_IMAGE_TYPE_D71:
+        case DISK_IMAGE_TYPE_G71:
+            return 9;
+        case DISK_IMAGE_TYPE_D67:
+            return 4;               /* FIXME: is this correct ? */
+        case DISK_IMAGE_TYPE_D80:
+        case DISK_IMAGE_TYPE_D82:
+            /* FIXME */
+        default:
+            log_message(disk_image_log,
+                        "Unknown disk type %u.  Cannot calculate header gap size",
+                        format);
+    }
+    return 1;
+}
+
+/*-----------------------------------------------------------------------*/
+/* length of a SYNC */
+
+/* FIXME: the value of 5 bytes for sync is also hardcoded in gcr.c */
+unsigned int disk_image_sync_size(unsigned int format, unsigned int track)
+{
+    switch (format) {
+        case DISK_IMAGE_TYPE_D64:
+#ifdef HAVE_X64_IMAGE
+        case DISK_IMAGE_TYPE_X64:   /* FIXME: X64 can contain a lot of different
+                                              formats, not just D64 (BW) */
+#endif
+        case DISK_IMAGE_TYPE_G64:
+        case DISK_IMAGE_TYPE_P64:
+        case DISK_IMAGE_TYPE_D71:
+        case DISK_IMAGE_TYPE_G71:
+            return 5; /* 40 bits */
+        case DISK_IMAGE_TYPE_D67:
+            return 5;               /* FIXME: is this correct ? */
+        case DISK_IMAGE_TYPE_D80:
+        case DISK_IMAGE_TYPE_D82:
+            /* FIXME */
+        default:
+            log_message(disk_image_log,
+                        "Unknown disk type %u.  Cannot calculate sync size",
+                        format);
+    }
+    return 1;
+}
+
 /*-----------------------------------------------------------------------*/
 /* Check out of bound */
 
@@ -305,13 +408,17 @@ static const char *disk_image_type(const disk_image_t *image)
         case DISK_IMAGE_TYPE_D67: return "D67";
         case DISK_IMAGE_TYPE_G64: return "G64";
         case DISK_IMAGE_TYPE_P64: return "P64";
+#ifdef HAVE_X64_IMAGE
         case DISK_IMAGE_TYPE_X64: return "X64";
+#endif
         case DISK_IMAGE_TYPE_D71: return "D71";
         case DISK_IMAGE_TYPE_G71: return "G71";
         case DISK_IMAGE_TYPE_D81: return "D81";
         case DISK_IMAGE_TYPE_D1M: return "D1M";
         case DISK_IMAGE_TYPE_D2M: return "D2M";
         case DISK_IMAGE_TYPE_D4M: return "D4M";
+        case DISK_IMAGE_TYPE_DHD: return "DHD";
+        case DISK_IMAGE_TYPE_D90: return "D90";
         default: return NULL;
     }
 }
@@ -324,7 +431,7 @@ static const char *disk_image_type(const disk_image_t *image)
  * \param[in]   unit    unit the image is attached to
  */
 void disk_image_attach_log(const disk_image_t *image, signed int lognum,
-                           unsigned int unit)
+                           unsigned int unit, unsigned int drive)
 {
     const char *type = disk_image_type(image);
 
@@ -334,8 +441,8 @@ void disk_image_attach_log(const disk_image_t *image, signed int lognum,
 
     switch (image->device) {
         case DISK_IMAGE_DEVICE_FS:
-            log_verbose("Unit %u: %s disk image attached: %s.",
-                        unit, type, fsimage_name_get(image));
+            log_verbose("Unit %u drive %u: %s disk image attached: %s.",
+                        unit, drive, type, fsimage_name_get(image));
             break;
     }
 }
@@ -348,7 +455,7 @@ void disk_image_attach_log(const disk_image_t *image, signed int lognum,
  * \param[in]   unit    unit the image is detached from
  */
 void disk_image_detach_log(const disk_image_t *image, signed int lognum,
-                           unsigned int unit)
+                           unsigned int unit, unsigned int drive)
 {
     const char *type = disk_image_type(image);
 
@@ -358,8 +465,8 @@ void disk_image_detach_log(const disk_image_t *image, signed int lognum,
 
     switch (image->device) {
         case DISK_IMAGE_DEVICE_FS:
-            log_verbose("Unit %u: %s disk image detached: %s.",
-                        unit, type, fsimage_name_get(image));
+            log_verbose("Unit %u drive %u: %s disk image detached: %s.",
+                        unit, drive, type, fsimage_name_get(image));
             break;
     }
 }
@@ -405,6 +512,11 @@ int disk_image_fsimage_create(const char *name, unsigned int type)
     return fsimage_create(name, type);
 }
 
+int disk_image_fsimage_create_dxm(const char *name, const char *diskname, unsigned int type)
+{
+    return fsimage_create_dxm(name, diskname, type);
+}
+
 /*-----------------------------------------------------------------------*/
 
 void disk_image_name_set(disk_image_t *image, const char *name)
@@ -648,3 +760,20 @@ int disk_image_cmdline_options_init(void)
 {
     return 0;
 }
+
+/*-----------------------------------------------------------------------*/
+
+uint32_t disk_image_size(const disk_image_t *image)
+{
+    switch (image->device) {
+        case DISK_IMAGE_DEVICE_FS:
+            return fsimage_size(image);
+#ifdef HAVE_REALDEVICE
+        case DISK_IMAGE_DEVICE_REAL:
+            break;
+#endif
+        default:
+            log_error(disk_image_log, "Unknown image device %u.", image->device);
+    }
+    return 0;
+}
