@@ -50,13 +50,15 @@ int fsimage_check_sector(const disk_image_t *image, unsigned int track,
 {
     unsigned int sectors = 0, i;
 
-    if (track < 1) {
+    if (image->type != DISK_IMAGE_TYPE_D90 && track < 1) {
         return FSIMAGE_BAD_TRKNUM;
     }
 
     switch (image->type) {
         case DISK_IMAGE_TYPE_D64:
+#ifdef HAVE_X64_IMAGE
         case DISK_IMAGE_TYPE_X64:
+#endif
             if (track > MAX_TRACKS_1541) {
                return FSIMAGE_BAD_TRKNUM;
             }
@@ -177,6 +179,31 @@ int fsimage_check_sector(const disk_image_t *image, unsigned int track,
             }
             sectors = (track - 1) * 256 + sector;
             break;
+        case DISK_IMAGE_TYPE_DHD:
+            /* for DHD images, just assume track/sector can be 16 bits each,
+            which gives us LARGE disks */
+            if (track > 0xffff) {
+               return FSIMAGE_BAD_TRKNUM;
+            }
+            if (sector > 0xffff) {
+                return FSIMAGE_BAD_SECNUM;
+            }
+            sectors = (track - 1) * 65536 + sector;
+            break;
+        case DISK_IMAGE_TYPE_D90:
+            /* track 0 is allowed here as the fdc passes it */
+            if (track > image->tracks) {
+               return FSIMAGE_BAD_TRKNUM;
+            }
+            if (sector >= image->sectors) {
+                return FSIMAGE_BAD_SECNUM;
+            }
+            /* use the mapping found in the FDC:
+              sectors = track * (image->sectors >> 5) + (sector >> 5);
+              sectors = (sectors << 5) + (sector & 31);
+            we use a more optimized function here */
+            sectors = track * (image->sectors & (~31)) + sector;
+            break;
         default:
             return -1;
     }
