@@ -65,26 +65,31 @@ static int set_drive_true_emulation(int val, void *param)
     machine_bus_status_truedrive_set((unsigned int)drive_true_emulation);
 
     if (val) {
-        for (dnr = 0; dnr < DRIVE_NUM; dnr++) {
-            drive = drive_context[dnr]->drive;
-            if (drive->type != DRIVE_TYPE_NONE) {
-                drive->enable = 1;
-                if (drive->type == DRIVE_TYPE_2000 || drive->type == DRIVE_TYPE_4000) {
-                    drivecpu65c02_reset_clk(drive_context[dnr]);
+        for (dnr = 0; dnr < NUM_DISK_UNITS; dnr++) {
+            diskunit_context_t *unit = diskunit_context[dnr];
+
+            if (unit->type != DRIVE_TYPE_NONE) {
+                unit->enable = 1;
+                if (unit->type == DRIVE_TYPE_2000 || unit->type == DRIVE_TYPE_4000 ||
+                    unit->type == DRIVE_TYPE_CMDHD) {
+                    drivecpu65c02_reset_clk(unit);
                 } else {
-                    drivecpu_reset_clk(drive_context[dnr]);
+                    drivecpu_reset_clk(unit);
                 }
             }
         }
-        for (dnr = 0; dnr < DRIVE_NUM; dnr++) {
-            drive_enable(drive_context[dnr]);
+        for (dnr = 0; dnr < NUM_DISK_UNITS; dnr++) {
+            drive_enable(diskunit_context[dnr]);
         }
     } else {
-        for (dnr = 0; dnr < DRIVE_NUM; dnr++) {
-            drive = drive_context[dnr]->drive;
-            drive_disable(drive_context[dnr]);
+        for (dnr = 0; dnr < NUM_DISK_UNITS; dnr++) {
+            diskunit_context_t *unit = diskunit_context[dnr];
+            drive = unit->drives[0];
+
+            drive_disable(diskunit_context[dnr]);
             if (drive->image != NULL) {
-                vdrive_bam_reread_bam(dnr + 8);
+                /* TODO: drive 1? */
+                vdrive_bam_reread_bam(dnr + 8, 0);
             }
         }
     }
@@ -113,7 +118,9 @@ static int set_drive_extend_image_policy(int val, void *param)
         case DRIVE_EXTEND_NEVER:
         case DRIVE_EXTEND_ASK:
         case DRIVE_EXTEND_ACCESS:
-            drive_context[vice_ptr_to_int(param)]->drive->extend_image_policy = val;
+            /* TODO: add resource for drive 1 */
+            diskunit_context[vice_ptr_to_int(param)]->drives[0]->extend_image_policy = val;
+            diskunit_context[vice_ptr_to_int(param)]->drives[1]->extend_image_policy = val;
             return 0;
         default:
             return -1;
@@ -124,11 +131,13 @@ static int drive_resources_type(int val, void *param)
 {
     unsigned int type, dnr;
     int busses;
-    drive_t *drive, *drive0;
+    diskunit_context_t *unit;
+    drive_t *drive;
     char *rtc_device = NULL;
 
     dnr = vice_ptr_to_uint(param);
-    drive = drive_context[dnr]->drive;
+    unit = diskunit_context[dnr];
+    drive = unit->drives[0];
 
     type = (unsigned int)val;
     busses = iec_available_busses();
@@ -140,44 +149,26 @@ static int drive_resources_type(int val, void *param)
         } else
         if (busses & IEC_BUS_IEEE) {
             type = DRIVE_TYPE_2031;
+        } else
+        if (busses & IEC_BUS_TCBM) {
+            type = DRIVE_TYPE_1551;
         } else {
             type = DRIVE_TYPE_NONE;
         }
     }
 
-    if (is_drive0(dnr)) {
-        if (drive_check_dual(type)) {
-            int drive1 = mk_drive1(dnr);
-
-            /* dual disk drives disable second emulated unit */
-            log_warning(drive->log,
-                        "Dual disk drive %u disables emulated drive %d", dnr, drive1);
-
-            drive_resources_type(DRIVE_TYPE_NONE, int_to_void_ptr(drive1));
-        }
-    } else {
-        drive0 = drive_context[mk_drive0(dnr)]->drive;
-        if (drive0->enable && drive_check_dual(drive0->type)) {
-            /* dual disk drives disable second emulated unit */
-            log_warning(drive->log,
-                        "Dual disk drive %u disables emulated drive %u", mk_drive0(dnr), dnr);
-
-            type = DRIVE_TYPE_NONE;
-        }
-    }
-
     if (type == DRIVE_TYPE_2000 || type == DRIVE_TYPE_4000) {
-        if (drive->type != DRIVE_TYPE_2000 && drive->type != DRIVE_TYPE_4000) {
+        if (unit->type != DRIVE_TYPE_2000 && unit->type != DRIVE_TYPE_4000) {
             rtc_device = lib_msprintf("FD%d", dnr + 8);
-            drive->ds1216 = ds1216e_init(rtc_device);
-            drive->ds1216->hours12 = 1;
+            unit->ds1216 = ds1216e_init(rtc_device);
+            unit->ds1216->hours12 = 1;
             lib_free(rtc_device);
         }
     } else {
-        if (drive->type == DRIVE_TYPE_2000 || drive->type == DRIVE_TYPE_4000) {
-            if (drive->ds1216) {
-                ds1216e_destroy(drive->ds1216, drive->rtc_save);
-                drive->ds1216 = NULL;
+        if (unit->type == DRIVE_TYPE_2000 || unit->type == DRIVE_TYPE_4000) {
+            if (unit->ds1216) {
+                ds1216e_destroy(unit->ds1216, unit->rtc_save);
+                unit->ds1216 = NULL;
             }
         }
     }
@@ -193,6 +184,7 @@ static int drive_resources_type(int val, void *param)
         case DRIVE_TYPE_1581:
         case DRIVE_TYPE_2000:
         case DRIVE_TYPE_4000:
+        case DRIVE_TYPE_CMDHD:
         case DRIVE_TYPE_2031:
         case DRIVE_TYPE_1001:
         case DRIVE_TYPE_2040:
@@ -200,31 +192,35 @@ static int drive_resources_type(int val, void *param)
         case DRIVE_TYPE_4040:
         case DRIVE_TYPE_8050:
         case DRIVE_TYPE_8250:
-            if (drive->type != type) {
+        case DRIVE_TYPE_9000:
+            if (unit->type != type) {
+                /* the drives that support half tracks are all single drives */
                 drive->current_half_track = 2 * 18;
                 if ((type == DRIVE_TYPE_1001)
                     || (type == DRIVE_TYPE_8050)
                     || (type == DRIVE_TYPE_8250)) {
                     drive->current_half_track = 2 * 38;
+                } else if (type == DRIVE_TYPE_9000) {
+                    drive->current_half_track = 2 * 76;
                 }
             }
-            drive->type = type;
+            unit->type = type;
             if (drive_true_emulation) {
-                drive->enable = 1;
-                drive_enable(drive_context[dnr]);
+                unit->enable = 1;
+                drive_enable(diskunit_context[dnr]);
                 /* 1551 drive does not use the IEC bus */
                 machine_bus_status_drivetype_set(dnr + 8, drive_check_bus(type,
                                                                           IEC_BUS_IEC));
             } else {
-                drive_enable_update_ui(drive_context[dnr]);
+                drive_enable_update_ui(diskunit_context[dnr]);
             }
-            drive_set_disk_drive_type(type, drive_context[dnr]);
-            driverom_initialize_traps(drive);
+            drive_set_disk_drive_type(type, diskunit_context[dnr]);
+            driverom_initialize_traps(diskunit_context[dnr]);
             machine_drive_idling_method(dnr);
             return 0;
         case DRIVE_TYPE_NONE:
-            drive->type = type;
-            drive_disable(drive_context[dnr]);
+            unit->type = type;
+            drive_disable(diskunit_context[dnr]);
             machine_bus_status_drivetype_set(dnr + 8, 0);
             return 0;
         default:
@@ -241,10 +237,10 @@ static resource_int_t res_drive_type[] = {
 int drive_resources_type_init(unsigned int default_type)
 {
     unsigned int dnr, type;
-    drive_t *drive;
 
-    for (dnr = 0; dnr < DRIVE_NUM; dnr++) {
-        drive = drive_context[dnr]->drive;
+    for (dnr = 0; dnr < NUM_DISK_UNITS; dnr++) {
+        diskunit_context_t *unit = diskunit_context[dnr];
+
         if (dnr == 0) {
             type = default_type;
         } else {
@@ -253,7 +249,7 @@ int drive_resources_type_init(unsigned int default_type)
 
         res_drive_type[0].name = lib_msprintf("Drive%iType", dnr + 8);
         res_drive_type[0].factory_value = (int)type;
-        res_drive_type[0].value_ptr = (int *)&(drive->type);
+        res_drive_type[0].value_ptr = (int *)&(unit->type);
         res_drive_type[0].param = uint_to_void_ptr(dnr);
 
         if (resources_register_int(res_drive_type) < 0) {
@@ -268,11 +264,8 @@ int drive_resources_type_init(unsigned int default_type)
 
 static int set_drive_idling_method(int val, void *param)
 {
-    unsigned int dnr;
-    drive_t *drive;
-
-    dnr = vice_ptr_to_uint(param);
-    drive = drive_context[dnr]->drive;
+    unsigned int dnr = vice_ptr_to_uint(param);
+    diskunit_context_t *unit = diskunit_context[dnr];
 
     /* FIXME: Maybe we should call `drive_cpu_execute()' here?  */
     switch (val) {
@@ -284,49 +277,69 @@ static int set_drive_idling_method(int val, void *param)
             return -1;
     }
 
-    drive->idling_method = val;
+    unit->idling_method = val;
 
     if (!rom_loaded) {
         return 0;
     }
 
-    driverom_initialize_traps(drive);
+    driverom_initialize_traps(unit);
+
     return 0;
 }
 
 static int set_drive_rpm(int val, void *param)
 {
     unsigned int dnr;
-    drive_t *drive;
+    drive_t *drive, *drive1;
 
     dnr = vice_ptr_to_uint(param);
-    drive = drive_context[dnr]->drive;
+    drive = diskunit_context[dnr]->drives[0];
+    drive1 = diskunit_context[dnr]->drives[1];
 
     drive->rpm = val;
+    drive1->rpm = val;
+
     return 0;
 }
 
-static int set_drive_rpm_wobble(int val, void *param)
+static int set_drive_wobble_frequency(int val, void *param)
 {
     unsigned int dnr;
-    drive_t *drive;
+    drive_t *drive, *drive1;
 
     dnr = vice_ptr_to_uint(param);
-    drive = drive_context[dnr]->drive;
+    drive = diskunit_context[dnr]->drives[0];
+    drive1 = diskunit_context[dnr]->drives[1];
 
-    drive->rpm_wobble = val;
+    drive->wobble_frequency = val;
+    drive1->wobble_frequency = val;
+    return 0;
+}
+
+static int set_drive_wobble_amplitude(int val, void *param)
+{
+    unsigned int dnr;
+    drive_t *drive, *drive1;
+
+    dnr = vice_ptr_to_uint(param);
+    drive = diskunit_context[dnr]->drives[0];
+    drive1 = diskunit_context[dnr]->drives[1];
+
+    drive->wobble_amplitude = val;
+    drive1->wobble_amplitude = val;
     return 0;
 }
 
 static int set_drive_rtc_save(int val, void *param)
 {
     unsigned int dnr;
-    drive_t *drive;
+    diskunit_context_t *unit;
 
     dnr = vice_ptr_to_uint(param);
-    drive = drive_context[dnr]->drive;
+    unit = diskunit_context[dnr];
 
-    drive->rtc_save = val ? 1 : 0;
+    unit->rtc_save = val ? 1 : 0;
 
     return 0;
 }
@@ -348,8 +361,10 @@ static resource_int_t res_drive[] = {
       NULL, set_drive_idling_method, NULL },
     { NULL, 30000, RES_EVENT_SAME, NULL,
       NULL, set_drive_rpm, NULL },
-    { NULL, 50, RES_EVENT_SAME, NULL,
-      NULL, set_drive_rpm_wobble, NULL },
+    { NULL, 5000, RES_EVENT_SAME, NULL,
+      NULL, set_drive_wobble_frequency, NULL },
+    { NULL, 2000, RES_EVENT_SAME, NULL,
+      NULL, set_drive_wobble_amplitude, NULL },
     RESOURCE_INT_LIST_END
 };
 
@@ -362,7 +377,6 @@ static resource_int_t res_drive_rtc[] = {
 int drive_resources_init(void)
 {
     unsigned int dnr;
-    drive_t *drive;
     int has_iec;
     int i;
 
@@ -378,25 +392,30 @@ int drive_resources_init(void)
             has_iec = 1;
     }
 
-    for (dnr = 0; dnr < DRIVE_NUM; dnr++) {
-        drive = drive_context[dnr]->drive;
+    for (dnr = 0; dnr < NUM_DISK_UNITS; dnr++) {
+        diskunit_context_t *unit = diskunit_context[dnr];
+        drive_t *drive0 = unit->drives[0];
 
+        /* TODO: add resources for drive 1 */
         res_drive[0].name = lib_msprintf("Drive%iExtendImagePolicy", dnr + 8);
-        res_drive[0].value_ptr = (int *)&(drive->extend_image_policy);
+        res_drive[0].value_ptr = (int *)&(drive0->extend_image_policy);
         res_drive[0].param = uint_to_void_ptr(dnr);
         res_drive[1].name = lib_msprintf("Drive%iIdleMethod", dnr + 8);
-        res_drive[1].value_ptr = &(drive->idling_method);
+        res_drive[1].value_ptr = &(unit->idling_method);
         res_drive[1].param = uint_to_void_ptr(dnr);
         res_drive[2].name = lib_msprintf("Drive%iRPM", dnr + 8);
-        res_drive[2].value_ptr = &(drive->rpm);
+        res_drive[2].value_ptr = &(drive0->rpm);
         res_drive[2].param = uint_to_void_ptr(dnr);
-        res_drive[3].name = lib_msprintf("Drive%iWobble", dnr + 8);
-        res_drive[3].value_ptr = &(drive->rpm_wobble);
+        res_drive[3].name = lib_msprintf("Drive%iWobbleFrequency", dnr + 8);
+        res_drive[3].value_ptr = &(drive0->wobble_frequency);
         res_drive[3].param = uint_to_void_ptr(dnr);
+        res_drive[4].name = lib_msprintf("Drive%iWobbleAmplitude", dnr + 8);
+        res_drive[4].value_ptr = &(drive0->wobble_amplitude);
+        res_drive[4].param = uint_to_void_ptr(dnr);
 
         if (has_iec) {
             res_drive_rtc[0].name = lib_msprintf("Drive%iRTCSave", dnr + 8);
-            res_drive_rtc[0].value_ptr = &(drive->rtc_save);
+            res_drive_rtc[0].value_ptr = &(unit->rtc_save);
             res_drive_rtc[0].param = uint_to_void_ptr(dnr);
             if (resources_register_int(res_drive_rtc) < 0) {
                 return -1;
