@@ -75,7 +75,7 @@
 
 static int drive_init_was_called = 0;
 
-drive_context_t *drive_context[DRIVE_NUM];
+diskunit_context_t *diskunit_context[NUM_DISK_UNITS];
 
 /* Generic drive logging goes here.  */
 static log_t drive_log = LOG_ERR;
@@ -85,60 +85,58 @@ int rom_loaded = 0;
 
 /* ------------------------------------------------------------------------- */
 
-static int drive_led_color[DRIVE_NUM];
+static int drive_led_color[NUM_DISK_UNITS];
 
 /* ------------------------------------------------------------------------- */
 
 void drive_set_disk_memory(uint8_t *id, unsigned int track, unsigned int sector,
-                           struct drive_context_s *drv)
+                           struct diskunit_context_s *unit)
 {
-    drive_t *drive;
-
-    drive = drv->drive;
-
-    if (drive->type == DRIVE_TYPE_1540
-        || drive->type == DRIVE_TYPE_1541
-        || drive->type == DRIVE_TYPE_1541II
-        || drive->type == DRIVE_TYPE_1570
-        || drive->type == DRIVE_TYPE_1571
-        || drive->type == DRIVE_TYPE_1571CR) {
-        drv->drive->drive_ram[0x12] = id[0];
-        drv->drive->drive_ram[0x13] = id[1];
-        drv->drive->drive_ram[0x16] = id[0];
-        drv->drive->drive_ram[0x17] = id[1];
-        drv->drive->drive_ram[0x18] = track;
-        drv->drive->drive_ram[0x19] = sector;
-        drv->drive->drive_ram[0x22] = track;
+    if (unit->type == DRIVE_TYPE_1540
+        || unit->type == DRIVE_TYPE_1541
+        || unit->type == DRIVE_TYPE_1541II
+        || unit->type == DRIVE_TYPE_1570
+        || unit->type == DRIVE_TYPE_1571
+        || unit->type == DRIVE_TYPE_1571CR) {
+        unit->drive_ram[0x12] = id[0];
+        unit->drive_ram[0x13] = id[1];
+        unit->drive_ram[0x16] = id[0];
+        unit->drive_ram[0x17] = id[1];
+        unit->drive_ram[0x18] = track;
+        unit->drive_ram[0x19] = sector;
+        unit->drive_ram[0x22] = track;
     }
 }
 
 void drive_set_last_read(unsigned int track, unsigned int sector, uint8_t *buffer,
-                         struct drive_context_s *drv)
+                         struct diskunit_context_s *unit)
 {
     drive_t *drive;
     int side = 0;
 
-    drive = drv->drive;
+    drive = unit->drives[0];
 
+    /* TODO: drive 1 ? */
     drive_gcr_data_writeback(drive);
 
-    if (drive->type == DRIVE_TYPE_1570
-        || drive->type == DRIVE_TYPE_1571
-        || drive->type == DRIVE_TYPE_1571CR) {
+    if (unit->type == DRIVE_TYPE_1570
+        || unit->type == DRIVE_TYPE_1571
+        || unit->type == DRIVE_TYPE_1571CR) {
         if (track > (DRIVE_HALFTRACKS_1571 / 2)) {
             track -= (DRIVE_HALFTRACKS_1571 / 2);
             side = 1;
         }
     }
+    /* TODO: drive 1 ? */
     drive_set_half_track(track * 2, side, drive);
 
-    if (drive->type == DRIVE_TYPE_1540
-        || drive->type == DRIVE_TYPE_1541
-        || drive->type == DRIVE_TYPE_1541II
-        || drive->type == DRIVE_TYPE_1570
-        || drive->type == DRIVE_TYPE_1571
-        || drive->type == DRIVE_TYPE_1571CR) {
-        memcpy(&(drv->drive->drive_ram[0x0400]), buffer, 256);
+    if (unit->type == DRIVE_TYPE_1540
+        || unit->type == DRIVE_TYPE_1541
+        || unit->type == DRIVE_TYPE_1541II
+        || unit->type == DRIVE_TYPE_1570
+        || unit->type == DRIVE_TYPE_1571
+        || unit->type == DRIVE_TYPE_1571CR) {
+        memcpy(&(unit->drive_ram[0x0400]), buffer, 256);
     }
 }
 
@@ -148,7 +146,7 @@ void drive_set_last_read(unsigned int track, unsigned int sector, uint8_t *buffe
    once before anything else).  Return 0 on success, -1 on error.  */
 int drive_init(void)
 {
-    unsigned int dnr;
+    unsigned int unit;
     drive_t *drive;
 
     if (rom_loaded) {
@@ -162,17 +160,26 @@ int drive_init(void)
 
     drive_log = log_open("Drive");
 
-    for (dnr = 0; dnr < DRIVE_NUM; dnr++) {
+    for (unit = 0; unit < NUM_DISK_UNITS; unit++) {
+        diskunit_context_t *diskunit =  diskunit_context[unit];
         char *logname;
+        unsigned int d;
 
-        drive = drive_context[dnr]->drive;
-        logname = lib_msprintf("Drive %i", dnr + 8);
-        drive->log = log_open(logname);
+        logname = lib_msprintf("Unit %u", unit + 8);
+        diskunit->log = log_open(logname);
         lib_free(logname);
 
-        drive_clk[dnr] = 0L;
-        drive->clk = &drive_clk[dnr];
-        drive->mynumber = dnr;
+        diskunit_clk[unit] = 0L;
+
+        for (d = 0; d < NUM_DRIVES; d++) {
+            drive = diskunit->drives[d];
+
+            drive->clk = &diskunit_clk[unit];
+            drive->unit = unit;
+            drive->drive = d;
+            drive->diskunit = diskunit_context[unit];
+        }
+
     }
 
     if (driverom_load_images() < 0) {
@@ -188,71 +195,82 @@ int drive_init(void)
 
     drive_overflow_init();
 
-    for (dnr = 0; dnr < DRIVE_NUM; dnr++) {
-        drive = drive_context[dnr]->drive;
+    for (unit = 0; unit < NUM_DISK_UNITS; unit++) {
+        diskunit_context_t *diskunit = diskunit_context[unit];
+        drive = diskunit->drives[0];
 
-        machine_drive_port_default(drive_context[dnr]);
+        machine_drive_port_default(diskunit);
 
-        if (drive_check_type(drive->type, dnr) < 1) {
-            resources_set_int_sprintf("Drive%iType", DRIVE_TYPE_NONE, dnr + 8);
+        if (drive_check_type(diskunit->type, unit) < 1) {
+            resources_set_int_sprintf("Drive%iType", DRIVE_TYPE_NONE, unit + 8);
         }
 
-        machine_drive_rom_setup_image(dnr);
+        machine_drive_rom_setup_image(unit);
     }
 
-    for (dnr = 0; dnr < DRIVE_NUM; dnr++) {
-        drive = drive_context[dnr]->drive;
-        drive->gcr = gcr_create_image();
-        drive->p64 = lib_calloc(1, sizeof(TP64Image));
-        P64ImageCreate(drive->p64);
-        drive->byte_ready_level = 1;
-        drive->byte_ready_edge = 1;
-        drive->GCR_dirty_track = 0;
-        drive->GCR_write_value = 0x55;
-        drive->GCR_track_start_ptr = NULL;
-        drive->GCR_current_track_size = 0;
-        drive->attach_clk = (CLOCK)0;
-        drive->detach_clk = (CLOCK)0;
-        drive->attach_detach_clk = (CLOCK)0;
-        drive->old_led_status = 0;
-        drive->old_half_track = 0;
-        drive->side = 0;
-        drive->GCR_image_loaded = 0;
-        drive->P64_image_loaded = 0;
-        drive->P64_dirty = 0;
-        drive->read_only = 0;
-        drive->clock_frequency = 1;
-        drive->led_last_change_clk = *(drive->clk);
-        drive->led_last_uiupdate_clk = *(drive->clk);
-        drive->led_active_ticks = 0;
+    for (unit = 0; unit < NUM_DISK_UNITS; unit++) {
+        diskunit_context_t *diskunit = diskunit_context[unit];
+        int d;
 
+        for (d = 0; d < NUM_DRIVES; d++) {
+            drive = diskunit->drives[d];
+
+            drive->gcr = gcr_create_image();
+            drive->p64 = lib_calloc(1, sizeof(TP64Image));
+            P64ImageCreate(drive->p64);
+            drive->byte_ready_level = 1;
+            drive->byte_ready_edge = 1;
+            drive->GCR_dirty_track = 0;
+            drive->GCR_write_value = 0x55;
+            drive->GCR_track_start_ptr = NULL;
+            drive->GCR_current_track_size = 0;
+            drive->attach_clk = (CLOCK)0;
+            drive->detach_clk = (CLOCK)0;
+            drive->attach_detach_clk = (CLOCK)0;
+            drive->old_led_status = 0;
+            drive->old_half_track = 0;
+            drive->side = 0;
+            drive->GCR_image_loaded = 0;
+            drive->P64_image_loaded = 0;
+            drive->P64_dirty = 0;
+            drive->read_only = 0;
+            drive->led_last_change_clk = *(drive->clk);
+            drive->led_last_uiupdate_clk = *(drive->clk);
+            drive->led_active_ticks = 0;
+            drive->read_write_mode = 1;
+
+            /* Position the R/W head on the directory track.  */
+            drive_set_half_track(36, 0, drive);
+            drive_set_active_led_color(diskunit->type, unit);
+        }
+    }
+
+    for (unit = 0; unit < NUM_DISK_UNITS; unit++) {
+        diskunit_context_t *diskunit = diskunit_context[unit];
+        drive = diskunit->drives[0];
+
+        driverom_initialize_traps(diskunit);
+
+        /* Sets diskunit->clock_frequency */
+        drivesync_clock_frequency(diskunit, diskunit->type);
+
+        /* TODO: rotation code is not drive1 aware */
+        rotation_init((diskunit->clock_frequency == 2) ? 1 : 0, unit);
         rotation_reset(drive);
 
-        /* Position the R/W head on the directory track.  */
-        drive_set_half_track(36, 0, drive);
-        drive_set_active_led_color(drive->type, dnr);
-    }
-
-    for (dnr = 0; dnr < DRIVE_NUM; dnr++) {
-        drive = drive_context[dnr]->drive;
-        driverom_initialize_traps(drive);
-
-        drivesync_clock_frequency(drive->type, drive);
-
-        rotation_init((drive->clock_frequency == 2) ? 1 : 0, dnr);
-
-        if (drive->type == DRIVE_TYPE_2000 || drive->type == DRIVE_TYPE_4000) {
-            drivecpu65c02_init(drive_context[dnr], drive->type);
+        if (diskunit->type == DRIVE_TYPE_2000 || diskunit->type == DRIVE_TYPE_4000 ||
+            diskunit->type == DRIVE_TYPE_CMDHD) {
+            drivecpu65c02_init(diskunit, diskunit->type);
         } else {
-            drivecpu_init(drive_context[dnr], drive->type);
+            drivecpu_init(diskunit, diskunit->type);
         }
 
         /* Make sure the sync factor is acknowledged correctly.  */
-        drivesync_factor(drive_context[dnr]);
+        drivesync_factor(diskunit);
 
         /* Make sure the traps are moved as needed.  */
-        if (drive->enable) {
-            drive_enable(drive_context[dnr]);
+        if (diskunit->enable) {
+            drive_enable(diskunit);
         }
     }
 
@@ -261,34 +279,53 @@ int drive_init(void)
 
 void drive_shutdown(void)
 {
-    unsigned int dnr;
+    unsigned int unr, dnr;
 
     if (!drive_init_was_called) {
         /* happens at the -help command line command*/
         return;
     }
 
-    for (dnr = 0; dnr < DRIVE_NUM; dnr++) {
-        if (drive_context[dnr]->drive->type == DRIVE_TYPE_2000 || drive_context[dnr]->drive->type == DRIVE_TYPE_4000) {
-            drivecpu65c02_shutdown(drive_context[dnr]);
+    for (unr = 0; unr < NUM_DISK_UNITS; unr++) {
+        diskunit_context_t *unit = diskunit_context[unr];
+
+        if (unit->type == DRIVE_TYPE_2000 || unit->type == DRIVE_TYPE_4000 ||
+            unit->type == DRIVE_TYPE_CMDHD) {
+            drivecpu65c02_shutdown(diskunit_context[unr]);
         } else {
-            drivecpu_shutdown(drive_context[dnr]);
+            drivecpu_shutdown(diskunit_context[unr]);
         }
-        if (drive_context[dnr]->drive->gcr) {
-            gcr_destroy_image(drive_context[dnr]->drive->gcr);
+
+        if (unit->ds1216) {
+            ds1216e_destroy(unit->ds1216, unit->rtc_save);
+            unit->ds1216 = NULL;
         }
-        if (drive_context[dnr]->drive->p64) {
-            P64ImageDestroy(drive_context[dnr]->drive->p64);
-            lib_free(drive_context[dnr]->drive->p64);
-        }
-        if (drive_context[dnr]->drive->ds1216) {
-            ds1216e_destroy(drive_context[dnr]->drive->ds1216, drive_context[dnr]->drive->rtc_save);
+
+        for (dnr = 0; dnr < NUM_DRIVES; dnr++) {
+            drive_t *drive = unit->drives[dnr];
+
+            if (drive->gcr) {
+                gcr_destroy_image(drive->gcr);
+            }
+            if (drive->p64) {
+                P64ImageDestroy(drive->p64);
+                lib_free(drive->p64);
+                drive->p64 = NULL;
+            }
         }
     }
 
-    for (dnr = 0; dnr < DRIVE_NUM; dnr++) {
-        lib_free(drive_context[dnr]->drive);
-        lib_free(drive_context[dnr]);
+    for (unr = 0; unr < NUM_DISK_UNITS; unr++) {
+        diskunit_context_t *unit = diskunit_context[unr];
+
+        for (dnr = 0; dnr < NUM_DRIVES; dnr++) {
+            drive_t *drive = unit->drives[dnr];
+
+            lib_free(drive);
+            unit->drives[dnr] = NULL;
+        }
+        lib_free(unit);
+        diskunit_context[unr] = NULL;
     }
 }
 
@@ -311,6 +348,7 @@ void drive_set_active_led_color(unsigned int type, unsigned int dnr)
             break;
         case DRIVE_TYPE_2000:   /* red power, green activity, red error, horizontal, line */
         case DRIVE_TYPE_4000:   /* red power, green activity, red error, horizontal, line */
+        case DRIVE_TYPE_CMDHD:   /* red power, green activity, red error, horizontal, line */
             drive_led_color[dnr] = DRIVE_LED1_GREEN | DRIVE_LED2_RED;
             break;
         case DRIVE_TYPE_2040:   /* red drive1, red power, red drive2, horizontal, round */
@@ -319,6 +357,9 @@ void drive_set_active_led_color(unsigned int type, unsigned int dnr)
         case DRIVE_TYPE_8050:   /* red drive1, green power, red drive2, horizontal, round */
             drive_led_color[dnr] = DRIVE_LED1_RED | DRIVE_LED2_RED;
             break;
+        case DRIVE_TYPE_9000:   /* red drive1, green power, red drive2, horizontal, round */
+            drive_led_color[dnr] = DRIVE_LED1_RED | DRIVE_LED2_RED;
+            break;
         case DRIVE_TYPE_8250:   /* red green, green power,green, horizontal, round */
             drive_led_color[dnr] = DRIVE_LED1_GREEN | DRIVE_LED2_GREEN; /* only the LP version is RED */
             break;
@@ -328,11 +369,10 @@ void drive_set_active_led_color(unsigned int type, unsigned int dnr)
     }
 }
 
-int drive_set_disk_drive_type(unsigned int type, struct drive_context_s *drv)
+int drive_set_disk_drive_type(unsigned int type, struct diskunit_context_s *drv)
 {
     unsigned int dnr;
-    drive_t *drive;
-    drive_t *drive1;
+    drive_t *drive0, *drive1;
 
     dnr = drv->mynumber;
 
@@ -340,36 +380,29 @@ int drive_set_disk_drive_type(unsigned int type, struct drive_context_s *drv)
         return -1;
     }
 
-    drive = drv->drive;
-    rotation_rotate_disk(drive);
+    drive0 = drv->drives[0];
+    drive1 = drv->drives[1];
 
-    drivesync_clock_frequency(type, drive);
+    /* TODO: drive 1? */
+    rotation_rotate_disk(drive0);
+    drivesync_clock_frequency(drv, type);
 
     rotation_init(0, dnr);
-    drive->type = type;
-    if (type == DRIVE_TYPE_2000 || type == DRIVE_TYPE_4000) {
+    drv->type = type;
+    if (type == DRIVE_TYPE_2000 || type == DRIVE_TYPE_4000 ||
+        type == DRIVE_TYPE_CMDHD) {
         drivecpu65c02_setup_context(drv, 0);
     } else {
         drivecpu_setup_context(drv, 0);
     }
-    drive->side = 0;
+    drive0->side = 0;
+    drive1->side = 0;
     machine_drive_rom_setup_image(dnr);
     drivesync_factor(drv);
     drive_set_active_led_color(type, dnr);
 
-    /* set up (relatively) easy detection of dual drives */
-    drive1 = drive_context[mk_drive1(dnr)]->drive;
-    drive->drive0 = NULL;
-    drive1->drive1 = NULL;
-    if (is_drive0(dnr) && drive_check_dual(type)) {
-        drive->drive1 = drive1;
-        drive1->drive0 = drive;
-    } else {
-        drive->drive1 = NULL;
-        drive1->drive0 = NULL;
-    }
-
-    if (type == DRIVE_TYPE_2000 || type == DRIVE_TYPE_4000) {
+    if (type == DRIVE_TYPE_2000 || type == DRIVE_TYPE_4000 ||
+        type == DRIVE_TYPE_CMDHD) {
         drivecpu65c02_init(drv, type);
     } else {
         drivecpu_init(drv, type);
@@ -380,45 +413,46 @@ int drive_set_disk_drive_type(unsigned int type, struct drive_context_s *drv)
 
 int drive_get_disk_drive_type(int dnr)
 {
-    if (dnr >= 0 && dnr < DRIVE_NUM) {
-        return drive_context[dnr]->drive->type;
+    if (dnr >= 0 && dnr < NUM_DISK_UNITS) {
+        return diskunit_context[dnr]->type;
     }
 
     return DRIVE_TYPE_NONE;
 }
 
-void drive_enable_update_ui(drive_context_t *drv)
+void drive_enable_update_ui(diskunit_context_t *drv)
 {
     int i;
-    unsigned int enabled_drives = 0;
+    unsigned int enabled_units = 0;
 
-    for (i = 0; i < DRIVE_NUM; i++) {
+    for (i = 0; i < NUM_DISK_UNITS; i++) {
         unsigned int the_drive;
-        drive_t *drive = drive_context[i]->drive;
+        diskunit_context_t *unit = diskunit_context[i];
+        /* TODO: drive 1 */
+        drive_t *drive = unit->drives[0];
 
         the_drive = 1 << i;
 
-        if (drive->enable || (drive->drive0 && drive->drive0->enable)) {
-            enabled_drives |= the_drive;
+        if (unit->enable) {
+            enabled_units |= the_drive;
             drive->old_led_status = -1;
             drive->old_half_track = -1;
             drive->old_side = -1;
         }
     }
 
-    ui_enable_drive_status(enabled_drives,
+    ui_enable_drive_status(enabled_units,
                            drive_led_color);
 }
 
 /* Activate full drive emulation. */
-int drive_enable(drive_context_t *drv)
+int drive_enable(diskunit_context_t *drv)
 {
     int drive_true_emulation = 0;
     unsigned int dnr;
-    drive_t *drive;
+    unsigned int drive;
 
     dnr = drv->mynumber;
-    drive = drv->drive;
 
     /* This must come first, because this might be called before the drive
        initialization.  */
@@ -433,19 +467,22 @@ int drive_enable(drive_context_t *drv)
         return 0;
     }
 
-    if (drive->type == DRIVE_TYPE_NONE) {
+    if (drv->type == DRIVE_TYPE_NONE) {
         return 0;
     }
 
     /* Recalculate drive geometry.  */
-    if (drive->image != NULL) {
-        drive_image_attach(drive->image, dnr + 8);
+    for (drive = 0; drive < NUM_DRIVES; drive++) {
+	if (drv->drives[drive]->image != NULL) {
+	    drive_image_attach(drv->drives[drive]->image, dnr, drive);
+	}
     }
 
     /* resync */
     drv->cpu->stop_clk = *(drv->clk_ptr);
 
-    if (drive->type == DRIVE_TYPE_2000 || drive->type == DRIVE_TYPE_4000) {
+    if (drv->type == DRIVE_TYPE_2000 || drv->type == DRIVE_TYPE_4000 ||
+        drv->type == DRIVE_TYPE_CMDHD) {
         drivecpu65c02_wake_up(drv);
     } else {
         drivecpu_wake_up(drv);
@@ -457,28 +494,29 @@ int drive_enable(drive_context_t *drv)
 }
 
 /* Disable full drive emulation.  */
-void drive_disable(drive_context_t *drv)
+void drive_disable(diskunit_context_t *drv)
 {
     int drive_true_emulation = 0;
-    drive_t *drive;
-
-    drive = drv->drive;
+    unsigned int drive;
 
     /* This must come first, because this might be called before the true
        drive initialization.  */
-    drive->enable = 0;
+    drv->enable = 0;
 
     resources_get_int("DriveTrueEmulation", &drive_true_emulation);
 
     if (rom_loaded) {
-        if (drive->type == DRIVE_TYPE_2000 || drive->type == DRIVE_TYPE_4000) {
+        if (drv->type == DRIVE_TYPE_2000 || drv->type == DRIVE_TYPE_4000 ||
+            drv->type == DRIVE_TYPE_CMDHD) {
             drivecpu65c02_sleep(drv);
         } else {
             drivecpu_sleep(drv);
         }
         machine_drive_port_default(drv);
 
-        drive_gcr_data_writeback(drive);
+	for (drive = 0; drive < NUM_DRIVES; drive++) {
+	    drive_gcr_data_writeback(drv->drives[drive]);
+	}
     }
 
     /* Make sure the UI is updated.  */
@@ -487,15 +525,15 @@ void drive_disable(drive_context_t *drv)
 
 monitor_interface_t *drive_cpu_monitor_interface_get(unsigned int dnr)
 {
-    return drive_context[dnr]->cpu->monitor_interface;
+    return diskunit_context[dnr]->cpu->monitor_interface;
 }
 
 void drive_cpu_early_init_all(void)
 {
     unsigned int dnr;
 
-    for (dnr = 0; dnr < DRIVE_NUM; dnr++) {
-        machine_drive_init(drive_context[dnr]);
+    for (dnr = 0; dnr < NUM_DISK_UNITS; dnr++) {
+        machine_drive_init(diskunit_context[dnr]);
     }
 }
 
@@ -503,20 +541,24 @@ void drive_cpu_prevent_clk_overflow_all(CLOCK sub)
 {
     unsigned int dnr;
 
-    for (dnr = 0; dnr < DRIVE_NUM; dnr++) {
-        drive_t *drive = drive_context[dnr]->drive;
-        if (drive->type == DRIVE_TYPE_2000 || drive->type == DRIVE_TYPE_4000) {
-            drivecpu65c02_prevent_clk_overflow(drive_context[dnr], sub);
+    for (dnr = 0; dnr < NUM_DISK_UNITS; dnr++) {
+        diskunit_context_t *unit = diskunit_context[dnr];
+
+        if (unit->type == DRIVE_TYPE_2000 || unit->type == DRIVE_TYPE_4000 ||
+            unit->type == DRIVE_TYPE_CMDHD) {
+            drivecpu65c02_prevent_clk_overflow(diskunit_context[dnr], sub);
         } else {
-            drivecpu_prevent_clk_overflow(drive_context[dnr], sub);
+            drivecpu_prevent_clk_overflow(diskunit_context[dnr], sub);
         }
     }
 }
 
 void drive_cpu_trigger_reset(unsigned int dnr)
 {
-    drive_t *drive = drive_context[dnr]->drive;
-    if (drive->type == DRIVE_TYPE_2000 || drive->type == DRIVE_TYPE_4000) {
+    diskunit_context_t *unit = diskunit_context[dnr];
+
+    if (unit->type == DRIVE_TYPE_2000 || unit->type == DRIVE_TYPE_4000 ||
+        unit->type == DRIVE_TYPE_CMDHD) {
         drivecpu65c02_trigger_reset(dnr);
     } else {
         drivecpu_trigger_reset(dnr);
@@ -527,36 +569,43 @@ void drive_cpu_trigger_reset(unsigned int dnr)
 void drive_reset(void)
 {
     unsigned int dnr;
-    drive_t *drive;
+    unsigned int d;
 
-    for (dnr = 0; dnr < DRIVE_NUM; dnr++) {
-        drive = drive_context[dnr]->drive;
+    for (dnr = 0; dnr < NUM_DISK_UNITS; dnr++) {
+        diskunit_context_t *unit = diskunit_context[dnr];
 
-        if (drive->type == DRIVE_TYPE_2000 || drive->type == DRIVE_TYPE_4000) {
-            drivecpu65c02_reset(drive_context[dnr]);
+        if (unit->type == DRIVE_TYPE_2000 || unit->type == DRIVE_TYPE_4000 ||
+            unit->type == DRIVE_TYPE_CMDHD) {
+            drivecpu65c02_reset(diskunit_context[dnr]);
         } else {
-            drivecpu_reset(drive_context[dnr]);
+            drivecpu_reset(diskunit_context[dnr]);
         }
 
-        drive->led_last_change_clk = *(drive->clk);
-        drive->led_last_uiupdate_clk = *(drive->clk);
-        drive->led_active_ticks = 0;
+	for (d = 0; d < NUM_DRIVES; d++) {
+	    drive_t *drive = unit->drives[d];
+
+	    drive->led_last_change_clk = *(drive->clk);
+	    drive->led_last_uiupdate_clk = *(drive->clk);
+	    drive->led_active_ticks = 0;
+	}
     }
 }
 
 /* Move the head to half track `num'.  */
 void drive_set_half_track(int num, int side, drive_t *dptr)
 {
+    unsigned int type = dptr->diskunit->type;
     int tmp;
-    if ((dptr->type == DRIVE_TYPE_1540
-         || dptr->type == DRIVE_TYPE_1541
-         || dptr->type == DRIVE_TYPE_1541II
-         || dptr->type == DRIVE_TYPE_1551
-         || dptr->type == DRIVE_TYPE_1570
-         || dptr->type == DRIVE_TYPE_2031) && (num > DRIVE_HALFTRACKS_1541)) {
+
+    if ((type == DRIVE_TYPE_1540
+         || type == DRIVE_TYPE_1541
+         || type == DRIVE_TYPE_1541II
+         || type == DRIVE_TYPE_1551
+         || type == DRIVE_TYPE_1570
+         || type == DRIVE_TYPE_2031) && (num > DRIVE_HALFTRACKS_1541)) {
         num = DRIVE_HALFTRACKS_1541;
     }
-    if ((dptr->type == DRIVE_TYPE_1571 || dptr->type == DRIVE_TYPE_1571CR)
+    if ((type == DRIVE_TYPE_1571 || type == DRIVE_TYPE_1571CR)
         && (num > DRIVE_HALFTRACKS_1571)) {
         num = DRIVE_HALFTRACKS_1571;
     }
@@ -599,7 +648,7 @@ void drive_move_head(int step, drive_t *drive)
         log_warning(drive_log, "ambiguous step count (%d)", step);
     }
     drive_gcr_data_writeback(drive);
-    drive_sound_head(drive->current_half_track, step, drive->mynumber);
+    drive_sound_head(drive->current_half_track, step, drive->unit);
     drive_set_half_track(drive->current_half_track + step, drive->side, drive);
 }
 
@@ -673,16 +722,20 @@ void drive_gcr_data_writeback(drive_t *drive)
 void drive_gcr_data_writeback_all(void)
 {
     drive_t *drive;
-    unsigned int i;
+    unsigned int i, j;
 
-    for (i = 0; i < DRIVE_NUM; i++) {
-        drive = drive_context[i]->drive;
-        drive_gcr_data_writeback(drive);
-        if (drive->P64_image_loaded && drive->image && drive->image->p64) {
-            if (drive->image->type == DISK_IMAGE_TYPE_P64) {
-                if (drive->P64_dirty) {
-                    drive->P64_dirty = 0;
-                    disk_image_write_p64_image(drive->image);
+    for (i = 0; i < NUM_DISK_UNITS; i++) {
+        for (j = 0; j < 2; j++) {
+            drive = diskunit_context[i]->drives[j];
+            if (drive) {
+                drive_gcr_data_writeback(drive);
+                if (drive->P64_image_loaded && drive->image && drive->image->p64) {
+                    if (drive->image->type == DISK_IMAGE_TYPE_P64) {
+                        if (drive->P64_dirty) {
+                            drive->P64_dirty = 0;
+                            disk_image_write_p64_image(drive->image);
+                        }
+                    }
                 }
             }
         }
@@ -691,55 +744,55 @@ void drive_gcr_data_writeback_all(void)
 
 /* ------------------------------------------------------------------------- */
 
-static void drive_led_update(drive_t *drive, drive_t *drive0)
+static void drive_led_update(diskunit_context_t *unit, drive_t *drive, int base)
 {
     int my_led_status = 0;
     CLOCK led_period;
-    unsigned int led_pwm;
+    unsigned int led_pwm1;
 
     /* Actually update the LED status only if the `trap idle'
        idling method is being used, as the LED status could be
        incorrect otherwise.  */
 
-    if (drive0->idling_method != DRIVE_IDLE_SKIP_CYCLES) {
+    if (unit->idling_method != DRIVE_IDLE_SKIP_CYCLES) {
         my_led_status = drive->led_status;
     }
 
-    /* Update remaining led clock ticks. */
+    /* Update remaining led clock ticks. TODO: move clk to diskunit. */
     if (drive->led_status & 1) {
-        drive->led_active_ticks += *(drive0->clk)
+        drive->led_active_ticks += *(drive->clk)
                                    - drive->led_last_change_clk;
     }
-    drive->led_last_change_clk = *(drive0->clk);
+    drive->led_last_change_clk = *(drive->clk);
 
-    led_period = *(drive0->clk) - drive->led_last_uiupdate_clk;
-    drive->led_last_uiupdate_clk = *(drive0->clk);
+    led_period = *(drive->clk) - drive->led_last_uiupdate_clk;
+    drive->led_last_uiupdate_clk = *(drive->clk);
 
     if (led_period == 0) {
         return;
     }
 
     if (drive->led_active_ticks > led_period) {
-        /* during startup it has been observer that led_pwm > 1000,
+        /* during startup it has been observer that led_pwm1 > 1000,
            which potentially breaks several UIs */
         /* this also happens when the drive is reset from UI
            and the LED was on */
-        led_pwm = 1000;
+        led_pwm1 = 1000;
     } else {
-        led_pwm = drive->led_active_ticks * 1000 / led_period;
+        led_pwm1 = drive->led_active_ticks * 1000 / led_period;
     }
-    assert(led_pwm <= MAX_PWM);
-    if (led_pwm > MAX_PWM) {
-        led_pwm = MAX_PWM;
+    assert(led_pwm1 <= MAX_PWM);
+    if (led_pwm1 > MAX_PWM) {
+        led_pwm1 = MAX_PWM;
     }
 
     drive->led_active_ticks = 0;
 
-    if (led_pwm != drive->led_last_pwm
+    if (led_pwm1 != drive->led_last_pwm
         || my_led_status != drive->old_led_status) {
-        ui_display_drive_led(drive->mynumber, led_pwm,
+        ui_display_drive_led(drive->unit, base, led_pwm1,
                              (my_led_status & 2) ? 1000 : 0);
-        drive->led_last_pwm = led_pwm;
+        drive->led_last_pwm = led_pwm1;
         drive->old_led_status = my_led_status;
     }
 }
@@ -754,26 +807,32 @@ void drive_update_ui_status(void)
     }
 
     /* Update the LEDs and the track indicators.  */
-    for (i = 0; i < DRIVE_NUM; i++) {
-        drive_t *drive = drive_context[i]->drive;
-        drive_t *drive0 = drive->drive0;
-        int dual = drive0 && drive0->enable;
+    for (i = 0; i < NUM_DISK_UNITS; i++) {
+        diskunit_context_t *unit = diskunit_context[i];
+        drive_t *drive0 = unit->drives[0];
+        drive_t *drive1 = unit->drives[1];
 
-        if (drive->enable || dual) {
-            if (!drive0) {
-                drive0 = drive;
+        if (unit->enable) {
+
+            drive_led_update(unit, drive0, 0);
+            /* drive_led_update(unit, drive0, 0); */ /* FIXME */
+            if (drive0->current_half_track != drive0->old_half_track
+                || drive0->side != drive0->old_side) {
+                drive0->old_half_track = drive0->current_half_track;
+                drive0->old_side = drive0->side;
+                ui_display_drive_track(i, 0,
+                                       drive0->current_half_track + (drive0->side * DRIVE_HALFTRACKS_1571));
             }
-
-            drive_led_update(drive, drive0);
-
-            if (drive->current_half_track != drive->old_half_track
-                || drive->side != drive->old_side) {
-                drive->old_half_track = drive->current_half_track;
-                drive->old_side = drive->side;
-                dual = dual || drive->drive1;   /* also include drive 0 */
-                ui_display_drive_track(i,
-                                       dual ? 0 : 8,
-                                       drive->current_half_track + (drive->side * DRIVE_HALFTRACKS_1571));
+            /* update LED and track of the second drive for dual drives */
+            if (drive_check_dual(unit->type)) {
+                /* drive_led_update(unit, drive1, 1); */ /* FIXME */
+                if (drive1->current_half_track != drive1->old_half_track
+                    || drive1->side != drive1->old_side) {
+                    drive1->old_half_track = drive1->current_half_track;
+                    drive1->old_side = drive1->side;
+                    ui_display_drive_track(i, 1,
+                                        drive1->current_half_track + (drive1->side * DRIVE_HALFTRACKS_1571));
+                }
             }
         }
     }
@@ -781,27 +840,28 @@ void drive_update_ui_status(void)
 
 int drive_num_leds(unsigned int dnr)
 {
-    drive_t *drive = drive_context[dnr]->drive;
+    diskunit_context_t *unit = diskunit_context[dnr];
 
-    switch (drive->type) {
+    switch (unit->type) {
     case DRIVE_TYPE_2040:
     case DRIVE_TYPE_3040:
     case DRIVE_TYPE_4040:
     case DRIVE_TYPE_8050:
     case DRIVE_TYPE_8250:
+    case DRIVE_TYPE_9000:
     case DRIVE_TYPE_2000:
     case DRIVE_TYPE_4000:
+    case DRIVE_TYPE_CMDHD:
         return 2;
     default:
         return 1;
     }
 }
 
-void drive_cpu_execute_one(drive_context_t *drv, CLOCK clk_value)
+void drive_cpu_execute_one(diskunit_context_t *drv, CLOCK clk_value)
 {
-    drive_t *drive = drv->drive;
-
-    if (drive->type == DRIVE_TYPE_2000 || drive->type == DRIVE_TYPE_4000) {
+    if (drv->type == DRIVE_TYPE_2000 || drv->type == DRIVE_TYPE_4000 ||
+        drv->type == DRIVE_TYPE_CMDHD) {
         drivecpu65c02_execute(drv, clk_value);
     } else {
         drivecpu_execute(drv, clk_value);
@@ -811,21 +871,20 @@ void drive_cpu_execute_one(drive_context_t *drv, CLOCK clk_value)
 void drive_cpu_execute_all(CLOCK clk_value)
 {
     unsigned int dnr;
-    drive_t *drive;
 
-    for (dnr = 0; dnr < DRIVE_NUM; dnr++) {
-        drive = drive_context[dnr]->drive;
-        if (drive->enable) {
-            drive_cpu_execute_one(drive_context[dnr], clk_value);
+    for (dnr = 0; dnr < NUM_DISK_UNITS; dnr++) {
+        diskunit_context_t *unit = diskunit_context[dnr];
+
+        if (unit->enable) {
+            drive_cpu_execute_one(unit, clk_value);
         }
     }
 }
 
-void drive_cpu_set_overflow(drive_context_t *drv)
+void drive_cpu_set_overflow(diskunit_context_t *drv)
 {
-    drive_t *drive = drv->drive;
-
-    if (drive->type == DRIVE_TYPE_2000 || drive->type == DRIVE_TYPE_4000) {
+    if (drv->type == DRIVE_TYPE_2000 || drv->type == DRIVE_TYPE_4000 ||
+        drv->type == DRIVE_TYPE_CMDHD) {
         /* nothing */
     } else {
         drivecpu_set_overflow(drv);
@@ -839,17 +898,20 @@ void drive_vsync_hook(void)
 
     drive_update_ui_status();
 
-    for (dnr = 0; dnr < DRIVE_NUM; dnr++) {
-        drive_t *drive = drive_context[dnr]->drive;
-        if (drive->enable) {
-            if (drive->idling_method != DRIVE_IDLE_SKIP_CYCLES) {
-                drive_cpu_execute_one(drive_context[dnr], maincpu_clk);
+    for (dnr = 0; dnr < NUM_DISK_UNITS; dnr++) {
+        diskunit_context_t *unit = diskunit_context[dnr];
+        drive_t *drive = unit->drives[0];
+
+        if (unit->enable) {
+            if (unit->idling_method != DRIVE_IDLE_SKIP_CYCLES) {
+                drive_cpu_execute_one(diskunit_context[dnr], maincpu_clk);
             }
-            if (drive->idling_method == DRIVE_IDLE_NO_IDLE) {
+            if (unit->idling_method == DRIVE_IDLE_NO_IDLE) {
                 /* if drive is never idle, also rotate the disk. this prevents
                  * huge peaks in cpu usage when the drive must catch up with
                  * a longer period of time.
                  */
+                /* TODO: drive 1 */
                 rotation_rotate_disk(drive);
             }
             /* printf("drive_vsync_hook drv %d @clk:%d\n", dnr, maincpu_clk); */
@@ -859,12 +921,23 @@ void drive_vsync_hook(void)
 
 /* ------------------------------------------------------------------------- */
 
-static void drive_setup_context_for_drive(drive_context_t *drv,
-                                          unsigned int dnr)
+static void drive_setup_context_for_unit(diskunit_context_t *drv,
+                                          unsigned int unr)
 {
-    drv->mynumber = dnr;
-    drv->drive = lib_calloc(1, sizeof(drive_t));
-    drv->clk_ptr = &drive_clk[dnr];
+    unsigned int d;
+
+    drv->mynumber = unr;
+
+    for (d = 0; d < NUM_DRIVES; d++) {
+        drv->drives[d] = lib_calloc(1, sizeof(drive_t));
+        /* TODO: init functions for allocated memory */
+        drv->drives[d]->image = NULL;
+        drv->drives[d]->diskunit = drv;
+        drv->drives[d]->unit = unr;
+        drv->drives[d]->drive = d;
+    }
+
+    drv->clk_ptr = &diskunit_clk[unr];
 
     drivecpu_setup_context(drv, 1); /* no need for 65c02, only allocating common stuff */
 
@@ -873,10 +946,29 @@ static void drive_setup_context_for_drive(drive_context_t *drv,
 
 void drive_setup_context(void)
 {
-    unsigned int dnr;
+    unsigned int unr;
 
-    for (dnr = 0; dnr < DRIVE_NUM; dnr++) {
-        drive_context[dnr] = lib_calloc(1, sizeof(drive_context_t));
-        drive_setup_context_for_drive(drive_context[dnr], dnr);
+    for (unr = 0; unr < NUM_DISK_UNITS; unr++) {
+        diskunit_context[unr] = lib_calloc(1, sizeof(diskunit_context_t));
+        drive_setup_context_for_unit(diskunit_context[unr], unr);
     }
 }
+
+int drive_has_buttons(unsigned int dnr)
+{
+    diskunit_context_t *unit = diskunit_context[dnr];
+    if (unit->type == DRIVE_TYPE_2000 || unit->type == DRIVE_TYPE_4000) {
+        return 8; /* single swap */
+    } else if (unit->type == DRIVE_TYPE_CMDHD) {
+        return 1 | 2 | 4; /* write protect, swap 8, swap 9 */
+    }
+
+    return 0;
+}
+
+void drive_cpu_trigger_reset_button(unsigned int dnr, unsigned int button)
+{
+    diskunit_context_t *unit = diskunit_context[dnr];
+    unit->button = button;
+    drive_cpu_trigger_reset(dnr);
+}
