@@ -1,5 +1,5 @@
 /** \file   drivecpu65c02.c
- * \brief   65C02 processor emulation of CMD fd2000/4000 disk drives
+ * \brief   65C02 processor emulation of CMD FD2000/4000 and HD drives
  *
  * \author  Kajtar Zsolt <soci@c64.rulez.org>
  *
@@ -54,9 +54,9 @@
 
 static void drivecpu65c02_set_bank_base(void *context);
 
-static interrupt_cpu_status_t *drivecpu_int_status_ptr[DRIVE_NUM];
+static interrupt_cpu_status_t *drivecpu_int_status_ptr[NUM_DISK_UNITS];
 
-void drivecpu65c02_setup_context(struct drive_context_s *drv, int i)
+void drivecpu65c02_setup_context(struct diskunit_context_s *drv, int i)
 {
     monitor_interface_t *mi;
     drivecpu_context_t *cpu;
@@ -93,9 +93,10 @@ void drivecpu65c02_setup_context(struct drive_context_s *drv, int i)
     mi->z80_cpu_regs = NULL;
     mi->h6809_cpu_regs = NULL;
     mi->int_status = cpu->int_status;
-    mi->clk = &(drive_clk[drv->mynumber]);
+    mi->clk = &(diskunit_clk[drv->mynumber]);
     mi->current_bank = 0;
     mi->mem_bank_list = NULL;
+    mi->mem_bank_list_nos = NULL;
     mi->mem_bank_from_name = NULL;
     mi->get_line_cycle = NULL;
     
@@ -145,18 +146,19 @@ void drivecpu65c02_setup_context(struct drive_context_s *drv, int i)
 
 /* ------------------------------------------------------------------------- */
 
-static void cpu_reset(drive_context_t *drv)
+static void cpu_reset(diskunit_context_t *drv)
 {
     int preserve_monitor;
 
     preserve_monitor = drv->cpu->int_status->global_pending_int & IK_MONITOR;
 
-    log_message(drv->drive->log, "RESET.");
+    log_message(drv->log, "RESET.");
 
     interrupt_cpu_status_reset(drv->cpu->int_status);
 
     *(drv->clk_ptr) = 6;
-    rotation_reset(drv->drive);
+    rotation_reset(drv->drives[0]);
+    rotation_reset(drv->drives[1]);
     machine_drive_reset(drv);
 
     if (preserve_monitor) {
@@ -164,14 +166,14 @@ static void cpu_reset(drive_context_t *drv)
     }
 }
 
-void drivecpu65c02_reset_clk(drive_context_t *drv)
+void drivecpu65c02_reset_clk(diskunit_context_t *drv)
 {
     drv->cpu->last_clk = maincpu_clk;
     drv->cpu->last_exc_cycles = 0;
     drv->cpu->stop_clk = 0;
 }
 
-void drivecpu65c02_reset(drive_context_t *drv)
+void drivecpu65c02_reset(diskunit_context_t *drv)
 {
     int preserve_monitor;
 
@@ -192,10 +194,10 @@ void drivecpu65c02_reset(drive_context_t *drv)
 
 void drivecpu65c02_trigger_reset(unsigned int dnr)
 {
-    interrupt_trigger_reset(drivecpu_int_status_ptr[dnr], drive_clk[dnr] + 1);
+    interrupt_trigger_reset(drivecpu_int_status_ptr[dnr], diskunit_clk[dnr] + 1);
 }
 
-void drivecpu65c02_shutdown(drive_context_t *drv)
+void drivecpu65c02_shutdown(diskunit_context_t *drv)
 {
     drivecpu_context_t *cpu;
 
@@ -221,36 +223,38 @@ void drivecpu65c02_shutdown(drive_context_t *drv)
     lib_free(cpu);
 }
 
-void drivecpu65c02_init(drive_context_t *drv, int type)
+/* TODO: check type is always already set, and remove it as parameter */
+void drivecpu65c02_init(diskunit_context_t *drv, int type)
 {
-    drivemem_init(drv, type);
+    drv->type = type;
+    drivemem_init(drv);
     drivecpu65c02_reset(drv);
 }
 
-void drivecpu65c02_wake_up(drive_context_t *drv)
+void drivecpu65c02_wake_up(diskunit_context_t *drv)
 {
     /* FIXME: this value could break some programs, or be way too high for
        others.  Maybe we should put it into a user-definable resource.  */
     if (maincpu_clk - drv->cpu->last_clk > 0xffffff
         && *(drv->clk_ptr) > 934639) {
-        log_message(drv->drive->log, "Skipping cycles.");
+        log_message(drv->log, "Skipping cycles.");
         drv->cpu->last_clk = maincpu_clk;
     }
 }
 
-void drivecpu65c02_sleep(drive_context_t *drv)
+void drivecpu65c02_sleep(diskunit_context_t *drv)
 {
     /* Currently does nothing.  But we might need this hook some day.  */
 }
 
 /* Make sure the drive clock counters never overflow; return nonzero if
    they have been decremented to prevent overflow.  */
-CLOCK drivecpu65c02_prevent_clk_overflow(drive_context_t *drv, CLOCK sub)
+CLOCK drivecpu65c02_prevent_clk_overflow(diskunit_context_t *drv, CLOCK sub)
 {
     if (sub != 0) {
         /* First, get in sync with what the main CPU has done.  Notice that
            `clk' has already been decremented at this point.  */
-        if (drv->drive->enable) {
+        if (drv->enable) {
             if (drv->cpu->last_clk < sub) {
                 /* Hm, this is kludgy.  :-(  */
                 drive_cpu_execute_all(maincpu_clk + sub);
@@ -266,11 +270,11 @@ CLOCK drivecpu65c02_prevent_clk_overflow(drive_context_t *drv, CLOCK sub)
 }
 
 /* Handle a ROM trap. */
-inline static uint32_t drive_trap_handler(drive_context_t *drv)
+inline static uint32_t drive_trap_handler(diskunit_context_t *drv)
 {
-    if (R65C02_REGS_GET_PC(&(drv->cpu->cpu_R65C02_regs)) == (uint16_t)drv->drive->trap) {
-        R65C02_REGS_SET_PC(&(drv->cpu->cpu_R65C02_regs), drv->drive->trapcont);
-        if (drv->drive->idling_method == DRIVE_IDLE_TRAP_IDLE) {
+    if (R65C02_REGS_GET_PC(&(drv->cpu->cpu_R65C02_regs)) == (uint16_t)drv->trap) {
+        R65C02_REGS_SET_PC(&(drv->cpu->cpu_R65C02_regs), drv->trapcont);
+        if (drv->idling_method == DRIVE_IDLE_TRAP_IDLE) {
             CLOCK next_clk;
 
             next_clk = alarm_context_next_pending_clk(drv->cpu->alarm_context);
@@ -353,7 +357,7 @@ inline static int interrupt_check_irq_delay(interrupt_cpu_status_t *cs,
 
 /* Execute up to the current main CPU clock value.  This automatically
    calculates the corresponding number of clock ticks in the drive.  */
-void drivecpu65c02_execute(drive_context_t *drv, CLOCK clk_value)
+void drivecpu65c02_execute(diskunit_context_t *drv, CLOCK clk_value)
 {
     CLOCK cycles;
     int tcycles;
@@ -438,10 +442,10 @@ void drivecpu65c02_execute(drive_context_t *drv, CLOCK clk_value)
 
 static void drivecpu65c02_set_bank_base(void *context)
 {
-    drive_context_t *drv;
+    diskunit_context_t *drv;
     drivecpu_context_t *cpu;
 
-    drv = (drive_context_t *)context;
+    drv = (diskunit_context_t *)context;
     cpu = drv->cpu;
 
     JUMP(reg_pc);
@@ -452,7 +456,7 @@ static void drivecpu65c02_set_bank_base(void *context)
 #define SNAP_MAJOR 1
 #define SNAP_MINOR 1
 
-int drivecpu65c02_snapshot_write_module(drive_context_t *drv, snapshot_t *s)
+int drivecpu65c02_snapshot_write_module(diskunit_context_t *drv, snapshot_t *s)
 {
     snapshot_module_t *m;
     drivecpu_context_t *cpu;
@@ -486,9 +490,15 @@ int drivecpu65c02_snapshot_write_module(drive_context_t *drv, snapshot_t *s)
         goto fail;
     }
 
-    if (drv->drive->type == DRIVE_TYPE_2000
-        || drv->drive->type == DRIVE_TYPE_4000) {
-        if (SMW_BA(m, drv->drive->drive_ram, 0x2000) < 0) {
+    if (drv->type == DRIVE_TYPE_2000
+        || drv->type == DRIVE_TYPE_4000) {
+        if (SMW_BA(m, drv->drive_ram, 0x2000) < 0) {
+            goto fail;
+        }
+    }
+
+    if (drv->type == DRIVE_TYPE_CMDHD) {
+        if (SMW_BA(m, drv->drive_ram, 0x10000) < 0) {
             goto fail;
         }
     }
@@ -506,7 +516,7 @@ fail:
     return -1;
 }
 
-int drivecpu65c02_snapshot_read_module(drive_context_t *drv, snapshot_t *s)
+int drivecpu65c02_snapshot_read_module(diskunit_context_t *drv, snapshot_t *s)
 {
     uint8_t major, minor;
     snapshot_module_t *m;
@@ -549,7 +559,7 @@ int drivecpu65c02_snapshot_read_module(drive_context_t *drv, snapshot_t *s)
     R65C02_REGS_SET_PC(&(cpu->cpu_R65C02_regs), pc);
     R65C02_REGS_SET_STATUS(&(cpu->cpu_R65C02_regs), status);
 
-    log_message(drv->drive->log, "RESET (For undump).");
+    log_message(drv->log, "RESET (For undump).");
 
     interrupt_cpu_status_reset(cpu->int_status);
 
@@ -559,9 +569,15 @@ int drivecpu65c02_snapshot_read_module(drive_context_t *drv, snapshot_t *s)
         goto fail;
     }
 
-    if (drv->drive->type == DRIVE_TYPE_2000
-        || drv->drive->type == DRIVE_TYPE_4000) {
-        if (SMR_BA(m, drv->drive->drive_ram, 0x2000) < 0) {
+    if (drv->type == DRIVE_TYPE_2000
+        || drv->type == DRIVE_TYPE_4000) {
+        if (SMR_BA(m, drv->drive_ram, 0x2000) < 0) {
+            goto fail;
+        }
+    }
+
+    if (drv->type == DRIVE_TYPE_CMDHD) {
+        if (SMR_BA(m, drv->drive_ram, 0x10000) < 0) {
             goto fail;
         }
     }
