@@ -34,7 +34,7 @@ extern void drive_image_init(void);
 extern void drive_image_init_track_size_d64(struct drive_s *drive);
 extern int drive_check_image_format(unsigned int format, unsigned int dnr);
 
-extern int drive_image_attach(struct disk_image_s *image, unsigned int unit);
-extern int drive_image_detach(struct disk_image_s *image, unsigned int unit);
+extern int drive_image_attach(struct disk_image_s *image, unsigned int unit, unsigned int drive);
+extern int drive_image_detach(struct disk_image_s *image, unsigned int unit, unsigned int drive);
 
 #endif
