@@ -30,6 +30,7 @@
 #include "types.h"
 
 struct drive_s;
+struct diskunit_context_s;
 struct snapshot_s;
 
 #define DRIVE_ROM1540_SIZE            0x4000
@@ -45,14 +46,16 @@ struct snapshot_s;
 #define DRIVE_ROM1581_SIZE            0x8000
 #define DRIVE_ROM2000_SIZE            0x8000
 #define DRIVE_ROM4000_SIZE            0x8000
+#define DRIVE_ROMCMDHD_SIZE           0x4000
 #define DRIVE_ROM2031_SIZE            0x4000
 #define DRIVE_ROM1001_SIZE            0x4000 /* same as ROM8050 and ROM8250 !*/
+#define DRIVE_ROM9000_SIZE            0x4000
 #define DRIVE_ROM2040_SIZE            0x2000
 #define DRIVE_ROM3040_SIZE            0x3000
 #define DRIVE_ROM4040_SIZE            0x3000
 
 extern void driverom_init(void);
-extern void driverom_initialize_traps(struct drive_s *drive);
+extern void driverom_initialize_traps(struct diskunit_context_s *drive);
 extern int driverom_load(const char *resource_name, uint8_t *drive_rom, unsigned
                          int *loaded, int min, int max, const char *name,
                          unsigned int type, unsigned int *size);
