@@ -28,12 +28,11 @@
 #define VICE_DRIVESYNC_H
 
 struct drive_s;
-struct drive_context_s;
+struct diskunit_context_s;
 
-extern void drivesync_factor(struct drive_context_s *drv);
-extern void drivesync_set_1571(int new_sync, struct drive_context_s *drv);
-extern void drivesync_set_4000(struct drive_context_s *drv, int new_sync);
-extern void drivesync_clock_frequency(unsigned int type,
-                                      struct drive_s *drive);
+extern void drivesync_factor(struct diskunit_context_s *drv);
+extern void drivesync_set_1571(struct diskunit_context_s *drv, int new_sync);
+extern void drivesync_set_4000(struct diskunit_context_s *drv, int new_sync);
+extern void drivesync_clock_frequency(struct diskunit_context_s *drive, unsigned int type);
 
 #endif
