@@ -46,8 +46,8 @@
 #define debug(_x_)
 #endif
 
-#define STEP_RATE ((16 - drv->step_rate) * drv->mycontext->drive->clock_frequency * 500000 / drv->rate)
-#define BYTE_RATE (drv->mycontext->drive->clock_frequency * 8000 / drv->rate)
+#define STEP_RATE ((16 - drv->step_rate) * drv->mycontext->clock_frequency * 500000 / drv->rate)
+#define BYTE_RATE (drv->mycontext->clock_frequency * 8000 / drv->rate)
 
 typedef enum pc8477_state_e {
     PC8477_WAIT, PC8477_COMMAND, PC8477_READ, PC8477_WRITE, PC8477_EXEC, PC8477_RESULT
@@ -143,7 +143,7 @@ struct pc8477_s {
     pc8477_cmd_t command;
     pc8477_state_t state;
     int int_step, sub_step;
-    struct drive_context_s *mycontext;
+    struct diskunit_context_s *mycontext;
 
     /* Floppy drives */
     struct {
@@ -173,7 +173,7 @@ struct pc8477_s {
     int rate;
 
     int sector; /* sector register */
-    int is8477; /* dp8473 or pc 8477 */
+    int is8477; /* dp8473=0 or pc8477!=0 */
 
     alarm_t *seek_alarm;
     int byte_count;
@@ -204,7 +204,11 @@ static void seek_alarm_handler(CLOCK offset, void *data)
             drv->fdds[i].seeking = 1;
             if (drv->fdds[i].recalibrating && drv->fdds[i].seek_pulses == 0
                 && !fdd_track0(drv->fdds[i].fdd)) {
-                drv->st[0] |= PC8477_ST0_EC;
+                /* CMD FD2000's mechanism also sets bit 5 and 6 on
+                   recalibration timeout, it is hard coded in ROM to check for
+                   $71 on result (@ $878a) and try again */
+                /* The FD4000 uses 85 pulses, so this shouldn't timeout */
+                drv->st[0] |= (PC8477_ST0_EC | 0x60);
             }
             break;
         }
@@ -240,7 +244,7 @@ static void clk_overflow_callback(CLOCK sub, void *data)
 }
 
 /* Functions using drive context.  */
-void pc8477d_init(drive_context_t *drv)
+void pc8477d_init(diskunit_context_t *drv)
 {
     char *name;
 
@@ -255,7 +259,7 @@ void pc8477d_init(drive_context_t *drv)
     lib_free(name);
 }
 
-void pc8477_setup_context(drive_context_t *drv)
+void pc8477_setup_context(diskunit_context_t *drv)
 {
     int i;
     drv->pc8477 = lib_calloc(1, sizeof(pc8477_t));
@@ -268,7 +272,7 @@ void pc8477_setup_context(drive_context_t *drv)
     }
     drv->pc8477->fdds[0].motor_on = (pc8477_motor_on_callback_t)drivesync_set_4000;
     drv->pc8477->fdds[0].motor_on_data = (void *)drv;
-    drv->pc8477->fdds[1].fdd = fdd_init(1, drv->drive);
+    drv->pc8477->fdds[1].fdd = fdd_init(1, drv->drives[0]);
     drv->pc8477->fdds[1].motor_on = (pc8477_motor_on_callback_t)fdd_set_motor;
     drv->pc8477->fdds[1].motor_on_data = (void *)drv->pc8477->fdds[1].fdd;
     drv->pc8477->mycontext = drv;
@@ -498,7 +502,8 @@ static pc8477_state_t pc8477_execute(pc8477_t *drv)
             return PC8477_EXEC;
         case PC8477_CMD_RECALIBRATE:
             debug((pc8477_log, "RECALIBRATE #%d", drv->current->num));
-            drv->current->seek_pulses = drv->is8477 ? -77 : -85;
+            /* the pc8477 returns -85, the dp8473 returns -77 */
+            drv->current->seek_pulses = drv->is8477 ? -85 : -77;
             drv->current->track = 0;
             drv->current->recalibrating = 1;
             if (!drv->seeking_active) {
@@ -1259,17 +1264,17 @@ int pc8477_irq(pc8477_t *drv)
     return drv->irq;
 }
 
-void pc8477d_store(drive_context_t *drv, uint16_t addr, uint8_t byte)
+void pc8477d_store(diskunit_context_t *drv, uint16_t addr, uint8_t byte)
 {
     pc8477_store(drv->pc8477, (uint16_t)(addr & 7), byte);
 }
 
-uint8_t pc8477d_read(drive_context_t *drv, uint16_t addr)
+uint8_t pc8477d_read(diskunit_context_t *drv, uint16_t addr)
 {
     return pc8477_read(drv->pc8477, (uint16_t)(addr & 7));
 }
 
-uint8_t pc8477d_peek(drive_context_t *drv, uint16_t addr)
+uint8_t pc8477d_peek(diskunit_context_t *drv, uint16_t addr)
 {
     return pc8477_peek(drv->pc8477, (uint16_t)(addr & 7));
 }
@@ -1278,7 +1283,7 @@ uint8_t pc8477d_peek(drive_context_t *drv, uint16_t addr)
 
 int pc8477_attach_image(disk_image_t *image, unsigned int unit)
 {
-    if (unit < 8 || unit > 8 + DRIVE_NUM) {
+    if (unit < 8 || unit > 8 + NUM_DISK_UNITS) {
         return -1;
     }
 
@@ -1287,19 +1292,19 @@ int pc8477_attach_image(disk_image_t *image, unsigned int unit)
         case DISK_IMAGE_TYPE_D1M:
         case DISK_IMAGE_TYPE_D2M:
         case DISK_IMAGE_TYPE_D4M:
-            disk_image_attach_log(image, pc8477_log, unit);
+            disk_image_attach_log(image, pc8477_log, unit, 0);
             break;
         default:
             return -1;
     }
 
-    fdd_image_attach(drive_context[unit - 8]->pc8477->fdds[1].fdd, image);
+    fdd_image_attach(diskunit_context[unit - 8]->pc8477->fdds[1].fdd, image);
     return 0;
 }
 
 int pc8477_detach_image(disk_image_t *image, unsigned int unit)
 {
-    if (image == NULL || unit < 8 || unit > 8 + DRIVE_NUM) {
+    if (image == NULL || unit < 8 || unit > 8 + NUM_DISK_UNITS) {
         return -1;
     }
 
@@ -1308,12 +1313,12 @@ int pc8477_detach_image(disk_image_t *image, unsigned int unit)
         case DISK_IMAGE_TYPE_D1M:
         case DISK_IMAGE_TYPE_D2M:
         case DISK_IMAGE_TYPE_D4M:
-            disk_image_detach_log(image, pc8477_log, unit);
+            disk_image_detach_log(image, pc8477_log, unit, 0);
             break;
         default:
             return -1;
     }
 
-    fdd_image_detach(drive_context[unit - 8]->pc8477->fdds[1].fdd);
+    fdd_image_detach(diskunit_context[unit - 8]->pc8477->fdds[1].fdd);
     return 0;
 }
