@@ -30,18 +30,18 @@
 #include "types.h"
 
 struct disk_image_s;
-struct drive_context_s;
+struct diskunit_context_s;
 typedef void (*pc8477_motor_on_callback_t)(void *data, int signal);
 typedef struct pc8477_s pc8477_t;
 
 /* FIXME: whats the deal with the different prefixes? */
-extern void pc8477d_init(struct drive_context_s *drv);
+extern void pc8477d_init(struct diskunit_context_s *drv);
 extern void pc8477_shutdown(pc8477_t *drv);
 
-extern void pc8477_setup_context(struct drive_context_s *drv);
-extern void pc8477d_store(struct drive_context_s *drv, uint16_t addr, uint8_t byte);
-extern uint8_t pc8477d_read(struct drive_context_s *drv, uint16_t addr);
-extern uint8_t pc8477d_peek(struct drive_context_s *drv, uint16_t addr);
+extern void pc8477_setup_context(struct diskunit_context_s *drv);
+extern void pc8477d_store(struct diskunit_context_s *drv, uint16_t addr, uint8_t byte);
+extern uint8_t pc8477d_read(struct diskunit_context_s *drv, uint16_t addr);
+extern uint8_t pc8477d_peek(struct diskunit_context_s *drv, uint16_t addr);
 extern void pc8477_reset(pc8477_t *drv, int is8477);
 extern int pc8477_irq(pc8477_t *drv);
 
