@@ -56,6 +56,7 @@ static const int wd1770_step_rate[2][4] = {
     {6000, 12000, 2000, 3000},   /* WD1772 */
 };
 
+/* Macros for wd1770_t *drv; */
 #define SETTLING (drv->clock_frequency * 30000)
 #define BYTE_RATE (drv->clock_frequency * 8000 / 250)
 #define STEP_RATE (drv->clock_frequency * wd1770_step_rate[drv->is1772][drv->cmd & WD_R])
@@ -180,8 +181,8 @@ static void clk_overflow_callback(CLOCK sub, void *data)
     }
 }
 
-/* Functions using drive context.  */
-void wd1770d_init(drive_context_t *drv)
+/* Functions using disk unit context.  */
+void wd1770d_init(diskunit_context_t *drv)
 {
     if (wd1770_log == LOG_ERR) {
         wd1770_log = log_open("WD1770");
@@ -189,10 +190,17 @@ void wd1770d_init(drive_context_t *drv)
 
     drv->wd1770 = lib_calloc(1, sizeof(wd1770_t));
     drv->wd1770->myname = lib_msprintf("WD1770%d", drv->mynumber);
-    drv->wd1770->fdd = fdd_init(4 * drv->mynumber, drv->drive);
+    drv->wd1770->fdd = fdd_init(4 * drv->mynumber, drv->drives[0]);
     drv->wd1770->cpu_clk_ptr = drv->clk_ptr;
     drv->wd1770->is1772 = 0;
     drv->wd1770->clock_frequency = 2;
+    /*
+     * Programming note: we should be able to get the clock_frequency from
+     * drv->clock_frequency. However we are called only once at the beginning,
+     * for any type of drive, and not when a drive of our type is actually
+     * connected. It just so happens that all drives with a wd177x have
+     * this clock frequency.
+     */
 
     clk_guard_add_callback(drv->cpu->clk_guard, clk_overflow_callback,
                            drv->wd1770);
@@ -942,39 +950,39 @@ void wd1770_reset(wd1770_t *drv)
 
 int wd1770_attach_image(disk_image_t *image, unsigned int unit)
 {
-    if (unit < 8 || unit > 8 + DRIVE_NUM) {
+    if (unit < 8 || unit > 8 + NUM_DISK_UNITS) {
         return -1;
     }
 
     switch (image->type) {
         case DISK_IMAGE_TYPE_D81:
         case DISK_IMAGE_TYPE_D1M:
-            disk_image_attach_log(image, wd1770_log, unit);
+            disk_image_attach_log(image, wd1770_log, unit, 0);
             break;
         default:
             return -1;
     }
 
-    fdd_image_attach(drive_context[unit - 8]->wd1770->fdd, image);
+    fdd_image_attach(diskunit_context[unit - 8]->wd1770->fdd, image);
     return 0;
 }
 
 int wd1770_detach_image(disk_image_t *image, unsigned int unit)
 {
-    if (image == NULL || unit < 8 || unit > 8 + DRIVE_NUM) {
+    if (image == NULL || unit < 8 || unit > 8 + NUM_DISK_UNITS) {
         return -1;
     }
 
     switch (image->type) {
         case DISK_IMAGE_TYPE_D81:
         case DISK_IMAGE_TYPE_D1M:
-            disk_image_detach_log(image, wd1770_log, unit);
+            disk_image_detach_log(image, wd1770_log, unit, 0);
             break;
         default:
             return -1;
     }
 
-    fdd_image_detach(drive_context[unit - 8]->wd1770->fdd);
+    fdd_image_detach(diskunit_context[unit - 8]->wd1770->fdd);
     return 0;
 }
 
@@ -993,17 +1001,17 @@ int wd1770_disk_change(wd1770_t *drv)
     return fdd_disk_change(drv->fdd);
 }
 
-void wd1770d_store(drive_context_t *drv, uint16_t addr, uint8_t byte)
+void wd1770d_store(diskunit_context_t *drv, uint16_t addr, uint8_t byte)
 {
     wd1770_store(drv->wd1770, (uint16_t)(addr & 3), byte);
 }
 
-uint8_t wd1770d_read(drive_context_t *drv, uint16_t addr)
+uint8_t wd1770d_read(diskunit_context_t *drv, uint16_t addr)
 {
     return wd1770_read(drv->wd1770, (uint16_t)(addr & 3));
 }
 
-uint8_t wd1770d_peek(drive_context_t *drv, uint16_t addr)
+uint8_t wd1770d_peek(diskunit_context_t *drv, uint16_t addr)
 {
     return wd1770_peek(drv->wd1770, (uint16_t)(addr & 3));
 }
