@@ -30,14 +30,12 @@
 
 #include "cmdline.h"
 #include "iec128dcr-cmdline-options.h"
-#include "translate.h"
 
-static const cmdline_option_t cmdline_options[] = {
-    { "-dos1571cr", SET_RESOURCE, 1,
+static const cmdline_option_t cmdline_options[] =
+{
+    { "-dos1571cr", SET_RESOURCE, CMDLINE_ATTRIB_NEED_ARGS,
       NULL, NULL, "DosName1571cr", NULL,
-      USE_PARAM_ID, USE_DESCRIPTION_ID,
-      IDCLS_P_NAME, IDCLS_SPECIFY_1571CR_DOS_ROM_NAME,
-      NULL, NULL },
+      "<Name>", "Specify name of 1571CR DOS ROM image" },
     CMDLINE_LIST_END
 };
 
