@@ -53,6 +53,7 @@ static uint8_t drive_rom1001[DRIVE_ROM1001_SIZE];
 static uint8_t drive_rom2040[DRIVE_ROM2040_SIZE];
 static uint8_t drive_rom3040[DRIVE_ROM3040_SIZE];
 static uint8_t drive_rom4040[DRIVE_ROM4040_SIZE];
+static uint8_t drive_rom9000[DRIVE_ROM9000_SIZE];
 #endif
 
 /* If nonzero, the ROM image has been loaded.  */
@@ -61,6 +62,7 @@ static unsigned int rom2040_loaded = 0;
 static unsigned int rom3040_loaded = 0;
 static unsigned int rom4040_loaded = 0;
 static unsigned int rom1001_loaded = 0;
+static unsigned int rom9000_loaded = 0;
 
 
 int ieeerom_load_2031(void)
@@ -98,32 +100,43 @@ int ieeerom_load_1001(void)
             DRIVE_TYPE_1001, NULL);
 }
 
-void ieeerom_setup_image(drive_t *drive)
+int ieeerom_load_9000(void)
+{
+    return driverom_load("DosName9000", drive_rom9000, &rom9000_loaded,
+            DRIVE_ROM9000_SIZE, DRIVE_ROM9000_SIZE, "D9090/9060",
+            DRIVE_TYPE_9000, NULL);
+}
+
+void ieeerom_setup_image(diskunit_context_t *unit)
 {
     if (rom_loaded) {
-        switch (drive->type) {
+        switch (unit->type) {
             case DRIVE_TYPE_2031:
-                memcpy(&(drive->rom[0x4000]), drive_rom2031,
+                memcpy(&(unit->rom[0x4000]), drive_rom2031,
                        DRIVE_ROM2031_SIZE);
                 break;
             case DRIVE_TYPE_2040:
-                memcpy(&(drive->rom[DRIVE_ROM_SIZE - DRIVE_ROM2040_SIZE]),
+                memcpy(&(unit->rom[DRIVE_ROM_SIZE - DRIVE_ROM2040_SIZE]),
                        drive_rom2040, DRIVE_ROM2040_SIZE);
                 break;
             case DRIVE_TYPE_3040:
-                memcpy(&(drive->rom[DRIVE_ROM_SIZE - DRIVE_ROM3040_SIZE]),
+                memcpy(&(unit->rom[DRIVE_ROM_SIZE - DRIVE_ROM3040_SIZE]),
                        drive_rom3040, DRIVE_ROM3040_SIZE);
                 break;
             case DRIVE_TYPE_4040:
-                memcpy(&(drive->rom[DRIVE_ROM_SIZE - DRIVE_ROM4040_SIZE]),
+                memcpy(&(unit->rom[DRIVE_ROM_SIZE - DRIVE_ROM4040_SIZE]),
                        drive_rom4040, DRIVE_ROM4040_SIZE);
                 break;
             case DRIVE_TYPE_1001:
             case DRIVE_TYPE_8050:
             case DRIVE_TYPE_8250:
-                memcpy(&(drive->rom[0x4000]), drive_rom1001,
+                memcpy(&(unit->rom[0x4000]), drive_rom1001,
                        DRIVE_ROM1001_SIZE);
                 break;
+            case DRIVE_TYPE_9000:
+                memcpy(&(unit->rom[0x4000]), drive_rom9000,
+                       DRIVE_ROM9000_SIZE);
+                break;
         }
     }
 }
@@ -160,9 +173,15 @@ int ieeerom_check_loaded(unsigned int type)
                 return -1;
             }
             break;
+        case DRIVE_TYPE_9000:
+            if (rom9000_loaded < 1 && rom_loaded) {
+                return -1;
+            }
+            break;
         case DRIVE_TYPE_ANY:
             if ((!rom2031_loaded && !rom2040_loaded && !rom3040_loaded
-                 && !rom4040_loaded && !rom1001_loaded) && rom_loaded) {
+                 && !rom4040_loaded && !rom1001_loaded && !rom9000_loaded )
+                 && rom_loaded) {
                 return -1;
             }
             break;
