@@ -58,12 +58,12 @@ int tcbmrom_load_1551(void)
             DRIVE_TYPE_1551, NULL);
 }
 
-void tcbmrom_setup_image(drive_t *drive)
+void tcbmrom_setup_image(diskunit_context_t *unit)
 {
     if (rom_loaded) {
-        switch (drive->type) {
+        switch (unit->type) {
             case DRIVE_TYPE_1551:
-                memcpy(&(drive->rom[0x4000]), drive_rom1551,
+                memcpy(&(unit->rom[0x4000]), drive_rom1551,
                        DRIVE_ROM1551_SIZE);
                 break;
         }
