@@ -30,16 +30,16 @@
 
 #include "types.h"
 
-struct drive_context_s;
+struct diskunit_context_s;
 struct drive_s;
 
-extern void via2d_setup_context(struct drive_context_s *ctxptr);
+extern void via2d_setup_context(struct diskunit_context_s *ctxptr);
 
-extern void via2d_init(struct drive_context_s *ctxptr);
-extern void via2d_store(struct drive_context_s *ctxptr, uint16_t addr, uint8_t byte);
-extern uint8_t via2d_read(struct drive_context_s *ctxptr, uint16_t addr);
-extern uint8_t via2d_peek(struct drive_context_s *ctxptr, uint16_t addr);
-extern int via2d_dump(struct drive_context_s *ctxptr, uint16_t addr);
+extern void via2d_init(struct diskunit_context_s *ctxptr);
+extern void via2d_store(struct diskunit_context_s *ctxptr, uint16_t addr, uint8_t byte);
+extern uint8_t via2d_read(struct diskunit_context_s *ctxptr, uint16_t addr);
+extern uint8_t via2d_peek(struct diskunit_context_s *ctxptr, uint16_t addr);
+extern int via2d_dump(struct diskunit_context_s *ctxptr, uint16_t addr);
 
 extern void via2d_update_pcr(int pcrval, struct drive_s *dptr);
 
