@@ -30,17 +30,19 @@
 #include "types.h"
 
 #define FILEIO_COMMAND_READ        0
-#define FILEIO_COMMAND_WRITE       1
-#define FILEIO_COMMAND_APPEND      2
-#define FILEIO_COMMAND_APPEND_READ 3
-#define FILEIO_COMMAND_STAT        4  /* works with directories */
+#define FILEIO_COMMAND_READ_WRITE  1
+#define FILEIO_COMMAND_WRITE       2  /* Will not overwrite existing file */
+#define FILEIO_COMMAND_OVERWRITE   3  /* Overwrites existing file, if any */
+#define FILEIO_COMMAND_APPEND      4
+#define FILEIO_COMMAND_APPEND_READ 5
+#define FILEIO_COMMAND_STAT        6  /* works with directories */
 #define FILEIO_COMMAND_MASK        15
 #define FILEIO_COMMAND_FSNAME      16
 
 #define FILEIO_FORMAT_RAW (1 << 0)
 #define FILEIO_FORMAT_P00 (1 << 1)
 
-#define FILEIO_TYPE_DEL 0
+#define FILEIO_TYPE_DEL 0       /* should match CBMDOS_FT_xxx */
 #define FILEIO_TYPE_SEQ 1
 #define FILEIO_TYPE_PRG 2
 #define FILEIO_TYPE_USR 3
@@ -68,7 +70,7 @@ typedef struct fileio_info_s fileio_info_t;
 
 extern fileio_info_t *fileio_open(const char *file_name, const char *path,
                                   unsigned int format, unsigned int command,
-                                  unsigned int type);
+                                  unsigned int type, int *reclenp);
 extern void fileio_close(fileio_info_t *info);
 extern unsigned int fileio_read(fileio_info_t *info, uint8_t *buf, unsigned int len);
 extern unsigned int fileio_write(fileio_info_t *info, uint8_t *buf, unsigned int len);
@@ -78,5 +80,7 @@ extern unsigned int fileio_rename(const char *src_name, const char *dest_name,
 extern unsigned int fileio_scratch(const char *file_name, const char *path,
                                    unsigned int format);
 extern unsigned int fileio_get_bytes_left(fileio_info_t *info);
+extern unsigned int fileio_seek(fileio_info_t *info, off_t offset, int whence);
+extern unsigned int fileio_tell(fileio_info_t *info);
 
 #endif
