@@ -54,7 +54,7 @@ struct fliplist_s {
     unsigned int unit;
 };
 
-static fliplist_t fliplist[DRIVE_NUM] = {
+static fliplist_t fliplist[NUM_DISK_UNITS] = {
     (fliplist_t)NULL,
     (fliplist_t)NULL
 };
@@ -112,7 +112,7 @@ void fliplist_resources_shutdown(void)
 {
     int i;
 
-    for (i = 0; i < DRIVE_NUM; i++) {
+    for (i = 0; i < NUM_DISK_UNITS; i++) {
         fliplist_clear_list(8 + i);
     }
 
@@ -291,7 +291,9 @@ bool fliplist_attach_head(unsigned int unit, int direction)
         fliplist[unit - 8] = fliplist[unit - 8]->prev;
     }
 
-    if (file_system_attach_disk(fliplist[unit - 8]->unit,
+    if (file_system_attach_disk(
+                fliplist[unit - 8]->unit,
+                0,  /* drive */
                 fliplist[unit - 8]->image) < 0) {
         /* shouldn't happen, so ignore it */
         return false;   /* handle it anyway */
@@ -380,7 +382,7 @@ int fliplist_save_list(unsigned int unit, const char *filename)
             while (flip != fliplist[unit - 8]);
         }
         unit++;
-    } while (all_units && ((unit - 8) < DRIVE_NUM));
+    } while (all_units && ((unit - 8) < NUM_DISK_UNITS));
 
     if (fp) {
         fclose(fp);
@@ -412,7 +414,7 @@ int fliplist_load_list(unsigned int unit, const char *filename, int autoattach)
     }
     if (unit == FLIPLIST_ALL_UNITS) {
         all_units = 1;
-        for (i = 0; i < DRIVE_NUM; i++) {
+        for (i = 0; i < NUM_DISK_UNITS; i++) {
             fliplist_clear_list(i + 8);
         }
     } else {
@@ -486,7 +488,7 @@ int fliplist_load_list(unsigned int unit, const char *filename, int autoattach)
         current_drive = unit;
 
         if (all_units) {
-            for (i = 0; i < DRIVE_NUM; i++) {
+            for (i = 0; i < NUM_DISK_UNITS; i++) {
                 show_fliplist(i + 8);
             }
         } else {
