@@ -3,6 +3,7 @@
  *
  * Written by
  *  Marco van den Heuvel <blackystardust68@yahoo.com>
+ *  groepaz <groepaz@gmx.net>
  *
  * This file is part of VICE, the Versatile Commodore Emulator.
  * See README for copyright notice.
@@ -45,6 +46,25 @@
 #include "util.h"
 #include "vsync.h"
 
+#define DEBUGNATIVEDRV
+
+#ifdef  DEBUGNATIVEDRV
+#define DBG(_x_)    printf _x_
+#else
+#define DBG(_x_)
+#endif
+
+/*
+    TODO:
+
+    disable debug output
+    test vic screenshots, fix automatic positioning of the gfx window
+    test vdc screenshots, fix automatic positioning of the gfx window
+    test crts screenshots, fix automatic positioning of the gfx window
+    when all is done, remove #if 0'ed code
+*/
+
+#if 0
 void native_smooth_scroll_borderize_colormap(native_data_t *source, uint8_t bordercolor, uint8_t xcover, uint8_t ycover)
 {
     int i, j, k;
@@ -55,6 +75,9 @@ void native_smooth_scroll_borderize_colormap(native_data_t *source, uint8_t bord
     int ysize;
     int yendamount = 0;
 
+    DBG(("native_smooth_scroll_borderize_colormap bordercolor: %d xcover: %d ycover: %d\n",
+         bordercolor, xcover, ycover));
+
     if (xcover == 255) {
         xstart = 0;
         xsize = source->xsize;
@@ -106,6 +129,7 @@ void native_smooth_scroll_borderize_colormap(native_data_t *source, uint8_t bord
         }
     }
 }
+#endif
 
 native_data_t *native_borderize_colormap(native_data_t *source, uint8_t bordercolor, int xsize, int ysize)
 {
@@ -116,6 +140,9 @@ native_data_t *native_borderize_colormap(native_data_t *source, uint8_t borderco
     int yendamount = 0;
     native_data_t *dest = lib_malloc(sizeof(native_data_t));
 
+    DBG(("native_borderize_colormap bordercolor: %d xsize: %d ysize: %d\n",
+         bordercolor, xsize, ysize));
+
     dest->filename = source->filename;
 
     if (source->xsize < xsize) {
@@ -186,6 +213,9 @@ native_data_t *native_crop_and_borderize_colormap(native_data_t *source, uint8_t
     int i, j, k, l;
     native_data_t *dest = lib_malloc(sizeof(native_data_t));
 
+    DBG(("native_crop_and_borderize_colormap bordercolor: %d xsize: %d ysize: %d oversize handling: %d\n",
+         bordercolor, xsize, ysize, oversize_handling));
+
     dest->filename = source->filename;
 
     startx = (xsize - source->xsize) / 2;
@@ -316,6 +346,9 @@ native_data_t *native_scale_colormap(native_data_t *source, int xsize, int ysize
     int i, j;
     int xmult, ymult;
 
+    DBG(("native_scale_colormap xsize: %d ysize: %d\n", xsize, ysize));
+    DBG(("               source xsize: %d ysize: %d\n", source->xsize, source->ysize));
+
     dest->filename = source->filename;
 
     dest->xsize = xsize;
@@ -328,7 +361,8 @@ native_data_t *native_scale_colormap(native_data_t *source, int xsize, int ysize
 
     for (i = 0; i < ysize; i++) {
         for (j = 0; j < xsize; j++) {
-            dest->colormap[(i * xsize) + j] = source->colormap[(((i * ymult) >> 8) * source->xsize) + ((j * xmult) >> 8)];
+            dest->colormap[(i * xsize) + j] =
+                source->colormap[(((i * ymult) >> 8) * source->xsize) + ((j * xmult) >> 8)];
         }
     }
 
@@ -338,11 +372,15 @@ native_data_t *native_scale_colormap(native_data_t *source, int xsize, int ysize
     return dest;
 }
 
+/* scale and/or crop and/or borderize according to the options */
 native_data_t *native_resize_colormap(native_data_t *source, int xsize, int ysize, uint8_t bordercolor, int oversize_handling, int undersize_handling)
 {
     native_data_t *data = source;
     int mc_data_present = source->mc_data_present;
 
+    DBG(("native_crop_and_borderize_colormap bordercolor: %d xsize: %d ysize: %d oversize handling: %d undersize handling: %d\n",
+         bordercolor, xsize, ysize, oversize_handling, undersize_handling));
+
     if (data->xsize > xsize) {
         if (oversize_handling == NATIVE_SS_OVERSIZE_SCALE) {
             data = native_scale_colormap(data, xsize, data->ysize);
@@ -418,6 +456,42 @@ native_color_sort_t *native_sort_colors_colormap(native_data_t *source, int colo
     return colors;
 }
 
+/* returns 1 if any 8x8 cell contains more than two colors */
+int native_is_colormap_multicolor(native_data_t *source)
+{
+    int blocksx = source->xsize / 8;
+    int blocksy = source->ysize / 8;
+    int i, j, k, l;
+    int multicolor = 0;
+    native_data_t *dest = lib_malloc(sizeof(native_data_t));
+    native_color_sort_t *colors = NULL;
+
+    dest->xsize = 8;
+    dest->ysize = 8;
+    dest->colormap = lib_malloc(8 * 8);
+
+    for (i = 0; (i < blocksy) && (multicolor == 0); i++) {
+        for (j = 0; (j < blocksx) && (multicolor == 0); j++) {
+            /* get block */
+            for (k = 0; k < 8; k++) {
+                for (l = 0; l < 8; l++) {
+                    dest->colormap[(k * 8) + l] = 
+                    source->colormap[(i * 8 * source->xsize) + (j * 8)
+                                        + (k * source->xsize) + l];
+                }
+            }
+            colors = native_sort_colors_colormap(dest, 16);
+            if (colors[2].amount > 0) {
+                multicolor = 1;
+            }
+            lib_free(colors);
+        }
+    }
+    lib_free(dest->colormap);
+    lib_free(dest);
+    return multicolor;
+}
+
 static uint8_t vicii_color_bw_translate[16] = {
     0,    /* vicii black       (0) -> vicii black (0) */
     1,    /* vicii white       (1) -> vicii white (1) */
@@ -538,6 +612,7 @@ static uint8_t vicii_closest_color[16][16] = {
     { 15, 13, 3, 12, 14, 10, 7, 1, 4, 5, 11, 8, 6, 9, 2, 0 }
 };
 
+/* altcolors[n].color == 255 marks the end of the list */
 static inline uint8_t vicii_color_to_nearest_color(uint8_t color, native_color_sort_t *altcolors)
 {
     int i, j;
@@ -558,13 +633,231 @@ void vicii_color_to_nearest_vicii_color_colormap(native_data_t *source, native_c
 
     for (i = 0; i < source->ysize; i++) {
         for (j = 0; j < source->xsize; j++) {
-            source->colormap[(i * source->xsize) + j] = vicii_color_to_nearest_color(source->colormap[(i * source->xsize) + j], colors);
+            source->colormap[(i * source->xsize) + j] =
+                vicii_color_to_nearest_color(source->colormap[(i * source->xsize) + j], colors);
         }
     }
 }
 
 /* ------------------------------------------------------------------------ */
 
+static native_data_t *native_generic_render(screenshot_t *screenshot, const char *filename, int xsize, int ysize)
+{
+    native_data_t *data = lib_malloc(sizeof(native_data_t));
+    int i, j;
+    int leftborder, topborder;
+    uint8_t *linebuffer;
+
+    DBG(("native_generic_render xsize: %d ysize: %d\n", xsize, ysize));
+    data->filename = filename;
+
+    /* size of the native picture */
+    data->xsize = xsize;
+    data->ysize = ysize;
+    data->colormap = lib_malloc(data->xsize * data->ysize);
+
+    linebuffer = lib_malloc(screenshot->width * screenshot->height);
+
+    DBG(("screenshot->width: %u\n", screenshot->width));
+    DBG(("screenshot->height: %u\n", screenshot->height));
+    DBG(("screenshot->max_width: %u\n", screenshot->max_width));
+    DBG(("screenshot->x_offset: %u\n", screenshot->x_offset));
+    DBG(("screenshot->first_displayed_line: %u\n", screenshot->first_displayed_line));
+    DBG(("screenshot->gfx_position.x %u\n", screenshot->gfx_position.x));
+    DBG(("screenshot->gfx_position.y %u\n", screenshot->gfx_position.y));
+
+    leftborder = screenshot->gfx_position.x;
+    topborder = screenshot->gfx_position.y - screenshot->first_displayed_line;
+
+    DBG(("                      leftborder: %d\n", leftborder));
+    DBG(("                      topborder: %d\n", topborder));
+
+    /* get screenshot data in palette format */
+    for (i = 0; i < data->ysize; i++) {
+        (screenshot->convert_line)(screenshot,
+                                   linebuffer + (i * screenshot->width),
+                                   i + topborder, SCREENSHOT_MODE_PALETTE);
+    }
+
+    /* create picture in the native size of the videochip, without border */
+    for (i = 0; i < data->ysize; i++) {
+        for (j = 0; j < data->xsize; j++) {
+            data->colormap[(i * data->xsize) + (j)] =
+            linebuffer[(i * screenshot->width) + (j + leftborder)]; 
+        }
+    }
+    data->mc_data_present = native_is_colormap_multicolor(data);
+    DBG(("                      mc_data_present: %d\n", data->mc_data_present));
+    return data;
+}
+
+native_data_t *native_vicii_render(screenshot_t *screenshot, const char *filename)
+{
+    native_data_t *data = native_generic_render(screenshot, filename, 320, 200);
+#if 0
+    uint8_t *regs = screenshot->video_regs;
+    if (((regs[0x16] & 8) == 0) || ((regs[0x11] & 8) == 0)) {
+        native_smooth_scroll_borderize_colormap(data, (uint8_t)(regs[0x20] & 0xf), (uint8_t)((regs[0x16] & 8) ? 255 : regs[0x16] & 7), (uint8_t)((regs[0x11] & 8) ? 255 : regs[0x11] & 7));
+    }
+#endif
+    return data;
+}
+
+native_data_t *native_ted_render(screenshot_t *screenshot, const char *filename)
+{
+    native_data_t *data = native_generic_render(screenshot, filename, 320, 200);
+#if 0
+    uint8_t *regs = screenshot->video_regs;
+    if (((regs[0x07] & 8) == 0) || ((regs[0x06] & 8) == 0)) {
+        native_smooth_scroll_borderize_colormap(data, brdrcolor, (uint8_t)((regs[0x07] & 8) ? 255 : regs[0x07] & 7), (uint8_t)((regs[0x06] & 8) ? 255 : regs[0x06] & 7));
+    }
+#endif
+    return data;
+}
+
+native_data_t *native_vic_render(screenshot_t *screenshot, const char *filename)
+{
+    native_data_t *data;
+    uint8_t *regs = screenshot->video_regs;
+    int xsize, ysize;
+    xsize = (regs[0x02] & 0x7f) * 16; /* columns */
+    ysize = ((regs[0x03] & 0x7e) >> 1) * 8; /* rows */
+    if (regs[0x03] & 1) {
+        ysize <<= 1; /* 16 pixel high chars */
+    }
+    data = native_generic_render(screenshot, filename, xsize, ysize);
+    return data;
+}
+
+/* FIXME:
+   screenshot->gfx_position.x/y does not match actual black border
+
+   "hre" and "petdww_ram" is broken
+*/
+native_data_t *native_crtc_render(screenshot_t *screenshot, const char *filename)
+{
+    native_data_t *data;
+    uint8_t *regs = screenshot->video_regs;
+    uint8_t *petdww_ram = screenshot->bitmap_ptr;
+    int xsize;
+    int ysize;
+    uint8_t invert;
+    uint8_t charheight, charwidth;
+    int hre = 0;
+#if 0
+    int base;
+    int chars = 1;
+    int col80;
+#endif
+
+    DBG(("screenshot->bitmap_low_ptr[0]: %d\n", screenshot->bitmap_low_ptr[0]));
+    DBG(("screenshot->bitmap_high_ptr[0]: %d\n", screenshot->bitmap_high_ptr[0]));
+    DBG(("1: %d\n", regs[0x01]));
+    DBG(("2: %d\n", regs[0x02]));
+    DBG(("3: %d\n", regs[0x03]));
+    DBG(("4: %d\n", regs[0x04]));
+    DBG(("5: %d\n", regs[0x05]));
+    DBG(("6: %d\n", regs[0x06]));
+    DBG(("7: %d\n", regs[0x07]));
+    DBG(("8: %d\n", regs[0x08]));
+    DBG(("9: %d\n", regs[0x09]));
+
+    xsize = regs[0x01];
+    if (screenshot->bitmap_low_ptr[0] == 80) {
+        xsize <<= 1;
+    }
+
+    ysize = regs[0x06];
+    invert = (regs[0x0c] & 0x10) >> 4;
+
+    if (!invert) {      /* On 8296 only! */
+        hre = 1;
+#if 0
+        chars = 0;
+#endif
+        invert = 1;
+    }
+
+    /* charheight = screenshot->bitmap_high_ptr[0]; */
+    charwidth = 8; /* is this correct? */
+    charheight = regs[0x09] + 1; /* FIXME: handle inter-character gap */
+
+    DBG(("chars x: %d w: %d\n", xsize, charwidth));
+    DBG(("chars y: %d h: %d\n", ysize, charheight));
+
+    if (hre) {
+        xsize = 512;
+        ysize = 256;
+    } else {
+        xsize = xsize * charwidth;
+        ysize = ysize * charheight;
+    }
+
+    if (petdww_ram) {
+        /* FIXME:
+        * If we're in an 80 column screen we need to
+        * horizontally double all pixels, since DWW
+        * only has 40 characters worth of pixels.
+        */
+    }
+    DBG(("xsize: %d\n", xsize));
+    DBG(("ysize: %d\n", ysize));
+
+    /* FIXME: this should be defined elsewhere in the crtc code */
+    if (regs[0x01] == 40) {
+        screenshot->gfx_position.y = 41;
+    } else {
+        screenshot->gfx_position.y = 28;
+    }
+    screenshot->gfx_position.x = 33;
+
+    data = native_generic_render(screenshot, filename, xsize, ysize);
+    return data;
+}
+
+/* FIXME:
+   screenshot->gfx_position.x/y does not match actual black border
+*/
+native_data_t *native_vdc_render(screenshot_t *screenshot, const char *filename)
+{
+    native_data_t *data;
+    uint8_t *regs = screenshot->video_regs;
+    int xsize, ysize;
+    uint8_t displayed_chars_h = regs[1];
+    uint8_t displayed_chars_v = regs[6];
+    uint8_t scanlines_per_char = (regs[9] & 0x1f) + 1;
+    /* uint8_t char_h_size_alloc = regs[22] & 0xf; */
+    uint8_t char_h_size_displayed = (regs[22] & 0xf0) >> 4;
+    uint8_t char_h_size = (regs[22] & 0x0f);
+    uint8_t char_h_double_pixel = ((regs[25] & 0x10) ? 2 : 1);
+    uint8_t top_border_chars_h;
+
+    DBG(("native_vdc_render displayed_chars_h: %u\n", displayed_chars_h));
+    DBG(("                  displayed_chars_v: %u\n", displayed_chars_v));
+    DBG(("                  char_h_size_displayed: %u\n", char_h_size_displayed));
+    DBG(("                  char_h_size: %u\n", char_h_size));
+    DBG(("                  scanlines_per_char: %u\n", scanlines_per_char));
+    DBG(("                  char_h_double_pixel: %u\n", char_h_double_pixel));
+    DBG(("                  3: %u\n", regs[3]));
+    DBG(("                  4: %u\n", regs[4]));
+    DBG(("                  5: %u\n", regs[5]));
+    DBG(("                  6: %u\n", regs[6]));
+
+    xsize = displayed_chars_h * char_h_size * char_h_double_pixel;
+    ysize = displayed_chars_v * scanlines_per_char;
+
+    DBG(("                  xsize: %d\n", xsize));
+    DBG(("                  ysize: %d\n", ysize));
+
+    /* FIXME: this should be set elsewhere in the vdc code */
+    top_border_chars_h = ((regs[4] - displayed_chars_v) / 2) - 1;   /* FIXME: is this correct? */
+    screenshot->gfx_position.y = top_border_chars_h * scanlines_per_char;
+
+    data = native_generic_render(screenshot, filename, xsize, ysize);
+    return data;
+}
+
+#if 0
 native_data_t *native_vicii_text_mode_render(screenshot_t *screenshot, const char *filename)
 {
     uint8_t *regs = screenshot->video_regs;
@@ -603,7 +896,9 @@ native_data_t *native_vicii_text_mode_render(screenshot_t *screenshot, const cha
     }
     return data;
 }
+#endif
 
+#if 0
 native_data_t *native_vicii_extended_background_mode_render(screenshot_t *screenshot, const char *filename)
 {
     uint8_t *regs = screenshot->video_regs;
@@ -641,7 +936,9 @@ native_data_t *native_vicii_extended_background_mode_render(screenshot_t *screen
     }
     return data;
 }
+#endif
 
+#if 0
 native_data_t *native_vicii_multicolor_text_mode_render(screenshot_t *screenshot, const char *filename)
 {
     uint8_t *regs = screenshot->video_regs;
@@ -708,7 +1005,9 @@ native_data_t *native_vicii_multicolor_text_mode_render(screenshot_t *screenshot
     }
     return data;
 }
+#endif
 
+#if 0
 native_data_t *native_vicii_hires_bitmap_mode_render(screenshot_t *screenshot, const char *filename)
 {
     uint8_t *regs = screenshot->video_regs;
@@ -750,7 +1049,9 @@ native_data_t *native_vicii_hires_bitmap_mode_render(screenshot_t *screenshot, c
     }
     return data;
 }
+#endif
 
+#if 0
 native_data_t *native_vicii_multicolor_bitmap_mode_render(screenshot_t *screenshot, const char *filename)
 {
     uint8_t *regs = screenshot->video_regs;
@@ -809,6 +1110,7 @@ native_data_t *native_vicii_multicolor_bitmap_mode_render(screenshot_t *screensh
     }
     return data;
 }
+#endif
 
 /* ------------------------------------------------------------------------ */
 
@@ -988,14 +1290,17 @@ void ted_color_to_vicii_color_colormap(native_data_t *source, int ted_lum_handli
         for (j = 0; j < source->xsize; j++) {
             colorbyte = source->colormap[(i * source->xsize) + j];
             if (ted_lum_handling == NATIVE_SS_TED_LUM_DITHER) {
-                source->colormap[(i * source->xsize) + j] = ted_lum_to_vicii_color((uint8_t)(colorbyte & 0xf), (uint8_t)(colorbyte >> 4));
+                source->colormap[(i * source->xsize) + j] =
+                    ted_lum_to_vicii_color((uint8_t)(colorbyte & 0xf), (uint8_t)(colorbyte >> 4));
             } else {
-                source->colormap[(i * source->xsize) + j] = ted_to_vicii_color((uint8_t)(colorbyte & 0xf));
+                source->colormap[(i * source->xsize) + j] =
+                    ted_to_vicii_color((uint8_t)(colorbyte & 0xf));
             }
         }
     }
 }
 
+#if 0
 native_data_t *native_ted_text_mode_render(screenshot_t *screenshot, const char *filename)
 {
     uint8_t *regs = screenshot->video_regs;
@@ -1045,7 +1350,9 @@ native_data_t *native_ted_text_mode_render(screenshot_t *screenshot, const char
     }
     return data;
 }
+#endif
 
+#if 0
 native_data_t *native_ted_extended_background_mode_render(screenshot_t *screenshot, const char *filename)
 {
     uint8_t *regs = screenshot->video_regs;
@@ -1090,7 +1397,9 @@ native_data_t *native_ted_extended_background_mode_render(screenshot_t *screensh
     }
     return data;
 }
+#endif
 
+#if 0
 native_data_t *native_ted_hires_bitmap_mode_render(screenshot_t *screenshot, const char *filename)
 {
     uint8_t *regs = screenshot->video_regs;
@@ -1134,7 +1443,9 @@ native_data_t *native_ted_hires_bitmap_mode_render(screenshot_t *screenshot, con
     }
     return data;
 }
+#endif
 
+#if 0
 native_data_t *native_ted_multicolor_bitmap_mode_render(screenshot_t *screenshot, const char *filename)
 {
     uint8_t *regs = screenshot->video_regs;
@@ -1194,6 +1505,7 @@ native_data_t *native_ted_multicolor_bitmap_mode_render(screenshot_t *screenshot
     }
     return data;
 }
+#endif
 
 /* ------------------------------------------------------------------------ */
 
@@ -1227,11 +1539,12 @@ void vic_color_to_vicii_color_colormap(native_data_t *source)
 
     for (i = 0; i < source->ysize; i++) {
         for (j = 0; j < source->xsize; j++) {
-            source->colormap[(i * source->xsize) + j] = vic_to_vicii_color(source->colormap[(i * source->xsize) + j]);
+            source->colormap[(i * source->xsize) + j] =
+                vic_to_vicii_color(source->colormap[(i * source->xsize) + j]);
         }
     }
 }
-
+#if 0
 native_data_t *native_vic_render(screenshot_t *screenshot, const char *filename)
 {
     uint8_t *regs = screenshot->video_regs;
@@ -1315,8 +1628,9 @@ native_data_t *native_vic_render(screenshot_t *screenshot, const char *filename)
     }
     return data;
 }
-
+#endif
 /* ------------------------------------------------------------------------ */
+#if 0
 
 #define MA_WIDTH        64
 #define MA_LO           (MA_WIDTH - 1)          /* 6 bits */
@@ -1478,7 +1792,7 @@ native_data_t *native_crtc_render(screenshot_t *screenshot, const char *filename
     }
     return data;
 }
-
+#endif
 /* ------------------------------------------------------------------------ */
 
 static uint8_t vdc_vicii_translate[16] = {
@@ -1511,11 +1825,13 @@ void vdc_color_to_vicii_color_colormap(native_data_t *source)
 
     for (i = 0; i < source->ysize; i++) {
         for (j = 0; j < source->xsize; j++) {
-            source->colormap[(i * source->xsize) + j] = vdc_to_vicii_color(source->colormap[(i * source->xsize) + j]);
+            source->colormap[(i * source->xsize) + j] =
+                vdc_to_vicii_color(source->colormap[(i * source->xsize) + j]);
         }
     }
 }
 
+#if 0
 native_data_t *native_vdc_text_mode_render(screenshot_t *screenshot, const char *filename)
 {
     uint8_t *regs = screenshot->video_regs;
@@ -1577,217 +1893,4 @@ native_data_t *native_vdc_text_mode_render(screenshot_t *screenshot, const char
     }
     return data;
 }
-
-/*
-                              ------------------------
-                              | Register Definitions |
-                              ------------------------
-
-Reg#     7    6    5    4    3    2    1    0     Description              Notes
-------- ---- ---- ---- ---- ---- ---- ---- ----   ------------------------ -----
-  1     HzD7 HzD6 HzD5 HzD4 HzD3 HzD2 HzD1 HzD0   Horizontal Displayed      ^1
-  6     VeD7 VeD6 VeD5 VeD4 VeD3 VeD2 VeD1 VeD0   Vertical Displayed        ^3
-  9     .... .... .... CTV4 CTV3 CTV2 CTV1 CTV0   Character Total Vertical  ^5
- 12     Ds15 Ds14 Ds13 Ds12 Ds11 Ds10 Ds09 Ds08   Display Start Adrs (Hi)   ^7
- 13     Ds07 Ds06 Ds05 Ds04 Ds03 Ds02 Ds01 Ds00   Display Start Adrs (Lo)   ^7
- 20     At15 At14 At13 At12 At11 At10 At09 At08   Attribute Start Adrs (Hi) ^7
- 21     At07 At06 At05 At04 At03 At02 At01 At00   Attribute Start Adrs (Lo) ^7
- 22     HcP3 HcP2 HcP1 HcP0 IcS3 IcS2 IcS1 IcS0   Hz Chr Pxl Ttl/IChar Spc  ^A
- 23     .... .... .... VcP4 VcP3 VcP2 VcP1 VcP0   Vert. Character Pxl Spc   ^5
- 24     BlkM RvsS Vss5 Vss4 Vss3 Vss2 Vss1 Vss0   Block/Rvs Scr/V. Scroll ^9^B^C
- 25     Text Atri Semi Dble Hss3 Hss2 Hss1 Hss0   Diff. Mode Sw/H. Scroll  ^D,^E
- 26     Fgd3 Fgd2 Fgd1 Fgd0 Bgd3 Bgd2 Bgd1 Bgd0   ForeGround/BackGround Col ^F
- 27     Rin7 Rin6 Rin5 Rin4 Rin3 Rin2 Rin1 Rin0   Row/Adrs. Increment       ^G
- 28     CSa2 CSa1 CSa0 RamT .... .... .... ....   Character Set Addrs/Ram  ^H,^I
- 34     DeB7 DeB6 DeB5 DeB4 DeB3 DeB2 DeB1 DeB0   Display Enable Begin      ^J
- 35     DeE7 DeE6 DeE5 DeE4 DeE3 DeE2 DeE1 DeE0   Display Enable End        ^J
-
-                             +-----------------+
-                             | Register Usage: |
-                             +-----------------+
-
----  Register #1:     Horizontal Displayed
-
-  These two register function to define the display width of the screen.
-Register 0 will contain the number of characters minus 1 between sucessive
-horizontal sync pulses, the horizontal border and the interval between
-horizontal sync pulses. The normal value for this is usually set to 126.
-Register 1 specifies how many of the positions as specified in register 0 can
-actually be used to display characters.  The default value for this is 80.
-The VDC can take values less than 80 and thus, will only display that many
-characters. A useful effect can be a sweep from the right by incrementing
-the value here from 1 to 80. Register #2 specifies the starting character
-position at which the vertical sync pulse begins. Thus, it also determines
-where on the active screen characters appear. A default value of 102,
-increasing the value moves the screen to the left, decreasing it moves it to
-the right.
-
-     Register #6:     Vertical Displayed
-
-  Register #4 of this register determines the total number of screen rows,
-including the rows for the active display, and the top and bottom borders in
-addition to that of the vertical sync width. The value held here is normally
-a value of 32 for NTSC systems (US Standard) or 39 for PAL(European) systems.
-Register #5 holds in bits 0-4 a "fine-adjust" where any extra scan lines that
-are necessary to make up the display can be specified here. The value here is
-normally a 0 in both the NTSC and PAL initializations by the kernal and bits
-5-7 are unused, always returning a binary 1111. Register #6 specifies the total
-number of the vertical character positions (as set in Register 4) that can be
-used for actual display of characters. Thus, this register usually holds a
-value of 25 for a standard 25-row display.
-
-^5 : Register #9:     Total Scan Lines Per Character
-----
-
-  Bits 0-4 of this register are the only relevant ones, the rest returning a
-binary 1. Bits 0-4 determine the character height in scan-lines of displayed
-characters and allow up to scan-line heights of 32 scan lines. The VDC normally
-sets aside 16 bytes for each character (normally, each byte is equivlent to
-1 scan line) so the value here could be increased to 16-1 and a double-height
-character set could be loaded in. Note, however that values less than 16 will
-tell the VDC to use a 8,192 byte character set (normal) while specifying values
-greater than 16 will make it use 32 bytes per character even if some of the
-bytes are not used.
-
-^7 : Register #12:    Display Start Address (Hi)
----- Register #13:    Display Start Address (Lo)
-     Register #20:    Attribute Start Addrs (Hi)
-     Register #21:    Attribute Start Addrs (Lo)
-
-  Note first, that all of these registers are grouped in Hi byte, Lo byte order
-which is usually different from the 6502 convention of low byte, hi byte (ie:
-in normal 6502 ml, $c000 is stored as $00 $c0, however in the 8563 it would be
-stored as $c0 $00).  Registers 12 and 13 determine, where in VDC memory the
-8563 is the start of the screen. Incrementing this value by 80 (the number of
-characters per line) and with a little additional work can provide a very
-effecient way of having a screen that "seems" to be larger than just 80x25.
-The cursor position in registers 14 and 15 reflect the actual character in
-memory that the cursor currently lies over. If it's not on the display screen,
-then it is not displayed. Registers 20 and 21 reflect where in the 8563 memory
-attribute memory is held. Attribute memory refers to the character attributes
-such as flash, inverse video, color etc that can be set for each character.
-
-^A : Register #22:    Character Horizontal Size Control
-----
-
-  Bits 0-3 of this register determines how many horizontal pixels are used
-for each displayed character. Values greater than 8 here result in apparent
-gaps in the display. Inter-character spacing can be achieved by setting this
-value greater than that of bits 4-7. Bits 4-7 determine the width of each
-character position in pixels. Thus, while bits 0-3 allocate n-pixels, bits
-4-7 specify how many of those pixels are used for character display.
-
-^B : Register #24:5   Reverse Screen Bit
----- Register #24:6   Blink Rate for Characters.
-
-  Bit #6 specifies for the VDC for all pixels normally unset on the VDC screen
-to be set, and all set pixels to be unset.  Bit #5 specifies the blink rate
-for all characters with the blink attribute. Setting this to a binary 1
-specifies a blink rate of 1/32 the refresh rate, while a binary 0 is equivlant
-to a blink rate 1/16th of the refresh rate.
-
-^C : Register #24:0-4 Vertical Smooth Scroll
-----
-
-  The 8563 provides for a smooth scroll, allowing bits 0-4 to function as an
-indicator of the number of bits to scroll the screen vertically upward.
-
-^D : Register #25:7   Text or Graphics Mode Indicator Bit
----- Register #25:6   Monochrome Mode Bit
-     Register #25:5   Semi-Graphics Mode
-     Register #25:4   Double-Pixel Mode
-
-  The 8563 allows the implementation of a graphics mode, in where all of the 16k
-of the screen may be bit-mapped sequentially resulting in a resolution of
-640x200 (see Craig Bruce's 8563 Line-Plotting routine in the first issue for a
-more detailed explanation of this feature). Setting this bit to 1 specifies
-graphics mode, binary 0 indicates text mode.  Bit 6 indicates to the 8563 where
-to obtain its color information etc, about the characters. Bit 6 when it is a
-binary 0 results in the 8563 taking it's color information from bits 4-7 of
-register 26. When this bit is a binary 1, the attribute memory is used to
-obtain color, flash, reverse information. Also note than when this bit is a
-binary 1 that only the first of the two character sets is available. Bit #5
-indicates a semi-graphics mode that allows the rightmost pixel of any characters
-to be repeated through-out the intercharacter spacing gap. Activating it on the
-normal display will result in what appears to be a "digital" character font. The
-8563 with bit #4 allows a pixel-double feature which results in all displayed
-horizontal pixels having twice their usual size. Thus, a 40 column screen is
-easily obtainable although the values in registers #00-#02 must be halved.
-
-^E : Register #25:    Horizontal Smooth Control
-----
-
-  This register is analogous to register #24 Vertical Smooth Control and
-functions similairly. Increasing this bits moves the screen one pixel to the
-right, while decreasing them moves the screen one pixel to the left.
-
-^F : Register #26:    ForeGround / BackGround Color Register
-----
-
-  This register, in bits 0-3 specifies the background color of the display while
-bits 4-7 specify the foreground character colors when attributes are disabled
-(via bit 6 of register #25).  Note, these are not the usual C= colors but are
-instead organized as follows:
-
-   Bit Value  Decimal Value   Color
-   ----------------------------------       +-----------------------------+
-    %0000       0 / $00       Black         |  Note: Bit 0 = Intensity    |
-    %0001       1 / $01       Dark Gray     |        Bit 1 = Blue         |
-    %0010       2 / $02       Dark Blue     | RGBI   Bit 2 = Green        |
-    %0011       3 / $03       Light Blue    |        Bit 3 = Red          |
-    %0100       4 / $04       Dark Green    |                             |
-    %0101       5 / $05       Light Green   +-----------------------------+
-    %0110       6 / $06       Dark Cyan
-    %0111       7 / $07       Light Cyan
-    %1000       8 / $08       Dark Red
-    %1001       9 / $09       Light Red
-    %1010      10 / $0A       Dark Purple
-    %1011      11 / $0B       Light Purple
-    %1100      12 / $0C       Dark Yellow
-    %1101      13 / $0D       Light Yellow
-    %1110      14 / $0E       Light Gray (Dark White)
-    %1111      15 / $0F       White
-
-^G : Register #27:    Row Address Display Increment
-----
-
-  This register specifies the number of bytes to skip, when displaying
-characters on the 8563 screen. Normally, this byte holds a value of $00
-indicating no bytes to skip; however typically programs that "scroll" the
-screen do so by setting this to 80 or 160 allowing the program to then alter
-the Screen Start (Registers #12 and #13) and appear to "scroll". Note the
-normal C= 128 Kernal Screen Editor does not support this function.
-
-^H : Register #28:7-5 Character Set Address
-----
-
-  These bits indicate the address of screen memory * 8k. Thus the values in
-these bits may be multiplied by 8192 to obtain the starting character set
-position (normall these bits hold a value of $01 indicating the character
-set begins at 8192).  Note that the character set is not in ROM, but is usually
-copied to 8192 when the computer is first turned on and the 8563 is initialized.
-(Examine the INIT80 routine at $CE0C in bank 15).
-
-                           +--------------------------+
-                           | 8563 Memory Organization |
-                           +--------------------------+
-
-  Normally, the extra memory of the C=128's equipped with 64k goes unused (48k
-worth) unless programs like Basic-8 etc, take advantage of it. There are various
-mod files describing the upgrade from 16k to 64k and it is _strongly_ advised
-(although the author has not yet done so) and be aware that ***OPENING YOUR
-COMPUTER JUST TO LOOK, YOU MAY MESS IT UP*** and it is _strongly_ advised that
-you contact a person experienced with electronics to perform the upgrade for
-you.  Note also that some mail order companies are offering an "up-grade board"
-which plugs into the 8563 slot and does not involve desoldering the RAM chips.
-
-  Now, the 8563 uses the 16k of memory (it ignores the extra 48k of memory when
-it's got 64k, thus the following applies also to the 8563's equipped with 64k
-of memory) and normally, has the following memory map:
-
-  $0000 - $07ff - Screen Memory
-  $0800 - $0fff - Attribute Memory
-  $1000 - $1fff - Unused
-  $2000 - $2fff - UpperCase / Graphic Character Set   (Char Set #1)
-  $3000 - $3fff - LowerCase / UpperCase Character Set (Char Set #2)
-*/
+#endif
