@@ -38,6 +38,7 @@
 #include "via.h"
 #include "types.h"
 #include "serial.h"
+#include "drive/iec/cmdhd.h"
 
 
 #define IECBUS_DEVICE_NONE      0
@@ -214,7 +215,7 @@ static void iecbus_cpu_write_conf0(uint8_t data, CLOCK clock)
     iecbus.iec_fast_1541 = data;
 }
 
-/* Only the first drive is enabled.  */
+/* Only the first disk unit is enabled.  */
 static uint8_t iecbus_cpu_read_conf1(CLOCK clock)
 {
     drive_cpu_execute_all(clock);
@@ -226,10 +227,9 @@ static uint8_t iecbus_cpu_read_conf1(CLOCK clock)
 
 static void iecbus_cpu_write_conf1(uint8_t data, CLOCK clock)
 {
-    drive_t *drive;
+    diskunit_context_t *unit = diskunit_context[0];
 
-    drive = drive_context[0]->drive;
-    drive_cpu_execute_one(drive_context[0], clock);
+    drive_cpu_execute_one(unit, clock);
 
     DEBUG_IEC_CPU_WRITE(data);
 
@@ -237,27 +237,32 @@ static void iecbus_cpu_write_conf1(uint8_t data, CLOCK clock)
 
     if (iec_old_atn != (iecbus.cpu_bus & 0x10)) {
         iec_old_atn = iecbus.cpu_bus & 0x10;
-        switch (drive->type) {
+        switch (unit->type) {
             case DRIVE_TYPE_1581:
                 if (!iec_old_atn) {
-                    ciacore_set_flag(drive_context[0]->cia1581);
+                    ciacore_set_flag(unit->cia1581);
                 }
                 break;
             case DRIVE_TYPE_2000:
             case DRIVE_TYPE_4000:
-                viacore_signal(drive_context[0]->via4000, VIA_SIG_CA2,
+                viacore_signal(unit->via4000, VIA_SIG_CA2,
                                iec_old_atn ? 0 : VIA_SIG_RISE);
                 break;
+            case DRIVE_TYPE_CMDHD:
+                viacore_signal(unit->cmdhd->via10, VIA_SIG_CA1,
+                               iec_old_atn ? VIA_SIG_RISE : VIA_SIG_FALL);
+                break;
             default:
-                viacore_signal(drive_context[0]->via1d1541, VIA_SIG_CA1,
+                viacore_signal(unit->via1d1541, VIA_SIG_CA1,
                                iec_old_atn ? 0 : VIA_SIG_RISE);
         }
     }
 
-    switch (drive->type) {
+    switch (unit->type) {
         case DRIVE_TYPE_1581:
         case DRIVE_TYPE_2000:
         case DRIVE_TYPE_4000:
+        case DRIVE_TYPE_CMDHD:
             iecbus.drv_bus[8] = (((iecbus.drv_data[8] << 3) & 0x40)
                                  | ((iecbus.drv_data[8] << 6)
                                     & ((iecbus.drv_data[8] | iecbus.cpu_bus) << 3)
@@ -266,13 +271,13 @@ static void iecbus_cpu_write_conf1(uint8_t data, CLOCK clock)
         default:
             iecbus.drv_bus[8] = (((iecbus.drv_data[8] << 3) & 0x40)
                                  | ((iecbus.drv_data[8] << 6)
-                                    & ((~iecbus.drv_data[8] ^ iecbus.cpu_bus) << 3)
+                                    & ((uint32_t)(~iecbus.drv_data[8] ^ iecbus.cpu_bus) << 3)
                                     & 0x80));
     }
     iec_update_ports();
 }
 
-/* Only the second drive is enabled.  */
+/* Only the second disk unit is enabled.  */
 static uint8_t iecbus_cpu_read_conf2(CLOCK clock)
 {
     drive_cpu_execute_all(clock);
@@ -284,10 +289,9 @@ static uint8_t iecbus_cpu_read_conf2(CLOCK clock)
 
 static void iecbus_cpu_write_conf2(uint8_t data, CLOCK clock)
 {
-    drive_t *drive;
+    diskunit_context_t *unit = diskunit_context[1];
 
-    drive = drive_context[1]->drive;
-    drive_cpu_execute_one(drive_context[1], clock);
+    drive_cpu_execute_one(unit, clock);
 
     DEBUG_IEC_CPU_WRITE(data);
 
@@ -295,27 +299,32 @@ static void iecbus_cpu_write_conf2(uint8_t data, CLOCK clock)
 
     if (iec_old_atn != (iecbus.cpu_bus & 0x10)) {
         iec_old_atn = iecbus.cpu_bus & 0x10;
-        switch (drive->type) {
+        switch (unit->type) {
             case DRIVE_TYPE_1581:
                 if (!iec_old_atn) {
-                    ciacore_set_flag(drive_context[1]->cia1581);
+                    ciacore_set_flag(unit->cia1581);
                 }
                 break;
             case DRIVE_TYPE_2000:
             case DRIVE_TYPE_4000:
-                viacore_signal(drive_context[1]->via4000, VIA_SIG_CA2,
+                viacore_signal(unit->via4000, VIA_SIG_CA2,
                                iec_old_atn ? 0 : VIA_SIG_RISE);
                 break;
+            case DRIVE_TYPE_CMDHD:
+                viacore_signal(unit->cmdhd->via10, VIA_SIG_CA1,
+                               iec_old_atn ? VIA_SIG_RISE : VIA_SIG_FALL);
+                break;
             default:
-                viacore_signal(drive_context[1]->via1d1541, VIA_SIG_CA1,
+                viacore_signal(unit->via1d1541, VIA_SIG_CA1,
                                iec_old_atn ? 0 : VIA_SIG_RISE);
         }
     }
 
-    switch (drive->type) {
+    switch (unit->type) {
         case DRIVE_TYPE_1581:
         case DRIVE_TYPE_2000:
         case DRIVE_TYPE_4000:
+        case DRIVE_TYPE_CMDHD:
             iecbus.drv_bus[9] = (((iecbus.drv_data[9] << 3) & 0x40)
                                  | ((iecbus.drv_data[9] << 6)
                                     & ((iecbus.drv_data[9] | iecbus.cpu_bus) << 3)
@@ -355,35 +364,43 @@ static void iecbus_cpu_write_conf3(uint8_t data, CLOCK clock)
     if (iec_old_atn != (iecbus.cpu_bus & 0x10)) {
         iec_old_atn = iecbus.cpu_bus & 0x10;
 
-        for (dnr = 0; dnr < DRIVE_NUM; dnr++) {
+        for (dnr = 0; dnr < NUM_DISK_UNITS; dnr++) {
             if (iecbus_device[8 + dnr] == IECBUS_DEVICE_TRUEDRIVE) {
-                switch (drive_context[dnr]->drive->type) {
+                diskunit_context_t *unit = diskunit_context[dnr];
+
+                switch (diskunit_context[dnr]->type) {
                     case DRIVE_TYPE_1581:
                         if (!iec_old_atn) {
-                            ciacore_set_flag(drive_context[dnr]->cia1581);
+                            ciacore_set_flag(unit->cia1581);
                         }
                         break;
                     case DRIVE_TYPE_2000:
                     case DRIVE_TYPE_4000:
-                        viacore_signal(drive_context[dnr]->via4000, VIA_SIG_CA2,
+                        viacore_signal(unit->via4000, VIA_SIG_CA2,
                                        iec_old_atn ? 0 : VIA_SIG_RISE);
                         break;
+                    case DRIVE_TYPE_CMDHD:
+                        viacore_signal(unit->cmdhd->via10, VIA_SIG_CA1,
+                                       iec_old_atn ? VIA_SIG_RISE : VIA_SIG_FALL);
+                        break;
                     default:
-                        viacore_signal(drive_context[dnr]->via1d1541, VIA_SIG_CA1,
+                        viacore_signal(unit->via1d1541, VIA_SIG_CA1,
                                        iec_old_atn ? 0 : VIA_SIG_RISE);
                 }
             }
         }
     }
 
-    for (dnr = 0; dnr < DRIVE_NUM; dnr++) {
+    for (dnr = 0; dnr < NUM_DISK_UNITS; dnr++) {
         if (iecbus_device[8 + dnr] == IECBUS_DEVICE_TRUEDRIVE) {
             unsigned int unit;
             unit = dnr + 8;
-            switch (drive_context[dnr]->drive->type) {
+
+            switch (diskunit_context[dnr]->type) {
                 case DRIVE_TYPE_1581:
                 case DRIVE_TYPE_2000:
                 case DRIVE_TYPE_4000:
+                case DRIVE_TYPE_CMDHD:
                     iecbus.drv_bus[unit] = (((iecbus.drv_data[unit] << 3) & 0x40)
                                             | ((iecbus.drv_data[unit] << 6)
                                                & ((iecbus.drv_data[unit]
@@ -420,11 +437,11 @@ static void calculate_callback_index(void)
             iecbus_callback_write = iecbus_cpu_write_conf0;
             break;
         case IECBUS_DEVICE_TRUEDRIVE << 0:
-                iecbus_callback_read = iecbus_cpu_read_conf1;
+            iecbus_callback_read = iecbus_cpu_read_conf1;
             iecbus_callback_write = iecbus_cpu_write_conf1;
             break;
         case IECBUS_DEVICE_TRUEDRIVE << 2:
-                iecbus_callback_read = iecbus_cpu_read_conf2;
+            iecbus_callback_read = iecbus_cpu_read_conf2;
             iecbus_callback_write = iecbus_cpu_write_conf2;
             break;
         default:
