@@ -56,27 +56,40 @@
 #define DBG(x)
 #endif
 
+#define NUM_STARTUP_DISK_IMAGES 8
 static char *autostart_string = NULL;
-static char *startup_disk_images[4];
+static char *startup_disk_images[NUM_STARTUP_DISK_IMAGES];
 static char *startup_tape_image;
 static unsigned int autostart_mode = AUTOSTART_MODE_NONE;
 
+
+/** \brief  Get autostart mode
+ *
+ * \return  autostart mode
+ */
 int cmdline_get_autostart_mode(void)
 {
     return autostart_mode;
 }
 
+
+void cmdline_set_autostart_mode(int mode)
+{
+    autostart_mode = mode;
+}
+
+
 static void cmdline_free_autostart_string(void)
 {
     lib_free(autostart_string);
     autostart_string = NULL;
 }
 
-static void cmdline_free_startup_images(void)
+void initcmdline_shutdown(void)
 {
     int unit;
 
-    for (unit = 0; unit < 4; unit++) {
+    for (unit = 0; unit < NUM_STARTUP_DISK_IMAGES; unit++) {
         if (startup_disk_images[unit] != NULL) {
             lib_free(startup_disk_images[unit]);
         }
@@ -97,7 +110,7 @@ static int cmdline_help(const char *param, void *extra_param)
 
 static int cmdline_features(const char *param, void *extra_param)
 {
-    feature_list_t *list = vice_get_feature_list();
+    const feature_list_t *list = vice_get_feature_list();
 
     printf("Compile time options:\n");
     while (list->symbol) {
@@ -117,6 +130,11 @@ static int cmdline_config(const char *param, void *extra_param)
     return 0;
 }
 
+static int cmdline_add_config(const char *param, void *extra_param)
+{
+    return resources_load(param);
+}
+
 static int cmdline_dumpconfig(const char *param, void *extra_param)
 {
     return resources_dump(param);
@@ -124,13 +142,6 @@ static int cmdline_dumpconfig(const char *param, void *extra_param)
 
 static int cmdline_default(const char *param, void *extra_param)
 {
-    /* the cartridge system uses internal state variables so the default cartridge
-       can be unset without changing the attached cartridge and/or attach another
-       cartridge without changing the default. to completely restore the default,
-       which is no default cartridge, and no currently attached cartridge, call
-       the respective functions of the cartridge system here */
-    cartridge_unset_default();
-    cartridge_detach_image(-1);
     return resources_set_defaults();
 }
 
@@ -192,6 +203,13 @@ static int cmdline_attach(const char *param, void *extra_param)
             lib_free(startup_disk_images[unit - 8]);
             startup_disk_images[unit - 8] = lib_strdup(param);
             break;
+        case 64:
+        case 65:
+        case 66:
+        case 67:
+            lib_free(startup_disk_images[unit - 64 + 4]);
+            startup_disk_images[unit - 64 + 4] = lib_strdup(param);
+            break;
         default:
             archdep_startup_log_error("cmdline_attach(): unexpected unit number %d?!\n", unit);
     }
@@ -203,7 +221,7 @@ static const cmdline_option_t common_cmdline_options[] =
 {
     { "-help", CALL_FUNCTION, CMDLINE_ATTRIB_NONE,
       cmdline_help, NULL, NULL, NULL,
-      NULL, "Show a list of the available options and exit normally" },
+      NULL, "Show a list of the available options an_vice_xit normally" },
     { "-?", CALL_FUNCTION, CMDLINE_ATTRIB_NONE,
       cmdline_help, NULL, NULL, NULL,
       NULL, "Show a list of the available options and exit normally" },
@@ -219,6 +237,9 @@ static const cmdline_option_t common_cmdline_options[] =
     { "-config", CALL_FUNCTION, CMDLINE_ATTRIB_NEED_ARGS,
       cmdline_config, NULL, NULL, NULL,
       "<filename>", "Specify config file" },
+    { "-addconfig", CALL_FUNCTION, CMDLINE_ATTRIB_NEED_ARGS,
+      cmdline_add_config, NULL, NULL, NULL,
+      "<filename>", "Specify extra config file for loading additional resources." },
     { "-dumpconfig", CALL_FUNCTION, CMDLINE_ATTRIB_NEED_ARGS,
       cmdline_dumpconfig, NULL, NULL, NULL,
       "<filename>", "Dump all resources to specified config file" },
@@ -255,16 +276,28 @@ static const cmdline_option_t cmdline_options[] =
       "<Name>", "Attach <name> as a tape image" },
     { "-8", CALL_FUNCTION, CMDLINE_ATTRIB_NEED_ARGS,
       cmdline_attach, (void *)8, NULL, NULL,
-      "<Name>", "Attach <name> as a disk image in drive #8" },
+      "<Name>", "Attach <name> as a disk image in unit #8" },
+    { "-8d1", CALL_FUNCTION, CMDLINE_ATTRIB_NEED_ARGS,
+      cmdline_attach, (void *)64, NULL, NULL,
+      "<Name>", "Attach <name> as a disk image in unit #8 drive #1" },
     { "-9", CALL_FUNCTION, CMDLINE_ATTRIB_NEED_ARGS,
       cmdline_attach, (void *)9, NULL, NULL,
-      "<Name>", "Attach <name> as a disk image in drive #9" },
+      "<Name>", "Attach <name> as a disk image in unit #9" },
+    { "-9d1", CALL_FUNCTION, CMDLINE_ATTRIB_NEED_ARGS,
+      cmdline_attach, (void *)65, NULL, NULL,
+      "<Name>", "Attach <name> as a disk image in unit #9 drive #1" },
     { "-10", CALL_FUNCTION, CMDLINE_ATTRIB_NEED_ARGS,
       cmdline_attach, (void *)10, NULL, NULL,
-      "<Name>", "Attach <name> as a disk image in drive #10" },
+      "<Name>", "Attach <name> as a disk image in unit #10" },
+    { "-10d1", CALL_FUNCTION, CMDLINE_ATTRIB_NEED_ARGS,
+      cmdline_attach, (void *)66, NULL, NULL,
+      "<Name>", "Attach <name> as a disk image in unit #10 drive #1" },
     { "-11", CALL_FUNCTION, CMDLINE_ATTRIB_NEED_ARGS,
       cmdline_attach, (void *)11, NULL, NULL,
-      "<Name>", "Attach <name> as a disk image in drive #11" },
+      "<Name>", "Attach <name> as a disk image in unit #11" },
+    { "-11d1", CALL_FUNCTION, CMDLINE_ATTRIB_NEED_ARGS,
+      cmdline_attach, (void *)67, NULL, NULL,
+      "<Name>", "Attach <name> as a disk image in unit #11 drive #1" },
     CMDLINE_LIST_END
 };
 
@@ -281,7 +314,6 @@ int initcmdline_init(void)
         }
     }
 
-    archdep_vice_atexit(cmdline_free_startup_images);
     return 0;
 }
 
@@ -349,7 +381,14 @@ void initcmdline_check_attach(void)
 
         /* `-autostart' */
         if (autostart_string != NULL) {
-            autostart_autodetect_opt_prgname(autostart_string, 0, autostart_mode);
+            if (autostart_autodetect_opt_prgname(autostart_string, 0, autostart_mode) < 0) {
+                log_error(LOG_DEFAULT,
+                        "Failed to autostart '%s'", autostart_string);
+                if (autostart_string != NULL) {
+                    lib_free(autostart_string);
+                }
+                archdep_vice_exit(1);
+            }
         }
         /* `-8', `-9', `-10' and `-11': Attach specified disk image.  */
         {
@@ -357,13 +396,22 @@ void initcmdline_check_attach(void)
 
             for (i = 0; i < 4; i++) {
                 if (startup_disk_images[i] != NULL
-                    && file_system_attach_disk(i + 8, startup_disk_images[i])
+                    && file_system_attach_disk(i + 8, 0, startup_disk_images[i])
                     < 0) {
                     log_error(LOG_DEFAULT,
                               "Cannot attach disk image `%s' to unit %d.",
                               startup_disk_images[i], i + 8);
                 }
             }
+            for (i = 4; i < 8; i++) {
+                if (startup_disk_images[i] != NULL
+                    && file_system_attach_disk(i + 4, 1, startup_disk_images[i])
+                    < 0) {
+                    log_error(LOG_DEFAULT,
+                              "Cannot attach disk image `%s' to unit %d drive 1.",
+                              startup_disk_images[i], i + 4);
+                }
+            }
         }
 
         /* `-1': Attach specified tape image.  */
