@@ -31,6 +31,8 @@
 #ifndef VICE_MOUSE_H
 #define VICE_MOUSE_H
 
+#include <stdbool.h>
+
 #include "types.h"
 
 typedef struct mouse_func_s {
@@ -47,8 +49,12 @@ extern void mouse_init(void);
 extern void mouse_shutdown(void);
 
 extern int _mouse_enabled;
+extern int _mouse_still_frames;
 extern int mouse_type;
 
+extern void mouse_move(float dx, float dy);
+extern void mouse_get_int16(int16_t *x, int16_t *y);
+
 extern void neos_mouse_set_machine_parameter(long clock_rate);
 extern void neos_mouse_store(uint8_t val);
 extern uint8_t neos_mouse_read(void);
