@@ -78,17 +78,17 @@ static uint8_t joyport_sampler_read(int port)
 }
 
 static joyport_t joyport_sampler_device = {
-    "Sampler (2bit)",
-    JOYPORT_RES_ID_SAMPLER,
-    JOYPORT_IS_NOT_LIGHTPEN,
-    JOYPORT_POT_OPTIONAL,
-    joyport_sampler_enable,
-    joyport_sampler_read,
-    NULL,               /* no store digital */
-    NULL,               /* no pot-x read */
-    NULL,               /* no pot-y read */
-    NULL,               /* no data for a snapshot */
-    NULL                /* no data for a snapshot */
+    "Sampler (2bit)",        /* name of the device */
+    JOYPORT_RES_ID_SAMPLER,  /* device is a sampler, only 1 sampler can be active at the same time */
+    JOYPORT_IS_NOT_LIGHTPEN, /* device is NOT a lightpen */
+    JOYPORT_POT_OPTIONAL,    /* device does NOT use the potentiometer lines */
+    joyport_sampler_enable,  /* device enable function */
+    joyport_sampler_read,    /* digital line read function */
+    NULL,                    /* NO digital line store function */
+    NULL,                    /* NO pot-x read function */
+    NULL,                    /* NO pot-x read function */
+    NULL,                    /* NO device write snapshot function */
+    NULL                     /* NO device read snapshot function */
 };
 
 /* currently only used to register the joyport device */
