@@ -1,5 +1,5 @@
 /*
- * lib.h - Library functions.
+ * lib.h - Library function wrappers, mostly for memory alloc/free tracking.
  *
  * Written by
  *  Andreas Boose <viceteam@t-online.de>
@@ -41,25 +41,13 @@
 #define LIB_DEBUG_PINPOINT
 #endif
 
-extern void lib_init_rand(void);
+extern void lib_init(void);
 extern unsigned int lib_unsigned_rand(unsigned int min, unsigned int max);
 extern float lib_float_rand(float min, float max);
 
 extern char *lib_msprintf(const char *fmt, ...);
 extern char *lib_mvsprintf(const char *fmt, va_list args);
 
-extern void lib_debug_check(void);
-
-#if defined(__CYGWIN32__) || defined(__CYGWIN__) || defined(WIN32_COMPILE)
-
-extern size_t lib_tcstostr(char *str, const char *tcs, size_t len);
-extern size_t lib_strtotcs(char *tcs, const char *str, size_t len);
-
-extern int lib_snprintf(char *str, size_t len, const char *fmt, ...);
-#define lib_sntprintf lib_snprintf
-
-#endif /* CYGWIN or WIN32_COMPILE */
-
 #ifdef LIB_DEBUG_PINPOINT
 extern void *lib_malloc_pinpoint(size_t size, const char *name, unsigned int line);
 extern void *lib_calloc_pinpoint(size_t nmemb, size_t size, const char *name, unsigned int line);
@@ -76,7 +64,7 @@ extern char *lib_strdup_pinpoint(const char *str, const char *name, unsigned int
 #define lib_realloc(x, y) lib_realloc_pinpoint(x, y, __FILE__, __LINE__)
 #define lib_strdup(x) lib_strdup_pinpoint(x, __FILE__, __LINE__)
 
-#endif //not defined COMPILING_LIB_DOT_C
+#endif /* !COMPILING_LIB_DOT_C */
 
 #if defined(AMIGA_SUPPORT) || defined(__VBCC__)
 extern void *lib_AllocVec_pinpoint(unsigned long size, unsigned long attributes, char *name, unsigned int line);
@@ -91,6 +79,7 @@ extern void lib_FreeMem_pinpoint(void *ptr, unsigned long size, char *name, unsi
 #endif
 
 #else
+/* !defined LIB_DEBUG_PINPOINT */
 
 extern void *lib_malloc(size_t size);
 extern void *lib_calloc(size_t nmemb, size_t size);
@@ -106,6 +95,8 @@ extern void *lib_AllocMem(unsigned long size, unsigned long attributes);
 extern void lib_FreeMem(void *ptr, unsigned long size);
 #endif
 
-#endif
+#endif /* LIB_DEBUG_PINPOINT */
+
+extern char *lib_strdup_trimmed(char *str);
 
 #endif
