@@ -26,10 +26,6 @@
 
 #include <inttypes.h>
 
-#ifdef IDE_COMPILE
-#include "libavutil/internal.h"
-#endif
-
 #include "libavutil/avassert.h"
 #include "libavutil/frame.h"
 #include "libavutil/imgutils.h"
@@ -1018,17 +1014,6 @@ static av_cold int decode_init(AVCodecContext *avctx)
 }
 
 AVCodec ff_fourxm_decoder = {
-#ifdef IDE_COMPILE
-    "4xm",
-    "4X Movie",
-    AVMEDIA_TYPE_VIDEO,
-    AV_CODEC_ID_4XM,
-    CODEC_CAP_DR1,
-    0, 0, 0, 0, 0, 0, 0, 0, sizeof(FourXContext),
-    0, 0, 0, 0, 0, decode_init,
-    0, 0, decode_frame,
-    decode_end,
-#else
 	.name           = "4xm",
     .long_name      = NULL_IF_CONFIG_SMALL("4X Movie"),
     .type           = AVMEDIA_TYPE_VIDEO,
@@ -1038,5 +1023,4 @@ AVCodec ff_fourxm_decoder = {
     .close          = decode_end,
     .decode         = decode_frame,
     .capabilities   = CODEC_CAP_DR1,
-#endif
 };
