@@ -47,13 +47,11 @@
 
 #define AAC_MAX_CHANNELS 6
 
-#if !defined(IDE_COMPILE) || (defined(IDE_COMPILE) && (_MSC_VER >= 1400))
 #define ERROR_IF(cond, ...) \
     if (cond) { \
         av_log(avctx, AV_LOG_ERROR, __VA_ARGS__); \
         return AVERROR(EINVAL); \
     }
-#endif
 
 float ff_aac_pow34sf_tab[428];
 
@@ -242,17 +240,10 @@ WINDOW_FUNC(eight_short)
 static void (*const apply_window[4])(AVFloatDSPContext *fdsp,
                                      SingleChannelElement *sce,
                                      const float *audio) = {
-#ifdef IDE_COMPILE
-    apply_only_long_window,
-    apply_long_start_window,
-    apply_eight_short_window,
-    apply_long_stop_window
-#else
 	[ONLY_LONG_SEQUENCE]   = apply_only_long_window,
     [LONG_START_SEQUENCE]  = apply_long_start_window,
     [EIGHT_SHORT_SEQUENCE] = apply_eight_short_window,
     [LONG_STOP_SEQUENCE]   = apply_long_stop_window
-#endif
 };
 
 static void apply_window_and_mdct(AACEncContext *s, SingleChannelElement *sce,
@@ -714,32 +705,9 @@ static av_cold int dsp_init(AVCodecContext *avctx, AACEncContext *s)
 static av_cold int alloc_buffers(AVCodecContext *avctx, AACEncContext *s)
 {
     int ch;
-#ifdef IDE_COMPILE
-    {
-		s->buffer.samples = av_mallocz_array(s->channels, 3 * 1024 * sizeof(s->buffer.samples[0]));
-		if (!s->buffer.samples) {
-			av_log(avctx, AV_LOG_ERROR, "Cannot allocate memory.\n");
-			goto alloc_fail;
-		}
-	};
-    {
-		s->cpe = av_mallocz_array(s->chan_map[0], sizeof(ChannelElement));
-		if (!s->cpe) {
-			av_log(avctx, AV_LOG_ERROR, "Cannot allocate memory.\n");
-			goto alloc_fail;
-		}
-	};
-    {
-		avctx->extradata = av_mallocz(FF_INPUT_BUFFER_PADDING_SIZE + 32);
-		if (!(avctx->extradata) && (FF_INPUT_BUFFER_PADDING_SIZE + 32) != 0) {
-			av_log(avctx, AV_LOG_ERROR, "Cannot allocate memory.\n"); goto alloc_fail;
-		}
-	};
-#else
 	FF_ALLOCZ_ARRAY_OR_GOTO(avctx, s->buffer.samples, s->channels, 3 * 1024 * sizeof(s->buffer.samples[0]), alloc_fail);
     FF_ALLOCZ_ARRAY_OR_GOTO(avctx, s->cpe, s->chan_map[0], sizeof(ChannelElement), alloc_fail);
     FF_ALLOCZ_OR_GOTO(avctx, avctx->extradata, 5 + FF_INPUT_BUFFER_PADDING_SIZE, alloc_fail);
-#endif
 
     for(ch = 0; ch < s->channels; ch++)
         s->planar_samples[ch] = s->buffer.samples + 3 * 1024 * ch;
@@ -765,7 +733,6 @@ static av_cold int aac_encode_init(AVCodecContext *avctx)
 
     s->channels = avctx->channels;
 
-#if !defined(IDE_COMPILE) || (defined(IDE_COMPILE) && (_MSC_VER >= 1400))
     ERROR_IF(i == 16,
              "Unsupported sample rate %d\n", avctx->sample_rate);
     ERROR_IF(s->channels > AAC_MAX_CHANNELS,
@@ -774,24 +741,6 @@ static av_cold int aac_encode_init(AVCodecContext *avctx)
              "Unsupported profile %d\n", avctx->profile);
     ERROR_IF(1024.0 * avctx->bit_rate / avctx->sample_rate > 6144 * s->channels,
              "Too many bits per frame requested\n");
-#else
-    if (i == 16) {
-        av_log(avctx, AV_LOG_ERROR, "Unsupported sample rate %d\n", avctx->sample_rate);
-        return AVERROR(EINVAL);
-    }
-    if (s->channels > AAC_MAX_CHANNELS) {
-        av_log(avctx, AV_LOG_ERROR, "Unsupported number of channels: %d\n", s->channels);
-        return AVERROR(EINVAL);
-    }
-    if (avctx->profile != FF_PROFILE_UNKNOWN && avctx->profile != FF_PROFILE_AAC_LOW) {
-        av_log(avctx, AV_LOG_ERROR, "Unsupported profile %d\n", avctx->profile);
-        return AVERROR(EINVAL);
-    }
-    if (1024.0 * avctx->bit_rate / avctx->sample_rate > 6144 * s->channels) {
-        av_log(avctx, AV_LOG_ERROR, "Too many bits per frame requested\n");
-        return AVERROR(EINVAL);
-    }
-#endif
 
     s->samplerate_index = i;
 
@@ -840,17 +789,6 @@ fail:
 
 #define AACENC_FLAGS AV_OPT_FLAG_ENCODING_PARAM | AV_OPT_FLAG_AUDIO_PARAM
 static const AVOption aacenc_options[] = {
-#ifdef IDE_COMPILE
-	{"stereo_mode", "Stereo coding method", offsetof(AACEncContext, options.stereo_mode), AV_OPT_TYPE_INT, {0}, -1, 1, AACENC_FLAGS, "stereo_mode"},
-    {"auto", "Selected by the Encoder", 0, AV_OPT_TYPE_CONST, {-1}, INT_MIN, INT_MAX, AACENC_FLAGS, "stereo_mode"},
-    {"ms_off", "Disable Mid/Side coding", 0, AV_OPT_TYPE_CONST, {0}, INT_MIN, INT_MAX, AACENC_FLAGS, "stereo_mode"},
-    {"ms_force", "Force Mid/Side for the whole frame if possible", 0, AV_OPT_TYPE_CONST, {1}, INT_MIN, INT_MAX, AACENC_FLAGS, "stereo_mode"},
-    {"aac_coder", "", offsetof(AACEncContext, options.aac_coder), AV_OPT_TYPE_INT, {AAC_CODER_TWOLOOP}, 0, AAC_CODER_NB-1, AACENC_FLAGS, "aac_coder"},
-    {"faac", "FAAC-inspired method", 0, AV_OPT_TYPE_CONST, {AAC_CODER_FAAC}, INT_MIN, INT_MAX, AACENC_FLAGS, "aac_coder"},
-    {"anmr", "ANMR method", 0, AV_OPT_TYPE_CONST, {AAC_CODER_ANMR}, INT_MIN, INT_MAX, AACENC_FLAGS, "aac_coder"},
-    {"twoloop", "Two loop searching method", 0, AV_OPT_TYPE_CONST, {AAC_CODER_TWOLOOP}, INT_MIN, INT_MAX, AACENC_FLAGS, "aac_coder"},
-    {"fast", "Constant quantizer", 0, AV_OPT_TYPE_CONST, {AAC_CODER_FAST}, INT_MIN, INT_MAX, AACENC_FLAGS, "aac_coder"},
-#else
 	{"stereo_mode", "Stereo coding method", offsetof(AACEncContext, options.stereo_mode), AV_OPT_TYPE_INT, {.i64 = 0}, -1, 1, AACENC_FLAGS, "stereo_mode"},
         {"auto",     "Selected by the Encoder", 0, AV_OPT_TYPE_CONST, {.i64 = -1 }, INT_MIN, INT_MAX, AACENC_FLAGS, "stereo_mode"},
         {"ms_off",   "Disable Mid/Side coding", 0, AV_OPT_TYPE_CONST, {.i64 =  0 }, INT_MIN, INT_MAX, AACENC_FLAGS, "stereo_mode"},
@@ -860,7 +798,6 @@ static const AVOption aacenc_options[] = {
         {"anmr",     "ANMR method",               0, AV_OPT_TYPE_CONST, {.i64 = AAC_CODER_ANMR},    INT_MIN, INT_MAX, AACENC_FLAGS, "aac_coder"},
         {"twoloop",  "Two loop searching method", 0, AV_OPT_TYPE_CONST, {.i64 = AAC_CODER_TWOLOOP}, INT_MIN, INT_MAX, AACENC_FLAGS, "aac_coder"},
         {"fast",     "Constant quantizer",        0, AV_OPT_TYPE_CONST, {.i64 = AAC_CODER_FAST},    INT_MIN, INT_MAX, AACENC_FLAGS, "aac_coder"},
-#endif
 	{NULL}
 };
 
@@ -878,26 +815,7 @@ static const int mpeg4audio_sample_rates[16] = {
     24000, 22050, 16000, 12000, 11025, 8000, 7350
 };
 
-#ifdef IDE_COMPILE
-static const enum AVSampleFormat tmp1[] = { AV_SAMPLE_FMT_FLTP,
-                                                     AV_SAMPLE_FMT_NONE };
-#endif
-
 AVCodec ff_aac_encoder = {
-#ifdef IDE_COMPILE
-    "aac",
-    "AAC (Advanced Audio Coding)",
-    AVMEDIA_TYPE_AUDIO,
-    AV_CODEC_ID_AAC,
-    CODEC_CAP_SMALL_LAST_FRAME | CODEC_CAP_DELAY | CODEC_CAP_EXPERIMENTAL,
-    0, 0, mpeg4audio_sample_rates,
-    tmp1,
-    0, 0, &aacenc_class,
-    0, sizeof(AACEncContext),
-    0, 0, 0, 0, 0, aac_encode_init,
-    0, aac_encode_frame,
-    0, aac_encode_end,
-#else
 	.name           = "aac",
     .long_name      = NULL_IF_CONFIG_SMALL("AAC (Advanced Audio Coding)"),
     .type           = AVMEDIA_TYPE_AUDIO,
@@ -912,5 +830,4 @@ AVCodec ff_aac_encoder = {
     .sample_fmts    = (const enum AVSampleFormat[]){ AV_SAMPLE_FMT_FLTP,
                                                      AV_SAMPLE_FMT_NONE },
     .priv_class     = &aacenc_class,
-#endif
 };
