@@ -813,15 +813,8 @@ static int binkb_decode_plane(BinkContext *c, AVFrame *frame, GetBitContext *gb,
     int v, col[2];
     const uint8_t *scan;
     int xoff, yoff;
-#if !defined(IDE_COMPILE) || (defined(IDE_COMPILE) && (_MSC_VER >= 1400))
 	LOCAL_ALIGNED_16(int16_t, block, [64]);
     LOCAL_ALIGNED_16(int32_t, dctblock, [64]);
-#else
-	uint8_t la_block[sizeof(int16_t[64]) + (16)];
-	int16_t (*block) = (void *)((((uintptr_t)la_block)+(16)-1)&~((16)-1));
-    uint8_t la_dctblock[sizeof(int32_t[64]) + (16)];
-	int32_t (*dctblock) = (void *)((((uintptr_t)la_dctblock)+(16)-1)&~((16)-1));
-#endif
 	int coordmap[64];
     int ybias = is_key ? -15 : 0;
     int qp;
@@ -966,18 +959,9 @@ static int bink_decode_plane(BinkContext *c, AVFrame *frame, GetBitContext *gb,
     int v, col[2];
     const uint8_t *scan;
     int xoff, yoff;
-#if !defined(IDE_COMPILE) || (defined(IDE_COMPILE) && (_MSC_VER >= 1400))
 	LOCAL_ALIGNED_16(int16_t, block, [64]);
     LOCAL_ALIGNED_16(uint8_t, ublock, [64]);
     LOCAL_ALIGNED_16(int32_t, dctblock, [64]);
-#else
-	uint8_t la_block[sizeof(int16_t [64] ) + (16)];
-	int16_t (*block) = (void *)((((uintptr_t)la_block)+(16)-1)&~((16)-1));
-    uint8_t la_ublock[sizeof(uint8_t [64] ) + (16)];
-	uint8_t (*ublock) = (void *)((((uintptr_t)la_ublock)+(16)-1)&~((16)-1));
-    uint8_t la_dctblock[sizeof(int32_t [64] ) + (16)];
-	int32_t (*dctblock) = (void *)((((uintptr_t)la_dctblock)+(16)-1)&~((16)-1));
-#endif
 	int coordmap[64];
 
     const int stride = frame->linesize[plane_idx];
@@ -1359,18 +1343,6 @@ static void flush(AVCodecContext *avctx)
 }
 
 AVCodec ff_bink_decoder = {
-#ifdef IDE_COMPILE
-    "binkvideo",
-    "Bink video",
-    AVMEDIA_TYPE_VIDEO,
-    AV_CODEC_ID_BINKVIDEO,
-    CODEC_CAP_DR1,
-    0, 0, 0, 0, 0, 0, 0, 0, sizeof(BinkContext),
-    0, 0, 0, 0, 0, decode_init,
-    0, 0, decode_frame,
-    decode_end,
-    flush,
-#else
 	.name           = "binkvideo",
     .long_name      = NULL_IF_CONFIG_SMALL("Bink video"),
     .type           = AVMEDIA_TYPE_VIDEO,
@@ -1381,5 +1353,4 @@ AVCodec ff_bink_decoder = {
     .decode         = decode_frame,
     .flush          = flush,
     .capabilities   = CODEC_CAP_DR1,
-#endif
 };
