@@ -25,10 +25,6 @@
  * @author Stefan Gehrer <stefan.gehrer@gmx.de>
  */
 
-#ifdef IDE_COMPILE
-#include "libavutil/internal.h"
-#endif
-
 #include "libavutil/avassert.h"
 #include "avcodec.h"
 #include "get_bits.h"
@@ -1245,18 +1241,6 @@ static int cavs_decode_frame(AVCodecContext *avctx, void *data, int *got_frame,
 }
 
 AVCodec ff_cavs_decoder = {
-#ifdef IDE_COMPILE
-    "cavs",
-    "Chinese AVS (Audio Video Standard) (AVS1-P2, JiZhun profile)",
-    AVMEDIA_TYPE_VIDEO,
-    AV_CODEC_ID_CAVS,
-    CODEC_CAP_DR1 | CODEC_CAP_DELAY,
-    0, 0, 0, 0, 0, 0, 0, 0, sizeof(AVSContext),
-    0, 0, 0, 0, 0, ff_cavs_init,
-    0, 0, cavs_decode_frame,
-    ff_cavs_end,
-    cavs_flush,
-#else
 	.name           = "cavs",
     .long_name      = NULL_IF_CONFIG_SMALL("Chinese AVS (Audio Video Standard) (AVS1-P2, JiZhun profile)"),
     .type           = AVMEDIA_TYPE_VIDEO,
@@ -1267,5 +1251,4 @@ AVCodec ff_cavs_decoder = {
     .decode         = cavs_decode_frame,
     .capabilities   = CODEC_CAP_DR1 | CODEC_CAP_DELAY,
     .flush          = cavs_flush,
-#endif
 };
