@@ -33,10 +33,6 @@
 #include "acelp_vectors.h"
 #include "lsp.h"
 
-#ifdef IDE_COMPILE
-#include "libavutil/libm.h"
-#endif
-
 #define MIN_LSP_SEP (0.05 / (2.0 * M_PI))
 #define MIN_DELAY      20
 #define MAX_DELAY     120
@@ -910,16 +906,6 @@ erasure:
 }
 
 AVCodec ff_evrc_decoder = {
-#ifdef IDE_COMPILE
-    "evrc",
-    "EVRC (Enhanced Variable Rate Codec)",
-    AVMEDIA_TYPE_AUDIO,
-    AV_CODEC_ID_EVRC,
-    CODEC_CAP_DR1,
-    0, 0, 0, 0, 0, 0, 0, 0, sizeof(EVRCContext),
-    0, 0, 0, 0, 0, evrc_decode_init,
-    0, 0, evrc_decode_frame,
-#else
 	.name           = "evrc",
     .long_name      = NULL_IF_CONFIG_SMALL("EVRC (Enhanced Variable Rate Codec)"),
     .type           = AVMEDIA_TYPE_AUDIO,
@@ -928,5 +914,4 @@ AVCodec ff_evrc_decoder = {
     .decode         = evrc_decode_frame,
     .capabilities   = CODEC_CAP_DR1,
     .priv_data_size = sizeof(EVRCContext),
-#endif
 };
