@@ -28,12 +28,7 @@
 #include <stddef.h>
 #include <stdint.h>
 
-#ifdef IDE_COMPILE
-#include "ffmpeg-config.h"
-#include "ide-config.h"
-#else
 #include "config.h"
-#endif
 
 #include "libavutil/attributes.h"
 #include "copy_block.h"
