@@ -1708,57 +1708,6 @@ copy_avg_fn(4)
 #undef copy_avg_fn
 
 static const int8_t vp9_subpel_filters[3][15][8] = {
-#ifdef IDE_COMPILE
-    {
-        { -3, -1, 32, 64, 38, 1, -3, 0 },
-        { -2, -2, 29, 63, 41, 2, -3, 0 },
-        { -2, -2, 26, 63, 43, 4, -4, 0 },
-        { -2, -3, 24, 62, 46, 5, -4, 0 },
-        { -2, -3, 21, 60, 49, 7, -4, 0 },
-        { -1, -4, 18, 59, 51, 9, -4, 0 },
-        { -1, -4, 16, 57, 53, 12, -4, -1 },
-        { -1, -4, 14, 55, 55, 14, -4, -1 },
-        { -1, -4, 12, 53, 57, 16, -4, -1 },
-        { 0, -4, 9, 51, 59, 18, -4, -1 },
-        { 0, -4, 7, 49, 60, 21, -3, -2 },
-        { 0, -4, 5, 46, 62, 24, -3, -2 },
-        { 0, -4, 4, 43, 63, 26, -2, -2 },
-        { 0, -3, 2, 41, 63, 29, -2, -2 },
-        { 0, -3, 1, 38, 64, 32, -1, -3 },
-    }, {
-        { 0, 1, -5, 126, 8, -3, 1, 0 },
-        { -1, 3, -10, 122, 18, -6, 2, 0 },
-        { -1, 4, -13, 118, 27, -9, 3, -1 },
-        { -1, 4, -16, 112, 37, -11, 4, -1 },
-        { -1, 5, -18, 105, 48, -14, 4, -1 },
-        { -1, 5, -19, 97, 58, -16, 5, -1 },
-        { -1, 6, -19, 88, 68, -18, 5, -1 },
-        { -1, 6, -19, 78, 78, -19, 6, -1 },
-        { -1, 5, -18, 68, 88, -19, 6, -1 },
-        { -1, 5, -16, 58, 97, -19, 5, -1 },
-        { -1, 4, -14, 48, 105, -18, 5, -1 },
-        { -1, 4, -11, 37, 112, -16, 4, -1 },
-        { -1, 3, -9, 27, 118, -13, 4, -1 },
-        { 0, 2, -6, 18, 122, -10, 3, -1 },
-        { 0, 1, -3, 8, 126, -5, 1, 0 },
-    }, {
-        { -1, 3, -7, 127, 8, -3, 1, 0 },
-        { -2, 5, -13, 125, 17, -6, 3, -1 },
-        { -3, 7, -17, 121, 27, -10, 5, -2 },
-        { -4, 9, -20, 115, 37, -13, 6, -2 },
-        { -4, 10, -23, 108, 48, -16, 8, -3 },
-        { -4, 10, -24, 100, 59, -19, 9, -3 },
-        { -4, 11, -24, 90, 70, -21, 10, -4 },
-        { -4, 11, -23, 80, 80, -23, 11, -4 },
-        { -4, 10, -21, 70, 90, -24, 11, -4 },
-        { -3, 9, -19, 59, 100, -24, 10, -4 },
-        { -3, 8, -16, 48, 108, -23, 10, -4 },
-        { -2, 6, -13, 37, 115, -20, 9, -4 },
-        { -2, 5, -10, 27, 121, -17, 7, -3 },
-        { -1, 3, -6, 17, 125, -13, 5, -2 },
-        { 0, 1, -3, 8, 127, -7, 3, -1 },
-    }
-#else
 	[FILTER_8TAP_REGULAR] = {
         {  0,  1,  -5, 126,   8,  -3,  1,  0 },
         { -1,  3, -10, 122,  18,  -6,  2,  0 },
@@ -1808,7 +1757,6 @@ static const int8_t vp9_subpel_filters[3][15][8] = {
         {  0, -3,   2,  41,  63,  29, -2, -2 },
         {  0, -3,   1,  38,  64,  32, -1, -3 },
     }
-#endif
 };
 
 #define FILTER_8TAP(src, x, F, stride) \
