@@ -26,10 +26,6 @@
  * http://tools.ietf.org/html/draft-pantos-http-live-streaming
  */
 
-#ifdef IDE_COMPILE
-#include "libavutil/internal.h"
-#endif
-
 #include "libavutil/avstring.h"
 #include "libavutil/avassert.h"
 #include "libavutil/intreadwrite.h"
@@ -1486,21 +1482,10 @@ static int recheck_discard_flags(AVFormatContext *s, int first)
 static void fill_timing_for_id3_timestamped_stream(struct playlist *pls)
 {
     if (pls->id3_offset >= 0) {
-#ifdef IDE_COMPILE
-        AVRational tmp;
-
-		tmp.num = 1;
-		tmp.den = MPEG_TIME_BASE;
-		pls->pkt.dts = pls->id3_mpegts_timestamp +
-                                 av_rescale_q(pls->id3_offset,
-                                              pls->ctx->streams[pls->pkt.stream_index]->time_base,
-                                              tmp);
-#else
 		pls->pkt.dts = pls->id3_mpegts_timestamp +
                                  av_rescale_q(pls->id3_offset,
                                               pls->ctx->streams[pls->pkt.stream_index]->time_base,
                                               MPEG_TIME_BASE_Q);
-#endif
 		if (pls->pkt.duration)
             pls->id3_offset += pls->pkt.duration;
         else
@@ -1512,19 +1497,9 @@ static void fill_timing_for_id3_timestamped_stream(struct playlist *pls)
     }
 
     if (pls->pkt.duration) {
-#ifdef IDE_COMPILE
-        AVRational tmp;
-		
-		tmp.num = 1;
-		tmp.den = MPEG_TIME_BASE;
-		pls->pkt.duration = av_rescale_q(pls->pkt.duration,
-                                         pls->ctx->streams[pls->pkt.stream_index]->time_base,
-                                         tmp);
-#else
 		pls->pkt.duration = av_rescale_q(pls->pkt.duration,
                                          pls->ctx->streams[pls->pkt.stream_index]->time_base,
                                          MPEG_TIME_BASE_Q);
-#endif
 	}
 	pls->pkt.pts = AV_NOPTS_VALUE;
 }
@@ -1532,15 +1507,7 @@ static void fill_timing_for_id3_timestamped_stream(struct playlist *pls)
 static AVRational get_timebase(struct playlist *pls)
 {
     if (pls->is_id3_timestamped) {
-#ifdef IDE_COMPILE
-        AVRational tmp;
-		
-		tmp.num = 1;
-		tmp.den = MPEG_TIME_BASE;
-		return tmp;
-#else
 		return MPEG_TIME_BASE_Q;
-#endif
 	}
 
     return pls->ctx->streams[pls->pkt.stream_index]->time_base;
@@ -1549,19 +1516,8 @@ static AVRational get_timebase(struct playlist *pls)
 static int compare_ts_with_wrapdetect(int64_t ts_a, struct playlist *pls_a,
                                       int64_t ts_b, struct playlist *pls_b)
 {
-#ifdef IDE_COMPILE
-	int64_t scaled_ts_a;
-	int64_t scaled_ts_b;
-    AVRational tmp;
-	
-	tmp.num = 1;
-	tmp.den = MPEG_TIME_BASE;
-	scaled_ts_a = av_rescale_q(ts_a, get_timebase(pls_a), tmp);
-    scaled_ts_b = av_rescale_q(ts_b, get_timebase(pls_b), tmp);
-#else
 	int64_t scaled_ts_a = av_rescale_q(ts_a, get_timebase(pls_a), MPEG_TIME_BASE_Q);
     int64_t scaled_ts_b = av_rescale_q(ts_b, get_timebase(pls_b), MPEG_TIME_BASE_Q);
-#endif
 
     return av_compare_mod(scaled_ts_a, scaled_ts_b, LLN(1) << 33);
 }
@@ -1596,17 +1552,8 @@ static int hls_read_packet(AVFormatContext *s, AVPacket *pkt)
 
                     if (c->first_timestamp == AV_NOPTS_VALUE &&
                         pls->pkt.dts       != AV_NOPTS_VALUE) {
-#ifdef IDE_COMPILE
-                        AVRational tmp;
-						
-						tmp.num = 1;
-						tmp.den = AV_TIME_BASE;
-						c->first_timestamp = av_rescale_q(pls->pkt.dts,
-                            get_timebase(pls), tmp);
-#else
 						c->first_timestamp = av_rescale_q(pls->pkt.dts,
                             get_timebase(pls), AV_TIME_BASE_Q);
-#endif
 					}
 				}
 
@@ -1660,19 +1607,9 @@ static int hls_read_packet(AVFormatContext *s, AVPacket *pkt)
         reset_packet(&c->playlists[minplaylist]->pkt);
 
         if (pkt->dts != AV_NOPTS_VALUE) {
-#ifdef IDE_COMPILE
-            AVRational tmp;
-			
-			tmp.num = 1;
-			tmp.den = AV_TIME_BASE;
-			c->cur_timestamp = av_rescale_q(pkt->dts,
-                                            pls->ctx->streams[pls->pkt.stream_index]->time_base,
-                                            tmp);
-#else
 			c->cur_timestamp = av_rescale_q(pkt->dts,
                                             pls->ctx->streams[pls->pkt.stream_index]->time_base,
                                             AV_TIME_BASE_Q);
-#endif
 		}
         return 0;
     }
@@ -1783,16 +1720,6 @@ static int hls_probe(AVProbeData *p)
 }
 
 AVInputFormat ff_hls_demuxer = {
-#ifdef IDE_COMPILE
-    "hls,applehttp",
-    "Apple HTTP Live Streaming",
-    0, 0, 0, 0, 0, 0, 0, sizeof(HLSContext),
-    hls_probe,
-    hls_read_header,
-    hls_read_packet,
-    hls_close,
-    hls_read_seek,
-#else
 	.name           = "hls,applehttp",
     .long_name      = NULL_IF_CONFIG_SMALL("Apple HTTP Live Streaming"),
     .priv_data_size = sizeof(HLSContext),
@@ -1801,5 +1728,4 @@ AVInputFormat ff_hls_demuxer = {
     .read_packet    = hls_read_packet,
     .read_close     = hls_close,
     .read_seek      = hls_read_seek,
-#endif
 };
