@@ -19,10 +19,6 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
-#ifdef __MSDOS__
-#define HAVE_INT32_T
-#endif
-
 #include "config.h"
 
 #if CONFIG_ZLIB
