@@ -30,10 +30,6 @@
 
 #include <inttypes.h>
 
-#ifdef IDE_COMPILE
-#include "libavutil/internal.h"
-#endif
-
 #include "libavutil/avassert.h"
 #include "libavutil/channel_layout.h"
 #include "libavutil/intreadwrite.h"
@@ -748,15 +744,6 @@ static int iff_read_packet(AVFormatContext *s,
 }
 
 AVInputFormat ff_iff_demuxer = {
-#ifdef IDE_COMPILE
-    "iff",
-    "IFF (Interchange File Format)",
-    AVFMT_GENERIC_INDEX | AVFMT_NO_BYTE_SEEK,
-    0, 0, 0, 0, 0, 0, sizeof(IffDemuxContext),
-    iff_probe,
-    iff_read_header,
-    iff_read_packet,
-#else
 	.name           = "iff",
     .long_name      = NULL_IF_CONFIG_SMALL("IFF (Interchange File Format)"),
     .priv_data_size = sizeof(IffDemuxContext),
@@ -764,5 +751,4 @@ AVInputFormat ff_iff_demuxer = {
     .read_header    = iff_read_header,
     .read_packet    = iff_read_packet,
     .flags          = AVFMT_GENERIC_INDEX | AVFMT_NO_BYTE_SEEK,
-#endif
 };
