@@ -396,19 +396,9 @@ static int nsv_parse_NSVs_header(AVFormatContext *s)
     if(i&0x80) { /* odd way of giving native framerates from docs */
         int t=(i & 0x7F)>>2;
         if(t<16) {
-#ifdef IDE_COMPILE
-			framerate.num = 1;
-			framerate.den = t+1;
-#else
 			framerate = (AVRational){1, t+1};
-#endif
 		} else {
-#ifdef IDE_COMPILE
-			framerate.num = t-15;
-			framerate.den = 1;
-#else
 			framerate = (AVRational){t-15, 1};
-#endif
 		}
         if(i&1){
             framerate.num *= 1000;
@@ -420,12 +410,7 @@ static int nsv_parse_NSVs_header(AVFormatContext *s)
         else              framerate.num *= 30;
     }
     else {
-#ifdef IDE_COMPILE
-		framerate.num = i;
-		framerate.den = 1;
-#else
 		framerate= (AVRational){i, 1};
-#endif
 	}
 
     nsv->avsync = avio_rl16(pb);
@@ -768,16 +753,6 @@ static int nsv_probe(AVProbeData *p)
 }
 
 AVInputFormat ff_nsv_demuxer = {
-#ifdef IDE_COMPILE
-    "nsv",
-    "Nullsoft Streaming Video",
-    0, 0, 0, 0, 0, 0, 0, sizeof(NSVContext),
-    nsv_probe,
-    nsv_read_header,
-    nsv_read_packet,
-    nsv_read_close,
-    nsv_read_seek,
-#else
 	.name           = "nsv",
     .long_name      = NULL_IF_CONFIG_SMALL("Nullsoft Streaming Video"),
     .priv_data_size = sizeof(NSVContext),
@@ -786,5 +761,4 @@ AVInputFormat ff_nsv_demuxer = {
     .read_packet    = nsv_read_packet,
     .read_close     = nsv_read_close,
     .read_seek      = nsv_read_seek,
-#endif
 };
