@@ -24,10 +24,6 @@
  * RTP protocol
  */
 
-#ifdef __MSDOS__
-#define HAVE_INT32_T
-#endif
-
 #include "libavutil/parseutils.h"
 #include "libavutil/avstring.h"
 #include "avformat.h"
