@@ -142,8 +142,6 @@ blocktype_d[2]        block type to use for previous granule
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
-#elif IDE_COMPILE
-# include "lame-config.h"
 #endif
 
 #include "lame.h"
