@@ -25,8 +25,6 @@
 /* $Id: quantize_pvt.c,v 1.169.2.2 2012/02/07 13:40:37 robert Exp $ */
 #ifdef HAVE_CONFIG_H
 # include <config.h>
-#elif IDE_COMPILE
-# include "lame-config.h"
 #endif
 
 
