@@ -192,15 +192,9 @@ void x264_me_search_ref( x264_t *h, x264_me_t *m, int16_t (*mvc)[2], int i_mvc,
     pixel *p_fenc = m->p_fenc[0];
     pixel *p_fref_w = m->p_fref_w;
 
-#if !defined(IDE_COMPILE) || (defined(IDE_COMPILE) && (_MSC_VER >= 1400))
 	ALIGNED_ARRAY_N( pixel, pix,[16*16] );
     ALIGNED_ARRAY_8( int16_t, mvc_temp,[16],[2] );
     ALIGNED_ARRAY_16( int, costs,[16] );
-#else
-	__declspec(align(32))pixel pix[16*16];
-    __declspec(align(8))int16_t mvc_temp[16][2];
-    __declspec(align(16))int costs[16];
-#endif
 
     int mv_x_min = h->mb.mv_limit_fpel[0][0];
     int mv_y_min = h->mb.mv_limit_fpel[0][1];
@@ -652,13 +646,8 @@ void x264_me_search_ref( x264_t *h, x264_me_t *m, int16_t (*mvc)[2], int i_mvc,
              * because sum(abs(diff)) >= abs(diff(sum)). */
             uint16_t *sums_base = m->integral;
 
-#if !defined(IDE_COMPILE) || (defined(IDE_COMPILE) && (_MSC_VER >= 1400))
 			ALIGNED_16( static pixel zero[8*FENC_STRIDE] ) = {0};
             ALIGNED_ARRAY_16( int, enc_dc,[4] );
-#else
-			__declspec(align(16))static pixel zero[8*16] = {0};
-            __declspec(align(16))int enc_dc[4];
-#endif
 
 			int sad_size = i_pixel <= PIXEL_8x8 ? PIXEL_8x8 : PIXEL_4x4;
             int delta = x264_pixel_size[sad_size].w;
@@ -701,11 +690,7 @@ void x264_me_search_ref( x264_t *h, x264_me_t *m, int16_t (*mvc)[2], int i_mvc,
 						int j;
 						pixel *ref = p_fref_w+min_x+my*stride;
 
-#if !defined(IDE_COMPILE) || (defined(IDE_COMPILE) && (_MSC_VER >= 1400))
 						ALIGNED_ARRAY_16( int, sads,[4] ); /* padded to [4] for asm */
-#else
-						__declspec(align(16))int sads[4];
-#endif
 
 						h->pixf.sad_x3[i_pixel]( p_fenc, ref+xs[i], ref+xs[i+1], ref+xs[i+2], stride, sads );
                         for( j = 0; j < 3; j++ )
@@ -911,13 +896,8 @@ static void refine_subpel( x264_t *h, x264_me_t *m, int hpel_iters, int qpel_ite
     int chroma_v_shift = CHROMA_V_SHIFT;
     int mvy_offset = chroma_v_shift & MB_INTERLACED & m->i_ref ? (h->mb.i_mb_y & 1)*4 - 2 : 0;
 
-#if !defined(IDE_COMPILE) || (defined(IDE_COMPILE) && (_MSC_VER >= 1400))
     ALIGNED_ARRAY_N( pixel, pix,[64*18] ); // really 17x17x2, but round up for alignment
     ALIGNED_ARRAY_16( int, costs,[4] );
-#else
-	__declspec(align(32))pixel pix[64*18];
-    __declspec(align(16))int costs[4];
-#endif
 
 	int bmx = m->mv[0];
     int bmy = m->mv[1];
@@ -1081,15 +1061,9 @@ static void ALWAYS_INLINE x264_me_refine_bidir( x264_t *h, x264_me_t *m0, x264_m
     const int bw = x264_pixel_size[i_pixel].w;
     const int bh = x264_pixel_size[i_pixel].h;
 
-#if !defined(IDE_COMPILE) || (defined(IDE_COMPILE) && (_MSC_VER >= 1400))
 	ALIGNED_ARRAY_N( pixel, pixy_buf,[2],[9][16*16] );
     ALIGNED_ARRAY_N( pixel, pixu_buf,[2],[9][16*16] );
     ALIGNED_ARRAY_N( pixel, pixv_buf,[2],[9][16*16] );
-#else
-	__declspec(align(32))pixel pixy_buf[2][9][16*16];
-    __declspec(align(32))pixel pixu_buf[2][9][16*16];
-    __declspec(align(32))pixel pixv_buf[2][9][16*16];
-#endif
 
 	pixel *src[3][2][9];
     int chromapix = h->luma2chroma_pixel[i_pixel];
@@ -1118,15 +1092,10 @@ static void ALWAYS_INLINE x264_me_refine_bidir( x264_t *h, x264_me_t *m0, x264_m
     const uint16_t *p_cost_m1y;
 	int pass;
 
-#if !defined(IDE_COMPILE) || (defined(IDE_COMPILE) && (_MSC_VER >= 1400))
 	/* each byte of visited represents 8 possible m1y positions, so a 4D array isn't needed */
     ALIGNED_ARRAY_N( uint8_t, visited,[8],[8][8] );
     /* all permutations of an offset in up to 2 of the dimensions */
     ALIGNED_4( static const int8_t dia4d[33][4] ) =
-#else
-	__declspec(align(32))uint8_t visited[8][8][8];
-    __declspec(align(4))static const int8_t dia4d[33][4] =
-#endif
 	{
         {0,0,0,0},
         {0,0,0,1}, {0,0,0,-1}, {0,0,1,0}, {0,0,-1,0},
