@@ -29,6 +29,8 @@
 #ifndef VICE_MONTYPES_H
 #define VICE_MONTYPES_H
 
+#include <stdbool.h>
+
 #include "monitor.h"
 #include "types.h"
 
@@ -39,70 +41,71 @@
 
 /* Types */
 
-#ifndef bool
-typedef int bool;
-#endif
-
 /* 65xx: 6502/6509/6510/7501/8500/8501/8502/65C02/65SC02
    658xx: 65802/65816
    6x09: 6809/6309
+
+   These values are used in the binary monitor API, so it
+   is important that they remain consistent.
  */
-enum t_reg_id {
-    e_A,		/* 65xx/c64dtv/658xx/6x09/z80 */
-    e_X,		/* 65xx/c64dtv/658xx/6x09 */
-    e_Y,		/* 65xx/c64dtv/658xx/6x09 */
-    e_PC,		/* 65xx/c64dtv/658xx/6x09/z80 */
-    e_SP,		/* 65xx/c64dtv/658xx/6x09/z80 */
-    e_FLAGS,	/* 65xx/c64dtv/658xx/6x09 */
-    e_AF,		/* z80 */
-    e_BC,		/* z80 */
-    e_DE,		/* z80 */
-    e_HL,		/* z80 */
-    e_IX,		/* z80 */
-    e_IY,		/* z80 */
-    e_I,		/* z80 */
-    e_R,		/* z80 */
-    e_AF2,		/* z80 */
-    e_BC2,		/* z80 */
-    e_DE2,		/* z80 */
-    e_HL2,		/* z80 */
-    e_R3,		/* c64dtv */
-    e_R4,		/* c64dtv */
-    e_R5,		/* c64dtv */
-    e_R6,		/* c64dtv */
-    e_R7,		/* c64dtv */
-    e_R8,		/* c64dtv */
-    e_R9,		/* c64dtv */
-    e_R10,		/* c64dtv */
-    e_R11,		/* c64dtv */
-    e_R12,		/* c64dtv */
-    e_R13,		/* c64dtv */
-    e_R14,		/* c64dtv */
-    e_R15,		/* c64dtv */
-    e_ACM,		/* c64dtv */
-    e_YXM,		/* c64dtv */
-    e_B,		/* 658xx/6x09/z80 */
-    e_C,		/* 658xx/z80 */
-    e_DPR,		/* 658xx */
-    e_PBR,		/* 658xx */
-    e_DBR,		/* 658xx */
-    e_D,		/* 6x09/z80 */
-    e_U,		/* 6x09 */
-    e_DP,		/* 6x09 */
-    e_E,		/* 658xx/6309/z80 */
-    e_F,		/* 6309 */
-    e_W,		/* 6309 */
-    e_Q,		/* 6309 */
-    e_V,		/* 6309 */
-    e_MD,		/* 6309 */
-    e_H,		/* z80 */
-    e_L,		/* z80 */
-    e_IXL,		/* z80 */
-    e_IXH,		/* z80 */
-    e_IYL,		/* z80 */
-    e_IYH,		/* z80 */
-    e_Rasterline,	/* Rasterline */
-    e_Cycle	/* Cycle */
+enum t_reg_id       {
+    e_A            = 0x00, /* 65xx/c64dtv/658xx/6x09/z80 */
+    e_X            = 0x01, /* 65xx/c64dtv/658xx/6x09     */
+    e_Y            = 0x02, /* 65xx/c64dtv/658xx/6x09     */
+    e_PC           = 0x03, /* 65xx/c64dtv/658xx/6x09/z80 */
+    e_SP           = 0x04, /* 65xx/c64dtv/658xx/6x09/z80 */
+    e_FLAGS        = 0x05, /* 65xx/c64dtv/658xx/6x09     */
+    e_AF           = 0x06, /* z80                        */
+    e_BC           = 0x07, /* z80                        */
+    e_DE           = 0x08, /* z80                        */
+    e_HL           = 0x09, /* z80                        */
+    e_IX           = 0x0a, /* z80                        */
+    e_IY           = 0x0b, /* z80                        */
+    e_I            = 0x0c, /* z80                        */
+    e_R            = 0x0d, /* z80                        */
+    e_AF2          = 0x0e, /* z80                        */
+    e_BC2          = 0x0f, /* z80                        */
+    e_DE2          = 0x10, /* z80                        */
+    e_HL2          = 0x11, /* z80                        */
+    e_R3           = 0x12, /* c64dtv                     */
+    e_R4           = 0x13, /* c64dtv                     */
+    e_R5           = 0x14, /* c64dtv                     */
+    e_R6           = 0x15, /* c64dtv                     */
+    e_R7           = 0x16, /* c64dtv                     */
+    e_R8           = 0x17, /* c64dtv                     */
+    e_R9           = 0x18, /* c64dtv                     */
+    e_R10          = 0x19, /* c64dtv                     */
+    e_R11          = 0x1a, /* c64dtv                     */
+    e_R12          = 0x1b, /* c64dtv                     */
+    e_R13          = 0x1c, /* c64dtv                     */
+    e_R14          = 0x1d, /* c64dtv                     */
+    e_R15          = 0x1e, /* c64dtv                     */
+    e_ACM          = 0x1f, /* c64dtv                     */
+    e_YXM          = 0x20, /* c64dtv                     */
+    e_B            = 0x21, /* 658xx/6x09/z80             */
+    e_C            = 0x22, /* 658xx/z80                  */
+    e_DPR          = 0x23, /* 658xx                      */
+    e_PBR          = 0x24, /* 658xx                      */
+    e_DBR          = 0x25, /* 658xx                      */
+    e_D            = 0x26, /* 6x09/z80                   */
+    e_U            = 0x27, /* 6x09                       */
+    e_DP           = 0x28, /* 6x09                       */
+    e_E            = 0x29, /* 658xx/6309/z80             */
+    e_F            = 0x2a, /* 6309                       */
+    e_W            = 0x2b, /* 6309                       */
+    e_Q            = 0x2c, /* 6309                       */
+    e_V            = 0x2d, /* 6309                       */
+    e_MD           = 0x2e, /* 6309                       */
+    e_H            = 0x2f, /* z80                        */
+    e_L            = 0x30, /* z80                        */
+    e_IXL          = 0x31, /* z80                        */
+    e_IXH          = 0x32, /* z80                        */
+    e_IYL          = 0x33, /* z80                        */
+    e_IYH          = 0x34, /* z80                        */
+    e_Rasterline   = 0x35, /* Rasterline                 */
+    e_Cycle        = 0x36, /* Cycle                      */
+    e_Zero         = 0x37, /* 6510/c64dtv                */
+    e_One          = 0x38  /* 6510/c64dtv                */
 };
 typedef enum t_reg_id REG_ID;
 
@@ -193,9 +196,6 @@ typedef void monitor_toggle_func_t(int value);
 #define STATE_BNAME    4
 #define STATE_CTYPE    5
 
-#define FIRST_SPACE e_comp_space
-#define LAST_SPACE e_disk11_space
-
 #define DEFAULT_DISASSEMBLY_SIZE 40
 
 #define any_watchpoints_load(mem) (watchpoints_load[(mem)] != NULL)
@@ -219,13 +219,14 @@ typedef void monitor_toggle_func_t(int value);
 
 /* Global variables */
 
-extern const char *_mon_space_strings[];
+extern const char * const _mon_space_strings[];
 
 struct console_s;
 struct monitor_cpu_type_s;
 
 extern struct console_s *console_log;
 extern int sidefx;
+extern int break_on_dummy_access;
 extern int exit_mon;
 
 extern RADIXTYPE default_radix;
@@ -234,7 +235,7 @@ extern bool asm_mode;
 extern MON_ADDR asm_mode_addr;
 extern struct monitor_cpu_type_s *monitor_cpu_for_memspace[NUM_MEMSPACES];
 extern MON_ADDR dot_addr[NUM_MEMSPACES];
-extern const char *mon_memspace_string[];
+extern const char * const mon_memspace_string[];
 extern int mon_stop_output;
 extern monitor_interface_t *mon_interfaces[NUM_MEMSPACES];
 extern bool force_array[NUM_MEMSPACES];
@@ -255,8 +256,10 @@ extern void mon_resource_set(const char *name, const char* value);
 extern void mon_screenshot_save(const char* filename, int format);
 extern void mon_show_dir(const char *path);
 extern void mon_show_pwd(void);
+extern void mon_make_dir(const char *path);
+extern void mon_remove_dir(const char *path);
 extern void mon_tape_ctrl(int command);
-extern void mon_display_screen(void);
+extern void mon_display_screen(long addr);
 extern void mon_instructions_step(int count);
 extern void mon_instructions_next(int count);
 extern void mon_instruction_return(void);
@@ -267,10 +270,15 @@ extern void mon_change_dir(const char *path);
 extern void mon_bank(MEMSPACE mem, const char *bank);
 extern const char *mon_get_current_bank_name(MEMSPACE mem);
 extern const char *mon_get_bank_name_for_bank(MEMSPACE mem, int banknum);
+extern int mon_get_bank_index_for_bank(MEMSPACE mem, int banknum);
+extern int mon_get_bank_flags_for_bank(MEMSPACE mem, int banknum);
+extern const int mon_banknum_validate(MEMSPACE mem, int banknum);
 extern int mon_banknum_from_bank(MEMSPACE mem, const char *bankname);
+
 extern void mon_display_io_regs(MON_ADDR addr);
 extern void mon_evaluate_default_addr(MON_ADDR *a);
 extern void mon_set_mem_val(MEMSPACE mem, uint16_t mem_addr, uint8_t val);
+extern void mon_set_mem_val_ex(MEMSPACE mem, int bank, uint16_t mem_addr, uint8_t val);
 extern bool mon_inc_addr_location(MON_ADDR *a, unsigned inc);
 extern void mon_start_assemble_mode(MON_ADDR addr, char *asm_line);
 extern long mon_evaluate_address_range(MON_ADDR *start_addr, MON_ADDR *end_addr,
@@ -280,12 +288,18 @@ extern bool check_drive_emu_level_ok(int drive_num);
 extern void mon_print_conditional(cond_node_t *cnode);
 extern void mon_delete_conditional(cond_node_t *cnode);
 extern int mon_evaluate_conditional(cond_node_t *cnode);
+extern int mon_write_snapshot(const char* name, int save_roms, int save_disks, int even_mode);
+extern int mon_read_snapshot(const char* name, int even_mode);
 extern bool mon_is_valid_addr(MON_ADDR a);
 extern bool mon_is_in_range(MON_ADDR start_addr, MON_ADDR end_addr,
                             unsigned loc);
 extern void mon_print_bin(int val, char on, char off);
 extern uint8_t mon_get_mem_val(MEMSPACE mem, uint16_t mem_addr);
 extern uint8_t mon_get_mem_val_ex(MEMSPACE mem, int bank, uint16_t mem_addr);
+/* the _nosfx variants must be used when the monitor must absolutely not cause
+   any sideeffect, be it emulated I/O or (re)triggering checkpoints */
+extern uint8_t mon_get_mem_val_ex_nosfx(MEMSPACE mem, int bank, uint16_t mem_addr);
+extern uint8_t mon_get_mem_val_nosfx(MEMSPACE mem, uint16_t mem_addr);
 extern void mon_get_mem_block(MEMSPACE mem, uint16_t mem_start, uint16_t mem_end, uint8_t *data);
 extern void mon_get_mem_block_ex(MEMSPACE mem, int bank, uint16_t mem_start, uint16_t mem_end, uint8_t *data);
 extern void mon_jump(MON_ADDR addr);
@@ -305,7 +319,7 @@ extern void mon_save_symbols(MEMSPACE mem, const char *filename);
 
 extern void mon_record_commands(char *filename);
 extern void mon_end_recording(void);
-extern void mon_playback_init(const char* filename);
+extern int mon_playback_commands(const char* filename);
 extern void monitor_change_device(MEMSPACE mem);
 
 extern void mon_export(void);
@@ -314,4 +328,7 @@ extern void mon_stopwatch_show(const char* prefix, const char* suffix);
 extern void mon_stopwatch_reset(void);
 extern void mon_maincpu_toggle_trace(int state);
 
+extern void mon_breakpoint_set_dummy_state(MEMSPACE mem, int state);
+
+extern void mon_update_all_checkpoint_state(void);
 #endif
