@@ -51,12 +51,12 @@ extern int palette_save(const char *file_name, const palette_t *palette);
 
 /* palette info for GUIs */
 typedef struct {
-    char *chip; /* chip this palette belongs to */
-    char *name; /* name to be used in menus */
-    char *file; /* filename of the palette file */
+    const char *chip; /* chip this palette belongs to */
+    const char *name; /* name to be used in menus */
+    const char *file; /* filename of the palette file */
 } palette_info_t;
 
 /* returns pointer to palette_info_t entries. may return an empty list. */
-extern palette_info_t *palette_get_info_list(void);
+extern const palette_info_t *palette_get_info_list(void);
 
 #endif
