@@ -49,17 +49,18 @@ static int sidcart_sound_machine_init(sound_t *psid, int speed, int cycles_per_s
     }
 }
 
+/* PET SID cartridge sound chip */
 static sound_chip_t sidcart_sound_chip = {
-    sid_sound_machine_open,
-    sidcart_sound_machine_init,
-    sid_sound_machine_close,
-    sid_sound_machine_calculate_samples,
-    sid_sound_machine_store,
-    sid_sound_machine_read,
-    sid_sound_machine_reset,
-    sid_sound_machine_cycle_based,
-    sid_sound_machine_channels,
-    0 /* chip enabled */
+    sid_sound_machine_open,              /* sound chip open function */ 
+    sidcart_sound_machine_init,          /* sound chip init function */
+    sid_sound_machine_close,             /* sound chip close function */
+    sid_sound_machine_calculate_samples, /* sound chip calculate samples function */
+    sid_sound_machine_store,             /* sound chip store function */
+    sid_sound_machine_read,              /* sound chip read function */
+    sid_sound_machine_reset,             /* sound chip reset function */
+    sid_sound_machine_cycle_based,       /* sound chip 'is_cycle_based()' function, RESID engine is cycle based, all other engines are NOT */
+    sid_sound_machine_channels,          /* sound chip 'get_amount_of_channels()' function, sound chip has 1 channel */
+    0                                    /* sound chip enabled flag, toggled upon device (de-)activation */
 };
 
 static uint16_t sidcart_sound_chip_offset = 0;
@@ -72,33 +73,35 @@ void sidcart_sound_chip_init(void)
 /* ------------------------------------------------------------------------- */
 
 static io_source_t sidcart_8f00_device = {
-    "SIDCART",
-    IO_DETACH_CART, /* dummy */
-    NULL,           /* dummy */
-    0x8f00, 0x8fff, 0x1f,
-    1, /* read is always valid */
-    sid_store,
-    sid_read,
-    NULL, /* no peek */
-    sid_dump,
-    0, /* dummy (not a cartridge) */
-    IO_PRIO_NORMAL,
-    0
+    "SIDCART",            /* name of the device */
+    IO_DETACH_RESOURCE,   /* use resource to detach the device when involved in a read-collision */
+    "SidCart",            /* resource to set to '0' */
+    0x8f00, 0x8fff, 0x1f, /* range for the device, regs:$8f00-$8f1f, mirrors:$8f20-$8fff */
+    1,                    /* read is always valid */
+    sid_store,            /* store function */
+    NULL,                 /* NO poke function */
+    sid_read,             /* read function */
+    sid_peek,             /* peek function */
+    sid_dump,             /* device state information dump function */
+    IO_CART_ID_NONE,      /* not a cartridge */
+    IO_PRIO_NORMAL,       /* normal priority, device read needs to be checked for collisions */
+    0                     /* insertion order, gets filled in by the registration function */
 };
 
 static io_source_t sidcart_e900_device = {
-    "SIDCART",
-    IO_DETACH_CART, /* dummy */
-    NULL,           /* dummy */
-    0xe900, 0xe9ff, 0x1f,
-    1, /* read is always valid */
-    sid_store,
-    sid_read,
-    NULL, /* no peek */
-    sid_dump,
-    0, /* dummy (not a cartridge) */
-    IO_PRIO_NORMAL,
-    0
+    "SIDCART",            /* name of the device */
+    IO_DETACH_RESOURCE,   /* use resource to detach the device when involved in a read-collision */
+    "SidCart",            /* resource to set to '0' */
+    0xe900, 0xe9ff, 0x1f, /* range for the device, regs:$e900-$e91f, mirrors:$e920-$e9ff */
+    1,                    /* read is always valid */
+    sid_store,            /* store function */
+    NULL,                 /* NO poke function */
+    sid_read,             /* read function */
+    sid_peek,             /* peek function */
+    sid_dump,             /* device state information dump function */
+    IO_CART_ID_NONE,      /* not a cartridge */
+    IO_PRIO_NORMAL,       /* normal priority, device read needs to be checked for collisions */
+    0                     /* insertion order, gets filled in by the registration function */
 };
 
 static io_source_list_t *sidcart_list_item = NULL;
