@@ -36,6 +36,6 @@ extern int digiblaster_cmdline_options_init(void);
 extern void digiblaster_sound_chip_init(void);
 extern int digiblaster_enabled(void);
 
-extern void digiblaster_set_address(WORD addr);
+extern void digiblaster_set_address(uint16_t addr);
 
 #endif
