@@ -38,14 +38,29 @@
 #include "ted.h"
 #include "tedtypes.h"
 #include "video.h"
+#include "vsync.h"
 
 ted_resources_t ted_resources = { 0 };
 static video_chip_cap_t video_chip_cap;
 
-static int set_border_mode(int val, void *param)
+static int next_border_mode;
+
+static void on_vsync_set_border_mode(void *unused)
 {
     int sync;
 
+    if (resources_get_int("MachineVideoStandard", &sync) < 0) {
+        sync = MACHINE_SYNC_PAL;
+    }
+
+    if (ted_resources.border_mode != next_border_mode) {
+        ted_resources.border_mode = next_border_mode;
+        machine_change_timing(sync, ted_resources.border_mode);
+    }
+}
+
+static int set_border_mode(int val, void *param)
+{
     switch (val) {
         case TED_NORMAL_BORDERS:
         case TED_FULL_BORDERS:
@@ -56,14 +71,9 @@ static int set_border_mode(int val, void *param)
             return -1;
     }
 
-    if (resources_get_int("MachineVideoStandard", &sync) < 0) {
-        sync = MACHINE_SYNC_PAL;
-    }
+    next_border_mode = val;
+    vsync_on_vsync_do(on_vsync_set_border_mode, NULL);
 
-    if (ted_resources.border_mode != val) {
-        ted_resources.border_mode = val;
-        machine_change_timing(sync, ted_resources.border_mode);
-    }
     return 0;
 }
 
