@@ -32,8 +32,8 @@
 
 #include "sound.h"
 
-extern void ted_sound_store(WORD addr, BYTE value);
-extern BYTE ted_sound_read(WORD addr);
+extern void ted_sound_store(uint16_t addr, uint8_t value);
+extern uint8_t ted_sound_read(uint16_t addr);
 
 extern void ted_sound_reset(sound_t *psid, CLOCK cpu_clk);
 
