@@ -52,7 +52,8 @@ rawfile_info_t *rawfile_open(const char *file_name, const char *path,
     char *complete;
     FILE *fd;
     const char *mode = NULL;
-    unsigned int isdir, len;
+    unsigned int isdir;
+    size_t len;
 
     if (path == NULL) {
         complete = lib_strdup(file_name);
@@ -65,7 +66,11 @@ rawfile_info_t *rawfile_open(const char *file_name, const char *path,
         case FILEIO_COMMAND_READ:
             mode = MODE_READ;
             break;
+        case FILEIO_COMMAND_READ_WRITE:
+            mode = MODE_READ_WRITE;
+            break;
         case FILEIO_COMMAND_WRITE:
+        case FILEIO_COMMAND_OVERWRITE:
             mode = MODE_WRITE;
             break;
         case FILEIO_COMMAND_APPEND:
@@ -81,7 +86,17 @@ rawfile_info_t *rawfile_open(const char *file_name, const char *path,
     if (ioutil_stat(complete, &len, &isdir) != 0) {
         /* if stat failed exit early, except in write mode
            (since opening a non existing file creates a new file) */
-        if (command != FILEIO_COMMAND_WRITE) {
+        if (command != FILEIO_COMMAND_WRITE &&
+	    command != FILEIO_COMMAND_OVERWRITE) {
+            lib_free(complete);
+            return NULL;
+        }
+    } else {
+        if (command == FILEIO_COMMAND_WRITE) {
+        /* A real drive doesn't overwrite an existing file,
+           so we should not either.
+	   Use FILEIO_COMMAND_OVERWRITE to go ahead and
+	   overwrite the file anyway. */
             lib_free(complete);
             return NULL;
         }
@@ -137,19 +152,25 @@ unsigned int rawfile_write(rawfile_info_t *info, uint8_t *buf, unsigned int len)
     return -1;
 }
 
-int rawfile_seek_set(rawfile_info_t *info, int offset)
-{
-    return fseek(info->fd, offset, SEEK_SET);
-}
-
 unsigned int rawfile_get_bytes_left(struct rawfile_info_s *info)
 {
-    unsigned int old_pos = ftell(info->fd);
-    unsigned int size;
+    /* this is fucked */
+    long old_pos = ftell(info->fd);
+    size_t size;
     fseek(info->fd, 0, SEEK_END);
     size = ftell(info->fd);
     fseek(info->fd, old_pos, SEEK_SET);
-    return size - old_pos;
+    return (unsigned int)(size - old_pos);
+}
+
+unsigned int rawfile_seek(struct rawfile_info_s *info, off_t offset, int whence)
+{
+    return fseek(info->fd, offset, whence);
+}
+
+unsigned int rawfile_tell(struct rawfile_info_s *info)
+{
+    return (unsigned int)ftell(info->fd);
 }
 
 unsigned int rawfile_ferror(rawfile_info_t *info)
