@@ -70,8 +70,8 @@ namespace reSID
 // DAC which controls the VCRs, the NMOS op-amps, and the output buffer.
 //
 //
-// SID 6581 filter / mixer / output
-// --------------------------------
+// SID filter / mixer / output
+// ---------------------------
 // 
 //                ---------------------------------------------------
 //               |                                                   |
@@ -307,185 +307,6 @@ namespace reSID
 // transistors as one.
 //
 // ----------------------------------------------------------------------------
-//
-// SID 8580 filter / mixer / output
-// --------------------------------
-// 
-//               +---------------------------------------------------+
-//               |    $17      +----Rf-+                             |
-//               |             |       |                             |
-//               |      D4&!D5 o- \-R3-o                             |
-//               |             |       |                    $17      |
-//               |     !D4&!D5 o- \-R2-o                             |
-//               |             |       |  +---R8-- \--+  !D6&D7      |
-//               |      D4&!D5 o- \-R1-o  |           |              |
-//               |             |       |  o---RC-- \--o   D6&D7      |
-//               |   +---------o--<A]--o--o           |              |
-//               |   |                    o---R4-- \--o  D6&!D7      |
-//               |   |                    |           |              |
-//               |   |                    +---Ri-- \--o !D6&!D7      |
-//               |   |                                |              |
-// $17           |   |                    (CAP2B)     |  (CAP1B)     |
-// 0=to mixer    |   +--R8--+  +---R8--+      +---C---o      +---C---o
-// 1=to filter   |          |  |       |      |       |      |       |
-//               +------R8--o--o--[A>--o--Rfc-o--[A>--o--Rfc-o--[A>--o
-//     ve (EXT IN)          |          |              |              |
-// D3  \ ---------------R8--o          |              | (CAP2A)      | (CAP1A)
-//     |   v3               |          | vhp          | vbp          | vlp
-// D2  |   \ -----------R8--o    +-----+              |              |
-//     |   |   v2           |    |                    |              |
-// D1  |   |   \ -------R8--o    |   +----------------+              |
-//     |   |   |   v1       |    |   |                               |
-// D0  |   |   |   \ ---R8--+    |   |   +---------------------------+
-//     |   |   |   |             |   |   |
-//     R6  R6  R6  R6            R6  R6  R6
-//     |   |   |   | $18         |   |   |  $18
-//     |    \  |   | D7: 1=open   \   \   \ D6 - D4: 0=open
-//     |   |   |   |             |   |   |
-//     +---o---o---o-------------o---o---+
-//                 |
-//                 |               D3 +--/ --1R2--+
-//                 |   +---R8--+      |           |  +---R2--+
-//                 |   |       |   D2 o--/ --2R2--o  |       |
-//                 +---o--[A>--o------o           o--o--[A>--o-- vo (AUDIO OUT)
-//                                 D1 o--/ --4R2--o (4.25R2)
-//                        $18         |           |
-//                        0=open   D0 +--/ --8R2--+ (8.75R2)
-//
-//
-//
-//
-// R1 = 15.3*Ri
-// R2 =  7.3*Ri
-// R3 =  4.7*Ri
-// Rf =  1.4*Ri
-// R4 =  1.4*Ri
-// R8 =  2.0*Ri
-// RC =  2.8*Ri
-//
-//
-//
-// Op-amps
-// -------
-// Unlike the 6581, the 8580 has real OpAmps.
-//
-// Temperature compensated differential amplifier:
-//
-//                9V
-//
-//                |
-//      +-------o-o-o-------+
-//      |       |   |       |
-//      |       R   R       |
-//      +--||   |   |   ||--+
-//         ||---o   o---||
-//      +--||   |   |   ||--+
-//      |       |   |       |
-//      o-----+ |   |       o--- Va
-//      |     | |   |       |
-//      +--|| | |   |   ||--+
-//         ||-o-+---+---||
-//      +--||   |   |   ||--+
-//      |       |   |       |
-//              |   |
-//     GND      |   |      GND
-//          ||--+   +--||
-// in- -----||         ||------ in+
-//          ||----o----||
-//                |
-//                8 Current sink
-//                |
-//
-//               GND
-//
-// Inverter + non-inverting output amplifier:
-//
-// Va ---o---||-------------------o--------------------+
-//       |                        |               9V   |
-//       |             +----------+----------+     |   |
-//       |        9V   |          |     9V   | ||--+   |
-//       |         |   |      9V  |      |   +-||      |
-//       |         R   |       |  |  ||--+     ||--+   |
-//       |         |   |   ||--+  +--||            o---o--- Vout
-//       |         o---o---||        ||--+     ||--+
-//       |         |       ||--+         o-----||
-//       |     ||--+           |     ||--+     ||--+
-//       +-----||              o-----||            |
-//             ||--+           |     ||--+
-//                 |           R         |        GND
-//                             |
-//                GND                   GND
-//                            GND
-//
-//
-//
-// Virtual ground
-// --------------
-// A PolySi resitive voltage divider provides the voltage
-// for the non-inverting input of the filter op-amps.
-//
-//     5V
-//          +----------+
-//      |   |   |\     |
-//      R1  +---|-\    |
-// 5V   |       |A >---o--- Vref
-//      o-------|+/
-//  |   |       |/
-// R10  R4
-//  |   |
-//  o---+
-//  |
-// R10
-//  |
-//
-// GND
-//
-// Rn = n*R1
-//
-//
-//
-// Rfc - freq control DAC resistance ladder
-// ----------------------------------------
-// The resistance for the bandpass and lowpass integrator stages of the filter are determined
-// by an 11 bit DAC driven by the FC register.
-// If all 11 bits are '0', the impedance of the DAC would be "infinitely high".
-// To get around this, there is an 11 input NOR gate below the DAC sensing those 11 bits.
-// If they are all 0, the NOR gate gives the gate control voltage to the 12 bit DAC LSB.
-//
-//
-//
-// Crystal stabilized precision switched capacitor voltage divider
-// ---------------------------------------------------------------
-// There is a FET working as a temperature sensor close to the DACs which changes the gate voltage
-// of the frequency control DACs according to the temperature, to reduce its effects on the filter curve.
-// An asynchronous 3 bit binary counter, running at the speed of PHI2, drives two big capacitors
-// which AC resistance is then used as a voltage divider.
-// This implicates that frequency difference between PAL and NTSC might shift the filter curve by 4% or such.
-//
-// https://en.wikipedia.org/wiki/Switched_capacitor
-//
-//                                |\  OpAmp has a smaller capacitor
-//                        Vref ---|+\            than the other OPs
-//                                |A >---o--- Vdac
-//                        o-------|-/    |
-//                        |       |/     |
-//                        |              |
-//       C1               |     C2       |
-//   +---||---o---+   +---o-----||-------o
-//   |        |   |   |   |              |
-//   o----+   |   -----   |              |
-//   |    |   |   -----   +----+   +-----+
-//   |    -----     |          |   |     |
-//   |    -----     |          -----     |
-//   |      |       |          -----     |
-//   |    +-----------+          |       |
-//        | /Q      Q |          +-------+
-//  GND   +-----------+      FET close to DAC
-//        |   clk/8   |      working as temperature sensor
-//        +-----------+
-//          |       |
-//         clk1    clk2
-//
 
 // Compile-time computation of op-amp summer and mixer table offsets.
 
@@ -592,22 +413,30 @@ protected:
   int v2;
   int v1;
 
+  // Cutoff frequency DAC voltage, resonance.
+  int Vddt_Vw_2, Vw_bias;
+  int _8_div_Q;
+  // FIXME: Temporarily used for MOS 8580 emulation.
+  int w0;
+  int _1024_div_Q;
+
   chip_model sid_model;
 
-  typedef struct {
+   typedef struct {
     unsigned short vx;
     short dvx;
   } opamp_t;
 
   typedef struct {
+    int vo_N16;  // Fixed point scaling for 16 bit op-amp output.
     int kVddt;   // K*(Vdd - Vth)
+    int n_snake;
     int voice_scale_s14;
     int voice_DC;
     int ak;
     int bk;
     int vc_min;
     int vc_max;
-    double vo_N16;  // Fixed point scaling for 16 bit op-amp output.
 
     // Reverse op-amp transfer function.
     unsigned short opamp_rev[1 << 16];
@@ -619,27 +448,8 @@ protected:
     unsigned short f0_dac[1 << 11];
   } model_filter_t;
 
-  // 6581 only
-  // Cutoff frequency DAC voltage, resonance.
-  int Vddt_Vw_2, Vw_bias;
-  int _8_div_Q;
-
-  static int n_snake;
-
-  // 8580 only
-  int n_dac;
-
-  static int n_param;
-
-  // DAC gate voltage
-  int kVgt;
-
-  // Lookup tables for resonance
-  static unsigned short resonance[16][1 << 16];
-
   int solve_gain(opamp_t* opamp, int n, int vi_t, int& x, model_filter_t& mf);
   int solve_integrate_6581(int dt, int vi_t, int& x, int& vc, model_filter_t& mf);
-  int solve_integrate_8580(int dt, int vi_t, int& x, int& vc, model_filter_t& mf);
 
   // VCR - 6581 only.
   static unsigned short vcr_kVg[1 << 16];
@@ -750,10 +560,16 @@ void Filter::clock(int voice1, int voice2, int voice3)
     Vhp = f.summer[offset + f.gain[_8_div_Q][Vbp] + Vlp + Vi];
   }
   else {
-    // MOS 8580.
-    Vlp = solve_integrate_8580(1, Vbp, Vlp_x, Vlp_vc, f);
-    Vbp = solve_integrate_8580(1, Vhp, Vbp_x, Vbp_vc, f);
-    Vhp = f.summer[offset + resonance[res][Vbp] + Vlp + Vi];
+    // MOS 8580. FIXME: Not yet using op-amp model.
+
+    // delta_t = 1 is converted to seconds given a 1MHz clock by dividing
+    // with 1 000 000.
+
+    int dVbp = w0*(Vhp >> 4) >> 16;
+    int dVlp = w0*(Vbp >> 4) >> 16;
+    Vbp -= dVbp;
+    Vlp -= dVlp;
+    Vhp = (Vbp*_1024_div_Q >> 10) - Vlp - Vi;
   }
 }
 
@@ -856,7 +672,7 @@ void Filter::clock(cycle_count delta_t, int voice1, int voice2, int voice3)
     // MOS 6581.
     while (delta_t) {
       if (unlikely(delta_t < delta_t_flt)) {
-        delta_t_flt = delta_t;
+	delta_t_flt = delta_t;
       }
 
       // Calculate filter outputs.
@@ -868,16 +684,24 @@ void Filter::clock(cycle_count delta_t, int voice1, int voice2, int voice3)
     }
   }
   else {
-    // MOS 8580.
+    // MOS 8580. FIXME: Not yet using op-amp model.
     while (delta_t) {
-      if (unlikely(delta_t < delta_t_flt)) {
-        delta_t_flt = delta_t;
+      if (delta_t < delta_t_flt) {
+	delta_t_flt = delta_t;
       }
 
+      // delta_t is converted to seconds given a 1MHz clock by dividing
+      // with 1 000 000. This is done in two operations to avoid integer
+      // multiplication overflow.
+
       // Calculate filter outputs.
-      Vlp = solve_integrate_8580(delta_t_flt, Vbp, Vlp_x, Vlp_vc, f);
-      Vbp = solve_integrate_8580(delta_t_flt, Vhp, Vbp_x, Vbp_vc, f);
-      Vhp = f.summer[offset + resonance[res][Vbp] + Vlp + Vi];
+      int w0_delta_t = w0*delta_t_flt >> 2;
+
+      int dVbp = w0_delta_t*(Vhp >> 4) >> 14;
+      int dVlp = w0_delta_t*(Vbp >> 4) >> 14;
+      Vbp -= dVbp;
+      Vlp -= dVlp;
+      Vhp = (Vbp*_1024_div_Q >> 10) - Vlp - Vi;
 
       delta_t -= delta_t_flt;
     }
@@ -1452,7 +1276,17 @@ for my $mix (0..2**@i-1) {
   }
 
   // Sum the inputs in the mixer and run the mixer output through the gain.
-  return (short)(f.gain[vol][f.mixer[offset + Vi]] - (1 << 15));
+  if (sid_model == 0) {
+    return (short)(f.gain[vol][f.mixer[offset + Vi]] - (1 << 15));
+  }
+  else {
+    // FIXME: Temporary code for MOS 8580, should use code above.
+    /* do hard clipping here, else some tunes manage to overflow this
+       (eg /MUSICIANS/L/Linus/64_Forever.sid, starting at 0:44) */
+    int tmp = Vi*(int)vol >> 4;
+    if (tmp < -32768) tmp = -32768;
+    if (tmp > 32767) tmp = 32767;
+    return (short)tmp;  }
 }
 
 
@@ -1536,7 +1370,7 @@ int Filter::solve_gain(opamp_t* opamp, int n, int vi, int& x, model_filter_t& mf
     // The dividend is scaled by m^2*2^27.
     int f = a*int(unsigned(b_vx)*unsigned(b_vx) >> 12) - c - int(unsigned(b_vo)*unsigned(b_vo) >> 5);
     // The divisor is scaled by m*2^11.
-    int df = (b_vo*(dvx + (1 << 11)) - a*(b_vx*dvx >> 7)) >> 15;
+    int df = ((b_vo*(dvx + (1 << 11)) >> 1) - (a*(b_vx*dvx >> 8))) >> 14;
     // The resulting quotient is thus scaled by m*2^16.
 
     // Newton-Raphson step: xk1 = xk - f(xk)/f'(xk)
@@ -1706,7 +1540,7 @@ int Filter::solve_integrate_6581(int dt, int vi, int& vx, int& vc, model_filter_
   unsigned int Vgdt_2 = Vgdt*Vgdt;
 
   // "Snake" current, scaled by (1/m)*2^13*m*2^16*m*2^16*2^-15 = m*2^30
-  int n_I_snake = n_snake*(int(Vgst*Vgst - Vgdt_2) >> 15);
+  int n_I_snake = mf.n_snake*(int(Vgst*Vgst - Vgdt_2) >> 15);
 
   // VCR gate voltage.       // Scaled by m*2^16
   // Vg = Vddt - sqrt(((Vddt - Vw)^2 + Vgdt^2)/2)
@@ -1741,50 +1575,6 @@ int Filter::solve_integrate_6581(int dt, int vi, int& vx, int& vc, model_filter_
   return vx + (vc >> 14);
 }
 
-/*
-The 8580 integrator is similar to those found in 6581
-but the resistance is formed by multiple NMOS transistors
-in parallel controlled by the fc bits where the gate voltage
-is driven by a temperature dependent voltage divider.
-
-                 ---C---
-                |       |
-  vi -----Rfc------[A>----- vo
-                vx
-
-  IRfc + ICr = 0
-  IRfc + C*(vc - vc0)/dt = 0
-  dt/C*(IRfc) + vc - vc0 = 0
-  vc = vc0 - n*(IRfc(vi,vx))
-  vc = vc0 - n*(IRfc(vi,g(vc)))
-
-IRfc = K*W/L*(Vgst^2 - Vgdt^2) = n*((Vgt - vx)^2 - (Vgt - vi)^2)
-*/
-RESID_INLINE
-int Filter::solve_integrate_8580(int dt, int vi, int& vx, int& vc, model_filter_t& mf)
-{
-  // Note that all variables are translated and scaled in order to fit
-  // in 16 bits. It is not necessary to explicitly translate the variables here,
-  // since they are all used in subtractions which cancel out the translation:
-  // (a - t) - (b - t) = a - b
-
-  // Dac voltages.
-  unsigned int Vgst = kVgt - vx;
-  unsigned int Vgdt = (vi < kVgt) ? kVgt - vi : 0;  // triode/saturation mode
-
-  // Dac current, scaled by (1/m)*2^13*m*2^16*m*2^16*2^-15 = m*2^30
-  int n_I_rfc = n_dac*(int(Vgst*Vgst - Vgdt*Vgdt) >> 15);
-
-  // Change in capacitor charge.
-  vc -= n_I_rfc*dt;
-
-  // vx = g(vc)
-  vx = mf.opamp_rev[(vc >> 15) + (1 << 15)];
-
-  // Return vo.
-  return vx + (vc >> 14);
-}
-
 #endif // RESID_INLINING || defined(RESID_FILTER_CC)
 
 } // namespace reSID
