@@ -27,8 +27,10 @@ namespace reSID
 
 // Number of cycles after which the shift register is reset
 // when the test bit is set.
-const cycle_count SHIFT_REGISTER_RESET_6581 = 0x8000;
-const cycle_count SHIFT_REGISTER_RESET_8580 = 0x950000;
+const cycle_count SHIFT_REGISTER_RESET_START_6581 =    9768; // 0x8000
+const cycle_count SHIFT_REGISTER_RESET_BIT_6581   =    1000;
+const cycle_count SHIFT_REGISTER_RESET_START_8580 = 2519864; // 0x950000
+const cycle_count SHIFT_REGISTER_RESET_BIT_8580   =  315000;
 
 // Number of cycles after which the waveform output fades to 0 when setting
 // the waveform register to 0.
@@ -38,8 +40,10 @@ const cycle_count SHIFT_REGISTER_RESET_8580 = 0x950000;
 //
 // This can't be found via sampling OSC3, it seems that
 // the actual analog output must be sampled and timed.
-const cycle_count FLOATING_OUTPUT_TTL_6581 = 200000;  // ~200ms
-const cycle_count FLOATING_OUTPUT_TTL_8580 = 5000000; // ~5s
+const cycle_count FLOATING_OUTPUT_TTL_START_6581 =  182000;  // ~200ms
+const cycle_count FLOATING_OUTPUT_TTL_BIT_6581   =    1500;
+const cycle_count FLOATING_OUTPUT_TTL_START_8580 = 4400000; // ~5s
+const cycle_count FLOATING_OUTPUT_TTL_BIT_8580   =   50000;
 
 // Waveform lookup tables.
 unsigned short WaveformGenerator::model_wave[2][8][1 << 12] = {
@@ -173,6 +177,8 @@ bool do_pre_writeback(reg8 waveform_prev, reg8 waveform, bool is6581)
     // This need more investigation
     if (waveform == 8)
         return false;
+    if (waveform_prev == 0xc)
+        return false;
     // What's happening here?
     if (is6581 &&
             ((((waveform_prev & 0x3) == 0x1) && ((waveform & 0x3) == 0x2))
@@ -218,7 +224,7 @@ void WaveformGenerator::writeCONTROL_REG(reg8 control)
     shift_pipeline = 0;
 
     // Set reset time for shift register.
-    shift_register_reset = (sid_model == MOS6581) ? SHIFT_REGISTER_RESET_6581 : SHIFT_REGISTER_RESET_8580;
+    shift_register_reset = (sid_model == MOS6581) ? SHIFT_REGISTER_RESET_START_6581 : SHIFT_REGISTER_RESET_START_8580;
 
     // The test bit sets pulse high.
     pulse_output = 0xfff;
@@ -249,12 +255,31 @@ void WaveformGenerator::writeCONTROL_REG(reg8 control)
   else if (waveform_prev) {
     // Change to floating DAC input.
     // Reset fading time for floating DAC input.
-    floating_output_ttl = (sid_model == MOS6581) ? FLOATING_OUTPUT_TTL_6581 : FLOATING_OUTPUT_TTL_8580;
+    floating_output_ttl = (sid_model == MOS6581) ? FLOATING_OUTPUT_TTL_START_6581 : FLOATING_OUTPUT_TTL_START_8580;
   }
 
   // The gate bit is handled by the EnvelopeGenerator.
 }
 
+void WaveformGenerator::wave_bitfade()
+{
+  waveform_output &= waveform_output >> 1;
+  osc3 = waveform_output;
+  if (waveform_output != 0)
+    floating_output_ttl = (sid_model == MOS6581) ? FLOATING_OUTPUT_TTL_BIT_6581 : FLOATING_OUTPUT_TTL_BIT_8580;
+}
+
+void WaveformGenerator::shiftreg_bitfade()
+{
+  shift_register |= 1;
+  shift_register |= shift_register << 1;
+
+  // New noise waveform output.
+  set_noise_output();
+  if (shift_register != 0x7fffff)
+    shift_register_reset = (sid_model == MOS6581) ? SHIFT_REGISTER_RESET_BIT_6581 : SHIFT_REGISTER_RESET_BIT_8580;
+}
+
 reg8 WaveformGenerator::readOSC()
 {
   return osc3 >> 4;
