@@ -125,7 +125,7 @@ void ds1216e_destroy(rtc_ds1216e_t *context, int save)
 
 /* ---------------------------------------------------------------------------------------------------- */
 
-static uint8_t pattern[64] = {
+static const uint8_t pattern[64] = {
     1, 0, 1, 0, 0, 0, 1, 1,
     0, 1, 0, 1, 1, 1, 0, 0,
     1, 1, 0, 0, 0, 1, 0, 1,
