@@ -323,7 +323,7 @@ static int convert_float_buffer(int size, int channels)
             c[3] = file_buffer[file_pointer + (i * frame_size) + 3];
         }
         memcpy(&f, c, sizeof(float));
-        f *= 0x7fffffff;
+        f *= (float)0x7fffffff;
         sample = (int32_t)f;
         sample_buffer1[i] = (uint8_t)((sample >> 24) + 0x80);
         if (sound_audio_channels == 2 && channels == SAMPLER_OPEN_STEREO) {
@@ -339,7 +339,7 @@ static int convert_float_buffer(int size, int channels)
                 c[3] = file_buffer[file_pointer + (i * frame_size) + 7];
             }
             memcpy(&f, c, sizeof(float));
-            f *= 0x7fffffff;
+            f *= (float)0x7fffffff;
             sample = (int32_t)f;
             sample_buffer2[i] = (uint8_t)((sample >> 24) + 0x80);
         }
@@ -1099,7 +1099,9 @@ static int handle_voc_file(int channels)
 
 static int is_voc_file(void)
 {
-    char header[] = { 0x43, 0x72, 0x65, 0x61, 0x74, 0x69, 0x76, 0x65, 0x20, 0x56, 0x6F, 0x69, 0x63, 0x65, 0x20, 0x46, 0x69, 0x6C, 0x65 };
+    static const char header[] = {
+        0x43, 0x72, 0x65, 0x61, 0x74, 0x69, 0x76, 0x65, 0x20, 0x56, 0x6F, 0x69, 0x63, 0x65, 0x20, 0x46, 0x69, 0x6C, 0x65
+    };
     size_t i;
 
     if (file_size < 26) {
@@ -1715,7 +1717,7 @@ static int handle_mp3_file(int channels)
     buffer_size = mpg123_length(mh);
 
     lib_free(file_buffer);
-    file_size = buffer_size * 2 * mp3_channels;
+    file_size = (unsigned int)(buffer_size * 2 * mp3_channels);
     file_buffer = lib_malloc(file_size);
 
     mp3_err = mpg123_read(mh, file_buffer, file_size, &done);
@@ -1728,7 +1730,7 @@ static int handle_mp3_file(int channels)
 
     sound_audio_type = AUDIO_TYPE_PCM;
     sound_audio_channels = mp3_channels;
-    sound_audio_rate = mp3_rate;
+    sound_audio_rate = (unsigned int)mp3_rate;
     sound_audio_bits = 16;
 
     return convert_pcm_buffer(file_size, channels);
@@ -1947,14 +1949,14 @@ static int handle_vorbis_file(int channels)
     }
 
     for (i = 0; i < ov.links; i++) {
-        vi = ov_info(&ov, i);
+        vi = ov_info(&ov, (int)i);
         sound_audio_channels = vi->channels;
         if (sound_audio_channels < 1 || sound_audio_channels > 2) {
             ov_clear(&ov);
             log_error(filedrv_log, "The ogg/vorbis file channels is not 1 or 2");
             return -1;
         }
-        sound_audio_rate = vi->rate;
+        sound_audio_rate = (unsigned int)(vi->rate);
         sound_audio_bits = 16;
         sound_audio_type = AUDIO_TYPE_PCM;
     }
@@ -1962,8 +1964,10 @@ static int handle_vorbis_file(int channels)
     pcmlength = ov_pcm_total(&ov, -1);
     vorbis_buffer = lib_malloc(pcmlength * sound_audio_channels * 2);
     i = 0;
-    while (i < pcmlength * sound_audio_channels * 2){
-        int ret = ov_read(&ov, (char*)vorbis_buffer + i, (pcmlength * 2 * sound_audio_channels) - i, 0, 2, 1, &dummy);
+    while (i < pcmlength * sound_audio_channels * 2) {
+        int ret = (int)ov_read(&ov, (char*)vorbis_buffer + i,
+                               (int)((pcmlength * 2 * sound_audio_channels) - i),
+                               0, 2, 1, &dummy);
         if (ret < 0) {
             ov_clear(&ov);
             lib_free(vorbis_buffer);
@@ -1982,7 +1986,7 @@ static int handle_vorbis_file(int channels)
 
     lib_free(file_buffer);
     file_buffer = vorbis_buffer;
-    file_size = pcmlength * 2 * sound_audio_channels;
+    file_size = (unsigned int)(pcmlength * 2 * sound_audio_channels);
     file_pointer = 0;
 
     return convert_pcm_buffer(file_size, channels);
@@ -2092,7 +2096,7 @@ static void file_load_sample(int channels)
     sample_file = fopen(sample_name, "rb");
     if (sample_file) {
         fseek(sample_file, 0, SEEK_END);
-        file_size = ftell(sample_file);
+        file_size = (unsigned int)ftell(sample_file);
         fseek(sample_file, 0, SEEK_SET);
         file_buffer = lib_malloc(file_size);
         if (fread(file_buffer, 1, file_size, sample_file) != file_size) {
@@ -2102,8 +2106,8 @@ static void file_load_sample(int channels)
         err = handle_file_type(channels);
         if (!err) {
             sound_sampling_started = 0;
-            sound_cycles_per_frame = machine_get_cycles_per_frame();
-            sound_frames_per_sec = machine_get_cycles_per_second() / sound_cycles_per_frame;
+            sound_cycles_per_frame = (unsigned int)machine_get_cycles_per_frame();
+            sound_frames_per_sec = (unsigned int)machine_get_cycles_per_second() / sound_cycles_per_frame;
             sound_samples_per_frame = sound_audio_rate / sound_frames_per_sec;
             current_channels = channels;
         } else {
