@@ -34,6 +34,24 @@
 #include "serial.h"
 #include "types.h"
 
+/*
+ * This code named "fsdrive" is not to be confused with "fsdevice".
+ *
+ * Together with serial/serial-iec-device.c it does much the same thing as
+ * parallel/parallel-trap.c, but in a somewhat different way, and it has
+ * serial/serial-iec-bus.c wedged in between...
+ *
+ * serial/serial-trap.c corresponds somewhat with serial-iec-bus.c when
+ * kernel traps are in use.
+ */
+/* #define FSDRIVE_DEBUG */
+#define FSDRIVE_DEBUG
+#ifdef FSDRIVE_DEBUG
+#define DBG(_x_)        log_debug _x_
+#else
+#define DBG(_x_)
+#endif
+
 #define SERIAL_NAMELENGTH 255
 
 static log_t fsdrive_log = LOG_ERR;
@@ -68,7 +86,8 @@ static uint8_t serialcommand(unsigned int device, uint8_t secondary)
     channel = secondary & 0x0f;
 
     if ((device & 0x0f) >= 8) {
-        vdrive = (void *)file_system_get_vdrive(device & 0x0f);
+        /* TODO serial devices only have a single drive */
+        vdrive = (void *)file_system_get_vdrive(device & 0x0f, 0);
     } else {
         vdrive = NULL;
     }
@@ -78,15 +97,26 @@ static uint8_t serialcommand(unsigned int device, uint8_t secondary)
         p->nextok[channel] = 0;
     }
     switch (secondary & 0xf0) {
+        case 0x20:
+        case 0x30:
+            DBG(("2x/3x: LISTEN, DEV = %d (no call to driver)", secondary & 0x1F));
+            break;
+        case 0x40:
+        case 0x50:
+            DBG(("4x/5x: TALK, DEV = %d (no call to driver)", secondary & 0x1F));
+            break;
         /*
          * Open Channel
          */
         case 0x60:
-            if (p->isopen[channel] == 1) {
-                p->isopen[channel] = 2;
+            DBG(("6x: OPEN CHANNEL, SA = %d (isopen[channel]=%d, 1 calls ->openf) SerialPtr=%d", secondary & 0x0F, p->isopen[channel], SerialPtr));
+            if (p->isopen[channel] == ISOPEN_AWAITING_NAME) {
+                p->isopen[channel] = ISOPEN_OPEN;
                 st = (uint8_t)((*(p->openf))(vdrive, NULL, 0, channel, NULL));
+
                 for (i = 0; i < SerialPtr; i++) {
                     (*(p->putf))(vdrive, ((uint8_t)(SerialBuffer[i])), channel);
+                    DBG(("SerialBuffer: %c", SerialBuffer[i]));
                 }
                 SerialPtr = 0;
             }
@@ -99,7 +129,8 @@ static uint8_t serialcommand(unsigned int device, uint8_t secondary)
          * Close File
          */
         case 0xE0:
-            p->isopen[channel] = 0;
+            DBG(("Ex: CLOSE FILE, SA = %d", secondary & 0x0F));
+            p->isopen[channel] = ISOPEN_CLOSED;
             st = (uint8_t)((*(p->closef))(vdrive, channel));
             break;
 
@@ -107,20 +138,21 @@ static uint8_t serialcommand(unsigned int device, uint8_t secondary)
          * Open File
          */
         case 0xF0:
-            if (p->isopen[channel]) {
+            DBG(("Fx: OPEN FILE, SA = %d", secondary & 0x0F));
+            if (p->isopen[channel] != ISOPEN_CLOSED) {
 #ifndef DELAYEDCLOSE
-                if (p->isopen[channel] == 2) {
+                if (p->isopen[channel] == ISOPEN_OPEN) {
                     log_warning(fsdrive_log, "Bogus close?");
                     (*(p->closef))(vdrive, channel);
                 }
-                p->isopen[channel] = 2;
+                p->isopen[channel] = ISOPEN_OPEN;
                 SerialBuffer[SerialPtr] = 0;
                 st = (uint8_t)((*(p->openf))(vdrive, SerialBuffer, SerialPtr,
                                           channel, NULL));
                 SerialPtr = 0;
 
                 if (st) {
-                    p->isopen[channel] = 0;
+                    p->isopen[channel] = ISOPEN_CLOSED;
                     (*(p->closef))(vdrive, channel);
 
                     log_error(fsdrive_log, "Cannot open file. Status $%02x.", st);
@@ -128,13 +160,13 @@ static uint8_t serialcommand(unsigned int device, uint8_t secondary)
 #else
                 if (SerialPtr != 0 || channel == 0x0f) {
                     (*(p->closef))(vdrive, channel);
-                    p->isopen[channel] = 2;
+                    p->isopen[channel] = ISOPEN_OPEN;
                     SerialBuffer[SerialPtr] = 0;
                     st = (uint8_t)((*(p->openf))(vdrive, SerialBuffer, SerialPtr,
                                               channel, NULL));
                     SerialPtr = 0;
                     if (st) {
-                        p->isopen[channel] = 0;
+                        p->isopen[channel] = ISOPEN_CLOSED;
                         (*(p->closef))(vdrive, channel);
 
                         log_error(fsdrive_log, "Cannot open file. Status $%02x.", st);
@@ -164,8 +196,9 @@ void fsdrive_open(unsigned int device, uint8_t secondary, void (*st_func)(uint8_
 #endif
 
     p = serial_device_get(device & 0x0f);
+    DBG(("fsdrive_open %u,%d", device & 0xF, secondary & 0xF));
 #ifndef DELAYEDCLOSE
-    if (p->isopen[secondary & 0x0f] == 2) {
+    if (p->isopen[secondary & 0x0f] == ISOPEN_OPEN) {
         if ((device & 0x0f) >= 8) {
             vdrive = (void *)file_system_get_vdrive(device & 0x0f);
         } else {
@@ -174,7 +207,7 @@ void fsdrive_open(unsigned int device, uint8_t secondary, void (*st_func)(uint8_
         (*(p->closef))(vdrive, secondary & 0x0f);
     }
 #endif
-    p->isopen[secondary & 0x0f] = 1;
+    p->isopen[secondary & 0x0f] = ISOPEN_AWAITING_NAME;
 }
 
 void fsdrive_close(unsigned int device, uint8_t secondary, void (*st_func)(uint8_t))
@@ -199,7 +232,8 @@ void fsdrive_listentalk(unsigned int device, uint8_t secondary, void (*st_func)(
         /* send listen/talk to emulated devices for flushing of
            REL file write buffer. */
         if ((device & 0x0f) >= 8) {
-            vdrive = (void *)file_system_get_vdrive(device & 0x0f);
+            /* single drive only */
+            vdrive = (void *)file_system_get_vdrive(device & 0x0f, 0);
             (*(p->listenf))(vdrive, secondary & 0x0f);
         }
     }
@@ -222,7 +256,8 @@ void fsdrive_unlisten(unsigned int device, uint8_t secondary, void (*st_func)(ui
         /* send unlisten to emulated devices for flushing of
            REL file write buffer. */
         if ((device & 0x0f) >= 8) {
-            vdrive = (void *)file_system_get_vdrive(device & 0x0f);
+            /* single drive only */
+            vdrive = (void *)file_system_get_vdrive(device & 0x0f, 0);
             (*(p->listenf))(vdrive, secondary & 0x0f);
         }
     }
@@ -241,15 +276,16 @@ void fsdrive_write(unsigned int device, uint8_t secondary, uint8_t data, void (*
     p = serial_device_get(device & 0x0f);
 
     if ((device & 0x0f) >= 8) {
-        vdrive = (void *)file_system_get_vdrive(device & 0x0f);
+        vdrive = (void *)file_system_get_vdrive(device & 0x0f, 0);
     } else {
         vdrive = NULL;
     }
 
     if (p->inuse) {
-        if (p->isopen[secondary & 0x0f] == 1) {
+        if (p->isopen[secondary & 0x0f] == ISOPEN_AWAITING_NAME) {
             /* Store name here */
             if (SerialPtr < SERIAL_NAMELENGTH) {
+                DBG(("SerialBuffer[%d] = '%c'", SerialPtr, data));
                 SerialBuffer[SerialPtr++] = data;
             }
         } else {
@@ -272,7 +308,7 @@ uint8_t fsdrive_read(unsigned int device, uint8_t secondary, void (*st_func)(uin
     p = serial_device_get(device & 0x0f);
 
     if ((device & 0x0f) >= 8) {
-        vdrive = (void *)file_system_get_vdrive(device & 0x0f);
+        vdrive = (void *)file_system_get_vdrive(device & 0x0f, 0);
     } else {
         vdrive = NULL;
     }
@@ -310,9 +346,9 @@ void fsdrive_reset(void)
         p = serial_device_get(i);
         if (p->inuse) {
             for (j = 0; j < 16; j++) {
-                if (p->isopen[j]) {
-                    vdrive = (void *)file_system_get_vdrive(i);
-                    p->isopen[j] = 0;
+                if (p->isopen[j] != ISOPEN_CLOSED) {
+                    vdrive = (void *)file_system_get_vdrive(i, 0);
+                    p->isopen[j] = ISOPEN_CLOSED;
                     (*(p->closef))(vdrive, j);
                 }
             }
