@@ -53,6 +53,7 @@
 #include "resources.h"
 #include "sound.h"
 #include "types.h"
+#include "tick.h"
 #include "uiapi.h"
 #include "util.h"
 #include "vsync.h"
@@ -168,7 +169,7 @@ static sound_register_devices_t sound_register_devices[] = {
 
 static uint16_t offset = 0;
 
-static sound_chip_t *sound_calls[20];
+static sound_chip_t *sound_calls[SOUND_CHIPS_MAX];
 
 uint16_t sound_chip_register(sound_chip_t *chip)
 {
@@ -177,7 +178,7 @@ uint16_t sound_chip_register(sound_chip_t *chip)
     sound_calls[offset >> 5] = chip;
     offset += 0x20;
 
-    assert((offset >> 5) < 20);
+    assert((offset >> 5) < SOUND_CHIPS_MAX);
 
     return offset - 0x20;
 }
@@ -233,6 +234,8 @@ static int sound_machine_calculate_samples(sound_t **psid, int16_t *pbuf, int nr
 {
     int i;
     int temp;
+    int initial_delta_t = *delta_t;
+    int delta_t_for_other_chips;
 
     if (sound_calls[0]->cycle_based() || (!sound_calls[0]->cycle_based() && sound_calls[0]->chip_enabled)) {
         temp = sound_calls[0]->calculate_samples(psid, pbuf, nr, soc, scc, delta_t);
@@ -243,7 +246,8 @@ static int sound_machine_calculate_samples(sound_t **psid, int16_t *pbuf, int nr
 
     for (i = 1; i < (offset >> 5); i++) {
         if (sound_calls[i]->chip_enabled) {
-            sound_calls[i]->calculate_samples(psid, pbuf, temp, soc, scc, delta_t);
+            delta_t_for_other_chips = initial_delta_t;
+            sound_calls[i]->calculate_samples(psid, pbuf, temp, soc, scc, &delta_t_for_other_chips);
         }
     }
     return temp;
@@ -316,7 +320,6 @@ static char *recorddevice_name = NULL; /* app_resources.soundDeviceName */
 static char *recorddevice_arg = NULL;  /* app_resources.soundDeviceArg */
 static int buffer_size;                /* app_resources.soundBufferSize */
 static int suspend_time;               /* app_resources.soundSuspendTime */
-static int speed_adjustment_setting;   /* app_resources.soundSpeedAdjustment */
 static int volume;
 static int amp;
 static int fragment_size;
@@ -347,6 +350,9 @@ int sid_state_changed;
 /* Sample based or cycle based sound engine. */
 static int cycle_based = 0;
 
+/* If a current playback device is used to control emulator timing */
+static int sound_is_timing_source = FALSE;
+
 static int set_output_option(int val, void *param)
 {
     switch (val) {
@@ -393,12 +399,9 @@ static int set_device_name(const char *val, void *param)
 {
     if (!val || val[0] == '\0') {
         /* Use the default sound device */
-#ifdef BEOS_COMPILE
-        if (archdep_is_haiku()) {
+        if (archdep_is_haiku() == 0) {
             util_string_set(&device_name, "bsp");
-        } else
-#endif
-        {
+        } else {
             util_string_set(&device_name, sound_register_devices[0].name);
         }
     } else {
@@ -469,25 +472,6 @@ static int set_suspend_time(int val, void *param)
     return 0;
 }
 
-static int set_speed_adjustment_setting(int val, void *param)
-{
-    if (val == SOUND_ADJUST_DEFAULT) {
-        speed_adjustment_setting = SOUND_ADJUST_EXACT;
-    } else {
-        switch (val) {
-            case SOUND_ADJUST_FLEXIBLE:
-            case SOUND_ADJUST_ADJUSTING:
-            case SOUND_ADJUST_EXACT:
-                break;
-            default:
-                return -1;
-        }
-        speed_adjustment_setting = val;
-    }
-
-    return 0;
-}
-
 static int set_volume(int val, void *param)
 {
     volume = val;
@@ -524,14 +508,12 @@ static const resource_int_t resources_int[] = {
       (void *)&playback_enabled, set_playback_enabled, NULL },
     { "SoundSampleRate", SOUND_SAMPLE_RATE, RES_EVENT_NO, NULL,
       (void *)&sample_rate, set_sample_rate, NULL },
-    { "SoundBufferSize", 0, RES_EVENT_NO, NULL,
+    { "SoundBufferSize", SOUND_SAMPLE_BUFFER_SIZE, RES_EVENT_NO, NULL,
       (void *)&buffer_size, set_buffer_size, NULL },
-    { "SoundFragmentSize", ARCHDEP_SOUND_FRAGMENT_SIZE, RES_EVENT_NO, NULL,
+    { "SoundFragmentSize", SOUND_FRAGMENT_MEDIUM, RES_EVENT_NO, NULL,
       (void *)&fragment_size, set_fragment_size, NULL },
     { "SoundSuspendTime", 0, RES_EVENT_NO, NULL,
       (void *)&suspend_time, set_suspend_time, NULL },
-    { "SoundSpeedAdjustment", SOUND_ADJUST_EXACT, RES_EVENT_NO, NULL,
-      (void *)&speed_adjustment_setting, set_speed_adjustment_setting, NULL },
     { "SoundVolume", 100, RES_EVENT_NO, NULL,
       (void *)&volume, set_volume, NULL },
     { "SoundOutput", ARCHDEP_SOUND_OUTPUT_MODE, RES_EVENT_NO, NULL,
@@ -577,9 +559,6 @@ static const cmdline_option_t cmdline_options[] =
     { "-soundfragsize", SET_RESOURCE, CMDLINE_ATTRIB_NEED_ARGS,
       NULL, NULL, "SoundFragmentSize", NULL,
       "<value>", "Set sound fragment size (0: very small, 1: small, 2: medium, 3: large, 4: very large)" },
-    { "-soundsync", SET_RESOURCE, CMDLINE_ATTRIB_NEED_ARGS,
-      NULL, NULL, "SoundSpeedAdjustment", NULL,
-      "<sync>", "Set sound speed adjustment (0: flexible, 1: adjusting, 2: exact)" },
     { "-soundoutput", SET_RESOURCE, CMDLINE_ATTRIB_NEED_ARGS,
       NULL, NULL, "SoundOutput", NULL,
       "<output mode>", "Sound output mode: (0: system decides mono/stereo, 1: always mono, 2: always stereo)" },
@@ -666,7 +645,7 @@ static unsigned int cycles_per_rfsh;
 static double rfsh_per_sec;
 
 /* Speed in percent, tracks relative_speed from vsync.c */
-static int speed_percent;
+static double speed_percent;
 
 /* Flag: Is warp mode enabled?  */
 static int warp_mode_enabled;
@@ -700,7 +679,7 @@ typedef struct {
     CLOCK lastclk;
 
     /* sample buffer */
-    int16_t buffer[SOUND_CHANNELS_MAX * SOUND_BUFSIZE];
+    int16_t *buffer;
 
     /* sample buffer pointer */
     int bufptr;
@@ -720,10 +699,6 @@ typedef struct {
     /* number of samples in kernel buffer */
     int bufsize;
 
-    /* constants related to adjusting sound */
-    int prevused;
-    int prevfill;
-
     /* is the device suspended? */
     int issuspended;
     int16_t lastsample[SOUND_CHANNELS_MAX];
@@ -764,14 +739,6 @@ const char *sound_device_name(unsigned int num)
 /* code to disable sid for a given number of seconds if needed */
 static time_t disabletime;
 
-static void suspendsound(const char *reason)
-{
-    disabletime = time(0);
-    log_warning(sound_log, "suspend, disabling sound for %d secs (%s)",
-                suspend_time, reason);
-    sound_state_changed = TRUE;
-}
-
 static void enablesound(void)
 {
     time_t diff;
@@ -799,10 +766,6 @@ static int sound_error(const char *msg)
 
     playback_enabled = 0;
 
-    if (!console_mode) {
-        ui_update_menus();
-    }
-
     return 1;
 }
 
@@ -879,7 +842,8 @@ static int sid_open(void)
 /* initialize SID engine */
 static int sid_init(void)
 {
-    int c, speed, speed_factor;
+    double speed_factor;
+    int c, speed;
 
     /* Special handling for cycle based as opposed to sample based sound
        engines. reSID is cycle based. */
@@ -937,13 +901,21 @@ int sound_open(void)
     int speed;
     int fragsize;
     int fragnr;
-    char frag_str[8];
     double bufsize;
 
     if (suspend_time > 0 && disabletime) {
         return 1;
     }
 
+    if (snddata.playdev) {
+        /*
+         * This can happen when loading a snapshot with multiple sids. Whilst fixing that is a good idea,
+         * this should be safe and simpler.
+         */
+        log_warning(sound_log, "ignoring sound_open when snddata.playdev is not NULL.");
+        return 0;
+    }
+
     /* Opening the sound device and initializing the sound engine
        might take some time. */
     vsync_suspend_speed_eval();
@@ -1018,11 +990,15 @@ int sound_open(void)
     }
     fragsize = 1 << i;
     fragnr = (int)((speed * bufsize + fragsize - 1) / fragsize);
-    if (fragnr < 3) {
-        fragnr = 3;
-    }
 
     if (pdev) {
+        
+        snddata.playdev = pdev;
+        snddata.fragsize = fragsize;
+        snddata.fragnr = fragnr;
+        snddata.bufsize = fragsize * fragnr;
+        snddata.bufptr = 0;
+        
         if (pdev->init) {
             channels_cap = channels;
             if (pdev->init(playparam, &speed, &fragsize, &fragnr, &channels_cap)) {
@@ -1040,23 +1016,19 @@ int sound_open(void)
                 snddata.sound_output_channels = channels;
             }
         }
+        snddata.buffer = lib_malloc(snddata.bufsize * snddata.sound_output_channels * sizeof(int16_t));
         snddata.issuspended = 0;
 
         for (c = 0; c < snddata.sound_output_channels; c++) {
             snddata.lastsample[c] = 0;
         }
 
-        snddata.playdev = pdev;
-        snddata.fragsize = fragsize;
-        snddata.fragnr = fragnr;
-        snddata.bufsize = fragsize * fragnr;
-        snddata.bufptr = 0;
-        /* log_message isn't guarenteed to handle "%f" */
-        sprintf(frag_str, "%.1f", (1000.0 * fragsize / speed));
         log_message(sound_log,
-                    "Opened device `%s', speed %dHz, fragment size %sms, buffer size %dms%s",
-                    pdev->name, speed, frag_str,
-                    (int)(1000.0 * snddata.bufsize / speed),
+                    "Opened device `%s', speed %dHz, fragment size %.2fms, buffer size %.2fms%s",
+                    pdev->name,
+                    speed,
+                    (1000.0 * fragsize / speed),
+                    (1000.0 * snddata.bufsize / speed),
                     snddata.sound_output_channels > 1 ? ", stereo" : "");
         sample_rate = speed;
 
@@ -1064,6 +1036,7 @@ int sound_open(void)
             return 1;
         }
 
+        sound_is_timing_source = pdev->is_timing_source ? TRUE : FALSE;
         sid_state_changed = FALSE;
 
         /* Fill up the sound hardware buffer. */
@@ -1154,14 +1127,18 @@ void sound_close(void)
 
     sid_close();
 
-    snddata.prevused = snddata.prevfill = 0;
-
     sdev_open = FALSE;
     sound_state_changed = FALSE;
+    sound_is_timing_source = FALSE;
+
+    lib_free(snddata.buffer);
+    snddata.buffer = NULL;
+    snddata.bufsize = 0;
 
     if (temp_buffer) {
         lib_free(temp_buffer);
         temp_buffer = NULL;
+        temp_buffer_size = 0;
     }
 
     /* Closing the sound device might take some time, and displaying
@@ -1172,10 +1149,14 @@ void sound_close(void)
 /* run sid */
 static int sound_run_sound(void)
 {
-    int nr = 0, i;
+#if 0
+    static int overflow_warning_count = 0;
+#endif
+
+    int nr = 0;
+    int i;
     int delta_t = 0;
     int16_t *bufferptr;
-    static int overflow_warning_count = 0;
 
     /* XXX: implement the exact ... */
     if (!playback_enabled || (suspend_time > 0 && disabletime)) {
@@ -1195,11 +1176,14 @@ static int sound_run_sound(void)
         bufferptr = snddata.buffer + snddata.bufptr * snddata.sound_output_channels;
         nr = sound_machine_calculate_samples(snddata.psid,
                                              bufferptr,
-                                             SOUND_BUFSIZE - snddata.bufptr,
+                                             snddata.bufsize - snddata.bufptr,
                                              snddata.sound_output_channels,
                                              snddata.sound_chip_channels,
                                              &delta_t);
         if (delta_t) {
+            sound_error("Sound buffer overflow (cycle based)");
+            return -1;
+#if 0
             if (overflow_warning_count < 25) {
                 log_warning(sound_log, "%s", "Sound buffer overflow (cycle based)");
                 overflow_warning_count++;
@@ -1209,40 +1193,37 @@ static int sound_run_sound(void)
                     overflow_warning_count++;
                 }
             }
-        }
-    } else {
-        /* Handling of sample based sound engines. */
-        nr = (int)((SOUNDCLK_CONSTANT(maincpu_clk) - snddata.fclk)
-                   / snddata.clkstep);
-        if (!nr) {
-            return 0;
-        }
-        if (snddata.bufptr + nr > SOUND_BUFSIZE) {
-#ifndef ANDROID_COMPILE
-            return sound_error("Sound buffer overflow.");
-#else
-            return 0;
 #endif
         }
-        bufferptr = snddata.buffer + snddata.bufptr * snddata.sound_output_channels;
-        sound_machine_calculate_samples(snddata.psid,
-                                        bufferptr,
-                                        nr,
-                                        snddata.sound_output_channels,
-                                        snddata.sound_chip_channels,
-                                        &delta_t);
-        snddata.fclk += nr * snddata.clkstep;
-    }
+     } else {
+         /* Handling of sample based sound engines. */
+         nr = (int)((SOUNDCLK_CONSTANT(maincpu_clk) - snddata.fclk)
+                    / snddata.clkstep);
+         if (!nr) {
+             return 0;
+         }
+         if (nr > snddata.bufsize - snddata.bufptr) {
+             nr = snddata.bufsize - snddata.bufptr;
+         }
+         bufferptr = snddata.buffer + snddata.bufptr * snddata.sound_output_channels;
+         sound_machine_calculate_samples(snddata.psid,
+                                         bufferptr,
+                                         nr,
+                                         snddata.sound_output_channels,
+                                         snddata.sound_chip_channels,
+                                         &delta_t);
+         snddata.fclk += nr * snddata.clkstep;
+     }
 
-    if (amp < 4096) {
-        if (amp) {
-            for (i = 0; i < (nr * snddata.sound_output_channels); i++) {
-                bufferptr[i] = bufferptr[i] * amp / 4096;
-            }
-        } else {
-            memset(bufferptr, 0, nr * snddata.sound_output_channels * sizeof(int16_t));
-        }
-    }
+     if (amp < 4096) {
+         if (amp) {
+             for (i = 0; i < (nr * snddata.sound_output_channels); i++) {
+                 bufferptr[i] = bufferptr[i] * amp / 4096;
+             }
+         } else {
+             memset(bufferptr, 0, nr * snddata.sound_output_channels * sizeof(int16_t));
+         }
+     }
 
     snddata.bufptr += nr;
     snddata.lastclk = maincpu_clk;
@@ -1280,22 +1261,25 @@ static void prevent_clk_overflow_callback(CLOCK sub, void *data)
     }
 }
 
-/* flush all generated samples from buffer to sounddevice. adjust sid runspeed
-   to match real running speed of program */
-double sound_flush()
+/* flush all generated samples from buffer to sounddevice. */
+bool sound_flush()
 {
-    int c, i, nr, space = 0, used;
-    int j;
-    static int drained_warning_count = 0;
-    char *state;
-    static time_t prev;
-    time_t now;
+    const unsigned long max_block_ms = 5000; /* If sound write blocks this long, assume it's broken */
+    const unsigned long block_warn_ms = 500; /* If sound write blocks at least this long before succeeding, log a warning */
+    
+    static unsigned long last_restart_tick = 0;
 
+    int c, i, nr, space;
+    char *state;
+    bool slept = false;
+    unsigned long first_block_tick = 0;
+    unsigned long total_block_ms;
+    
     if (!playback_enabled) {
         if (sdev_open) {
             sound_close();
         }
-        return 0;
+        goto done;
     }
 
     if (sound_state_changed) {
@@ -1309,19 +1293,19 @@ double sound_flush()
         enablesound();
     }
     if (sound_run_sound()) {
-        return 0;
+        goto done;
     }
 
     if (sid_state_changed) {
         if (sid_init() != 0) {
-            return 0;
+            goto done;
         }
         sid_state_changed = FALSE;
     }
 
     if (warp_mode_enabled && snddata.recdev == NULL) {
         snddata.bufptr = 0;
-        return 0;
+        goto done;
     }
     sound_resume();
 
@@ -1331,128 +1315,109 @@ double sound_flush()
         lib_free(state);
         if (i) {
             sound_error("cannot flush.");
-            return 0;
+            goto done;
         }
     }
 
     /* Calculate the number of samples to flush - whole fragments. */
     nr = snddata.bufptr - snddata.bufptr % snddata.fragsize;
     if (!nr) {
-        return 0;
+        goto done;
     }
 
-    /* adjust speed */
-    if (snddata.playdev->bufferspace) {
-        space = snddata.playdev->bufferspace();
-        if (space < 0 || space > snddata.bufsize) {
-            log_warning(sound_log, "fragment problems %d %d", space, snddata.bufsize);
-            sound_error("fragment problems.");
-            return 0;
-        }
-        /* we only write complete fragments, sound drivers that can tell
-         * better accuracy aren't utilized at this stage. */
-        space -= space % snddata.fragsize;
+    /*
+     * At this point we have to block until we have written at least one fragment.
+     * 
+     * The 'push against the audio device' sync method depends on this.
+     */
+    
+    while (!warp_mode_enabled) {
 
-        used = snddata.bufsize - space;
-        /* buffer emptied during vsync? Looks like underrun. */
-        if (used < snddata.fragsize) {
-            if (suspend_time > 0) {
-                now = time(0);
-                if (now == prev) {
-                    suspendsound("buffer overruns");
-                    return 0;
-                }
-                prev = now;
-            }
-
-            /* Calculate unused space in buffer, accounting for data we are
-             * about to write. */
-            j = snddata.bufsize - nr;
-
-            /* Fill up sound hardware buffer. */
-            if (j > 0) {
-                fill_buffer(j, 0);
-            }
-            snddata.prevfill = j;
-
-            /* Fresh start for vsync. */
-            if (drained_warning_count < 25) {
-                log_warning(sound_log, "Buffer drained");
-                drained_warning_count++;
-            } else {
-                if (drained_warning_count == 25) {
-                    log_warning(sound_log, "Buffer drained warning repeated 25 times, will now be ignored");
-                    drained_warning_count++;
-                }
-            }
-            vsync_sync_reset();
-        }
-        if (cycle_based || speed_adjustment_setting != SOUND_ADJUST_ADJUSTING) {
-            if (speed_percent > 0) {
-                snddata.clkfactor = SOUNDCLK_CONSTANT(speed_percent) / 100;
-            }
+        if (snddata.playdev->bufferspace) {
+            space = snddata.playdev->bufferspace();            
         } else {
-            if (snddata.prevfill) {
-                snddata.prevused = used;
+            /*
+             * Blocking driver like simple pulse - write everything we have.
+             * I'm not sure if this is the right thing to do, perhaps we should
+             * only be writin a single fragment at time?
+             */
+            space = nr;
+        }
+
+        space -= space % snddata.fragsize;
+        
+        if (space) {
+            if (nr > space) {
+                /* Write as much as we can */
+                nr = space;
             }
-            snddata.clkfactor = SOUNDCLK_MULT(snddata.clkfactor,
-                                              SOUNDCLK_CONSTANT(1.0)
-                                              + (SOUNDCLK_CONSTANT(0.9)
-                                                 * (used - snddata.prevused))
-                                              / snddata.bufsize);
-        }
-        snddata.prevused = used;
-        snddata.prevfill = 0;
-
-        if (!cycle_based && speed_adjustment_setting != SOUND_ADJUST_EXACT
-            && snddata.recdev == NULL) {
-            snddata.clkfactor = SOUNDCLK_MULT(snddata.clkfactor,
-                                              SOUNDCLK_CONSTANT(0.9)
-                                              + ((used + nr)
-                                                 * SOUNDCLK_CONSTANT(0.12))
-                                              / snddata.bufsize);
-        }
-        snddata.clkstep = SOUNDCLK_MULT(snddata.origclkstep,
-                                        snddata.clkfactor);
-        if (SOUNDCLK_CONSTANT(cycles_per_rfsh) / snddata.clkstep
-            >= snddata.bufsize) {
-            if (suspend_time > 0) {
-                suspendsound("running too slow");
-            } else {
-                sound_error("running too slow.");
-            }
-            return 0;
-        }
-        /* Not all sound drivers block during writing. We must avoid
-         * overwriting. */
-        if (nr > space) {
-            nr = space; /* warning: "space" may have become 0 due to fragment size
-                           alignment */
-        }
-    }
-
-    if (nr) {
-        /* Flush buffer, all channels are already mixed into it. */
-        if (snddata.playdev->write(snddata.buffer, nr * snddata.sound_output_channels)) {
-            sound_error("write to sound device failed.");
-            return 0;
-        }
-
-        if (snddata.recdev) {
-            if (snddata.recdev->write(snddata.buffer, nr * snddata.sound_output_channels)) {
+            
+            /* Flush buffer, all channels are already mixed into it. */
+            if (snddata.playdev->write(snddata.buffer, nr * snddata.sound_output_channels)) {
                 sound_error("write to sound device failed.");
-                return 0;
+                goto done;
             }
-        }
-    }
 
-    /* "No Limit" speed support: nuke the accumulated buffer. */
-    if (speed_percent == 0) {
-        nr = snddata.bufptr;
+            if (snddata.recdev) {
+                if (snddata.recdev->write(snddata.buffer, nr * snddata.sound_output_channels)) {
+                    sound_error("write to sound device failed.");
+                    goto done;
+                }
+            }
+
+            if (first_block_tick) {
+                total_block_ms = tick_delta(first_block_tick) / (tick_per_second() / 1000);
+                if (total_block_ms >= block_warn_ms) {
+                    log_warning(sound_log, "Sound device write was blocked for %lums", total_block_ms);
+                }
+                
+                /* not blocked anymore */
+                first_block_tick = 0;
+            }
+            break;
+        }
+        
+        /* Haven't written yet, try again after a minimal sleep */
+
+        if (!first_block_tick) {
+            first_block_tick = tick_now();
+        } else {
+            total_block_ms = tick_delta(first_block_tick) / (tick_per_second() / 1000);
+
+            if (total_block_ms >= max_block_ms) {
+
+                /*
+                 * Sound device may have stalled and might benefit from a restart.
+                 * But only try if we haven't tried a restart recently.
+                 */
+                
+                log_message(sound_log, "Writing to sound device still blocked after %lums", total_block_ms);
+                
+                if (tick_delta(last_restart_tick) / (tick_per_second() / 1000) >= 2 * max_block_ms) {
+                    log_message(sound_log, "Attempting restart");
+                    sound_close();
+                    last_restart_tick = tick_now();
+                    first_block_tick = 0;
+                    goto done;
+                }
+                
+                log_message(sound_log, "Last restart is too recent, disabling sound.");
+                sound_error("Sound device stalled");
+                goto done;
+            }
+
+            /* More to write, try again after a minimal sleep */
+            tick_sleep(tick_per_second() / 1000);
+            slept = true;
+        }
     }
 
     snddata.bufptr -= nr;
 
+    /*
+     * Move any incomplete fragments back to the start of the sample buffer
+     */ 
+
     for (c = 0; c < snddata.sound_output_channels; c++) {
         snddata.lastsample[c] = snddata.buffer[(nr - 1) * snddata.sound_output_channels + c];
         for (i = 0; i < snddata.bufptr; i++) {
@@ -1460,21 +1425,19 @@ double sound_flush()
                 snddata.buffer[(i + nr) * snddata.sound_output_channels + c];
         }
     }
+    
+done:
 
-    if (snddata.playdev->bufferspace
-        && (cycle_based || speed_adjustment_setting == SOUND_ADJUST_EXACT))
-    {
-        /* finetune VICE timer */
-        /* Read bufferspace() just before returning to minimize the possibility
-           of getting interrupted before vsync delay calculation. */
-        /* Aim for utilization of bufsize - fragsize. */
-        int remspace =
-            snddata.playdev->bufferspace() - snddata.bufptr;
-        /* Return delay in seconds. */
-        return (double)remspace / sample_rate;
+    if (!slept) {
+        mainlock_yield_once();
     }
 
-    return 0;
+    /*
+     * If the sound device is not a timing source, then we need
+     * the host to sleep to sync time with the emulator.
+     */
+    
+    return !sound_is_timing_source;
 }
 
 /* suspend sid (eg. before pause) */
@@ -1533,8 +1496,8 @@ void sound_set_machine_parameter(long clock_rate, long ticks_per_frame)
 {
     sid_state_changed = TRUE;
 
-    cycles_per_sec = clock_rate;
-    cycles_per_rfsh = ticks_per_frame;
+    cycles_per_sec = (unsigned int)clock_rate;
+    cycles_per_rfsh = (unsigned int)ticks_per_frame;
     rfsh_per_sec = (1.0 / ((double)cycles_per_rfsh / (double)cycles_per_sec));
 }
 
@@ -1568,6 +1531,8 @@ void sound_init(unsigned int clock_rate, unsigned int ticks_per_frame)
     log_message(sound_log, "Available sound devices:%s", devlist);
 
     lib_free(devlist);
+
+    archdep_sound_enable_default_device_tracking();
 }
 
 long sound_sample_position(void)
@@ -1629,11 +1594,22 @@ void sound_store(uint16_t addr, uint8_t val, int chipno)
 
 void sound_set_relative_speed(int value)
 {
-    if (value != speed_percent) {
-        sid_state_changed = TRUE;
+    double natural_fps;
+    double new_percent;
+    
+    if (value < 0) {
+        natural_fps = (double)machine_get_cycles_per_second() / machine_get_cycles_per_frame();
+        new_percent = 100.0 * (double)(0 - value) / natural_fps;
+    } else {
+        new_percent = value;
     }
 
-    speed_percent = value;
+    /* printf("sound new percent: %f\n", new_percent); */
+
+    if (new_percent != speed_percent) {
+        sid_state_changed = TRUE;
+        speed_percent = new_percent;
+    }
 }
 
 void sound_set_warp_mode(int value)
