@@ -160,7 +160,8 @@ static sound_device_t aiff_device =
     NULL,
     NULL,
     0,
-    2
+    2,
+    false
 };
 
 int sound_init_aiff_device(void)
