@@ -222,7 +222,8 @@ static sound_device_t bsp_device =
     bsp_suspend,
     bsp_resume,
     1,
-    2
+    2,
+    false
 };
 
 int sound_init_bsp_device(void)
