@@ -46,32 +46,43 @@
 
 #include <AudioToolbox/AudioToolbox.h>
 #include <CoreAudio/CoreAudio.h>
+#include <stdatomic.h>
+#include <libkern/OSAtomic.h>
 
 #include "lib.h"
 #include "log.h"
 #include "sound.h"
+#include "tick.h"
+#include "vsyncapi.h"
 
 /* Requested audio device name */
 CFStringRef requested_device_name_ref = NULL;
-char * requested_device_name = NULL;
+char *requested_device_name = NULL;
 
 /* resolved device id */
 static AudioDeviceID device = kAudioDeviceUnknown;
 
-/* type for atomic increments */
-typedef volatile int atomic_int_t;
 
 /* the cyclic buffer containing m fragments */
-static int16_t *soundbuffer;
+static volatile int16_t *ringbuffer;
 
-/* silence fragment */
-static int16_t *silence;
+/* the buffer used to pass non cyclic data to the driver  */
+static volatile int16_t *copybuffer;
+static volatile int copybuffer_size_bytes;
 
 /* current read position: no. of fragment in soundbuffer */
-static unsigned int read_position;
+static volatile int read_position_2;
 
 /* the next position to write: no. of fragment in soundbuffer */
-static unsigned int write_position;
+static volatile int write_position_2;
+
+/* current number of fragments in buffer */
+static volatile int fragments_in_queue_2;
+
+/* samples left in current fragment */
+static volatile int frames_left_in_fragment_2;
+
+
 
 /* frames in fragment  */
 static unsigned int frames_in_fragment;
@@ -85,63 +96,34 @@ static unsigned int swords_in_fragment;
 /* total number of fragments */
 static unsigned int fragment_count;
 
-/* current number of fragments in buffer */
-static atomic_int_t fragments_in_queue;
-
 /* number of interleaved channels (mono SID=1, stereo SID=2) */
 static int in_channels;
 
-/* samples left in current fragment */
-static unsigned int frames_left_in_fragment;
-
 /* bytes per input frame */
 static unsigned int in_frame_byte_size;
 
-/* ----- Atomic Increment/Decrement for Thread-Safe Audio Buffers -------- */
+/* How many times the audio code didn't get enough audio */
+static unsigned long underflow_count;
 
-#if defined(__x86_64__) || defined(__i386__)
-/* Intel Mac Implementation */
-
-static inline void atomic_increment(atomic_int_t * addr)
+#if 0
+static void coreaudio_buffer_stats(bool force)
 {
-    __asm__ __volatile__ ("lock ; incl %0"
-                          :"=m" (*addr)
-                          :"m" (*addr));
+    static unsigned long last_log_time_sec;
+
+    unsigned long this_log_time_sec;
+    unsigned long now;
+
+    now = tick_now();
+    this_log_time_sec = now / (tick_per_second());
+
+    if (!force && this_log_time_sec == last_log_time_sec) {
+        return;
+    }
+
+    last_log_time_sec = this_log_time_sec;
+
+    log_message(LOG_DEFAULT, "%d of %d (underflows: %lu)", fragments_in_queue_2, fragment_count, underflow_count);
 }
-
-static inline void atomic_decrement(atomic_int_t * addr)
-{
-    __asm__ __volatile__ ("lock ; decl %0"
-                          :"=m" (*addr)
-                          :"m" (*addr));
-}
-
-#else
-/* PowerPC Mac Implementation */
-
-static inline void atomic_add(atomic_int_t * addr, int val)
-{
-    register int tmp;
-    asm volatile("    lwarx  %0,0,%2  \n\t"  /* load value & reserve */
-                 "    addc   %0,%0,%3 \n\t"  /* add <val> */
-                 "    stwcx. %0,0,%2  \n\n"  /* store new value */
-                 "    bne-   $-12"           /* check if store was successful */
-                 : "=&r"(tmp), "=m"(addr)
-                 : "r"(addr), "r"(val), "m"(addr)
-                 : "cr0"
-                );
-}
-
-static inline void atomic_increment(atomic_int_t * addr)
-{
-    atomic_add(addr, 1);
-}
-
-static inline void atomic_decrement(atomic_int_t * addr)
-{
-    atomic_add(addr, -1);
-}
-
 #endif
 
 /* ----- Audio Converter ------------------------------------------------ */
@@ -154,41 +136,77 @@ static OSStatus converter_input(AudioConverterRef inAudioConverter,
                                 AudioStreamPacketDescription** outDataPacketDescription,
                                 void * inUserData)
 {
-    UInt32 num_frames = *ioNumberDataPackets;
-
-    int16_t *buffer;
-    if (fragments_in_queue) {
-        /* too many -> crop to available in current fragment */
-        if (num_frames > frames_left_in_fragment) {
-            num_frames = frames_left_in_fragment;
-        }
-
-        /* calc position in sound buffer */
-        int sample_offset_in_fragment = frames_in_fragment - frames_left_in_fragment;
-        buffer = soundbuffer + swords_in_fragment * read_position + sample_offset_in_fragment * in_channels;
-
-        /* update the samples left in the current fragment */
-        frames_left_in_fragment -= num_frames;
-
-        /* fetch next fragment */
-        if (frames_left_in_fragment == 0) {
-            read_position = (read_position + 1) % fragment_count;
-            atomic_decrement(&fragments_in_queue);
-            frames_left_in_fragment = frames_in_fragment;
-        }
-    } else {
-        if (num_frames > frames_in_fragment) {
-            num_frames = frames_in_fragment;
-        }
-
-        /* output silence */
-        buffer = silence;
+    UInt32 needed_frames = *ioNumberDataPackets;
+    int needed_bytes = needed_frames * in_frame_byte_size;
+    int this_read_position  = read_position_2;
+    int consumed_fragments = 0;
+    
+    int16_t *source;
+    int16_t *dest;
+    
+    /* ensure our copy buffer is large enough */
+    if (needed_bytes > copybuffer_size_bytes) {
+        lib_free((void *)copybuffer);
+        copybuffer_size_bytes = needed_bytes;
+        copybuffer = lib_malloc(copybuffer_size_bytes);
+        log_message(LOG_DEFAULT, "Copybuffer increase to %d bytes", copybuffer_size_bytes);
     }
-
+    
     /* prepare return buffer */
-    ioData->mBuffers[0].mDataByteSize = num_frames * in_frame_byte_size;
-    ioData->mBuffers[0].mData = buffer;
-    *ioNumberDataPackets = num_frames;
+    ioData->mBuffers[0].mNumberChannels = in_channels;
+    ioData->mBuffers[0].mData = (void *)copybuffer;
+    ioData->mBuffers[0].mDataByteSize = needed_bytes;
+    ioData->mNumberBuffers = 1;
+    
+    dest = (int16_t *)copybuffer;
+    
+    while (fragments_in_queue_2 && needed_frames) {
+        /*
+         * Assemble a single block of samples for the converter
+         */
+
+        /* calc position in ring buffer */
+        int sample_offset_in_fragment = frames_in_fragment - frames_left_in_fragment_2;
+        source = (int16_t *)ringbuffer + (swords_in_fragment * this_read_position) + (sample_offset_in_fragment * in_channels);
+
+        if (needed_frames < frames_left_in_fragment_2) {
+            /* The current fragement has more than enough, so it will be read from again next time. */
+            memcpy(dest, source, needed_frames * in_frame_byte_size);
+
+            dest += needed_frames * in_channels;
+            frames_left_in_fragment_2 -= needed_frames;
+
+            needed_frames = 0;
+        } else {
+            /* We'll need all the audio in the current frament. */
+            memcpy(dest, source, frames_left_in_fragment_2 * in_frame_byte_size);
+
+            dest += frames_left_in_fragment_2 * in_channels;
+            needed_frames -= frames_left_in_fragment_2;
+
+            this_read_position = (this_read_position + 1) % fragment_count;
+
+            consumed_fragments++;
+            frames_left_in_fragment_2 = frames_in_fragment;
+        }
+    }
+    
+    if (needed_frames) {
+        /* Underflow */
+        underflow_count++;
+        *ioNumberDataPackets = 0;
+        
+        //memset(dest, 0, needed_frames * in_frame_byte_size);
+    } else {
+        /*
+        * There was enough data, apply the read.
+        */
+        
+        while(consumed_fragments--) {
+            OSAtomicDecrement32(&fragments_in_queue_2);
+            read_position_2 = this_read_position;
+        }
+    }
 
     return kAudioHardwareNoError;
 }
@@ -200,7 +218,7 @@ static int converter_open(AudioStreamBasicDescription *in,
 
     /* need to to sample rate conversion? */
     if (out->mSampleRate != in->mSampleRate) {
-        log_warning(LOG_DEFAULT, "sound (coreaudio_init): sampling rate conversion %dHz->%dHz",
+        log_message(LOG_DEFAULT, "sound (coreaudio_init): sampling rate conversion %dHz->%dHz",
                     (int)in->mSampleRate, (int)out->mSampleRate);
     }
 
@@ -238,12 +256,11 @@ static void converter_close(void)
     }
 }
 
-static int string_buf_size_for_utf8_char_length(int utf8Chars)
+static CFIndex string_buf_size_for_utf8_char_length(CFIndex utf8Chars)
 {
     return utf8Chars * 4 + 1;
 }
 
-#if defined(MAC_OS_X_VERSION_10_6) && (MAC_OS_X_VERSION_MIN_REQUIRED>=MAC_OS_X_VERSION_10_6)
 static int determine_output_device_id()
 {
     OSStatus err;
@@ -254,6 +271,7 @@ static int determine_output_device_id()
         kAudioObjectPropertyScopeGlobal,
         kAudioObjectPropertyElementMaster
     };
+    bool requested_device_found = false;
 
     /* get default audio device id */
     property_address.mSelector = kAudioHardwarePropertyDefaultOutputDevice;
@@ -359,6 +377,7 @@ static int determine_output_device_id()
         if (kCFCompareEqualTo == CFStringCompare(requested_device_name_ref, device_name_ref, 0)) {
             /* matches the requested audio device */
             device = audio_devices[i];
+            requested_device_found = true;
         }
 
         lib_free(device_name);
@@ -390,111 +409,7 @@ static int determine_output_device_id()
 
     return 0;
 }
-#endif
 
-/* ----- Audio API before AudioUnits ------------------------------------- */
-
-#ifndef HAVE_AUDIO_UNIT
-
-/* bytes per output frame */
-static unsigned int out_frame_byte_size;
-
-/* proc id */
-#if defined(MAC_OS_X_VERSION_10_5) && (MAC_OS_X_VERSION_MIN_REQUIRED>=MAC_OS_X_VERSION_10_5)
-static AudioDeviceIOProcID procID;
-#endif
-
-static OSStatus audio_render(AudioDeviceID device,
-                             const AudioTimeStamp  * now,
-                             const AudioBufferList * input_data,
-                             const AudioTimeStamp  * input_time,
-                             AudioBufferList       * output_data,
-                             const AudioTimeStamp  * output_time,
-                             void                  * client_data)
-{
-    /* get the number of frames(=packets) in the output buffer */
-    UInt32 bufferPacketSize = output_data->mBuffers[0].mDataByteSize / out_frame_byte_size;
-
-    OSStatus result = AudioConverterFillComplexBuffer(converter,
-                                                      converter_input,
-                                                      NULL,
-                                                      &bufferPacketSize,
-                                                      output_data,
-                                                      NULL);
-
-    return result;
-}
-
-static int audio_open(AudioStreamBasicDescription *in)
-{
-    OSStatus err;
-    UInt32 size;
-
-    if (0 != determine_output_device_id()) {
-        return -1;
-    }
-
-    AudioStreamBasicDescription out;
-    /* get default output format */
-    size = sizeof(out);
-    err = AudioDeviceGetProperty(device, 0, false,
-                                 kAudioDevicePropertyStreamFormat,
-                                 &size, (void*)&out);
-    if (err != kAudioHardwareNoError) {
-        log_error(LOG_DEFAULT, "sound (coreaudio_init): stream format not support");
-        return -1;
-    }
-    /* store size of output frame */
-    out_frame_byte_size = out.mBytesPerPacket;
-
-    /* setup audio renderer callback */
-#if defined(MAC_OS_X_VERSION_10_5) && (MAC_OS_X_VERSION_MIN_REQUIRED>=MAC_OS_X_VERSION_10_5)
-    err = AudioDeviceCreateIOProcID( device, audio_render, NULL, &procID );
-#else
-    err = AudioDeviceAddIOProc( device, audio_render, NULL );
-#endif
-    if (err != kAudioHardwareNoError) {
-        log_error(LOG_DEFAULT,
-                  "sound (coreaudio_init): could not add IO proc: err=%d", (int)err);
-        return -1;
-    }
-
-    /* open audio converter */
-    return converter_open(in, &out);
-}
-
-static void audio_close(void)
-{
-#if defined(MAC_OS_X_VERSION_10_5) && (MAC_OS_X_VERSION_MIN_REQUIRED>=MAC_OS_X_VERSION_10_5)
-    AudioDeviceDestroyIOProcID(device, procID);
-#else
-    AudioDeviceRemoveIOProc(device, audio_render);
-#endif
-
-    converter_close();
-}
-
-static int audio_start(void)
-{
-    OSStatus err = AudioDeviceStart(device, audio_render);
-    if (err != kAudioHardwareNoError) {
-        return -1;
-    } else {
-        return 0;
-    }
-}
-
-static int audio_stop(void)
-{
-    OSStatus err = AudioDeviceStop(device, audio_render);
-    if (err != kAudioHardwareNoError) {
-        return -1;
-    } else {
-        return 0;
-    }
-}
-
-#else /* HAVE_AUDIO_UNIT */
 /* ------ Audio Unit API ------------------------------------------------- */
 
 #ifdef DEBUG
@@ -525,7 +440,6 @@ static OSStatus audio_render(void *inRefCon,
                                            NULL);
 }
 
-#if defined(MAC_OS_X_VERSION_10_6) && (MAC_OS_X_VERSION_MIN_REQUIRED>=MAC_OS_X_VERSION_10_6)
 static int audio_open(AudioStreamBasicDescription *in)
 {
     OSStatus err;
@@ -551,11 +465,12 @@ static int audio_open(AudioStreamBasicDescription *in)
     err = AudioComponentInstanceNew(output_component, &outputUnit);
     if (err) {
         log_error(LOG_DEFAULT, "sound (coreaudio_init): error opening output unit");
+        outputUnit = NULL;
         return -1;
     }
 
     /* select output device */
-    if(0 != determine_output_device_id()) {
+    if(determine_output_device_id()) {
         return -1;
     }
 
@@ -565,13 +480,12 @@ static int audio_open(AudioStreamBasicDescription *in)
                                0,
                                &device,
                                sizeof(device));
-
     if (err) {
         log_error(LOG_DEFAULT,
                   "sound (coreaudio_init): error setting device id");
         return -1;
     }
-
+    
     /* Set up a callback function to generate output to the output unit */
     AURenderCallbackStruct input;
     input.inputProc = audio_render;
@@ -587,9 +501,37 @@ static int audio_open(AudioStreamBasicDescription *in)
                   "sound (coreaudio_init): error setting render callback");
         return -1;
     }
+    
+    /*
+     * For accuracy, we want to convert directly from the emulator output
+     * to the format used by the audio hardware, in a single step.
+     */
 
-    /* Get output properties */
+    /* Get hardware output properties */
     size = sizeof(AudioStreamBasicDescription);
+    err = AudioUnitGetProperty(outputUnit,
+                               kAudioUnitProperty_StreamFormat,
+                               kAudioUnitScope_Output,
+                               0,
+                               &out,
+                               &size);
+    if (err) {
+        log_error(LOG_DEFAULT, "sound (coreaudio_init): error getting default input format");
+        return -1;
+    }
+    
+    /* Tell the AudioUnit that we'll give it samples in the same format */
+    err = AudioUnitSetProperty(outputUnit,
+                               kAudioUnitProperty_StreamFormat,
+                               kAudioUnitScope_Input,
+                               0,
+                               &out,
+                               size);
+    if (err) {
+        log_error(LOG_DEFAULT, "sound (coreaudio_init): error setting desired sample rate");
+    }
+    
+    /* Get the final format that we'll need to use - the above may have failed */
     err = AudioUnitGetProperty(outputUnit,
                                kAudioUnitProperty_StreamFormat,
                                kAudioUnitScope_Input,
@@ -597,115 +539,116 @@ static int audio_open(AudioStreamBasicDescription *in)
                                &out,
                                &size);
     if (err) {
-        log_error(LOG_DEFAULT,
-                  "sound (coreaudio_init): error setting desired input format");
+        log_error(LOG_DEFAULT, "sound (coreaudio_init): error getting final output format");
         return -1;
     }
 
+    /* Determine the range of supported buffer sizes */
+    AudioObjectPropertyAddress frame_size_range_address = {
+        kAudioDevicePropertyBufferFrameSizeRange,
+        kAudioObjectPropertyScopeOutput,
+        kAudioObjectPropertyElementMaster
+    };
+    AudioValueRange range = {0, 0};
+    size = sizeof(AudioValueRange);
+
+    err = AudioObjectGetPropertyData(device, &frame_size_range_address, 0, NULL, &size, &range);
+    if (err) {
+        log_error(LOG_DEFAULT, "sound (coreaudio_init): error getting kAudioDevicePropertyBufferFrameSizeRange");
+        return -1;
+    }
+    
+    size = range.mMinimum > frames_in_fragment ? range.mMinimum : frames_in_fragment;
+    
+    log_message(LOG_DEFAULT, "sound (coreaudio_init): audio frame buffer size in samples min: %f max: %f fragment size: %u, chosen: %d", range.mMinimum, range.mMaximum, frames_in_fragment, size);
+
+    /* set the buffer size */
+    AudioObjectPropertyAddress buffer_size_address = {
+        kAudioDevicePropertyBufferFrameSize,
+        kAudioObjectPropertyScopeGlobal,
+        kAudioObjectPropertyElementMaster
+    };
+    
+    err = AudioObjectSetPropertyData(device, &buffer_size_address, 0, NULL, sizeof(UInt32), &size);
+    if (err) {
+        log_error(LOG_DEFAULT, "sound (coreaudio_init): error setting buffer size to %d", size);
+        return -1;
+    }
+    err = AudioUnitSetProperty(outputUnit, kAudioUnitProperty_MaximumFramesPerSlice, kAudioUnitScope_Global, 0, &size, sizeof(size));
+    if (err) {
+        log_error(LOG_DEFAULT, "sound (coreaudio_init): error setting max frames per slice to %d", size);
+        return -1;
+    }
+    
+    /* open converter */
+    err = converter_open(in, &out);
+    if (err) {
+        log_error(LOG_DEFAULT, "sound (coreaudio_init): error initializing audio converter");
+        return -1;
+    }
+
+    return 0;
+}
+
+static int audio_start(void)
+{
+    OSStatus err;
+    
     /* Init unit */
     err = AudioUnitInitialize(outputUnit);
     if (err) {
         log_error(LOG_DEFAULT,
-                  "sound (coreaudio_init): error initializing audio unit");
+                  "sound (coreaudio_start): error initializing audio unit");
         return -1;
     }
-
-    /* open converter */
-    return converter_open(in, &out);
-}
-
-static void audio_close(void)
-{
-    OSStatus err;
-
-    converter_close();
-
-    /* Uninit unit */
-    err = AudioUnitUninitialize(outputUnit);
+    
+    err = AudioOutputUnitStart(outputUnit);
     if (err) {
-        log_error(LOG_DEFAULT, "sound (coreaudio_close): error uninitializing audio unit");
-    }
-
-    /* Close component */
-    AudioComponentInstanceDispose(outputUnit);
-}
-#else
-static int audio_open(AudioStreamBasicDescription *in)
-{
-    OSStatus err;
-    UInt32 size;
-    AudioStreamBasicDescription out;
-
-    /* get default audio device */
-    size = sizeof(device);
-    err = AudioHardwareGetProperty(kAudioHardwarePropertyDefaultOutputDevice,
-                                   &size, (void*)&device);
-    if (err != kAudioHardwareNoError) {
-        log_error(LOG_DEFAULT, "sound (coreaudio_init): Failed to get default output device");
+        log_error(LOG_DEFAULT, "sound (coreaudio_start): failed to start audio device");
         return -1;
     }
-
-    /* get default output format */
-    size = sizeof(out);
-    err = AudioDeviceGetProperty(device, 0, false,
-                                 kAudioDevicePropertyStreamFormat,
-                                 &size, (void*)&out);
-    if (err != kAudioHardwareNoError) {
-        log_error(LOG_DEFAULT, "sound (coreaudio_init): stream format not support");
-        return -1;
-    }
-    /* store size of output frame */
-    out_frame_byte_size = out.mBytesPerPacket;
-
-    /* setup audio renderer callback */
-#if defined(MAC_OS_X_VERSION_10_5) && (MAC_OS_X_VERSION_MIN_REQUIRED>=MAC_OS_X_VERSION_10_5)
-    err = AudioDeviceCreateIOProcID( device, audio_render, NULL, &procID );
-#else
-    err = AudioDeviceAddIOProc( device, audio_render, NULL );
-#endif
-    if (err != kAudioHardwareNoError) {
-        log_error(LOG_DEFAULT,
-                  "sound (coreaudio_init): could not add IO proc: err=%d", (int)err);
-        return -1;
-    }
-
-    /* open audio converter */
-    return converter_open(in, &out);
-}
-
-static void audio_close(void)
-{
-#if defined(MAC_OS_X_VERSION_10_5) && (MAC_OS_X_VERSION_MIN_REQUIRED>=MAC_OS_X_VERSION_10_5)
-    AudioDeviceDestroyIOProcID(device, procID);
-#else
-    AudioDeviceRemoveIOProc(device, audio_render);
-#endif
-
-    converter_close();
-}
-#endif
-
-static int audio_start(void)
-{
-    OSStatus err = AudioOutputUnitStart(outputUnit);
-    if (err) {
-        return -1;
-    } else {
-        return 0;
-    }
+    
+    log_message(LOG_DEFAULT, "sound (coreaudio_start): Started");
+    return 0;
 }
 
 static int audio_stop(void)
 {
-    OSStatus err = AudioOutputUnitStop(outputUnit);
+    OSStatus err;
+    
+    err = AudioOutputUnitStop(outputUnit);
     if (err) {
+        log_error(LOG_DEFAULT, "sound (audio_stop): error uninitializing audio unit");
         return -1;
-    } else {
-        return 0;
     }
+    
+    err = AudioUnitUninitialize(outputUnit);
+    if (err) {
+        log_error(LOG_DEFAULT, "sound (audio_stop): error uninitializing audio unit");
+        return -1;
+    }
+
+    return 0;
 }
 
-#endif /* HAVE_AUDIO_UNIT */
+static void audio_close(void)
+{
+    OSStatus err;
+    
+    if (outputUnit) {
+        audio_stop();
+        
+        log_message(LOG_DEFAULT, "AudioComponentInstanceDispose");
+        err = AudioComponentInstanceDispose(outputUnit);
+        if (err) {
+            log_error(LOG_DEFAULT, "sound (audio_close): error disposing of audio unit");
+        }
+
+        outputUnit = NULL;
+    }
+    
+    converter_close();
+}
 
 /* ---------- coreaudio VICE API ------------------------------------------ */
 
@@ -740,8 +683,7 @@ static int coreaudio_init(const char *param, int *speed,
     in_frame_byte_size = sizeof(int16_t) * in_channels;
 
     /* allocate sound buffers */
-    soundbuffer = lib_calloc(fragment_count, bytes_in_fragment);
-    silence = lib_calloc(1, bytes_in_fragment);
+    ringbuffer = lib_calloc(fragment_count, bytes_in_fragment);
 
     /* define desired input format */
     in.mChannelsPerFrame = *channels;
@@ -771,41 +713,48 @@ static int coreaudio_init(const char *param, int *speed,
 
 static int coreaudio_write(int16_t *pbuf, size_t nr)
 {
-    int i, count;
+    int i;
+    size_t count;
 
     /* number of fragments */
     count = nr / swords_in_fragment;
 
     for (i = 0; i < count; i++)
     {
-        if (fragments_in_queue == fragment_count) {
+        if (fragments_in_queue_2 == fragment_count) {
             log_warning(LOG_DEFAULT, "sound (coreaudio): buffer overrun");
-            return -1;
+            return 0;
         }
 
-        memcpy(soundbuffer + swords_in_fragment * write_position,
-               pbuf + i * swords_in_fragment,
+        memcpy((int16_t *)ringbuffer + (swords_in_fragment * write_position_2),
+               pbuf + (i * swords_in_fragment),
                bytes_in_fragment);
 
-        write_position = (write_position + 1) % fragment_count;
+        write_position_2 = (write_position_2 + 1) % fragment_count;
 
-        atomic_increment(&fragments_in_queue);
+        OSAtomicIncrement32(&fragments_in_queue_2);
     }
+    
+    /* coreaudio_buffer_stats(false); */
 
     return 0;
 }
 
 static int coreaudio_bufferspace(void)
 {
-    return (fragment_count - fragments_in_queue) * frames_in_fragment;
+    return (fragment_count - fragments_in_queue_2) * frames_in_fragment;
 }
 
 static void coreaudio_close(void)
 {
     audio_close();
 
-    lib_free(soundbuffer);
-    lib_free(silence);
+    lib_free((void *)ringbuffer);
+    lib_free((void *)copybuffer);
+
+    ringbuffer = NULL;
+    copybuffer = NULL;
+    copybuffer_size_bytes = 0;
 
     if (requested_device_name_ref) {
         CFRelease(requested_device_name_ref);
@@ -832,10 +781,10 @@ static int coreaudio_resume(void)
     int result;
 
     /* reset buffers before resume */
-    read_position = 0;
-    write_position = 0;
-    fragments_in_queue = 0;
-    frames_left_in_fragment = frames_in_fragment;
+    read_position_2 = 0;
+    write_position_2 = 0;
+    fragments_in_queue_2 = 0;
+    frames_left_in_fragment_2 = frames_in_fragment;
 
     result = audio_start();
     if (result < 0) {
@@ -856,11 +805,13 @@ static sound_device_t coreaudio_device =
     coreaudio_suspend,
     coreaudio_resume,
     1,
-    2
+    2,
+    true
 };
 
 int sound_init_coreaudio_device(void)
 {
     return sound_register_device(&coreaudio_device);
 }
-#endif
+
+#endif /* USE_COREAUDIO_SUPPORT */
