@@ -47,7 +47,8 @@ static sound_device_t dummy_device =
     NULL,
     NULL,
     0,
-    2
+    2,
+    false
 };
 
 int sound_init_dummy_device(void)
