@@ -116,7 +116,8 @@ static sound_device_t soundmovie_device =
     NULL,
     NULL,
     0,
-    2
+    2,
+    false
 };
 
 int sound_init_movie_device(void)
