@@ -170,7 +170,8 @@ static sound_device_t voc_device =
     NULL,
     NULL,
     0,
-    2
+    2,
+    false
 };
 
 int sound_init_voc_device(void)
