@@ -166,7 +166,8 @@ static sound_device_t vorbis_device =
     NULL,
     NULL,
     0,
-    2
+    2,
+    false
 };
 
 int sound_init_vorbis_device(void)
