@@ -137,7 +137,8 @@ static sound_device_t wav_device =
     NULL,
     NULL,
     0,
-    2
+    2,
+    false
 };
 
 int sound_init_wav_device(void)
