@@ -30,13 +30,20 @@
 #include "snapshot.h"
 #include "types.h"
 
-#define TAPEPORT_DEVICE_DATASETTE                0
-#define TAPEPORT_DEVICE_CP_CLOCK_F83             1
-#define TAPEPORT_DEVICE_DTL_BASIC_DONGLE         2
-#define TAPEPORT_DEVICE_SENSE_DONGLE             3
-#define TAPEPORT_DEVICE_TAPE_LOG                 4
-#define TAPEPORT_DEVICE_TAPE_DIAG_586220_HARNESS 5
-#define TAPEPORT_DEVICE_TAPECART                 6
+/* #define TAPEPORT_EXPERIMENTAL_DEVICES */
+
+enum {
+    TAPEPORT_DEVICE_DATASETTE = 0,
+    TAPEPORT_DEVICE_CP_CLOCK_F83,
+    TAPEPORT_DEVICE_DTL_BASIC_DONGLE,
+    TAPEPORT_DEVICE_SENSE_DONGLE,
+    TAPEPORT_DEVICE_TAPE_LOG,
+#ifdef TAPEPORT_EXPERIMENTAL_DEVICES
+    TAPEPORT_DEVICE_TAPE_DIAG_586220_HARNESS,
+#endif
+    TAPEPORT_DEVICE_TAPECART,
+    TAPEPORT_MAX_DEVICES
+};
 
 typedef struct tapeport_device_s {
     /* id of the device */
