@@ -32,7 +32,7 @@
 #include "types.h"
 
 typedef enum {
-    UI_JAM_RESET, UI_JAM_HARD_RESET, UI_JAM_MONITOR, UI_JAM_NONE
+    UI_JAM_INVALID = -1, UI_JAM_RESET, UI_JAM_HARD_RESET, UI_JAM_MONITOR, UI_JAM_NONE
 } ui_jam_action_t;
 
 typedef enum {
@@ -71,8 +71,8 @@ extern void ui_display_drive_track(unsigned int drive_number,
                                    unsigned int drive_base,
                                    unsigned int half_track_number);
 /* The pwm value will vary between 0 and 1000.  */
-extern void ui_display_drive_led(int drive_number, unsigned int pwm1, unsigned int led_pwm2);
-extern void ui_display_drive_current_image(unsigned int drive_number, const char *image);
+extern void ui_display_drive_led(unsigned int drive_number, unsigned int drive_base, unsigned int led_pwm1, unsigned int led_pwm2);
+extern void ui_display_drive_current_image(unsigned int unit_number, unsigned int drive_number, const char *image);
 extern int ui_extend_image_dialog(void);
 
 /* Tape related UI */
@@ -85,9 +85,6 @@ extern void ui_display_tape_current_image(const char *image);
 /* Show a CPU JAM dialog.  */
 extern ui_jam_action_t ui_jam_dialog(const char *format, ...);
 
-/* Update all menu entries.  */
-extern void ui_update_menus(void);
-
 /* Recording UI */
 extern void ui_display_playback(int playback_status, char *version);
 extern void ui_display_recording(int recording_status);
@@ -99,7 +96,4 @@ extern void ui_display_joyport(uint8_t *joyport);
 /* Volume UI */
 void ui_display_volume(int vol);
 
-/* Event related UI. */
-extern void ui_dispatch_next_event(void);
-
 #endif
