@@ -31,6 +31,11 @@
 
 #include "vice.h"
 
+#ifdef _FILE_OFFSET_BITS
+#undef _FILE_OFFSET_BITS
+#endif
+#define _FILE_OFFSET_BITS 64
+
 #include <assert.h>
 #include <ctype.h>
 #include <stdarg.h>
@@ -422,10 +427,10 @@ size_t util_file_length(FILE *fd)
 {
     size_t off, filesize;
 
-    off = ftell(fd);
-    fseek(fd, 0, SEEK_END);
-    filesize = ftell(fd);
-    fseek(fd, (long)off, SEEK_SET);
+    off = ftello(fd);
+    fseeko(fd, 0, SEEK_END);
+    filesize = ftello(fd);
+    fseeko(fd, (long)off, SEEK_SET);
     return filesize;
 }
 
@@ -435,7 +440,7 @@ int util_file_load(const char *name, uint8_t *dest, size_t size,
                    unsigned int load_flag)
 {
     FILE *fd;
-    size_t i, length, r = 0;
+    size_t length, r = 0;
     long start = 0;
 
     if (util_check_null_string(name)) {
@@ -456,34 +461,18 @@ int util_file_load(const char *name, uint8_t *dest, size_t size,
         length -= 2;
     }
 
-    if (length > size) {
+    if (length != size) {
         fclose(fd);
         return -1;
     }
 
-    if ((load_flag & UTIL_FILE_LOAD_FILL) == 0 && length != size) {
-        fclose(fd);
-        return -1;
-    }
-
-    for (i = 0; i < size; i += length) {
-        fseek(fd, start, SEEK_SET);
-        if (i + length > size) {
-            break;
-        }
-        r = fread((void *)&(dest[i]), length, 1, fd);
-/* log_debug("READ %i bytes to offset %i result %i.",length,i,r); */
-        if (r < 1) {
-            break;
-        }
-    }
-
+    fseek(fd, start, SEEK_SET);
+    r = fread(dest, 1, size, fd);
     fclose(fd);
 
-    if (r < 1) {
+    if (r < size) {
         return -1;
     }
-
     return 0;
 }
 
@@ -822,570 +811,6 @@ char *util_find_prev_line(const char *text, const char *pos)
 
 /* ------------------------------------------------------------------------- */
 
-/* The following are replacements for libc functions that could be missing.  */
-
-#if 0
-
-#if !defined HAVE_MEMMOVE
-void *memmove(void *target, const void *source, unsigned int length)
-{
-    char *tptr = (char *) target;
-    const char *sptr = (const char *) source;
-
-    if (tptr > sptr) {
-        tptr += length;
-        sptr += length;
-        while (length--) {
-            *(--tptr) = *(--sptr);
-        }
-    } else if (tptr < sptr) {
-        while (length--) {
-            *(tptr++) = *(sptr++);
-        }
-    }
-
-    return target;
-}
-#endif /* !defined HAVE_MEMMOVE */
-
-#if !defined HAVE_ATEXIT
-static void atexit_support_func(int status, void *arg)
-{
-    void (*f)(void) = (void (*)(void))arg;
-
-    f();
-}
-
-int atexit(void (*function)(void))
-{
-    return on_exit(atexit_support_func, (void *)function);
-}
-#endif /* !defined HAVE_ATEXIT */
-
-#if !defined HAVE_STRERROR
-char *strerror(int errnum)
-{
-    static char buffer[100];
-
-    sprintf(buffer, "Error %d", errnum);
-    return buffer;
-}
-#endif /* !defined HAVE_STRERROR */
-
-/* The following `strcasecmp()' and `strncasecmp()' implementations are
-   taken from:
-   GLIB - Library of useful routines for C programming
-   Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh
-   MacDonald.
-   The source is available from http://www.gtk.org/.  */
-
-#if !defined HAVE_STRCASECMP
-int strcasecmp(const char *s1, const char *s2)
-{
-    int c1, c2;
-
-    if (s1 == NULL || s2 == NULL) {
-        return 0;
-    }
-
-    while (*s1 && *s2) {
-        /* According to A. Cox, some platforms have islower's that don't work
-           right on non-uppercase.  */
-        c1 = isupper ((unsigned int)*s1) ? tolower ((unsigned int)*s1) : *s1;
-        c2 = isupper ((unsigned int)*s2) ? tolower ((unsigned int)*s2) : *s2;
-        if (c1 != c2) {
-            return (c1 - c2);
-        }
-        s1++; s2++;
-    }
-
-    return (((int)(unsigned char)*s1) - ((int)(unsigned char)*s2));
-}
-#endif
-
-#if !defined HAVE_STRNCASECMP
-int strncasecmp(const char *s1, const char *s2, size_t n)
-{
-    int c1, c2;
-
-    if (s1 == NULL || s2 == NULL) {
-        return 0;
-    }
-
-    while (n-- && *s1 && *s2) {
-        /* According to A. Cox, some platforms have islower's that don't work
-           right on non-uppercase.  */
-        c1 = isupper((unsigned int)*s1) ? tolower((unsigned int)*s1) : *s1;
-        c2 = isupper((unsigned int)*s2) ? tolower((unsigned int)*s2) : *s2;
-        if (c1 != c2) {
-            return (c1 - c2);
-        }
-        s1++;
-        s2++;
-    }
-
-    if (n) {
-        return (((int)(unsigned char)*s1) - ((int)(unsigned char)*s2));
-    } else {
-        return 0;
-    }
-}
-#endif
-
-#ifndef HAVE_STRTOK_R
-char *strtok_r(char *s, const char *delim, char **last)
-{
-    char *spanp;
-    int c, sc;
-    char *tok;
-
-    if (s == NULL && (s = *last) == NULL) {
-        return (NULL);
-    }
-
-cont:
-    c = *s++;
-    for (spanp = (char *)delim; (sc = *spanp++) != 0;) {
-        if (c == sc) {
-            goto cont;
-        }
-    }
-
-    if (c == 0) {
-        *last = NULL;
-        return (NULL);
-    }
-    tok = s - 1;
-
-    for (;;) {
-        c = *s++;
-        spanp = (char *)delim;
-        do {
-            if ((sc = *spanp++) == c) {
-                if (c == 0) {
-                    s = NULL;
-                } else {
-                    s[-1] = 0;
-                }
-                *last = s;
-                return (tok);
-            }
-        } while (sc != 0);
-    }
-}
-#endif
-
-#ifndef HAVE_STRTOUL
-unsigned long strtoul(const char *nptr, char **endptr, int base)
-{
-    const char *s = nptr;
-    unsigned long acc;
-    int c;
-    unsigned long cutoff;
-    int neg = 0;
-    int any;
-    int cutlim;
-
-
-    do {
-        c = *s++;
-    } while (isspace(c));
-
-    if (c == '-') {
-        neg = 1;
-        c = *s++;
-    } else if (c == '+') {
-        c = *s++;
-    }
-    if ((base == 0 || base == 16) && c == '0' && (*s == 'x' || *s == 'X')) {
-        c = s[1];
-        s += 2;
-        base = 16;
-    }
-    if (base == 0) {
-        base = c == '0' ? 8 : 10;
-    }
-
-    cutoff = (unsigned long)ULONG_MAX / (unsigned long)base;
-    cutlim = (unsigned long)ULONG_MAX % (unsigned long)base;
-
-    for (acc = 0, any = 0;; c = *s++) {
-        if (isdigit(c)) {
-            c -= '0';
-        } else if (isalpha(c)) {
-            c -= isupper(c) ? 'A' - 10 : 'a' - 10;
-        } else {
-            break;
-        }
-
-        if (c >= base) {
-            break;
-        }
-        if (any < 0 || acc > cutoff || (acc == cutoff && c > cutlim)) {
-            any = -1;
-        } else {
-            any = 1;
-            acc *= base;
-            acc += c;
-        }
-    }
-
-    if (endptr != 0) {
-        *endptr = (char *)(any ? s - 1 : nptr);
-    }
-
-    if (any < 0) {
-        acc = ULONG_MAX;
-        errno = ERANGE;
-    } else if (neg) {
-        return -acc;
-    }
-
-    return (acc);
-}
-#endif
-
-
-#ifndef HAVE_VSNPRINTF
-
-/* Taken from SDL */
-#ifndef HAVE_STRREV
-static char *strrev(char *string)
-{
-    size_t len = strlen(string);
-    char *a = &string[0];
-    char *b = &string[len - 1];
-    char c;
-
-    len /= 2;
-
-    while (len--) {
-        c = *a;
-        *a++ = *b;
-        *b-- = c;
-    }
-    return string;
-}
-#endif
-
-/* Taken from SDL */
-#ifndef HAVE_STRLWR
-static char *strlwr(char *string)
-{
-    char *bufp = string;
-
-    while (*bufp) {
-        *bufp = tolower((unsigned char)*bufp);
-        ++bufp;
-    }
-    return string;
-}
-#endif
-
-/* Taken from SDL */
-#ifndef HAVE_STRLCPY
-
-#define VICE_MIN(x, y) (((x) < (y)) ? (x) : (y))
-#define VICE_MAX(x, y) (((x) > (y)) ? (x) : (y))
-
-static size_t strlcpy(char *dst, const char *src, size_t maxlen)
-{
-    size_t srclen = strlen(src);
-    size_t len;
-
-    if (maxlen > 0) {
-        len = VICE_MIN(srclen, maxlen - 1);
-        memcpy(dst, src, len);
-        dst[len] = 0;
-    }
-    return srclen;
-}
-#endif
-
-#if !defined(HAVE_LTOA) || !defined(HAVE_ULTOA)
-static const char ntoa_table[] = {
-    '0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
-    'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J',
-    'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T',
-    'U', 'V', 'W', 'X', 'Y', 'Z'
-};
-#endif
-
-/* Taken from SDL */
-#ifndef HAVE_LTOA
-static char *ltoa(long value, char *string, int radix)
-{
-    char *bufp = string;
-
-    if (value < 0) {
-        *bufp++ = '-';
-        value = -value;
-    }
-    if (value) {
-        while (value > 0) {
-            *bufp++ = ntoa_table[value % radix];
-            value /= radix;
-        }
-    } else {
-        *bufp++ = '0';
-    }
-    *bufp = '\0';
-
-    if (*string == '-') {
-        strrev(string + 1);
-    } else {
-        strrev(string);
-    }
-
-    return string;
-}
-#endif
-
-/* Taken from SDL */
-#ifndef HAVE_ULTOA
-static char *ultoa(unsigned long value, char *string, int radix)
-{
-    char *bufp = string;
-
-    if (value) {
-        while (value > 0) {
-            *bufp++ = ntoa_table[value % radix];
-            value /= radix;
-        }
-    } else {
-        *bufp++ = '0';
-    }
-    *bufp = 0;
-
-    strrev(string);
-
-    return string;
-}
-#endif
-
-/* Taken from SDL */
-static size_t PrintLong(char *text, long value, int radix, size_t maxlen)
-{
-    char num[130];
-    size_t size;
-
-    ltoa(value, num, radix);
-    size = strlen(num);
-    if (size >= maxlen) {
-        size = maxlen - 1;
-    }
-    strlcpy(text, num, size + 1);
-
-    return size;
-}
-
-static size_t PrintUnsignedLong(char *text, unsigned long value, int radix, size_t maxlen)
-{
-    char num[130];
-    size_t size;
-
-    ultoa(value, num, radix);
-    size = strlen(num);
-    if (size >= maxlen) {
-        size = maxlen - 1;
-    }
-    strlcpy(text, num, size + 1);
-
-    return size;
-}
-
-static size_t PrintFloat(char *text, double arg, size_t maxlen)
-{
-    char *textstart = text;
-    const double precision = 0.00000001;
-    size_t len;
-    unsigned long value;
-    int mult;
-
-    if (arg) {
-        if (arg < 0) {
-            *text++ = '-';
-            --maxlen;
-            arg = -arg;
-        }
-        value = (unsigned long)arg;
-        len = PrintUnsignedLong(text, value, 10, maxlen);
-        text += len;
-        maxlen -= len;
-        arg -= value;
-        if (arg > precision && maxlen) {
-            mult = 10;
-            *text++ = '.';
-            while ((arg > precision) && maxlen) {
-                value = (unsigned long)(arg * mult);
-                len = PrintUnsignedLong(text, value, 10, maxlen);
-                text += len;
-                maxlen -= len;
-                arg -= (double)value / mult;
-                mult *= 10;
-            }
-        }
-    } else {
-        *text++ = '0';
-    }
-    return (text - textstart);
-}
-
-static size_t PrintString(char *text, const char *string, size_t maxlen)
-{
-    char *textstart = text;
-
-    while (*string && maxlen--) {
-        *text++ = *string++;
-    }
-    return (text - textstart);
-}
-
-int vsnprintf(char *text, size_t maxlen, const char *fmt, va_list ap)
-{
-    char *textstart = text;
-    int done;
-    size_t len;
-    int do_lowercase;
-    int radix;
-
-    enum {
-        DO_INT,
-        DO_LONG,
-        DO_LONGLONG
-    } inttype;
-
-
-    if (maxlen <= 0) {
-        return 0;
-    }
-    --maxlen;
-    while (*fmt && maxlen) {
-        if (*fmt == '%') {
-            done = 0;
-            len = 0;
-            do_lowercase = 0;
-            radix = 10;
-            inttype = DO_INT;
-
-            ++fmt;
-            while (*fmt == '.' || (*fmt >= '0' && *fmt <= '9')) {
-                ++fmt;
-            }
-            while (!done) {
-                switch (*fmt) {
-                    case '%':
-                        *text = '%';
-                        len = 1;
-                        done = 1;
-                        break;
-                    case 'c':
-                        *text = (char)va_arg(ap, int);
-                        len = 1;
-                        done = 1;
-                        break;
-                    case 'h':
-                        break;
-                    case 'l':
-                        if (inttype < DO_LONGLONG) {
-                            ++inttype;
-                        }
-                        break;
-                    case 'I':
-                        if (strncmp(fmt, "I64", 3) == 0) {
-                            fmt += 2;
-                            inttype = DO_LONGLONG;
-                        }
-                        break;
-                    case 'i':
-                    case 'd':
-                        switch (inttype) {
-                            case DO_INT:
-                                len = PrintLong(text, (long)va_arg(ap, int), radix, maxlen);
-                                break;
-                            case DO_LONG:
-                            case DO_LONGLONG:
-                                len = PrintLong(text, va_arg(ap, long), radix, maxlen);
-                                break;
-                        }
-                        done = 1;
-                        break;
-                    case 'p':
-                    case 'x':
-                        do_lowercase = 1;
-                    case 'X':
-                        if (radix == 10) {
-                            radix = 16;
-                        }
-                        if (*fmt == 'p') {
-                            inttype = DO_LONG;
-                        }
-                    case 'o':
-                        if (radix == 10) {
-                            radix = 8;
-                        }
-                    case 'u':
-                        switch (inttype) {
-                            case DO_INT:
-                                len = PrintUnsignedLong(text, (unsigned long)va_arg(ap, unsigned int), radix, maxlen);
-                                break;
-                            case DO_LONG:
-                            case DO_LONGLONG:
-                                len = PrintUnsignedLong(text, va_arg(ap, unsigned long), radix, maxlen);
-                                break;
-                        }
-                        if (do_lowercase) {
-                            strlwr(text);
-                        }
-                        done = 1;
-                        break;
-                    case 'f':
-                        len = PrintFloat(text, va_arg(ap, double), maxlen);
-                        done = 1;
-                        break;
-                    case 's':
-                        len = PrintString(text, va_arg(ap, char*), maxlen);
-                        done = 1;
-                        break;
-                    default:
-                        done = 1;
-                        break;
-                }
-                ++fmt;
-            }
-            text += len;
-            maxlen -= len;
-        } else {
-            *text++ = *fmt++;
-            --maxlen;
-        }
-    }
-    *text = 0;
-
-    return (text - textstart);
-}
-#endif
-
-/* Taken from SDL */
-#ifndef HAVE_SNPRINTF
-int snprintf(char *text, size_t maxlen, const char *fmt, ...)
-{
-    va_list ap;
-    int retval;
-
-    va_start(ap, fmt);
-    retval = vsnprintf(text, maxlen, fmt, ap);
-    va_end(ap);
-
-    return retval;
-}
-#endif
-
-#endif
-
-/*
-------------------------------------------------------------------------- */
-
 /** \brief  Add \a extension to a\ name if missing
  *
  * \param[in,out]   name        input and final 'string'
