@@ -38,39 +38,47 @@
 #define BAM_BIT_MAP_8050        6
 #define BAM_BIT_MAP_8250        BAM_BIT_MAP_8050
 #define BAM_BIT_MAP_4000        32
+#define BAM_BIT_MAP_9000        16
 #define BAM_VERSION_1541        165     /* position of DOS version (2A/2C)
                                            in BAM blk */
 #define BAM_VERSION_1581        25
 #define BAM_VERSION_8050        27
 #define BAM_VERSION_8250        BAM_VERSION_8050
 #define BAM_VERSION_4000        25
+#define BAM_VERSION_9000        27
 
 struct vdrive_s;
 
 extern int vdrive_bam_allocate_chain(struct vdrive_s *vdrive, unsigned int t,
                                      unsigned int s);
+extern int vdrive_bam_allocate_chain_255(struct vdrive_s *vdrive, unsigned int t,
+                                         unsigned int s);
 extern int vdrive_bam_alloc_first_free_sector(struct vdrive_s *vdrive,
                                               unsigned int *track,
                                               unsigned int *sector);
 extern int vdrive_bam_alloc_next_free_sector(struct vdrive_s *vdrive,
                                              unsigned int *track,
                                              unsigned int *sector);
+extern int vdrive_bam_alloc_next_free_sector_interleave(struct vdrive_s *vdrive,
+                                                        unsigned int *track,
+                                                        unsigned int *sector,
+                                                        unsigned int interleave);
 extern int vdrive_bam_allocate_sector(struct vdrive_s *vdrive,
                                       unsigned int track, unsigned int sector);
+extern int vdrive_bam_is_sector_allocated(struct vdrive_s *vdrive,
+                                          unsigned int track, unsigned int sector);
+
 extern void vdrive_bam_clear_all(struct vdrive_s *vdrive);
 extern void vdrive_bam_create_empty_bam(struct vdrive_s *vdrive,
                                         const char *name, uint8_t *id);
 extern int unsigned vdrive_bam_free_block_count(struct vdrive_s *vdrive);
 extern int vdrive_bam_free_sector(struct vdrive_s *vdrive,
                                   unsigned int track, unsigned int sector);
-extern int vdrive_bam_get_disk_id(unsigned int unit, uint8_t *id);
-extern int vdrive_bam_set_disk_id(unsigned int unit, uint8_t *id);
+extern int vdrive_bam_get_disk_id(unsigned int unit, unsigned int drive, uint8_t *id);
+extern int vdrive_bam_set_disk_id(unsigned int unit, unsigned int drive, uint8_t *id);
 extern int vdrive_bam_read_bam(struct vdrive_s *vdrive);
-extern int vdrive_bam_reread_bam(unsigned int unit);
+extern int vdrive_bam_reread_bam(unsigned int unit, unsigned int drive);
 extern int vdrive_bam_write_bam(struct vdrive_s *vdrive);
 extern int vdrive_bam_get_interleave(unsigned int type);
 
-extern int vdrive_bam_isset(uint8_t *bamp, unsigned int sector);
-extern uint8_t *vdrive_bam_get_track_entry(struct vdrive_s *vdrive, unsigned int track);
-
 #endif
