@@ -160,21 +160,21 @@ static const export_resource_t export_res = {
 
 /* ------------------------------------------------------------------------- */
 
-/* read 0x0400-0x0fff */
+/* read 0x0400-0x0fff (nvram 0x0400 - 0x0fff) */
 uint8_t megacart_ram123_read(uint16_t addr)
 {
     if (nvram_en_flop) {
-        return cart_nvram[addr & 0x1fff];
+        return cart_nvram[addr & 0x0fff];
     } else {
         return vic20_v_bus_last_data;
     }
 }
 
-/* store 0x0400-0x0fff */
+/* store 0x0400-0x0fff (nvram 0x0400 - 0x0fff) */
 void megacart_ram123_store(uint16_t addr, uint8_t value)
 {
     if (nvram_en_flop) {
-        cart_nvram[addr & 0x1fff] = value;
+        cart_nvram[addr & 0x0fff] = value;
     }
 }
 
@@ -278,14 +278,14 @@ void megacart_blk5_store(uint16_t addr, uint8_t value)
     }
 }
 
-/* read 0x9800-0x9bff */
+/* read 0x9800-0x9bff (nvram 0x1800 - 0x1bff) */
 static uint8_t megacart_io2_read(uint16_t addr)
 {
     uint8_t value;
 
     if (nvram_en_flop) {
         megacart_io2_device.io_source_valid = 1;
-        value = cart_nvram[addr & 0x1fff];
+        value = cart_nvram[0x1800 + (addr & 0x3ff)];
     } else {
         megacart_io2_device.io_source_valid = 0;
         value = vic20_cpu_last_data;
@@ -293,22 +293,22 @@ static uint8_t megacart_io2_read(uint16_t addr)
     return value;
 }
 
-/* store 0x9800-0x9bff */
+/* store 0x9800-0x9bff (nvram 0x1800 - 0x1bff) */
 static void megacart_io2_store(uint16_t addr, uint8_t value)
 {
     if (nvram_en_flop) {
-        cart_nvram[addr & 0x1fff] = value;
+        cart_nvram[0x1800 + (addr & 0x3ff)] = value;
     }
 }
 
-/* read 0x9c00-0x9fff */
+/* read 0x9c00-0x9fff (nvram 0x1c00 - 0x1fff) */
 static uint8_t megacart_io3_read(uint16_t addr)
 {
     uint8_t value;
 
     if (nvram_en_flop) {
         megacart_io3_device.io_source_valid = 1;
-        value = cart_nvram[addr & 0x1fff];
+        value = cart_nvram[0x1c00 + (addr & 0x3ff)];
     } else {
         megacart_io3_device.io_source_valid = 0;
         value = vic20_cpu_last_data;
@@ -326,14 +326,14 @@ static uint8_t megacart_io3_peek(uint16_t addr)
         return bank_low_reg;
     }
 
-    return cart_nvram[addr & 0x1fff];
+    return cart_nvram[0x1c00 + (addr & 0x3ff)];
 }
 
-/* store 0x9c00-0x9fff */
+/* store 0x9c00-0x9fff (nvram 0x1c00 - 0x1fff) */
 static void megacart_io3_store(uint16_t addr, uint8_t value)
 {
     if (nvram_en_flop) {
-        cart_nvram[addr & 0x1fff] = value;
+        cart_nvram[0x1c00 + (addr & 0x3ff)] = value;
     }
 
     if ((addr & 0x180) == 0x080) { /* $9c80 */
