@@ -56,6 +56,12 @@
 #include "vic20mem.h"
 #include "zfile.h"
 
+/*
+   UltiMem by Retro Innovations
+
+   see http://www.go4retro.com/products/ultimem/
+*/
+
 /* ------------------------------------------------------------------------- */
 /*
  * Cartridge RAM (512KiB or 1 MiB)
@@ -87,7 +93,7 @@ static uint8_t ultimem[17];
 
 /** Used bits in ultimem[] */
 static const uint8_t ultimem_mask[16] = {
-    ultimem_reg0_regs_disable | ultimem_reg0_led,
+    0xc7,
     0x3f, /* 00:IO3 config:IO2 config:RAM123 config */
     0xff, /* BLK5:BLK3:BLK2:BLK1 */
     0,
@@ -527,6 +533,9 @@ static void vic_um_io3_store(uint16_t addr, uint8_t value)
         switch (addr) {
             case 0:
                 value |= ultimem_reset[0];
+                if (value & 0x40) {
+                    machine_trigger_reset(MACHINE_RESET_MODE_HARD);
+                }
                 break;
             case 3:
                 return; /* not writable */
@@ -572,19 +581,25 @@ void vic_um_init(void)
 void vic_um_reset(void)
 {
     flash040core_reset(&flash_state);
-    memcpy(ultimem, ultimem_reset, sizeof ultimem);
-    switch (cart_rom_size) {
-        case CART_ROM_SIZE_8M:
-            ultimem[3] = ultimem_reg3_8m;
-            break;
-        case CART_ROM_SIZE_512K:
-            ultimem[3] = ultimem_reg3_512k;
-            break;
+    if (ultimem[0] & 0x40) {
+        /* soft reset triggered by write to $9ff0 */
+        ultimem[0] &= ~0x40;
+    } else {
+        memcpy(ultimem, ultimem_reset, sizeof ultimem);
+        switch (cart_rom_size) {
+            case CART_ROM_SIZE_8M:
+                ultimem[3] = ultimem_reg3_8m;
+                break;
+            case CART_ROM_SIZE_512K:
+                ultimem[3] = ultimem_reg3_512k;
+                break;
+        }
     }
 }
 
 void vic_um_config_setup(uint8_t *rawcart)
 {
+    memcpy(ultimem, ultimem_reset, sizeof ultimem);
 }
 
 int vic_um_bin_attach(const char *filename)
@@ -649,7 +664,7 @@ int vic_um_bin_attach(const char *filename)
 
 void vic_um_detach(void)
 {
-    int n = 0;
+    long n = 0;
     FILE *fd;
 
     /* try to write back cartridge contents if write back is enabled
@@ -755,8 +770,8 @@ int vic_um_snapshot_write_module(snapshot_t *s)
 
     if (0
         || (SMW_BA(m, ultimem, sizeof ultimem) < 0)
-        || (SMW_BA(m, cart_ram, cart_ram_size) < 0)
-        || (SMW_BA(m, cart_rom, cart_rom_size) < 0)) {
+        || (SMW_BA(m, cart_ram, (unsigned int)cart_ram_size) < 0)
+        || (SMW_BA(m, cart_rom, (unsigned int)cart_rom_size) < 0)) {
         snapshot_module_close(m);
         return -1;
     }
@@ -816,8 +831,8 @@ int vic_um_snapshot_read_module(snapshot_t *s)
     }
 
     if (0
-        || (SMR_BA(m, cart_ram, cart_ram_size) < 0)
-        || (SMR_BA(m, cart_rom, cart_rom_size) < 0)) {
+        || (SMR_BA(m, cart_ram, (unsigned int)cart_ram_size) < 0)
+        || (SMR_BA(m, cart_rom, (unsigned int)cart_rom_size) < 0)) {
         goto snapshot_error;
     }
 
