@@ -90,10 +90,16 @@ static read_func_ptr_t _mem_peek_tab[0x101];
 
 read_func_ptr_t *_mem_read_tab_ptr;
 store_func_ptr_t *_mem_write_tab_ptr;
+read_func_ptr_t *_mem_read_tab_ptr_dummy;
+store_func_ptr_t *_mem_write_tab_ptr_dummy;
 static uint8_t **_mem_read_base_tab_ptr;
 static int *mem_read_limit_tab_ptr;
 
-/* Current watchpoint state. 1 = watchpoints active, 0 = no watchpoints */
+/* Current watchpoint state. 
+          0 = no watchpoints
+    bit0; 1 = watchpoints active
+    bit1; 2 = watchpoints trigger on dummy accesses
+*/
 static int watchpoints_active = 0;
 
 /* ------------------------------------------------------------------------- */
@@ -516,13 +522,23 @@ void mem_toggle_watchpoints(int flag, void *context)
     if (flag) {
         _mem_read_tab_ptr = _mem_read_tab_watch;
         _mem_write_tab_ptr = _mem_write_tab_watch;
+        if (flag > 1) {
+            /* enable watchpoints on dummy accesses */
+            _mem_read_tab_ptr_dummy = _mem_read_tab_watch;
+            _mem_write_tab_ptr_dummy = _mem_write_tab_watch;
+        } else {
+            _mem_read_tab_ptr_dummy = _mem_read_tab_nowatch;
+            _mem_write_tab_ptr_dummy = _mem_write_tab_nowatch;
+        }
     } else {
+        /* all watchpoints disabled */
         _mem_read_tab_ptr = _mem_read_tab_nowatch;
         _mem_write_tab_ptr = _mem_write_tab_nowatch;
+        _mem_read_tab_ptr_dummy = _mem_read_tab_nowatch;
+        _mem_write_tab_ptr_dummy = _mem_write_tab_nowatch;
     }
     watchpoints_active = flag;
 }
-
 /* ------------------------------------------------------------------------- */
 
 /* Initialize RAM for power-up.  */
@@ -594,16 +610,25 @@ void mem_inject_key(uint16_t addr, uint8_t value)
 /* Banked memory access functions for the monitor */
 
 /* Exported banked memory access functions for the monitor */
+#define MAXBANKS (2)
 
-static const char *banknames[] = { "default", "cpu", NULL };
+/* by convention, a "bank array" has a 2-hex-digit bank index appended */
+static const char *banknames[MAXBANKS + 1] = { "default", "cpu", NULL };
 
-static const int banknums[] = { 0, 0 };
+static const int banknums[MAXBANKS + 1] = { 0, 0, -1 };
+static const int bankindex[MAXBANKS + 1] = { -1, -1, -1 };
+static const int bankflags[MAXBANKS + 1] = { 0, 0, -1 };
 
 const char **mem_bank_list(void)
 {
     return banknames;
 }
 
+const int *mem_bank_list_nos(void) {
+    return banknums;
+}
+
+/* return bank number for a given literal bank name */
 int mem_bank_from_name(const char *name)
 {
     int i = 0;
@@ -617,6 +642,33 @@ int mem_bank_from_name(const char *name)
     return -1;
 }
 
+/* return current index for a given bank */
+int mem_bank_index_from_bank(int bank)
+{
+    int i = 0;
+
+    while (banknums[i] > -1) {
+        if (banknums[i] == bank) {
+            return bankindex[i];
+        }
+        i++;
+    }
+    return -1;
+}
+
+int mem_bank_flags_from_bank(int bank)
+{
+    int i = 0;
+
+    while (banknums[i] > -1) {
+        if (banknums[i] == bank) {
+            return bankflags[i];
+        }
+        i++;
+    }
+    return -1;
+}
+
 uint8_t mem_bank_read(int bank, uint16_t addr, void *context)
 {
     return mem_read(addr);
